@@ -1,7 +1,7 @@
 #! perl
 
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: Configure.pl 19428 2007-06-28 22:50:36Z particle $
+# $Id: Configure.pl 20603 2007-08-11 02:11:28Z jkeenan $
 
 =head1 NAME
 
@@ -40,9 +40,10 @@ is setting.
 Tells Configure.pl to output information about i<every> setting added or
 changed.
 
-=item C<--verbose-step={N|regex}>
+=item C<--verbose-step={N|name|regex}>
 
-Run C<--verbose=2> for step number C<N> or matching description.
+Run one step with C<--verbose=2>; step indicated  by step number C<N>, by step
+name (I<e.g.>, C<--verbose-step=inter::make>), or by matching description.
 
 =item C<--nomanicheck>
 
@@ -289,8 +290,6 @@ F<lib/Parrot/Configure/Step.pm>, F<docs/configuration.pod>
 use 5.008_000;
 use strict;
 use warnings;
-use Data::Dumper;
-$Data::Dumper::Indent = 1;
 use lib 'lib';
 
 use Parrot::BuildUtil;
@@ -321,10 +320,9 @@ my $args = process_options( {
     argv            => [ @ARGV ],
     script          => $0,
     parrot_version  => $parrot_version,
-    svnid           => '$Id: Configure.pl 19428 2007-06-28 22:50:36Z particle $',
+    svnid           => '$Id: Configure.pl 20603 2007-08-11 02:11:28Z jkeenan $',
 } );
-exit unless defined $args;
-my %args = %{$args};
+exit(1) unless defined $args;
 
 my $opttest = Parrot::Configure::Options::Test->new($args);
 # configuration tests will only be run if you requested them
@@ -332,7 +330,7 @@ my $opttest = Parrot::Configure::Options::Test->new($args);
 $opttest->run_configure_tests();
 
 # from Parrot::Configure::Messages
-print_introduction($parrot_version) unless exists $args{step};
+print_introduction($parrot_version) unless exists $args->{step};
 
 my $conf = Parrot::Configure->new;
 
@@ -340,15 +338,15 @@ my $conf = Parrot::Configure->new;
 $conf->add_steps(get_steps_list());
 
 # from Parrot::Configure::Data
-$conf->options->set(%args);
+$conf->options->set(%{$args});
 
-if ( exists $args{step} ) {
+if ( exists $args->{step} ) {
     # from Parrot::Configure::Data
     $conf->data()->slurp();
     $conf->data()->slurp_temp()
-        if $args{step} =~ /gen::makefiles/;
+        if $args->{step} =~ /gen::makefiles/;
     # from Parrot::Configure
-    $conf->runstep( $args{step} );
+    $conf->run_single_step( $args->{step} );
     print "\n";
 }
 else {
@@ -362,8 +360,7 @@ else {
 $opttest->run_build_tests();
 
 # from Parrot::Configure::Messages
-print_conclusion($conf->data->get('make')) unless exists $args{step};
-
+print_conclusion($conf->data->get('make')) unless exists $args->{step};
 exit(0);
 
 ################### DOCUMENTATION ###################
