@@ -18,19 +18,15 @@
 
 /* HEADERIZER BEGIN: static */
 
-static void analyse_life_block( PARROT_INTERP,
-    NOTNULL(Basic_block* bb),
-    NOTNULL(SymReg* r) )
+static void analyse_life_block( NOTNULL(Basic_block* bb), NOTNULL(SymReg* r) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3);
+        __attribute__nonnull__(2);
 
-static void analyse_life_symbol( PARROT_INTERP,
+static void analyse_life_symbol(
     NOTNULL(const struct _IMC_Unit *unit),
     NOTNULL(SymReg* r) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3);
+        __attribute__nonnull__(2);
 
 static void bb_add_edge(
     NOTNULL(IMC_Unit *unit),
@@ -85,6 +81,8 @@ static void free_loops( NOTNULL(IMC_Unit *unit) )
 static void init_basic_blocks( NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static Basic_block* make_basic_block( PARROT_INTERP,
     NOTNULL(IMC_Unit *unit),
     NOTNULL(Instruction* ins) )
@@ -450,18 +448,19 @@ bb_add_edge(NOTNULL(IMC_Unit *unit), NOTNULL(Basic_block *from), NOTNULL(Basic_b
        on the predecessors of 'from', it won't be on the successors
        of 'to'. */
 
-    e = mem_sys_allocate(sizeof(*e));
+    e             = mem_allocate_typed(Edge);
 
-    e->succ_next = from->succ_list;
-    e->from = from;
+    e->succ_next  = from->succ_list;
+    e->from       = from;
 
-    e->pred_next = to->pred_list;
-    e->to = to;
+    e->pred_next  = to->pred_list;
+    e->to         = to;
 
-    from->succ_list = to->pred_list = e;
+    to->pred_list = from->succ_list = e;
 
     /* memory housekeeping */
     e->next = 0;
+
     if (unit->edge_list == 0)
         unit->edge_list = e;
     else {
@@ -509,6 +508,7 @@ bb_remove_edge(NOTNULL(IMC_Unit *unit), NOTNULL(Edge *edge))
             if (prev->next == edge) {
                 prev->next = edge->next;
                 free(edge);
+                break;
             }
         }
     }
@@ -549,11 +549,11 @@ life_analysis(PARROT_INTERP, NOTNULL(const struct _IMC_Unit *unit))
 
     IMCC_info(interp, 2, "life_analysis\n");
     for (i = 0; i < unit->n_symbols; i++)
-        analyse_life_symbol(interp, unit, reglist[i]);
+        analyse_life_symbol(unit, reglist[i]);
 }
 
 static void
-analyse_life_symbol(PARROT_INTERP, NOTNULL(const struct _IMC_Unit *unit),
+analyse_life_symbol(NOTNULL(const struct _IMC_Unit *unit),
         NOTNULL(SymReg* r))
 {
     int i;
@@ -564,14 +564,14 @@ analyse_life_symbol(PARROT_INTERP, NOTNULL(const struct _IMC_Unit *unit),
 
     if (r->life_info)
         free_life_info(unit, r);
-    r->life_info = mem_sys_allocate_zeroed(unit->n_basic_blocks *
-            sizeof (Life_range*));
+    r->life_info = (Life_range **)mem_sys_allocate_zeroed(unit->n_basic_blocks *
+            sizeof (Life_range *));
 
     /* First we make a pass to each block to gather the information
      * that can be obtained locally */
 
     for (i=0; i < unit->n_basic_blocks; i++) {
-        analyse_life_block(interp, unit->bb_list[i], r);
+        analyse_life_block(unit->bb_list[i], r);
     }
 
     /* Now we need to consider the relations between blocks */
@@ -634,21 +634,15 @@ free_life_info(NOTNULL(const struct _IMC_Unit *unit), NOTNULL(SymReg *r))
  */
 
 static void
-analyse_life_block(PARROT_INTERP, NOTNULL(Basic_block* bb), NOTNULL(SymReg* r))
+analyse_life_block(NOTNULL(Basic_block* bb), NOTNULL(SymReg* r))
 {
-    Instruction* ins, *special;
-    Life_range* l;
-    int is_alias;
+    Life_range* const l = make_life_range(r, bb->index);
 
-    l = make_life_range(r, bb->index);
+    Instruction *special = NULL;
+    Instruction *ins;
 
-    special = NULL;
     for (ins = bb->start; ins ; ins = ins->next) {
-        if (ins==NULL) {
-            IMCC_fatal(interp, 1,"analyse_life_block: "
-                    "Index %i of %i has NULL instruction\n",
-                    ins->index, bb->end->index);
-        }
+        int is_alias;
         /*
          * if we have a setp_ind opcode, it may write all PMC
          * registers
@@ -793,11 +787,13 @@ compute_dominators(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
 
     const int n = unit->n_basic_blocks;
     IMCC_info(interp, 2, "compute_dominators\n");
-    dominators = unit->dominators = malloc(sizeof (Set*) * n);
-    unit->idoms = malloc(sizeof (int) * n);
+
+    dominators  = unit->dominators = (Set **)malloc(sizeof (Set*) * n);
+    unit->idoms = (int *)malloc(sizeof (int) * n);
 
     dominators[0] = set_make(n);
     set_add(dominators[0], 0);
+
     for (i = 1; i < n; i++) {
         if (unit->bb_list[i]->pred_list) {
             dominators[i] = set_make_full(n);
@@ -809,11 +805,11 @@ compute_dominators(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
     }
 
 #if USE_BFS
-    q = calloc(n, sizeof (int));
+    q       = calloc(n, sizeof (int));
     visited = set_make(n);
     set_add(visited, 0);
-    len=1;
-    cur=0;
+    len     = 1;
+    cur     = 0;
 
     while (cur < len) {
         Edge *edge;
@@ -837,11 +833,12 @@ compute_dominators(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
 
         /* TODO: This 'for' should be a breadth-first search for speed */
         for (i = 1; i < n; i++) {
-            Set *s = set_copy(dominators[i]);
+            Set  *s = set_copy(dominators[i]);
             Edge *edge;
 
-            for (edge=unit->bb_list[i]->pred_list;
-                    edge; edge=edge->pred_next) {
+            for (edge = unit->bb_list[i]->pred_list;
+                edge;
+                edge = edge->pred_next) {
                 pred_index = edge->from->index;
                 set_intersec_inplace(s, dominators[pred_index]);
             }
@@ -858,8 +855,10 @@ compute_dominators(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
         }
     }
 #endif
+
     /* calc idoms */
     unit->idoms[0] = unit->bb_list[0]->index;
+
     for (b = 1; b < n; b++) {
         unit->idoms[b] = 0;
         for (i = n - 1; i > 0; i--) {
@@ -875,10 +874,12 @@ compute_dominators(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
                         }
                     }
                 }
+
                 if (!wrong) {
                     unit->idoms[b] = unit->bb_list[i]->index;
                     break;
                 }
+
             }
        }
    }
@@ -900,7 +901,8 @@ compute_dominance_frontiers(PARROT_INTERP, NOTNULL(struct _IMC_Unit *unit))
     int i, b;
 
     const int n = unit->n_basic_blocks;
-    Set ** const dominance_frontiers = unit->dominance_frontiers = malloc(sizeof (Set*) * n);
+    Set ** const dominance_frontiers = unit->dominance_frontiers =
+        (Set **)malloc(sizeof (Set *) * n);
 
     IMCC_info(interp, 2, "compute_dominance_frontiers\n");
 
@@ -1150,23 +1152,25 @@ mark_loop(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const Edge *e))
         }
     }
 
-    /* now 'loop' contains the set of nodes inside the loop.
-    */
-    n_loops = unit->n_loops;
+    /* now 'loop' contains the set of nodes inside the loop.  */
+    n_loops  = unit->n_loops;
     loop_info = unit->loop_info;
+
     if (!loop_info)
-        loop_info = unit->loop_info = mem_sys_allocate(
-                (n_loops+1)*sizeof (Loop_info *));
+        loop_info = unit->loop_info = (Loop_info **)mem_sys_allocate(
+                (n_loops + 1) * sizeof (Loop_info *));
     else
-        loop_info = unit->loop_info = mem_sys_realloc(loop_info,
-                (n_loops+1)*sizeof (Loop_info *));
-    loop_info[n_loops] = mem_sys_allocate(sizeof (Loop_info));
-    loop_info[n_loops]->loop = loop;
-    loop_info[n_loops]->exits = exits;
-    loop_info[n_loops]->depth = footer->loop_depth;
+        loop_info = unit->loop_info = (Loop_info **)mem_sys_realloc(loop_info,
+                (n_loops + 1) * sizeof (Loop_info *));
+
+    loop_info[n_loops]            = mem_allocate_typed(Loop_info);
+    loop_info[n_loops]->loop      = loop;
+    loop_info[n_loops]->exits     = exits;
+    loop_info[n_loops]->depth     = footer->loop_depth;
     loop_info[n_loops]->n_entries = i;
-    loop_info[n_loops]->header = header->index;
+    loop_info[n_loops]->header    = header->index;
     loop_info[n_loops]->preheader = natural_preheader(unit, loop_info[n_loops]);
+
     unit->n_loops++;
 }
 
@@ -1201,6 +1205,7 @@ search_predecessors_not_in(NOTNULL(const Basic_block *node), NOTNULL(Set* s))
    }
 
 }
+
 /*** Utility functions ***/
 
 static void
@@ -1208,11 +1213,12 @@ init_basic_blocks(NOTNULL(IMC_Unit *unit))
 {
     if (unit->bb_list != NULL)
         clear_basic_blocks(unit);
-    unit->bb_list =
-        calloc(unit->bb_list_size = 256,
-            sizeof (Basic_block*));
+
+    unit->bb_list = (Basic_block **)calloc(unit->bb_list_size = 256,
+            sizeof (Basic_block *));
+
     unit->n_basic_blocks = 0;
-    unit->edge_list = 0;
+    unit->edge_list      = 0;
 }
 
 void
@@ -1231,31 +1237,35 @@ clear_basic_blocks(NOTNULL(struct _IMC_Unit *unit))
     free_loops(unit);
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static Basic_block*
 make_basic_block(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(Instruction* ins))
 {
     int n;
-    Basic_block * const bb = mem_sys_allocate(sizeof (Basic_block));
+    Basic_block * const bb = mem_allocate_typed(Basic_block);
 
     if (ins == NULL) {
         PANIC(interp, "make_basic_block: called with NULL argument\n");
     }
 
-    bb->start = ins;
-    bb->end = ins;
+    bb->start      = ins;
+    bb->end        = ins;
 
-    bb->pred_list = NULL;
-    bb->succ_list = NULL;
-    n = unit->n_basic_blocks;
-    ins->bbindex = n;
-    bb->index = n;
+    bb->pred_list  = NULL;
+    bb->succ_list  = NULL;
+
+    n              = unit->n_basic_blocks;
+    ins->bbindex   = n;
+    bb->index      = n;
     bb->loop_depth = 0;
+
     if (n == unit->bb_list_size) {
         unit->bb_list_size *= 2;
-        unit->bb_list =
-            mem_sys_realloc(unit->bb_list,
-                unit->bb_list_size*sizeof (Basic_block*));
+        unit->bb_list = (Basic_block **)mem_sys_realloc(unit->bb_list,
+                unit->bb_list_size * sizeof (Basic_block *));
     }
+
     unit->bb_list[n] = bb;
     unit->n_basic_blocks++;
 
@@ -1263,13 +1273,13 @@ make_basic_block(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(Instruction* in
 }
 
 PARROT_MALLOC
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 Life_range *
 make_life_range(NOTNULL(SymReg *r), int idx)
 {
-   Life_range * const l = mem_sys_allocate_zeroed(sizeof(*l));
+   Life_range * const l = mem_allocate_zeroed_typed(Life_range);
+   r->life_info[idx]    = l;
 
-   r->life_info[idx] = l;
    return l;
 }
 
