@@ -1,4 +1,4 @@
-/* $Id: imc.h 19915 2007-07-16 22:08:22Z petdance $ */
+/* $Id: imc.h 20285 2007-07-28 16:42:44Z petdance $ */
 
 #ifndef PARROT_IMCC_IMC_H_GUARD
 #define PARROT_IMCC_IMC_H_GUARD
@@ -122,8 +122,12 @@ Instruction * IMCC_create_itcall_label( PARROT_INTERP )
 void IMCC_itcall_sub( PARROT_INTERP, SymReg* sub )
         __attribute__nonnull__(1);
 
-Instruction * INS_LABEL( PARROT_INTERP, IMC_Unit *unit, SymReg *r0, int emit )
-        __attribute__nonnull__(1);
+Instruction * INS_LABEL( PARROT_INTERP,
+    NOTNULL(IMC_Unit *unit),
+    SymReg *r0,
+    int emit )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 /* HEADERIZER END: compilers/imcc/imcparser.c */
 
@@ -154,6 +158,8 @@ int check_op( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * imcc_compile( PARROT_INTERP,
     NOTNULL(const char *s),
     int pasm_file,
@@ -162,10 +168,12 @@ PMC * imcc_compile( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(4);
 
+PARROT_CANNOT_RETURN_NULL
 void * IMCC_compile_file( PARROT_INTERP, NOTNULL(const char *s) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
 void * IMCC_compile_file_s( PARROT_INTERP,
     NOTNULL(const char *s),
     NOTNULL(STRING **error_message) )
@@ -265,11 +273,12 @@ Instruction * multi_keyed( PARROT_INTERP,
 void op_fullname(
     NOTNULL(char *dest),
     NOTNULL(const char *name),
-    SymReg * args[],
+    NOTNULL(SymReg *args[]),
     int narg,
     int keyvec )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 void register_compilers( PARROT_INTERP )
         __attribute__nonnull__(1);
@@ -372,8 +381,8 @@ typedef enum _imcc_reg_allocator_t {
     IMCC_GRAPH_ALLOCATOR
 } imcc_reg_allocator;
 
-PARROT_API void IMCC_push_parser_state(Interp*);
-PARROT_API void IMCC_pop_parser_state(Interp*, void *yyscanner);
+PARROT_API void IMCC_push_parser_state(PARROT_INTERP);
+PARROT_API void IMCC_pop_parser_state(PARROT_INTERP, void *yyscanner);
 
 typedef struct _imc_info_t {
     struct _imc_info_t *prev;
@@ -407,7 +416,7 @@ typedef struct _imc_info_t {
     int cur_pmc_type;
     SymReg *cur_call;
     SymReg *cur_obj;
-    char *adv_named_id;
+    const char *adv_named_id;
 
     /* Lex globals */
     int in_pod;
