@@ -1,8 +1,7 @@
-/* $Id: instructions.c 19915 2007-07-16 22:08:22Z petdance $ */
+/* $Id: instructions.c 20342 2007-07-30 05:57:58Z petdance $ */
 
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #define _PARSER
 #include "imc.h"
 #include "pbc.h"
@@ -33,12 +32,12 @@ int n_comp_units;
 
 /* HEADERIZER BEGIN: static */
 
-static int e_file_close( PARROT_INTERP, void *param )
+static int e_file_close( PARROT_INTERP, SHIM(void *param) )
         __attribute__nonnull__(1);
 
 static int e_file_emit( PARROT_INTERP,
-    void *param,
-    IMC_Unit *unit,
+    SHIM(void *param),
+    SHIM(IMC_Unit *unit),
     NOTNULL(const Instruction *ins) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(4);
@@ -53,8 +52,8 @@ const char types[] = "INPS";
 Emitter emitters[2] = {
     {e_file_open,
      e_file_emit,
-     (int (*)(Interp *, void *, IMC_Unit *))NULLfunc,
-     (int (*)(Interp *, void *, IMC_Unit *))NULLfunc,
+     NULL,
+     NULL,
      e_file_close},
 
     {e_pbc_open,
@@ -123,12 +122,12 @@ imcc_init_tables(PARROT_INTERP)
         size_t i;
         for (i = 0; i < N_ELEMENTS(reads); i++) {
             const int n = interp->op_lib->op_code(reads[i], 1);
-            assert(n);
+            PARROT_ASSERT(n);
             r_special[i] = n;
         }
         for (i = 0; i < N_ELEMENTS(writes); i++) {
             const int n = interp->op_lib->op_code(writes[i], 1);
-            assert(n);
+            PARROT_ASSERT(n);
             w_special[i] = n;
         }
     }
@@ -318,7 +317,7 @@ get_branch_reg(NOTNULL(const Instruction *ins))
     const int r = get_branch_regno(ins);
     if (r >= 0)
         return ins->r[r];
-    return 0;
+    return NULL;
 }
 
 /* some useful instruction routines */
@@ -451,7 +450,7 @@ emitb(PARROT_INTERP, NULLOK(struct _IMC_Unit *unit), NULLOK(Instruction *i))
 {
 
     if (!unit || !i)
-        return 0;
+        return NULL;
     if (!unit->instructions)
         unit->last_ins = unit->instructions = i;
     else {
@@ -499,15 +498,15 @@ ins_print(PARROT_INTERP, NOTNULL(FILE *fd), NOTNULL(const Instruction *ins))
             continue;
         if (p->type & VT_CONSTP)
             p = p->reg;
-        if (p->color >= 0 && (p->type & VTREGISTER)) {
+        if (p->color >= 0 && REG_NEEDS_ALLOC(p)) {
             sprintf(regb[i], "%c%d", p->set, (int)p->color);
             regstr[i] = regb[i];
         }
         else if (IMCC_INFO(interp)->allocated &&
                 (IMCC_INFO(interp)->optimizer_level & OPT_J) &&
                 p->set != 'K' &&
-                p->color < 0 && (p->type & VTREGISTER)) {
-                    sprintf(regb[i], "r%c%d", tolower(p->set), -1 - (int)p->color);
+                p->color < 0 && REG_NEEDS_ALLOC(p)) {
+                    sprintf(regb[i], "r%c%d", tolower((unsigned char)p->set), -1 - (int)p->color);
                     regstr[i] = regb[i];
         }
         else if (p->type & VTREGKEY) {
@@ -522,7 +521,7 @@ ins_print(PARROT_INTERP, NOTNULL(FILE *fd), NOTNULL(const Instruction *ins))
                         k->reg &&
                         k->reg->color < 0)
                     sprintf(regb[i]+strlen(regb[i]), "r%c%d",
-                            tolower(k->reg->set), -1 - (int)k->reg->color);
+                            tolower((unsigned char)k->reg->set), -1 - (int)k->reg->color);
                 else
                     strcat(regb[i], k->name);   /* XXX */
                 if (k->nextkey)
