@@ -115,14 +115,15 @@ static int dead_code_remove( PARROT_INTERP, IMC_Unit * unit )
 static int eval_ins( PARROT_INTERP, char *op, size_t ops, SymReg **r )
         __attribute__nonnull__(1);
 
-static Basic_block * find_outer(
+Basic_block * find_outer(
     NOTNULL(IMC_Unit *unit),
     NOTNULL(Basic_block *blk) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static int if_branch( PARROT_INTERP, IMC_Unit *unit )
-        __attribute__nonnull__(1);
+static int if_branch( PARROT_INTERP, NOTNULL(IMC_Unit *unit) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
 static int is_ins_save( PARROT_INTERP,
@@ -135,25 +136,25 @@ static int is_ins_save( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
-static int is_invariant( PARROT_INTERP,
+int is_invariant( PARROT_INTERP,
     NOTNULL(IMC_Unit * unit),
     NOTNULL(Instruction *ins) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-static int loop_one( PARROT_INTERP, NOTNULL(IMC_Unit *unit), int bnr )
+int loop_one( PARROT_INTERP, NOTNULL(IMC_Unit *unit), int bnr )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static int loop_optimization( PARROT_INTERP, NOTNULL(IMC_Unit *unit) )
+int loop_optimization( PARROT_INTERP, NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static int max_loop_depth( NOTNULL(IMC_Unit *unit) )
+int max_loop_depth( NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(1);
 
-static int move_ins_out( PARROT_INTERP,
+int move_ins_out( PARROT_INTERP,
     NOTNULL(IMC_Unit *unit),
     NOTNULL(Instruction **ins),
     NOTNULL(Basic_block *bb) )
@@ -174,7 +175,7 @@ static int used_once( PARROT_INTERP, NOTNULL(IMC_Unit *unit) )
 
 /* HEADERIZER END: static */
 #if DO_LOOP_OPTIMIZATION
-static int loop_optimization(Interp *, IMC_Unit *);
+int loop_optimization(Interp *, IMC_Unit *);
 #endif
 
 /*
@@ -281,7 +282,7 @@ get_neg_op(NOTNULL(const char *op), NOTNULL(int *n))
  *
  */
 static int
-if_branch(PARROT_INTERP, IMC_Unit *unit)
+if_branch(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
 {
     Instruction *ins, *last;
     int reg, changed = 0;
@@ -343,6 +344,7 @@ strength_reduce(PARROT_INTERP, IMC_Unit * unit)
     Instruction *ins, *tmp;
     SymReg *r;
     int changes = 0;
+    FLOATVAL f;
 
     IMCC_info(interp, 2, "\tstrength_reduce\n");
     for (ins = unit->instructions; ins; ins = ins->next) {
@@ -425,8 +427,8 @@ strength_reduce(PARROT_INTERP, IMC_Unit * unit)
                       IMCC_int_from_reg(interp, ins->r[1]) == 1)
           || ((ins->opnum == PARROT_OP_add_n_nc ||
                 ins->opnum == PARROT_OP_sub_n_nc) &&
-                      atof(ins->r[1]->name) == 0.0)
-          || ((ins->opnum == PARROT_OP_mul_n_nc ||
+                (f = atof(ins->r[1]->name), FLOAT_IS_ZERO(f)))
+          || ((ins->opnum  == PARROT_OP_mul_n_nc  ||
                 ins->opnum == PARROT_OP_div_n_nc ||
                 ins->opnum == PARROT_OP_fdiv_n_nc) &&
                       atof(ins->r[1]->name) == 1.0)) {
@@ -494,9 +496,9 @@ strength_reduce(PARROT_INTERP, IMC_Unit * unit)
                       IMCC_int_from_reg(interp, ins->r[1]) == 1)
           || ((ins->opnum == PARROT_OP_add_n_n_nc ||
                 ins->opnum == PARROT_OP_sub_n_n_nc) &&
-                      atof(ins->r[2]->name) == 0.0)
+                (f = atof(ins->r[2]->name), FLOAT_IS_ZERO(f)))
           || (ins->opnum == PARROT_OP_add_n_nc_n &&
-                      atof(ins->r[1]->name) == 0.0)
+             (f = atof(ins->r[1]->name), FLOAT_IS_ZERO(f)))
           || ((ins->opnum == PARROT_OP_mul_n_n_nc ||
                 ins->opnum == PARROT_OP_div_n_n_nc ||
                 ins->opnum == PARROT_OP_fdiv_n_n_nc) &&
@@ -529,10 +531,10 @@ strength_reduce(PARROT_INTERP, IMC_Unit * unit)
                 ins->opnum == PARROT_OP_mul_i_ic) &&
                       IMCC_int_from_reg(interp, ins->r[1]) == 0)
           || (ins->opnum == PARROT_OP_mul_n_n_nc &&
-                      atof(ins->r[2]->name) == 0.0)
+             (f = atof(ins->r[2]->name), FLOAT_IS_ZERO(f)))
           || ((ins->opnum == PARROT_OP_mul_n_nc_n ||
                 ins->opnum == PARROT_OP_mul_n_nc) &&
-                      atof(ins->r[1]->name) == 0.0)) {
+                (f = atof(ins->r[1]->name), FLOAT_IS_ZERO(f)))) {
             IMCC_debug(interp, DEBUG_OPT1, "opt1 %I => ", ins);
             r = mk_const(interp, str_dup("0"), ins->r[0]->set);
             --ins->r[1]->use_count;
@@ -552,8 +554,8 @@ strength_reduce(PARROT_INTERP, IMC_Unit * unit)
         if ((ins->opnum == PARROT_OP_set_i_ic &&
                       IMCC_int_from_reg(interp, ins->r[1]) == 0)
           || (ins->opnum == PARROT_OP_set_n_nc &&
-                      atof(ins->r[1]->name) == 0.0 &&
-                      ins->r[1]->name[0] != '-')) {
+             (f = atof(ins->r[1]->name), FLOAT_IS_ZERO(f)) &&
+              ins->r[1]->name[0] != '-')) {
             IMCC_debug(interp, DEBUG_OPT1, "opt1 %I => ", ins);
             --ins->r[1]->use_count;
             tmp = INS(interp, unit, "null", "", ins->r, 1, 0, 0);
@@ -721,7 +723,7 @@ eval_ins(PARROT_INTERP, char *op, size_t ops, SymReg **r)
     for (i = 0; i < op_info->op_count - 1; i++) {
         switch (op_info->types[i]) {
             case PARROT_ARG_IC:
-                assert(i && ops == (unsigned int)i);
+                PARROT_ASSERT(i && ops == (unsigned int)i);
                 /* set branch offset to zero */
                 eval[i + 1] = 0;
                 break;
@@ -766,7 +768,7 @@ eval_ins(PARROT_INTERP, char *op, size_t ops, SymReg **r)
     /* the returned pc is either incremented by op_count or is eval,
      * as the branch offset is 0 - return true if it branched
      */
-    assert(pc == eval + op_info->op_count || pc == eval);
+    PARROT_ASSERT(pc == eval + op_info->op_count || pc == eval);
     return pc == eval;
 }
 
@@ -838,8 +840,8 @@ IMCC_subst_constants(PARROT_INTERP, IMC_Unit * unit, NOTNULL(const char *name),
             /*
              * create instruction e.g. add_i_ic_ic => add_i_i_i
              */
-            sprintf(op, "%s_%c_%c_%c", name, tolower(r[0]->set),
-                    tolower(r[1]->set), tolower(r[2]->set));
+            sprintf(op, "%s_%c_%c_%c", name, tolower((unsigned char)r[0]->set),
+                    tolower((unsigned char)r[1]->set), tolower((unsigned char)r[2]->set));
             debug_fmt = "opt %s_x_xc_xc => ";
             break;
         }
@@ -852,8 +854,8 @@ IMCC_subst_constants(PARROT_INTERP, IMC_Unit * unit, NOTNULL(const char *name),
                 r[1]->type & (VTCONST|VT_CONSTP) &&
                 !strcmp(name, ops2[i])) {
             found = 3;
-            sprintf(op, "%s_%c_%c", name, tolower(r[0]->set),
-                    tolower(r[1]->set));
+            sprintf(op, "%s_%c_%c", name, tolower((unsigned char)r[0]->set),
+                    tolower((unsigned char)r[1]->set));
             debug_fmt = "opt %s_x_xc => ";
             break;
         }
@@ -867,8 +869,8 @@ IMCC_subst_constants(PARROT_INTERP, IMC_Unit * unit, NOTNULL(const char *name),
                 r[1]->type & (VTCONST|VT_CONSTP)  &&
                 !strcmp(name, ops3[i])) {
             found = 2;
-            sprintf(op, "%s_%c_%c_ic", name, tolower(r[0]->set),
-                    tolower(r[1]->set));
+            sprintf(op, "%s_%c_%c_ic", name, tolower((unsigned char)r[0]->set),
+                    tolower((unsigned char)r[1]->set));
             debug_fmt = "opt %s_xc_xc_ic => ";
             break;
         }
@@ -881,7 +883,7 @@ IMCC_subst_constants(PARROT_INTERP, IMC_Unit * unit, NOTNULL(const char *name),
                 r[0]->type & (VTCONST|VT_CONSTP) &&
                 !strcmp(name, ops4[i])) {
             found = 1;
-            sprintf(op, "%s_%c_ic", name, tolower(r[0]->set));
+            sprintf(op, "%s_%c_ic", name, tolower((unsigned char)r[0]->set));
             debug_fmt = "opt %s_xc_ic => ";
             break;
         }
@@ -1068,15 +1070,13 @@ branch_cond_loop_swap(PARROT_INTERP, IMC_Unit *unit, Instruction *branch,
 {
     int changed = 0;
     int args;
-    const char *neg_op = get_neg_op(cond->op, &args);
+    const char * const neg_op = get_neg_op(cond->op, &args);
     if (neg_op) {
-        char *label;
-        int count, size, found;
+        const size_t size  = strlen(branch->r[0]->name) + 10; /* + '_post999' */
+        char * const label = (char *)malloc(size);
+        int count;
+        int found = 0;
 
-        size = strlen(branch->r[0]->name) + 10; /* + '_post999' */
-
-        label = malloc(size);
-        found = 0;
         for (count = 1; count != 999; ++count) {
             sprintf(label, "%s_post%d", branch->r[0]->name, count);
             if (get_sym(interp, label) == 0) {
@@ -1090,7 +1090,7 @@ branch_cond_loop_swap(PARROT_INTERP, IMC_Unit *unit, Instruction *branch,
             SymReg *regs[3], *r;
 
             /* cond_op has 2 or 3 args */
-            assert(args <= 3);
+            PARROT_ASSERT(args <= 3);
 
             r = mk_local_label(interp, str_dup(label));
             tmp = INS_LABEL(interp, unit, r, 0);
@@ -1111,7 +1111,7 @@ branch_cond_loop_swap(PARROT_INTERP, IMC_Unit *unit, Instruction *branch,
 
             regs[get_branch_regno(cond)] =
                 mk_label_address(interp, str_dup(label));
-            tmp = INS(interp, unit, (char*)neg_op, "", regs, args, 0, 0);
+            tmp = INS(interp, unit, (const char*)neg_op, "", regs, args, 0, 0);
 
             IMCC_debug(interp, DEBUG_OPT1,
             "loop %s -> %s converted to post-test, added label %s\n",
@@ -1476,18 +1476,19 @@ is_ins_save(PARROT_INTERP, NOTNULL(IMC_Unit *unit),
 }
 
 #if DO_LOOP_OPTIMIZATION
-static int
+int
 max_loop_depth(NOTNULL(IMC_Unit *unit))
 {
-    int i, d;
-    d = 0;
+    int i;
+    int d = 0;
+
     for (i = 0; i < unit->n_basic_blocks; i++)
         if (unit->bb_list[i]->loop_depth > d)
             d = unit->bb_list[i]->loop_depth;
     return d;
 }
 
-static int
+int
 is_invariant(PARROT_INTERP, NOTNULL(IMC_Unit * unit), NOTNULL(Instruction *ins))
 {
     int ok = 0;
@@ -1510,7 +1511,7 @@ is_invariant(PARROT_INTERP, NOTNULL(IMC_Unit * unit), NOTNULL(Instruction *ins))
 
 #  define MOVE_INS_1_BL
 #  ifdef MOVE_INS_1_BL
-static Basic_block *
+Basic_block *
 find_outer(NOTNULL(IMC_Unit *unit), NOTNULL(Basic_block *blk))
 {
     int bb = blk->index;
@@ -1528,7 +1529,7 @@ find_outer(NOTNULL(IMC_Unit *unit), NOTNULL(Basic_block *blk))
 #  endif
 
 /* move the instruction ins before loop in bb */
-static int
+int
 move_ins_out(PARROT_INTERP, NOTNULL(IMC_Unit *unit),
         NOTNULL(Instruction **ins), NOTNULL(Basic_block *bb))
 {
@@ -1569,7 +1570,7 @@ move_ins_out(PARROT_INTERP, NOTNULL(IMC_Unit *unit),
     return 1;
 }
 
-static int
+int
 loop_one(PARROT_INTERP, NOTNULL(IMC_Unit *unit), int bnr)
 {
     Basic_block * const bb = unit->bb_list[bnr];
@@ -1597,7 +1598,7 @@ loop_one(PARROT_INTERP, NOTNULL(IMC_Unit *unit), int bnr)
 
 }
 
-static int
+int
 loop_optimization(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
 {
     int l, bb, loop_depth;
