@@ -8,14 +8,13 @@
  *
  * parser support functions
  *
- * $Id: parser_util.c 19914 2007-07-16 21:06:48Z petdance $
+ * $Id: parser_util.c 20690 2007-08-19 00:29:46Z chromatic $
  *
  */
 
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 
 #define _PARSER
 
@@ -33,13 +32,15 @@
 
 PARROT_WARN_UNUSED_RESULT
 static int change_op( PARROT_INTERP,
-    IMC_Unit *unit,
+    NOTNULL(IMC_Unit *unit),
     NOTNULL(SymReg **r),
     int num,
     int emit )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CANNOT_RETURN_NULL
 static void * imcc_compile_file( PARROT_INTERP,
     NOTNULL(const char *fullname),
     NOTNULL(STRING **error_message) )
@@ -74,10 +75,13 @@ static const char * to_infix( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
-static const char * try_rev_cmp( SHIM_INTERP,
-    IMC_Unit * unit,
-    const char *name,
-    SymReg ** r );
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
+static const char * try_rev_cmp(
+    NOTNULL(const char *name),
+    NOTNULL(SymReg **r) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
 static Instruction * var_arg_ins( PARROT_INTERP,
@@ -161,7 +165,7 @@ iNEW(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(SymReg *r0),
  * out, but please don't remove it. :) -Mel
  */
 void
-op_fullname(NOTNULL(char *dest), NOTNULL(const char *name), SymReg * args[],
+op_fullname(NOTNULL(char *dest), NOTNULL(const char *name), NOTNULL(SymReg *args[]),
         int narg, int keyvec)
 {
     int i;
@@ -197,17 +201,17 @@ op_fullname(NOTNULL(char *dest), NOTNULL(const char *name), SymReg * args[],
         if (args[i]->set == 'K')
             *dest++ = 'p';
         else
-            *dest++ = tolower(args[i]->set);
+            *dest++ = tolower((unsigned char)args[i]->set);
 
         if (args[i]->type & (VTCONST|VT_CONSTP)) {
 #if IMC_TRACE_HIGH
-            PIO_eprintf(NULL, " (%cc)%s", tolower(args[i]->set), args[i]->name);
+            PIO_eprintf(NULL, " (%cc)%s", tolower((unsigned char)args[i]->set), args[i]->name);
 #endif
             *dest++ = 'c';
         }
 #if IMC_TRACE_HIGH
         else
-            PIO_eprintf(NULL, " (%c)%s", tolower(args[i]->set), args[i]->name);
+            PIO_eprintf(NULL, " (%c)%s", tolower((unsigned char)args[i]->set), args[i]->name);
 #endif
     }
     *dest = '\0';
@@ -248,7 +252,7 @@ maybe_builtin(PARROT_INTERP, NOTNULL(const char *name),
     SymReg *sub, *meth, *rr[10];
     int first_arg, is_void;
 
-    assert(n < 15);
+    PARROT_ASSERT(n < 15);
     for (i = 0; i < n; ++i) {
         sig[i] = r[i]->set;
         rr[i] = r[i];
@@ -319,7 +323,7 @@ to_infix(PARROT_INTERP, NOTNULL(const char *name), NOTNULL(SymReg **r),
     SymReg *mmd;
     int is_n;
 
-    assert(*n >= 2);
+    PARROT_ASSERT(*n >= 2);
     is_n = (IMCC_INFO(interp)->state->pragmas & PR_N_OPERATORS) ||
         (name[0] == 'n' && name[1] == '_') ||
         (mmd_op == MMD_LOR || mmd_op == MMD_LAND || mmd_op == MMD_LXOR);
@@ -422,7 +426,7 @@ var_arg_ins(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const char *name),
     dirs = 1;           /* in constant */
     op_fullname(fullname, name, r, 1, 0);
     op = interp->op_lib->op_code(fullname, 1);
-    assert(op >= 0);
+    PARROT_ASSERT(op >= 0);
 
     ins = _mk_instruction(name, "", n, r, dirs);
     ins->opnum = op;
@@ -464,7 +468,9 @@ INS(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const char *name),
         (strcmp(name, "set_returns") == 0)) {
         return var_arg_ins(interp, unit, name, r, n, emit);
     }
-    if ((op = is_infix(name, n, r)) >= 0) {
+
+    op = is_infix(name, n, r);
+    if (op >= 0) {
         /* sub x, y, z  => infix .MMD_SUBTRACT, x, y, z */
         name = to_infix(interp, name, r, &n, op);
     }
@@ -490,12 +496,12 @@ INS(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const char *name),
     if (op < 0)         /* maybe we got a fullname */
         op = interp->op_lib->op_code(name, 1);
     if (op < 0) {         /* still wrong, try reverse compare */
-        const char * const n_name = try_rev_cmp(interp, unit, name, r);
+        const char * const n_name = try_rev_cmp(name, r);
         if (n_name) {
             DECL_CONST_CAST;
-            name = const_cast(n_name);
+            name = (char *)const_cast(n_name);
             op_fullname(fullname, name, r, n, keyvec);
-            op = interp->op_lib->op_code(fullname, 1);
+            op   = interp->op_lib->op_code(fullname, 1);
         }
     }
     if (op < 0)         /* still wrong, try to find an existing op */
@@ -552,7 +558,7 @@ INS(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const char *name),
                 break;
 
             default:
-                assert(0);
+                PARROT_ASSERT(0);
         };
         if (keyvec & KEY_BIT(i)) {
             len = strlen(format);
@@ -627,8 +633,7 @@ INS(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(const char *name),
     }
     else if (!strcmp(name, "set") && n == 2) {
         /* set Px, Py: both PMCs have the same address */
-        if (r[0]->set == r[1]->set &&
-                (r[1]->type & VTREGISTER))
+        if (r[0]->set == r[1]->set && REG_NEEDS_ALLOC(r[1]))
             ins->type |= ITALIAS;
     }
     else if (!strcmp(name, "compile"))
@@ -661,6 +666,8 @@ do_yylex_init(PARROT_INTERP, NOTNULL(yyscan_t* yyscanner))
     return retval;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 imcc_compile(PARROT_INTERP, NOTNULL(const char *s), int pasm_file,
              NOTNULL(STRING **error_message))
@@ -676,6 +683,7 @@ imcc_compile(PARROT_INTERP, NOTNULL(const char *s), int pasm_file,
     DECL_CONST_CAST;
     INTVAL regs_used[4] = {3,3,3,3};
     void *yyscanner;
+    Parrot_Context *ignored;
 
     do_yylex_init( interp, &yyscanner );
 
@@ -685,9 +693,9 @@ imcc_compile(PARROT_INTERP, NOTNULL(const char *s), int pasm_file,
     Parrot_block_DOD(interp);
     if (IMCC_INFO(interp)->last_unit) {
         /* got a reentrant compile */
-        imc_info = mem_sys_allocate_zeroed(sizeof (imc_info_t));
-        imc_info->ghash = IMCC_INFO(interp)->ghash;
-        imc_info->prev = IMCC_INFO(interp);
+        imc_info          = mem_allocate_zeroed_typed(imc_info_t);
+        imc_info->ghash   = IMCC_INFO(interp)->ghash;
+        imc_info->prev    = IMCC_INFO(interp);
         IMCC_INFO(interp) = imc_info;
     }
 
@@ -707,12 +715,14 @@ imcc_compile(PARROT_INTERP, NOTNULL(const char *s), int pasm_file,
     next = IMCC_INFO(interp)->state->next;
     if (imc_info)
         IMCC_INFO(interp)->state->next = NULL;
-    IMCC_INFO(interp)->state->pasm_file = pasm_file;
-    IMCC_INFO(interp)->state->file = name;
-    IMCC_INFO(interp)->expect_pasm = 0;
-    Parrot_push_context(interp, regs_used);
 
-    compile_string(interp, const_cast(s), yyscanner);
+    IMCC_INFO(interp)->state->pasm_file = pasm_file;
+    IMCC_INFO(interp)->state->file      = name;
+    IMCC_INFO(interp)->expect_pasm      = 0;
+
+    ignored = Parrot_push_context(interp, regs_used);
+
+    compile_string(interp, (char *)const_cast(s), yyscanner);
 
     Parrot_pop_context(interp);
     /*
@@ -755,6 +765,8 @@ imcc_compile(PARROT_INTERP, NOTNULL(const char *s), int pasm_file,
         mem_sys_free(imc_info);
         imc_info = IMCC_INFO(interp);
         IMCC_INFO(interp)->cur_unit = imc_info->last_unit;
+        if (IMCC_INFO(interp)->cur_namespace)
+            free_sym(IMCC_INFO(interp)->cur_namespace);
         IMCC_INFO(interp)->cur_namespace = imc_info->cur_namespace;
     }
     else
@@ -832,6 +844,8 @@ imcc_compile_pir_ex(PARROT_INTERP, NOTNULL(const char *s))
 /*
  * Compile a file by filename (can be either PASM or IMCC code)
  */
+
+PARROT_CANNOT_RETURN_NULL
 static void *
 imcc_compile_file(PARROT_INTERP, NOTNULL(const char *fullname),
                    NOTNULL(STRING **error_message))
@@ -842,6 +856,8 @@ imcc_compile_file(PARROT_INTERP, NOTNULL(const char *fullname),
     FILE *fp;
     struct _imc_info_t *imc_info = NULL;
     STRING *fs;
+    Parrot_Context *ignored;
+
     /* need at least 3 regs for compilation of constant math e.g.
      * add_i_ic_ic - see also IMCC_subst_constants()
      */
@@ -849,24 +865,20 @@ imcc_compile_file(PARROT_INTERP, NOTNULL(const char *fullname),
 
     if (IMCC_INFO(interp)->last_unit) {
         /* got a reentrant compile */
-        imc_info = mem_sys_allocate_zeroed(sizeof (imc_info_t));
-        imc_info->ghash = IMCC_INFO(interp)->ghash;
-        imc_info->prev = IMCC_INFO(interp);
+        imc_info          = mem_allocate_zeroed_typed(imc_info_t);
+        imc_info->prev    = IMCC_INFO(interp);
+        imc_info->ghash   = IMCC_INFO(interp)->ghash;
         IMCC_INFO(interp) = imc_info;
     }
 
     fs = string_make(interp, fullname, strlen(fullname), NULL, 0);
-    if (Parrot_stat_info_intval(interp, fs, STAT_ISDIR)) {
+    if (Parrot_stat_info_intval(interp, fs, STAT_ISDIR))
         real_exception(interp, NULL, E_IOError,
                 "imcc_compile_file: '%s' is a directory\n", fullname);
-        return NULL;
-    }
 
-    if (!(fp = fopen(fullname, "r"))) {
+    if (!(fp = fopen(fullname, "r")))
         IMCC_fatal(interp, E_IOError,
                 "imcc_compile_file: couldn't open '%s'\n", fullname);
-        return NULL;
-    }
 
 #if IMC_TRACE
     fprintf(stderr, "parser_util.c: imcc_compile_file '%s'\n", fullname);
@@ -884,7 +896,7 @@ imcc_compile_file(PARROT_INTERP, NOTNULL(const char *fullname),
      * which can destroy packfiles under construction
      */
     Parrot_block_DOD(interp);
-    Parrot_push_context(interp, regs_used);
+    ignored = Parrot_push_context(interp, regs_used);
 
     if (ext && strcmp(ext, ".pasm") == 0) {
         void *yyscanner;
@@ -931,6 +943,7 @@ imcc_compile_file(PARROT_INTERP, NOTNULL(const char *fullname),
  * Note: This function is provided for backward compatibility. This
  * function can go away in future.
  */
+PARROT_CANNOT_RETURN_NULL
 void *
 IMCC_compile_file(PARROT_INTERP, NOTNULL(const char *s))
 {
@@ -938,6 +951,7 @@ IMCC_compile_file(PARROT_INTERP, NOTNULL(const char *s))
     return imcc_compile_file(interp, s, &error_message);
 }
 
+PARROT_CANNOT_RETURN_NULL
 void *
 IMCC_compile_file_s(PARROT_INTERP, NOTNULL(const char *s),
                    NOTNULL(STRING **error_message))
@@ -960,13 +974,13 @@ register_compilers(PARROT_INTERP)
 
 PARROT_WARN_UNUSED_RESULT
 static int
-change_op(PARROT_INTERP, IMC_Unit *unit, NOTNULL(SymReg **r), int num, int emit)
+change_op(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(SymReg **r), int num, int emit)
 {
     int changed = 0;
 
     if (r[num]->type & (VTCONST|VT_CONSTP)) {
         /* make a number const */
-        SymReg *c = r[num];
+        const SymReg *c = r[num];
         SymReg *s;
         if (c->type & VT_CONSTP)
             c = c->reg;
@@ -1087,14 +1101,15 @@ try_find_op(PARROT_INTERP, IMC_Unit * unit, NOTNULL(const char *name),
     return -1;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static const char *
-try_rev_cmp(SHIM_INTERP, SHIM(IMC_Unit * unit), const char *name,
-        SymReg ** r)
+try_rev_cmp(NOTNULL(const char *name), NOTNULL(SymReg **r))
 {
     static struct br_pairs {
-        const char *op;
-        const char *nop;
-        int to_swap;
+        NOTNULL( const char * const op );
+        NOTNULL( const char * const nop );
+        const int to_swap;
     } br_pairs[] = {
         { "gt", "lt", 0 },
         { "ge", "le", 0 },
@@ -1137,7 +1152,7 @@ multi_keyed(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(char *name),
     if (keyf <= 1)
         return 0;
     /* XXX what to do, if we don't emit instruction? */
-    assert(emit);
+    PARROT_ASSERT(emit);
     UNUSED(emit);
     /* OP  _p_k    _p_k_p_k =>
      * set      py, p_k
@@ -1299,7 +1314,7 @@ imcc_vfprintf(PARROT_INTERP, NOTNULL(FILE *fd), NOTNULL(const char *format), va_
                 {
                 const char * const _string = va_arg(ap, char *);
                 memcpy(buf, cp, n = (fmt - cp));
-                assert(n<128);
+                PARROT_ASSERT(n<128);
                 buf[n] = '\0';
                 len += fprintf(fd, buf, _string);
                 }
@@ -1327,7 +1342,7 @@ char *
 str_dup(NOTNULL(const char *old))
 {
     const size_t bytes = strlen(old) + 1;
-    char * const copy = mem_sys_allocate(bytes);
+    char * const copy  = (char *)mem_sys_allocate(bytes);
     memcpy(copy, old, bytes);
 #ifdef MEMDEBUG
     debug(interp, 1,"line %d str_dup %s [%x]\n", line, old, copy);
@@ -1339,7 +1354,7 @@ PARROT_API
 void
 imcc_init(PARROT_INTERP)
 {
-    IMCC_INFO(interp) = mem_sys_allocate_zeroed(sizeof (imc_info_t));
+    IMCC_INFO(interp) = mem_allocate_zeroed_typed(imc_info_t);
     /* register PASM and PIR compilers to parrot core */
     register_compilers(interp);
 }
