@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2007, The Perl Foundation.
- * $Id: pbc.c 19912 2007-07-16 20:19:17Z petdance $
+ * $Id: pbc.c 20690 2007-08-19 00:29:46Z chromatic $
  */
 
 #include "imc.h"
@@ -137,7 +137,10 @@ static int get_old_size( PARROT_INTERP, NOTNULL(int *ins_line) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static void imcc_globals_destroy( SHIM_INTERP, int ex, void *param );
+static void imcc_globals_destroy( SHIM_INTERP,
+    SHIM(int ex),
+    SHIM(void *param) );
+
 static void make_new_sub( NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(1);
 
@@ -279,7 +282,8 @@ make_jit_info(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
     size_t size, old;
 
     if (!globals.cs->jit_info) {
-        char * const name = malloc(strlen(globals.cs->seg->base.name) + 5);
+        char * const name = (char *)
+            malloc(strlen(globals.cs->seg->base.name) + 5);
         sprintf(name, "%s_JIT", globals.cs->seg->base.name);
         globals.cs->jit_info =
             PackFile_Segment_new_seg(interp,
@@ -293,7 +297,7 @@ make_jit_info(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
     globals.cs->subs->n_basic_blocks = unit->n_basic_blocks;
 
     /* offset of block start and end, 4 * registers_used */
-    globals.cs->jit_info->data = realloc(globals.cs->jit_info->data,
+    globals.cs->jit_info->data = (opcode_t *)realloc(globals.cs->jit_info->data,
             size * sizeof (opcode_t) * 6);
 
     globals.cs->jit_info->size = size * 6;
@@ -481,16 +485,16 @@ fixup_globals(PARROT_INTERP)
                  *   convert to find_name, if the sub is a multi
                  */
                 if (s1) {
-                    assert(s1->unit);
+                    PARROT_ASSERT(s1->unit);
                     if (s1->unit->type & IMC_PCCSUB) {
                         ins     = s1->unit->instructions;
-                        assert(ins);
+                        PARROT_ASSERT(ins);
 
                         r1      = ins->r[0];
-                        assert(r1);
+                        PARROT_ASSERT(r1);
 
                         pcc_sub = r1->pcc_sub;
-                        assert(pcc_sub);
+                        PARROT_ASSERT(pcc_sub);
 
                         /* if the sub is multi, don't insert constant */
                         if (pcc_sub->nmulti)
@@ -502,7 +506,7 @@ fixup_globals(PARROT_INTERP)
                     SymReg *nam;
 
                     op = interp->op_lib->op_code("find_name_p_sc", 1);
-                    assert(op);
+                    PARROT_ASSERT(op);
 
                     interp->code->base.data[addr] = op;
 
@@ -556,7 +560,7 @@ IMCC_string_from_reg(PARROT_INTERP, NOTNULL(const SymReg *r))
          */
         const char *charset;
         char * const p = strchr(r->name, '"');
-        assert(p && p[-1] == ':');
+        PARROT_ASSERT(p && p[-1] == ':');
 
         p[-1]   = 0;
         charset = r->name;
@@ -649,7 +653,7 @@ mk_multi_sig(PARROT_INTERP, NOTNULL(SymReg *r))
                     ct->constants[r->color]->u.string);
         }
         else {
-            assert(r->set == 'K');
+            PARROT_ASSERT(r->set == 'K');
             sig_pmc = ct->constants[r->color]->u.key;
         }
 
@@ -704,14 +708,14 @@ create_lexinfo(PARROT_INTERP, IMC_Unit *unit, PMC *sub, int need_lex)
 
                 /* at least one lexical name */
                 n = r->reg;
-                assert(n);
+                PARROT_ASSERT(n);
 
                 while (n) {
                     k = n->color;
-                    assert(k >= 0);
+                    PARROT_ASSERT(k >= 0);
 
                     lex_name = constants[k]->u.string;
-                    assert(PObj_is_string_TEST(lex_name));
+                    PARROT_ASSERT(PObj_is_string_TEST(lex_name));
 
                     IMCC_debug(interp, DEBUG_PBC_CONST,
                             "add lexical '%s' to sub name '%s'\n",
@@ -1318,8 +1322,8 @@ verify_signature(PARROT_INTERP, NOTNULL(const Instruction *ins), opcode_t *pc)
     PMC    *changed_sig = NULL;
     PMC    *sig_arr     = interp->code->const_table->constants[pc[-1]]->u.key;
 
-    assert(PObj_is_PMC_TEST(sig_arr));
-    assert(sig_arr->vtable->base_type == enum_class_FixedIntegerArray);
+    PARROT_ASSERT(PObj_is_PMC_TEST(sig_arr));
+    PARROT_ASSERT(sig_arr->vtable->base_type == enum_class_FixedIntegerArray);
 
     no_consts = (ins->opnum == PARROT_OP_get_results_pc ||
         ins->opnum == PARROT_OP_get_params_pc);
@@ -1415,14 +1419,15 @@ e_pbc_emit(PARROT_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit), NOTNULL(co
          * this storage
          */
         if (interp->code->base.data) {
-            interp->code->base.data =
+            interp->code->base.data       = (opcode_t *)
                 mem_sys_realloc(interp->code->base.data, bytes);
-            interp->code->pic_index->data =
+            interp->code->pic_index->data = (opcode_t *)
                 mem_sys_realloc(interp->code->pic_index->data, bytes/2);
         }
         else {
-            interp->code->base.data = mem_sys_allocate(bytes);
-            interp->code->pic_index->data = mem_sys_allocate(bytes/2);
+            interp->code->base.data       = (opcode_t *)mem_sys_allocate(bytes);
+            interp->code->pic_index->data =
+                (opcode_t *)mem_sys_allocate(bytes/2);
         }
         interp->code->base.size       = oldsize + code_size;
         interp->code->pic_index->size = (oldsize + code_size) / 2;
@@ -1456,8 +1461,8 @@ e_pbc_emit(PARROT_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit), NOTNULL(co
         else {
             /* need a dummy to hold register usage */
             SymReg * const r = mk_sub_label(interp, str_dup("(null)"));
-            r->type    = VT_PCC_SUB;
-            r->pcc_sub = calloc(1, sizeof (pcc_sub_t));
+            r->type          = VT_PCC_SUB;
+            r->pcc_sub       = (pcc_sub_t *)calloc(1, sizeof (pcc_sub_t));
 
             add_const_pmc_sub(interp, r, oldsize, oldsize + code_size);
         }
@@ -1478,7 +1483,7 @@ e_pbc_emit(PARROT_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit), NOTNULL(co
 #endif
         if ((ins->type & ITBRANCH) &&
                 (addr = get_branch_reg(ins)) != 0 &&
-                !(addr->type & VTREGISTER)) {
+                !REG_NEEDS_ALLOC(addr)) {
             /* fixup local jumps - calc offset */
             if (addr->color == -1)
                 IMCC_fatal(interp, 1, "e_pbc_emit: "
@@ -1552,7 +1557,7 @@ e_pbc_emit(PARROT_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit), NOTNULL(co
                 case PARROT_ARG_KC:
                     r = ins->r[i];
                     if (r->set == 'K') {
-                        assert(r->color >= 0);
+                        PARROT_ASSERT(r->color >= 0);
                         *pc++ = r->color;
                     }
                     else {
