@@ -3,25 +3,25 @@
 
 /* HEADERIZER BEGIN: compilers/imcc/pbc.c */
 
-int e_pbc_close( PARROT_INTERP, void *param )
+int e_pbc_close( PARROT_INTERP, SHIM(void *param) )
         __attribute__nonnull__(1);
 
 int e_pbc_emit( PARROT_INTERP,
-    void *param,
+    SHIM(void *param),
     NOTNULL(IMC_Unit *unit),
     NOTNULL(const Instruction *ins) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
-int e_pbc_end_sub( PARROT_INTERP, void *param, NOTNULL(IMC_Unit *unit) )
+int e_pbc_end_sub( PARROT_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
-int e_pbc_new_sub( SHIM_INTERP, void *param, NOTNULL(IMC_Unit *unit) )
+int e_pbc_new_sub( SHIM_INTERP, SHIM(void *param), NOTNULL(IMC_Unit *unit) )
         __attribute__nonnull__(3);
 
-int e_pbc_open( PARROT_INTERP, void *param )
+int e_pbc_open( PARROT_INTERP, SHIM(void *param) )
         __attribute__nonnull__(1);
 
 INTVAL IMCC_int_from_reg( PARROT_INTERP, NOTNULL(const SymReg *r) )
