@@ -15,7 +15,6 @@
  */
 
 #include <string.h>
-#include <assert.h>
 #include "imc.h"
 #include "optimizer.h"
 
@@ -309,7 +308,7 @@ make_stat(NOTNULL(IMC_Unit *unit), NULLOK(int *sets), NULLOK(int *cols))
             if (r->color > unit->max_color)
                 unit->max_color = r->color;
             for (j = 0; j < 4; j++)
-                if (r->set == type[j] && (r->type & VTREGISTER)) {
+                if (r->set == type[j] && REG_NEEDS_ALLOC(r)) {
                     if (sets)
                         sets[j]++;
                     if (cols)
@@ -429,13 +428,13 @@ build_reglist(Parrot_Interp interp, NOTNULL(IMC_Unit *unit))
     count = unit->hash.entries;
     if (count == 0)
         return;
-    unit->reglist = mem_sys_allocate(count * sizeof (SymReg*));
+    unit->reglist = (SymReg **)mem_sys_allocate(count * sizeof (SymReg*));
 
     for (i = count = 0; i < hsh->size; i++) {
         SymReg *r;
         for (r = hsh->data[i]; r; r = r->next) {
             /* Add each symbol to reglist  */
-            if (r->type & VTREGISTER) {
+            if (REG_NEEDS_ALLOC(r)) {
                 unit->reglist[count++] = r;
             }
         }
@@ -716,7 +715,7 @@ try_allocate(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
     int x;
     char *avail;
     int t, n;
-    unsigned int *graph = unit->interference_graph;
+    unsigned int *graph     = unit->interference_graph;
     SymReg ** const reglist = unit->reglist;
 
     /*
@@ -725,7 +724,7 @@ try_allocate(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
     n = unit->n_symbols;
     if (unit->max_color >= n)
         n = unit->max_color + 1;
-    avail = mem_sys_allocate(n);
+    avail = (char *)mem_sys_allocate(n);
 
     for (x = 0; x < unit->n_symbols; ++x) {
         SymReg *r = reglist[x];
@@ -782,7 +781,7 @@ map_colors(NOTNULL(IMC_Unit* unit), int x, NOTNULL(unsigned int *graph), NOTNULL
             if (   r
                     && r->color != -1
                     && r->set == typ) {
-                assert(r->color - already_allocated >= 0);
+                PARROT_ASSERT(r->color - already_allocated >= 0);
                 avail[r->color - already_allocated] = 0;
             }
         }
@@ -796,31 +795,33 @@ map_colors(NOTNULL(IMC_Unit* unit), int x, NOTNULL(unsigned int *graph), NOTNULL
 static int
 first_avail(NOTNULL(IMC_Unit *unit), int reg_set, NULLOK(Set **avail))
 {
-    int i, n, first;
-    SymHash *hsh;
-    Set *allocated;
+    int      n         = unit->n_symbols > unit->max_color ?
+                         unit->n_symbols : unit->max_color;
+    Set     *allocated = set_make(n + 1);
+    SymHash *hsh       = &unit->hash;
 
-    n = unit->n_symbols;
-    if (unit->max_color > n)
-        n = unit->max_color;
-    allocated = set_make(n + 1);
-    hsh = &unit->hash;
+    int i, first;
+
+    /* find allocated registers */
     for (i = 0; i < hsh->size; i++) {
-        SymReg * r;
+        SymReg *r;
         for (r = hsh->data[i]; r; r = r->next) {
             if (r->set != reg_set)
                 continue;
-            if (r->type & VTREGISTER) {
+            if (REG_NEEDS_ALLOC(r)) {
                 if (r->color >= 0)
                     set_add(allocated, r->color);
             }
         }
     }
+
     first = set_first_zero(allocated);
+
     if (avail)
         *avail = allocated;
     else
         set_free(allocated);
+
     return first;
 }
 
@@ -848,8 +849,7 @@ allocate_uniq(PARROT_INTERP, NOTNULL(IMC_Unit *unit), int usage)
             for (r = hsh->data[i]; r; r = r->next) {
                 if (r->set != reg_set)
                     continue;
-                if ((r->type & VTREGISTER) &&
-                        r->color == -1 &&
+                if (REG_NEEDS_ALLOC(r) && r->color == -1 &&
                         (r->usage & usage)) {
                     if (set_contains(avail, first_reg))
                         first_reg = first_avail(unit, reg_set, NULL);
@@ -875,39 +875,40 @@ allocate_uniq(PARROT_INTERP, NOTNULL(IMC_Unit *unit), int usage)
 static void
 vanilla_reg_alloc(SHIM_INTERP, NOTNULL(IMC_Unit *unit))
 {
-    char type[] = "INSP";
     int i, j, reg_set, first_reg;
-    SymReg * r;
-    SymHash *hsh;
-    Set *avail;
-
-    hsh = &unit->hash;
+    char     type[] = "INSP";
+    Set     *avail;
+    SymReg  *r;
+    SymHash *hsh = &unit->hash;
 
     /* Clear the pre-assigned colors. */
     for (i = 0; i < hsh->size; i++) {
         for (r = hsh->data[i]; r; r = r->next) {
-            if (r->type & VTREGISTER) /* TODO Ignore non-volatiles */
-                r->color=-1;
+            /* TODO Ignore non-volatiles */
+            if (REG_NEEDS_ALLOC(r))
+                r->color = -1;
         }
     }
 
     /* Assign new colors. */
     for (j = 0; j < 4; j++) {
-        reg_set = type[j];
+        reg_set   = type[j];
         first_reg = first_avail(unit, reg_set, &avail);
+
         for (i = 0; i < hsh->size; i++) {
             for (r = hsh->data[i]; r; r = r->next) {
                 if (r->set != reg_set)
                     continue;
-                if ((r->type & VTREGISTER) && (r->color == -1 )
-                        ) {
+                if (REG_NEEDS_ALLOC(r) && (r->color == -1 )) {
                     if (set_contains(avail, first_reg))
                         first_reg = first_avail(unit, reg_set, NULL);
+
                     set_add(avail, first_reg);
                     r->color = first_reg++;
                 }
             }
         }
+
         set_free(avail);
         unit->first_avail[j] = first_reg;
     }
