@@ -29,11 +29,14 @@ static char * add_ns( PARROT_INTERP, NOTNULL(char *name) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static SymReg* mk_pmc_const_2( PARROT_INTERP,
-    IMC_Unit *unit,
+    NOTNULL(IMC_Unit *unit),
     NOTNULL(SymReg *left),
     NOTNULL(SymReg *rhs) )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
@@ -56,7 +59,7 @@ push_namespace(char * name)
 void
 pop_namespace(NULLOK(char *name))
 {
-    Namespace * ns = _namespace;
+    Namespace * const ns = _namespace;
 
     if (ns == NULL) {
         fprintf(stderr, "pop() on empty namespace stack\n");
@@ -341,6 +344,8 @@ mk_fullname(NOTNULL(const char *name))
 }
 
 /* Makes a new identifier */
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg *
 mk_ident(PARROT_INTERP, NOTNULL(char *name), int t)
 {
@@ -369,8 +374,10 @@ mk_ident(PARROT_INTERP, NOTNULL(char *name), int t)
     return r;
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg*
-mk_ident_ur(PARROT_INTERP, char *name, int t)
+mk_ident_ur(PARROT_INTERP, NOTNULL(char *name), int t)
 {
     SymReg * const r = mk_ident(interp, name, t);
     r->usage        |= U_NON_VOLATILE;
@@ -378,8 +385,11 @@ mk_ident_ur(PARROT_INTERP, char *name, int t)
     return r;
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static SymReg*
-mk_pmc_const_2(PARROT_INTERP, IMC_Unit *unit, NOTNULL(SymReg *left), NOTNULL(SymReg *rhs))
+mk_pmc_const_2(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(SymReg *left), NOTNULL(SymReg *rhs))
+    /* XXX This always returns NULL.  Probably shouldn't return anything then. */
 {
     SymReg *r[2];
     char   *name;
@@ -462,9 +472,10 @@ mk_const_ident(PARROT_INTERP,
 
 /* Makes a new constant */
 SymReg *
-_mk_const(NOTNULL(SymHash *hsh), NOTNULL(char *name), int t)
+_mk_const(NOTNULL(SymHash *hsh), NOTNULL(const char *name), int t)
 {
-    SymReg * const r = _mk_symreg(hsh, name, t);
+    DECL_CONST_CAST;
+    SymReg * const r = _mk_symreg(hsh, (char *)const_cast(name), t);
     r->type          = VTCONST;
 
     if (t == 'U') {
@@ -479,7 +490,7 @@ _mk_const(NOTNULL(SymHash *hsh), NOTNULL(char *name), int t)
 }
 
 SymReg *
-mk_const(PARROT_INTERP, char * name, int t)
+mk_const(PARROT_INTERP, NOTNULL(const char *name), int t)
 {
     SymHash * const h = &IMCC_INFO(interp)->ghash;
 
@@ -567,11 +578,12 @@ _mk_address(PARROT_INTERP, NOTNULL(SymHash *hsh), NOTNULL(char *name),
 SymReg *
 mk_address(PARROT_INTERP, char *name, int uniq)
 {
-    SymHash * const h = *name == '_'
+    const int begins_with_underscore = *name == '_';
+    SymHash * const h = begins_with_underscore
         ? &IMCC_INFO(interp)->ghash : &IMCC_INFO(interp)->cur_unit->hash;
     SymReg  * const s = _mk_address(interp, h, name, uniq);
 
-    if (*name == '_')
+    if (begins_with_underscore)
        s->usage |= U_FIXUP;
 
     return s;
@@ -733,7 +745,7 @@ link_keys(PARROT_INTERP, int nargs, NOTNULL(SymReg * keys[]), int force)
         /* if any component is a variable, we need to track it in
          * life analysis
          */
-        if (keys[i]->type & VTREGISTER)
+        if (REG_NEEDS_ALLOC(keys[i]))
             keychain->type |= VTREGKEY;
 
         key->nextkey = dup_sym(keys[i]);
@@ -742,7 +754,7 @@ link_keys(PARROT_INTERP, int nargs, NOTNULL(SymReg * keys[]), int force)
         /* for registers, point ->reg to the original, needed by
          * life analyses & coloring
          */
-        if (keys[i]->type & VTREGISTER)
+        if (REG_NEEDS_ALLOC(keys[i]))
             key->reg = keys[i];
     }
 
@@ -861,6 +873,8 @@ store_symreg(PARROT_INTERP, NOTNULL(SymReg *r))
 }
 
 /* Gets a symbol from the hash */
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg *
 _get_sym(NOTNULL(SymHash *hsh), NOTNULL(const char *name))
 {
@@ -875,10 +889,12 @@ _get_sym(NOTNULL(SymHash *hsh), NOTNULL(const char *name))
             return p;
     }
 
-    return 0;
+    return NULL;
 }
 
 /* Gets a symbol from the current unit symbol table */
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg *
 get_sym(PARROT_INTERP, NOTNULL(const char *name))
 {
@@ -886,6 +902,8 @@ get_sym(PARROT_INTERP, NOTNULL(const char *name))
 }
 
 /* find a symbol hash or ghash */
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg *
 _find_sym(PARROT_INTERP, NULLOK(const Namespace *nspace),
     NOTNULL(SymHash *hsh), NOTNULL(const char *name))
@@ -913,10 +931,12 @@ _find_sym(PARROT_INTERP, NULLOK(const Namespace *nspace),
     if (p)
         return p;
 
-    return 0;
+    return NULL;
 }
 
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg *
 find_sym(PARROT_INTERP, NOTNULL(const char *name))
 {
