@@ -30,6 +30,7 @@ enum VARTYPE {              /* variable type can be */
 
 /* this VARTYPE needs register allocation and such */
 #define VTREGISTER (VTREG | VTIDENTIFIER | VTREGKEY | VTPASM)
+#define REG_NEEDS_ALLOC(r) ((r)->type & VTREGISTER)
 
 enum LIFEFLAG {    /* The status of a var inside a basic block can be */
     LF_use       = 1 << 0, /* block uses the the var before defining it */
@@ -111,6 +112,8 @@ struct _IMC_Unit;
 
 /* HEADERIZER BEGIN: compilers/imcc/symreg.c */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg * _find_sym( PARROT_INTERP,
     NULLOK(const Namespace *nspace),
     NOTNULL(SymHash *hsh),
@@ -119,6 +122,8 @@ SymReg * _find_sym( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg * _get_sym( NOTNULL(SymHash *hsh), NOTNULL(const char *name) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -131,7 +136,7 @@ SymReg * _mk_address( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-SymReg * _mk_const( NOTNULL(SymHash *hsh), NOTNULL(char *name), int t )
+SymReg * _mk_const( NOTNULL(SymHash *hsh), NOTNULL(const char *name), int t )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
@@ -192,6 +197,8 @@ PARROT_MALLOC
 SymReg * dup_sym( NOTNULL(const SymReg *r) )
         __attribute__nonnull__(1);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg * find_sym( PARROT_INTERP, NOTNULL(const char *name) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -199,6 +206,8 @@ SymReg * find_sym( PARROT_INTERP, NOTNULL(const char *name) )
 void free_sym( NOTNULL(SymReg *r) )
         __attribute__nonnull__(1);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg * get_sym( PARROT_INTERP, NOTNULL(const char *name) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -217,8 +226,9 @@ SymReg * link_keys( PARROT_INTERP,
 SymReg * mk_address( PARROT_INTERP, char *name, int uniq )
         __attribute__nonnull__(1);
 
-SymReg * mk_const( PARROT_INTERP, char * name, int t )
-        __attribute__nonnull__(1);
+SymReg * mk_const( PARROT_INTERP, NOTNULL(const char *name), int t )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 SymReg * mk_const_ident( PARROT_INTERP,
     NOTNULL(char *name),
@@ -232,12 +242,17 @@ SymReg * mk_const_ident( PARROT_INTERP,
 char * mk_fullname( NOTNULL(const char *name) )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 SymReg * mk_ident( PARROT_INTERP, NOTNULL(char *name), int t )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-SymReg* mk_ident_ur( PARROT_INTERP, char *name, int t )
-        __attribute__nonnull__(1);
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+SymReg* mk_ident_ur( PARROT_INTERP, NOTNULL(char *name), int t )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 SymReg * mk_label_address( PARROT_INTERP, NOTNULL(char *name) )
         __attribute__nonnull__(1)
