@@ -63,7 +63,7 @@ new_pirvtable(void) {
 
     if (vtable == NULL) {
         fprintf(stderr, "Failed to allocate memory for vtable!\n");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     /* set all entries to 'default' by default. This way, another output module
@@ -150,3 +150,4 @@ destroy_pirvtable(pirvtable *vtable) {
  * vim: expandtab shiftwidth=4:
  */
 
+
