@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: gcc.pm 19704 2007-07-08 22:36:21Z paultcochrane $
+# $Id: gcc.pm 20663 2007-08-18 01:37:27Z chromatic $
 
 =head1 NAME
 
@@ -90,9 +90,21 @@ sub runstep {
         # determined in a non-strict environment.  An example is Solaris 8.
 
         my @opt_and_vers = (
-                  0 => "-W -Wall -Wstrict-prototypes -Wmissing-prototypes -Winline"
-                . " -Wshadow -Wpointer-arith -Wcast-qual"
-                . " -Wwrite-strings -Waggregate-return -Winline -Wno-unused"
+            0 => " -W"
+                . " -Wall"
+                . " -Wundef"
+                . " -Wmissing-declarations"
+                . " -Wstrict-prototypes"
+                . " -Wmissing-prototypes"
+                . " -Winline"
+                . " -Wshadow"
+                . " -Wpointer-arith"
+                . " -Wcast-qual"
+                . " -Wwrite-strings"
+                . " -Waggregate-return"
+                . " -Winline"
+                . " -Wno-unused"
+                . " -Wnested-externs -Wfloat-equal"
                 . ( $maint ? " -Wlarger-than-4096" : "" ),
 
             # others; ones we might like marked with ?
@@ -123,12 +135,17 @@ sub runstep {
             #      -O2 and -O3
             #      -falign-functions=16 is the real alignment, no exponent
             3.0 => "-falign-functions=16"
-                . " -Wformat-nonliteral -Wformat-security -Wpacked"
-                . " -Wdisabled-optimization -mno-accumulate-outgoing-args"
+                . " -Wformat-nonliteral"
+                . " -Wformat-security"
+                . " -Wpacked"
+                . " -Wdisabled-optimization"
+                . " -mno-accumulate-outgoing-args"
                 . " -Wno-shadow",
 
-            3.4 => " -Wextra -Wdeclaration-after-statement"
-                . " -Wold-style-definition -Wbad-function-cast",
+            3.4 => " -Wextra"
+                . " -Wdeclaration-after-statement"
+                . " -Wold-style-definition"
+                . " -Wbad-function-cast",
 
             # -Wsequence-point is part of -Wall
             # -Wfloat-equal may not be what we want
@@ -136,37 +153,96 @@ sub runstep {
             # us -Wpadded may prove interesting, or even noisy.
             # -Wunreachable-code might be useful in a non debugging version
             4.0 => "-fvisibility=hidden",
+            # Needed to prevent C++ compatibility issues
+            4.1 => " -Wc++-compat",
         );
+
         my @cage_opt_and_vers = (
-            0 =>
-" -std=c89 -Wall -Wextra -Wundef -Wbad-function-cast -Wmissing-declarations -Wredundant-decls -Wnested-externs -Wlong-long"
-                . " -Wfloat-equal -Wpadded -Wunreachable-code"
+            0 => " -std=c89"
+                . " -Wall"
+                . " -Wextra"
+                . " -Wundef"
+                . " -Wbad-function-cast"
+                . " -Wmissing-declarations"
+                . " -Wredundant-decls"
+                . " -Wnested-externs"
+                . " -Wlong-long"
+                . " -Wpadded"
+                . " -Wunreachable-code"
 
                 #. "-fsyntax-only "
                 #. "-pedantic -pedantic-errors "
                 #. " -w "
                 #. " -Werror "
-                . " -Wextra -Wall -Waggregate-return -Wcast-align  -Wcast-qual  -Wchar-subscripts "
-                . " -Wcomment -Wconversion  -Wno-deprecated-declarations -Wdisabled-optimization  -Wno-div-by-zero  -Wno-endif-labels "
-                . " -Werror-implicit-function-declaration -Wfloat-equal -Wformat  -Wformat=2 -Wno-format-extra-args -Wformat-nonliteral "
-                . " -Wformat-security  -Wformat-y2k -Wimplicit  -Wimplicit-function-declaration  -Wimplicit-int -Wimport  -Wno-import  -Winit-self "
-                . " -Winline -Winvalid-pch -Wlarger-than-4096 -Wlong-long -Wmain  -Wmissing-braces  "
-                . " -Wmissing-format-attribute  -Wmissing-noreturn -Wno-multichar  -Wnonnull  -Wpacked "
-                . " -Wpadded -Wparentheses  -Wpointer-arith  -Wredundant-decls -Wreturn-type  -Wsequence-point  -Wshadow -Wsign-compare "
-                . " -Wstrict-aliasing -Wstrict-aliasing=2 -Wswitch  -Wswitch-default -Wswitch-enum -Wsystem-headers  -Wtrigraphs  -Wundef "
+                . " -Waggregate-return"
+                . " -Wcast-align"
+                . " -Wcast-qual"
+                . " -Wchar-subscripts "
+                . " -Wcomment"
+                . " -Wconversion"
+                . " -Wno-deprecated-declarations"
+                . " -Wdisabled-optimization"
+                . " -Wno-div-by-zero"
+                . " -Wno-endif-labels"
+                . " -Werror-implicit-function-declaration"
+                . " -Wfloat-equal"
+                . " -Wformat"
+                . " -Wformat=2"
+                . " -Wno-format-extra-args"
+                . " -Wformat-nonliteral"
+                . " -Wformat-security"
+                . " -Wformat-y2k"
+                . " -Wimplicit"
+                . " -Wimplicit-function-declaration"
+                . " -Wimplicit-int"
+                . " -Wimport"
+                . " -Wno-import"
+                . " -Winit-self"
+                . " -Winline"
+                . " -Winvalid-pch"
+                . " -Wlarger-than-4096"
+                . " -Wmain"
+                . " -Wmissing-braces"
+                . " -Wmissing-format-attribute"
+                . " -Wmissing-noreturn"
+                . " -Wno-multichar"
+                . " -Wnonnull"
+                . " -Wpacked"
+                . " -Wparentheses"
+                . " -Wpointer-arith"
+                . " -Wreturn-type"
+                . " -Wsequence-point"
+                . " -Wshadow"
+                . " -Wsign-compare"
+                . " -Wstrict-aliasing"
+                . " -Wstrict-aliasing=2"
+                . " -Wswitch"
+                . " -Wswitch-default"
+                . " -Wswitch-enum"
+                . " -Wsystem-headers"
+                . " -Wtrigraphs"
+                . " -Wundef"
 
                 #. " -Wuninitialized "
                 #     requires -O
-                . " -Wunknown-pragmas  -Wunreachable-code -Wunused  -Wunused-function  -Wunused-label  -Wunused-parameter -Wunused-value "
-                . " -Wunused-variable  -Wwrite-strings "
+                . " -Wunknown-pragmas"
+                . " -Wunused"
+                . " -Wunused-function"
+                . " -Wunused-label"
+                . " -Wunused-parameter"
+                . " -Wunused-value"
+                . " -Wunused-variable"
+                . " -Wwrite-strings"
 
                 #."-Wmost (APPLE ONLY)"
 
                 #C-only Warning Options
-                . " -Wbad-function-cast  -Wmissing-declarations -Wmissing-prototypes -Wnested-externs  -Wold-style-definition -Wstrict-prototypes "
+                . " -Wmissing-prototypes"
+                . " -Wold-style-definition"
+                . " -Wstrict-prototypes"
 
                 #. " -Wtraditional "
-                . " -Wdeclaration-after-statement ",
+                . " -Wdeclaration-after-statement",
 
 #"-Wall -Wstrict-prototypes -Wmissing-prototypes -Winline -Wshadow -Wpointer-arith -Wcast-qual -Wwrite-strings -Waggregate-return -Winline -W -Wno-unused",
 
@@ -204,13 +280,13 @@ sub runstep {
             #3.0 => "-Wformat-nonliteral -Wformat-security -Wpacked "
             #    . "-Wdisabled-optimization -mno-accumulate-outgoing-args "
             #    . "-Wno-shadow -falign-functions=16 ",
-            4.0 => "-Wfatal-errors "
-                . "-Wmissing-field-initializers "
-                . "-Wmissing-include-dirs "
-                . "-Wvariadic-macros "
-                . "-Wno-discard-qual "
-                . "-Wno-pointer-sign ",
-            4.1 => "-Wc++-compat",
+            4.0 => " -Wfatal-errors"
+                . " -Wmissing-field-initializers"
+                . " -Wmissing-include-dirs"
+                . " -Wvariadic-macros"
+                . " -Wno-discard-qual"
+                . " -Wno-pointer-sign",
+            4.1 => " -Wc++-compat",
             4.2 => "",
 
             # -Wsequence-point is part of -Wall
@@ -222,7 +298,8 @@ sub runstep {
 
         $warns = "";
         my @warning_options = ( \@opt_and_vers );
-        push @warning_options, \@cage_opt_and_vers if $conf->options->get('cage');
+        push @warning_options, \@cage_opt_and_vers
+            if $conf->options->get('cage');
 
         foreach my $curr_opt_and_vers (@warning_options) {
             while ( my ( $vers, $opt ) = splice @$curr_opt_and_vers, 0, 2 ) {
