@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: makefiles.pm 19825 2007-07-13 11:58:36Z paultcochrane $
+# $Id: makefiles.pm 19976 2007-07-18 19:06:23Z pmichaud $
 
 =head1 NAME
 
@@ -39,8 +39,8 @@ my %makefiles = (
         conditioned_lines   => 1,
     },
 
-     'compilers/past/Makefile'    => { SOURCE => 'config/gen/makefiles/past.in' },
      'compilers/past-pm/Makefile' => { SOURCE => 'config/gen/makefiles/past-pm.in' },
+     'compilers/pct/Makefile'     => { SOURCE => 'config/gen/makefiles/pct.in' },
      'compilers/pge/Makefile'     => { SOURCE => 'config/gen/makefiles/pge.in' },
      'compilers/tge/Makefile'     => { SOURCE => 'config/gen/makefiles/tge.in' },
      'compilers/bcg/Makefile'     => { SOURCE => 'config/gen/makefiles/bcg.in' },
