@@ -11,13 +11,13 @@
 #include <sys/wait.h>
 
 INTVAL
-Parrot_Run_OS_Command(Parrot_Interp interpreter, STRING *command)
+Parrot_Run_OS_Command(Parrot_Interp interp, STRING *command)
 {
     pid_t child;
     child = fork();
     /* Did we fail? */
     if (-1 == child) {
-        internal_exception(NOSPAWN, "Can't spawn child process");
+        real_exception(interp, NULL, NOSPAWN, "Can't spawn child process");
     }
     /* Are we the parent or child? */
     if (child) {
@@ -32,7 +32,7 @@ Parrot_Run_OS_Command(Parrot_Interp interpreter, STRING *command)
            about to be something else */
         int status;
         status = execlp("sh", "sh", "-c",
-            string_to_cstring(interpreter, command), (void *)NULL);
+            string_to_cstring(interp, command), (void *)NULL);
         /* if we get here, something's horribly wrong... */
         if (status) {
             exit(status);
@@ -42,17 +42,17 @@ Parrot_Run_OS_Command(Parrot_Interp interpreter, STRING *command)
 }
 
 INTVAL
-Parrot_Run_OS_Command_Argv(Parrot_Interp interpreter, PMC *cmdargs)
+Parrot_Run_OS_Command_Argv(Parrot_Interp interp, PMC *cmdargs)
 {
     pid_t child;
-    int len = VTABLE_elements(interpreter, cmdargs);
+    int len = VTABLE_elements(interp, cmdargs);
     if (len == 0) {
-        internal_exception(NOSPAWN, "Empty argument array for execvp");
+        real_exception(interp, NULL, NOSPAWN, "Empty argument array for execvp");
     }
     child = fork();
     /* Did we fail? */
     if (-1 == child) {
-        internal_exception(NOSPAWN, "Can't spawn child process");
+        real_exception(interp, NULL, NOSPAWN, "Can't spawn child process");
     }
     /* Are we the parent or child? */
     if (child) {
@@ -72,8 +72,8 @@ Parrot_Run_OS_Command_Argv(Parrot_Interp interpreter, PMC *cmdargs)
 
         argv = (char **)mem_sys_allocate((len+1)*sizeof (char *));
         for (i = 0; i < len; ++i) {
-            s = VTABLE_get_string_keyed_int(interpreter, cmdargs, i);
-            argv[i] = string_to_cstring(interpreter, s);
+            s = VTABLE_get_string_keyed_int(interp, cmdargs, i);
+            argv[i] = string_to_cstring(interp, s);
         }
         cmd = argv[0];
         argv[i] = NULL;
@@ -88,15 +88,15 @@ Parrot_Run_OS_Command_Argv(Parrot_Interp interpreter, PMC *cmdargs)
 }
 
 void
-Parrot_Exec_OS_Command(Parrot_Interp interpreter, STRING *command) {
+Parrot_Exec_OS_Command(Parrot_Interp interp, STRING *command) {
     /* Be horribly profligate with memory, since we're
        about to be something else */
     int status;
     status = execlp("sh", "sh", "-c",
-            string_to_cstring(interpreter, command), (void *)NULL);
+            string_to_cstring(interp, command), (void *)NULL);
     /* if we get here, something's horribly wrong... */
     if (status) {
-        internal_exception(NOSPAWN, "Exec failed, code %i", status);
+        real_exception(interp, NULL, NOSPAWN, "Exec failed, code %i", status);
     }
 }
 
