@@ -13,7 +13,7 @@ Parrot_set_sighandler(int signum, Parrot_sighandler_t handler)
 {
     return signal(signum, handler);
 }
-#endif
+#endif /* PARROT_HAS_HEADER_SIGNAL */
 
 /*
  * Local variables:
