@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: /parrotcode/local/config/init/headers.pm 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: headers.pm 19457 2007-06-29 08:15:08Z paultcochrane $
 
 =head1 NAME
 
@@ -7,7 +7,8 @@ config/init/headers.pm - Nongenerated Headers
 
 =head1 DESCRIPTION
 
-Uses C<ExtUtils::Manifest> to determine which headers are nongenerated.
+Uses C<ExtUtils::Manifest> to find the C header files that are
+distributed with Parrot.
 
 =cut
 
@@ -21,7 +22,7 @@ use base qw(Parrot::Configure::Step::Base);
 use Parrot::Configure::Step;
 use ExtUtils::Manifest qw(maniread);
 
-our $description = 'Determining nongenerated header files';
+our $description = 'Finding header files distributed with Parrot';
 our @args;
 
 sub runstep {
