@@ -1,4 +1,4 @@
-/* $Id: japhc.c 18526 2007-05-13 20:42:55Z chromatic $ */
+/* $Id: japhc.c 19482 2007-06-29 18:23:17Z paultcochrane $ */
 
 /*
  * example compiler used by japh16.pasm
@@ -20,7 +20,7 @@
 #  define cdebug(x)
 #endif
 
-PMC* japh_compiler(Parrot_Interp interpreter, const char *s);
+PMC* japh_compiler(Parrot_Interp interp, const char *s);
 
 /*
  * loadlib calls the load and init hooks
@@ -28,13 +28,13 @@ PMC* japh_compiler(Parrot_Interp interpreter, const char *s);
  */
 
 void
-Parrot_lib_japhc_init(Parrot_Interp interpreter, PMC* lib)
+Parrot_lib_japhc_init(Parrot_Interp interp, PMC* lib)
 {
     STRING *cmp;
 
     cdebug((stderr, "japhc_init\n"));
-    cmp = const_string(interpreter, "JaPH_Compiler");
-    Parrot_compreg(interpreter, cmp, japh_compiler);
+    cmp = const_string(interp, "JaPH_Compiler");
+    Parrot_compreg(interp, cmp, japh_compiler);
 }
 
 
@@ -63,7 +63,7 @@ unescape(char *string)
 
 /* add constant string to constant_table */
 static int
-add_const_str(Parrot_Interp interpreter, PackFile_ConstTable *consts, char *str)
+add_const_str(Parrot_Interp interp, PackFile_ConstTable *consts, char *str)
 {
     int k, l;
     char *o;
@@ -98,10 +98,10 @@ add_const_str(Parrot_Interp interpreter, PackFile_ConstTable *consts, char *str)
                 k * sizeof (PackFile_Constant *));
 
     /* Allocate a new constant */
-    consts->constants[--k] = PackFile_Constant_new(interpreter);
+    consts->constants[--k] = PackFile_Constant_new(interp);
     consts->constants[k]->type = PFC_STRING;
     consts->constants[k]->u.string =
-        string_make(interpreter, buf, (UINTVAL) l, "iso-8859-1", 0 );
+        string_make(interp, buf, (UINTVAL) l, "iso-8859-1", 0 );
     free(o);
     return k;
 }
@@ -110,7 +110,7 @@ add_const_str(Parrot_Interp interpreter, PackFile_ConstTable *consts, char *str)
  * simple compiler - no error checking
  */
 PMC*
-japh_compiler(Parrot_Interp interpreter, const char *program)
+japh_compiler(Parrot_Interp interp, const char *program)
 {
     PackFile_ByteCode *cur_cs, *old_cs;
     PackFile_ConstTable *consts;
@@ -125,8 +125,8 @@ japh_compiler(Parrot_Interp interpreter, const char *program)
     /*
      * need some packfile segments
      */
-    cur_cs = PF_create_default_segs(interpreter, "JAPHc", 1);
-    old_cs = Parrot_switch_to_cs(interpreter, cur_cs, 0);
+    cur_cs = PF_create_default_segs(interp, "JAPHc", 1);
+    old_cs = Parrot_switch_to_cs(interp, cur_cs, 0);
     /*
      * alloc byte code mem
      */
@@ -140,46 +140,46 @@ japh_compiler(Parrot_Interp interpreter, const char *program)
     for (p = program; *p; ++p) {
         switch (*p) {
             case 'p':        /* print_sc */
-                *pc++ = interpreter->op_lib->op_code("print_sc", 1);
+                *pc++ = interp->op_lib->op_code("print_sc", 1);
                 /* const follows */
                 ++p;
                 switch (*p) {
                     case 'J':
-                        *pc++ = add_const_str(interpreter, consts, "Just ");
+                        *pc++ = add_const_str(interp, consts, "Just ");
                         break;
                     case 'a':
-                        *pc++ = add_const_str(interpreter, consts, "another ");
+                        *pc++ = add_const_str(interp, consts, "another ");
                         break;
                     case 'P':
-                        *pc++ = add_const_str(interpreter, consts, "Parrot ");
+                        *pc++ = add_const_str(interp, consts, "Parrot ");
                         break;
                     case 'H':
-                        *pc++ = add_const_str(interpreter, consts, "Hacker");
+                        *pc++ = add_const_str(interp, consts, "Hacker");
                         break;
                     case 'n':
-                        *pc++ = add_const_str(interpreter, consts, "\n");
+                        *pc++ = add_const_str(interp, consts, "\n");
                         break;
                 }
                 break;
             case 'e':        /* end */
-                *pc++ = interpreter->op_lib->op_code("invoke_p", 1);
+                *pc++ = interp->op_lib->op_code("invoke_p", 1);
                 *pc++ = 1;
                 break;
         }
     }
     if (old_cs) {
         /* restore old byte_code, */
-        (void)Parrot_switch_to_cs(interpreter, old_cs, 0);
+        (void)Parrot_switch_to_cs(interp, old_cs, 0);
     }
     /*
      * create sub PMC
      */
-    sub = pmc_new(interpreter, enum_class_Eval);
+    sub = pmc_new(interp, enum_class_Eval);
     sub_data = PMC_sub(sub);
     sub_data->seg = cur_cs;
     sub_data->address = cur_cs->base.data;
     sub_data->end = cur_cs->base.data + cur_cs->base.size;
-    sub_data->name = string_from_cstring(interpreter, "JaPHC", 0);
+    sub_data->name = string_from_literal(interp, "JaPHC");
     return sub;
 }
 
