@@ -1,7 +1,7 @@
 /* atomic/gcc_x86.h
  *  Copyright (C) 2006, The Perl Foundation.
  *  SVN Info
- *     $Id: gcc_x86.h 18945 2007-06-12 14:08:35Z fperrad $
+ *     $Id: gcc_x86.h 20696 2007-08-19 02:10:21Z chromatic $
  *  Overview:
  *     This header provides an implementation of atomic
  *     operations on x86 platforms with GCC-style
@@ -72,9 +72,10 @@ inline static void *parrot_i386_cmpxchg(void *volatile *ptr, void *expect,
 
 #define PARROT_ATOMIC_INT_CAS(result, a, expect, update) \
     do { \
-        if (expect == (long) parrot_i386_cmpxchg( \
+        void *res = parrot_i386_cmpxchg( \
                 (void * volatile *) &(a).val, \
-                (void *) expect, (void *) update)) { \
+                (void *) expect, (void *) update); \
+        if (expect == (long)res) { \
             result = 1; \
         } \
         else { \
