@@ -1,7 +1,7 @@
 /* compiler.h
- *  Copyright (C) 2006, The Perl Foundation.
+ *  Copyright (C) 2007, The Perl Foundation.
  *  SVN Info
- *     $Id: compiler.h 19886 2007-07-15 06:19:53Z petdance $
+ *     $Id: compiler.h 20435 2007-08-02 22:34:32Z particle $
  *  Overview:
  *     defines compiler capabilities
  */
@@ -16,6 +16,16 @@
  * ./Configure time.  For now, you'll have to do it by hand.
  */
 
+#ifdef HASATTRIBUTE_NEVER_WORKS
+#  error This attribute can never succeed.  Something has mis-sniffed your configuration.
+#endif
+#ifdef HASATTRIBUTE_DEPRECATED
+#  ifdef _MSC_VER
+#    define __attribute__deprecated__         __declspec(deprecated)
+#  else
+#    define __attribute__deprecated__           __attribute__((deprecated))
+#  endif
+#endif
 #ifdef HASATTRIBUTE_FORMAT
 #  define __attribute__format__(x,y,z)      __attribute__((format(x,y,z)))
 #endif
@@ -26,7 +36,11 @@
 #  define __attribute__nonnull__(a)         __attribute__((nonnull(a)))
 #endif
 #ifdef HASATTRIBUTE_NORETURN
-#  define __attribute__noreturn__           __attribute__((noreturn))
+#  ifdef _MSC_VER
+#    define __attribute__noreturn__         __declspec(noreturn)
+#  else
+#    define __attribute__noreturn__           __attribute__((noreturn))
+#  endif
 #endif
 #ifdef HASATTRIBUTE_PURE
 #  define __attribute__pure__               __attribute__((pure))
@@ -42,6 +56,9 @@
 #endif
 
 /* If we haven't defined the attributes yet, define them to blank. */
+#ifndef __attribute__deprecated__
+#  define __attribute__deprecated__
+#endif
 #ifndef __attribute__format__
 #  define __attribute__format__(x,y,z)
 #endif
@@ -74,7 +91,7 @@
  * get "_unused" added to them so that you can't accidentally use them
  * without removing the shim designation.
  */
-#define SHIM(a) /*@unused@*/ a##_unused __attribute__unused__ /*@null@*/
+#define SHIM(a) /*@unused@*/ /*@null@*/ a##_unused __attribute__unused__
 
 /* UNUSED() is the old way we handled shim arguments Should still be
    used in cases where the argument should, at some point be used.
@@ -84,7 +101,9 @@
 #define PARROT_CAN_RETURN_NULL      /*@null@*/
 #define PARROT_CANNOT_RETURN_NULL   /*@notnull@*/
 
-#define PARROT_MAY_IGNORE_RESULT
+#define PARROT_DEPRECATED           __attribute__deprecated__
+
+#define PARROT_IGNORABLE_RESULT
 #define PARROT_WARN_UNUSED_RESULT   __attribute__warn_unused_result__
 
 #define PARROT_PURE_FUNCTION        __attribute__pure__ __attribute__warn_unused_result__
@@ -95,6 +114,10 @@
 #define NOTNULL(x)                  /*@notnull@*/ x
 #define NULLOK(x)                   /*@null@*/ x
 
+#define ARGOUT(x)                   /*@notnull@*/ /*@out@*/ x
+#define ARGINOUT(x)                 /*@notnull@*/ /*@in@*/ /*@out@*/ x
+#define ARGOUT_NULLOK(x)            /*@out@*/ x
+
 
 #endif /* PARROT_COMPILER_H_GUARD */
 
