@@ -3,7 +3,7 @@
  *  Copyright (C) 2002, The Perl Foundation.
  *  License:  Artistic/GPL, see README and LICENSES for details
  *  SVN Info
- *     $Id: datatypes.h 18975 2007-06-13 16:48:54Z bernhard $
+ *     $Id: datatypes.h 19881 2007-07-14 22:05:00Z petdance $
  *  Overview:
  *     Parrot and native data types enums and type names.
  *
@@ -66,7 +66,7 @@ typedef enum {
     enum_type_sized,            /* arbitrary size type for list_new */
     enum_last_type              /* + one */
 
-} PARROT_DATA_TYPES;
+} PARROT_DATA_TYPE;
 
 /* &end_gen */
 struct _data_types {
@@ -128,14 +128,18 @@ extern const struct _data_types data_types[];
 
 /* HEADERIZER BEGIN: src/datatypes.c */
 
-PARROT_API INTVAL Parrot_get_datatype_enum( Interp *interp,
-    const STRING *type_name /*NN*/ )
-        __attribute__nonnull__(2)
-        __attribute__pure__
-        __attribute__warn_unused_result__;
+PARROT_API
+PARROT_WARN_UNUSED_RESULT
+INTVAL Parrot_get_datatype_enum( PARROT_INTERP,
+    NOTNULL(const STRING *type_name) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
-PARROT_API STRING * Parrot_get_datatype_name( Interp *interp, INTVAL type )
-        __attribute__warn_unused_result__;
+PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+STRING * Parrot_get_datatype_name( PARROT_INTERP, INTVAL type )
+        __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/datatypes.c */
 
