@@ -2,7 +2,7 @@
  * debug.h
  *
  * SVN Info
- *    $Id: debug.h 19914 2007-07-16 21:06:48Z petdance $
+ *    $Id: debug.h 20342 2007-07-30 05:57:58Z petdance $
  * Overview:
  *    Parrot debugger header files
  * History:
@@ -177,17 +177,21 @@ long PDB_add_label(
 void PDB_backtrace( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
 char PDB_break( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
 char PDB_check_condition( PARROT_INTERP, NOTNULL(PDB_condition_t *condition) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CAN_RETURN_NULL
 opcode_t * PDB_compile( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CAN_RETURN_NULL
 PDB_condition_t * PDB_cond( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -207,7 +211,7 @@ void PDB_disable_breakpoint( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void PDB_disassemble( PARROT_INTERP, const char *command )
+void PDB_disassemble( PARROT_INTERP, SHIM(const char *command) )
         __attribute__nonnull__(1);
 
 size_t PDB_disassemble_op( PARROT_INTERP,
@@ -227,6 +231,8 @@ void PDB_enable_breakpoint( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 char * PDB_escape( NOTNULL(const char *string), INTVAL length )
         __attribute__nonnull__(1);
 
@@ -237,6 +243,8 @@ void PDB_eval( PARROT_INTERP, NOTNULL(const char *command) )
 int PDB_extend_const_table( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PDB_breakpoint_t * PDB_find_breakpoint( PARROT_INTERP,
     NOTNULL(const char *command) )
         __attribute__nonnull__(1)
@@ -250,7 +258,8 @@ void PDB_get_command( PARROT_INTERP )
 
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
-char PDB_hasinstruction( const char *c );
+char PDB_hasinstruction( NOTNULL(const char *c) )
+        __attribute__nonnull__(1);
 
 void PDB_help( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
@@ -259,7 +268,7 @@ void PDB_help( PARROT_INTERP, NOTNULL(const char *command) )
 void PDB_info( PARROT_INTERP )
         __attribute__nonnull__(1);
 
-void PDB_init( PARROT_INTERP, const char *command )
+void PDB_init( PARROT_INTERP, SHIM(const char *command) )
         __attribute__nonnull__(1);
 
 void PDB_list( PARROT_INTERP, NOTNULL(const char *command) )
@@ -277,12 +286,14 @@ void PDB_print( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void PDB_print_user_stack( PARROT_INTERP, const char *command )
-        __attribute__nonnull__(1);
+void PDB_print_user_stack( PARROT_INTERP, NOTNULL(const char *command) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 char PDB_program_end( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_IGNORABLE_RESULT
 int PDB_run_command( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -291,8 +302,9 @@ void PDB_script_file( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void PDB_set_break( PARROT_INTERP, const char *command /*NULL*/ )
-        __attribute__nonnull__(1);
+void PDB_set_break( PARROT_INTERP, NOTNULL(const char *command) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 void PDB_skip_breakpoint( PARROT_INTERP, long i )
         __attribute__nonnull__(1);
@@ -300,7 +312,9 @@ void PDB_skip_breakpoint( PARROT_INTERP, long i )
 void PDB_trace( PARROT_INTERP, NULLOK(const char *command) )
         __attribute__nonnull__(1);
 
-int PDB_unescape( char *string );
+int PDB_unescape( NOTNULL(char *string) )
+        __attribute__nonnull__(1);
+
 void PDB_watchpoint( PARROT_INTERP, NOTNULL(const char *command) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
