@@ -1,7 +1,7 @@
 /* dod.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: dod.h 19808 2007-07-12 05:07:56Z petdance $
+ *     $Id: dod.h 20663 2007-08-18 01:37:27Z chromatic $
  *  Overview:
  *     Handles dead object destruction of the various headers
  *  History:
@@ -44,7 +44,8 @@
 #define DOD_trace_stack_FLAG    (UINTVAL)(1 << 0)   /* trace system areads and stack */
 #define DOD_trace_normal        (UINTVAL)(1 << 0)   /* the same */
 #define DOD_lazy_FLAG           (UINTVAL)(1 << 1)   /* timely destruction run */
-#define DOD_finish_FLAG         (UINTVAL)(1 << 2)   /* run async past sweep */
+#define DOD_finish_FLAG         (UINTVAL)(1 << 2)   /* on Parrot exit: mark (almost) all PMCs dead and */
+                                                    /* garbage collect. */
 #define DOD_no_trace_volatile_roots (UINTVAL)(1 << 3)
             /* trace all but volatile root set, i.e. registers */
 
@@ -133,19 +134,26 @@ void parrot_gc_gms_pobject_lives( PARROT_INTERP, NOTNULL(PObj *obj) )
         __attribute__nonnull__(2);
 
 void parrot_gc_gms_wb( PARROT_INTERP,
-    PMC *agg,
-    void *old,
-    NOTNULL(void *new) )
+    NOTNULL(PMC *agg),
+    NOTNULL(void *old),
+    NOTNULL(void *_new) )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
 void parrot_gc_gms_wb_key( PARROT_INTERP,
-    PMC *agg,
-    void *old,
-    void *old_key,
-    void *new,
-    void *new_key )
-        __attribute__nonnull__(1);
+    NOTNULL(PMC *agg),
+    NOTNULL(void *old),
+    NOTNULL(void *old_key),
+    NOTNULL(void *_new),
+    NOTNULL(void *new_key) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5)
+        __attribute__nonnull__(6);
 
 /* HEADERIZER END: src/gc/gc_gms.c */
 
