@@ -1,6 +1,6 @@
 /* dynext.h
 *
-* $Id: dynext.h 19886 2007-07-15 06:19:53Z petdance $
+* $Id: dynext.h 20342 2007-07-30 05:57:58Z petdance $
 *
 *   Parrot dynamic extensions
 */
@@ -13,19 +13,27 @@
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
-PMC * Parrot_clone_lib_into( Interp *d, Interp *s, PMC *lib_pmc );
+PMC * Parrot_clone_lib_into(
+    NOTNULL(Interp *d),
+    NOTNULL(Interp *s),
+    NOTNULL(PMC *lib_pmc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_API
 PMC * Parrot_init_lib( PARROT_INTERP,
-    PMC *(*load_func)(NULLOK(Interp *)),
-    void (*init_func)(Interp *,
+    PMC *(*load_func)(PARROT_INTERP),
+    void (*init_func)(PARROT_INTERP,
     NULLOK(PMC *)) )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
-PMC * Parrot_load_lib( PARROT_INTERP, NULLOK(STRING *lib), PMC *initializer )
+PMC * Parrot_load_lib( PARROT_INTERP,
+    NULLOK(STRING *lib),
+    SHIM(PMC *initializer) )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/dynext.c */
