@@ -1,7 +1,7 @@
 /* embed.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: embed.h 18945 2007-06-12 14:08:35Z fperrad $
+ *     $Id: embed.h 20435 2007-08-02 22:34:32Z particle $
  *  Overview:
  *     This is the Parrot embedding system--the only part of Parrot that
  *     the outside world should see.
@@ -60,8 +60,9 @@ PARROT_API Parrot_Opcode * Parrot_debug(Parrot_Interp, Parrot_Opcode *pc);
 
 PARROT_API void Parrot_disassemble(Parrot_Interp);
 
-PARROT_API void Parrot_exit(Parrot_Interp, int status)
-                __attribute__noreturn__;
+PARROT_API
+PARROT_DOES_NOT_RETURN
+void Parrot_exit(Parrot_Interp, int status);
 
 PARROT_API void Parrot_run_native(Parrot_Interp interp, native_func_t func);
 
