@@ -2,7 +2,7 @@
  * exec.h
  *
  * SVN Info
- *    $Id: exec.h 19833 2007-07-13 22:02:25Z petdance $
+ *    $Id: exec.h 20704 2007-08-19 05:02:46Z chromatic $
  * Overview:
  *    Exec header file.
  * History:
@@ -16,20 +16,20 @@
 
 #if EXEC_CAPABLE
 
-#  if PARROT_EXEC_OS_OPENBSD
+#  ifdef PARROT_EXEC_OS_OPENBSD
 #    ifdef PARROT_OPENBSD_ELF
 #      define EXEC_ELF
 #    else
 #      define EXEC_A_OUT
 #    endif
 #  endif
-#  if PARROT_EXEC_OS_DARWIN
+#  ifdef PARROT_EXEC_OS_DARWIN
 #    define EXEC_MACH_O
 #  endif
-#  if (PARROT_EXEC_OS_FREEBSD) || (PARROT_EXEC_OS_NETBSD) || (PARROT_EXEC_OS_LINUX)
+#  if defined(PARROT_EXEC_OS_FREEBSD) || defined(PARROT_EXEC_OS_NETBSD) || defined(PARROT_EXEC_OS_LINUX)
 #    define EXEC_ELF
 #  endif
-#  if PARROT_EXEC_OS_MSWIN32
+#  ifdef PARROT_EXEC_OS_MSWIN32
 #    define EXEC_COFF
 #  endif
 
@@ -94,24 +94,45 @@ typedef struct Parrot_exec_objfile_t {
 } Parrot_exec_objfile_t;
 
 /* HEADERIZER BEGIN: src/exec.c */
-void Parrot_exec(PARROT_INTERP, opcode_t *pc,
-    opcode_t *code_start, opcode_t *code_end);
-PARROT_API void Parrot_exec_add_text_rellocation_func(Parrot_exec_objfile_t *obj,
-    char *nptr, const char *func_name);
-PARROT_API int *Parrot_exec_add_text_rellocation_reg(Parrot_exec_objfile_t *obj,
-    char *nptr, const char *var, int offset, int disp);
-PARROT_API void Parrot_exec_add_text_rellocation(Parrot_exec_objfile_t *obj,
-    char *nptr, int type, const char *symbol, int disp);
-int Parrot_exec_add_symbol(Parrot_exec_objfile_t *obj, const char *symbol, int stype)
-    __attribute__nonnull__(1)
-    __attribute__nonnull__(2);
 
-void Parrot_exec_save(PARROT_INTERP, Parrot_exec_objfile_t *obj, const char *file);
+PARROT_API
+void Parrot_exec_add_text_rellocation(
+    NOTNULL(Parrot_exec_objfile_t *obj),
+    char *nptr,
+    int type,
+    const char *symbol,
+    int disp )
+        __attribute__nonnull__(1);
+
+PARROT_API
+void Parrot_exec_add_text_rellocation_func(
+    NOTNULL(Parrot_exec_objfile_t *obj),
+    char *nptr,
+    const char *func_name )
+        __attribute__nonnull__(1);
+
+PARROT_API
+int * Parrot_exec_add_text_rellocation_reg(
+    NOTNULL(Parrot_exec_objfile_t *obj),
+    char *nptr,
+    const char *var,
+    int offset,
+    int disp )
+        __attribute__nonnull__(1);
+
+void Parrot_exec( PARROT_INTERP,
+    opcode_t *pc,
+    opcode_t *code_start,
+    opcode_t *code_end )
+        __attribute__nonnull__(1);
+
+int Parrot_exec_add_symbol(
+    NOTNULL(Parrot_exec_objfile_t *obj),
+    NOTNULL(const char *symbol),
+    int stype )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
-void Parrot_exec_emit_mov_mr(PARROT_INTERP, char *mem, int reg);
-void Parrot_exec_emit_mov_mr_n(PARROT_INTERP, char *mem, int reg);
-void Parrot_exec_emit_mov_rm(PARROT_INTERP, int reg, char *mem);
-void Parrot_exec_emit_mov_rm_n(PARROT_INTERP, int reg, char *mem);
 /* HEADERIZER END: src/exec.c */
 
 #endif /* EXEC_CAPABLE */
