@@ -1,7 +1,7 @@
 /* hll.h
  *  Copyright (C) 2005-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: hll.h 19882 2007-07-14 22:33:30Z petdance $
+ *     $Id: hll.h 20661 2007-08-17 22:39:03Z chromatic $
  *  Overview:
  *     This is the API header for the HLL subsystem
  *  Data Structure and Algorithms:
@@ -22,6 +22,8 @@
 /* HEADERIZER BEGIN: src/hll.c */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC* Parrot_get_ctx_HLL_namespace( PARROT_INTERP )
         __attribute__nonnull__(1);
 
@@ -34,11 +36,15 @@ PARROT_WARN_UNUSED_RESULT
 INTVAL Parrot_get_HLL_id( PARROT_INTERP, NULLOK(STRING *hll_name) )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PARROT_API
 STRING * Parrot_get_HLL_name( PARROT_INTERP, INTVAL id )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC* Parrot_get_HLL_namespace( PARROT_INTERP, int hll_id )
         __attribute__nonnull__(1);
 
@@ -53,7 +59,7 @@ void Parrot_regenerate_HLL_namespaces( PARROT_INTERP )
 PARROT_API
 INTVAL Parrot_register_HLL( PARROT_INTERP,
     NULLOK(STRING *hll_name),
-    STRING *hll_lib )
+    NULLOK(STRING *hll_lib) )
         __attribute__nonnull__(1);
 
 PARROT_API
@@ -65,6 +71,13 @@ void Parrot_register_HLL_type( PARROT_INTERP,
 
 /* HEADERIZER END: src/hll.c */
 
+typedef enum HLL_enum_t {
+    e_HLL_name,
+    e_HLL_lib,
+    e_HLL_typemap,
+    e_HLL_MAX
+} HLL_enum_t;
+
 #endif /* PARROT_HLL_H_GUARD */
 
 /*
