@@ -99,7 +99,7 @@ int Parrot_init_arg_indexes_and_sig_pmc( PARROT_INTERP,
 
 PARROT_API
 int Parrot_init_arg_nci( PARROT_INTERP,
-    NOTNULL(call_state *st),
+    ARGOUT(call_state *st),
     NOTNULL(const char *sig) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -115,18 +115,18 @@ int Parrot_init_arg_op( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 PARROT_API
-int Parrot_init_arg_sig( PARROT_INTERP,
-    parrot_context_t *ctx,
+int Parrot_init_arg_sig( SHIM_INTERP,
+    NOTNULL(parrot_context_t *ctx),
     NOTNULL(const char *sig),
-    void *ap,
+    NULLOK(void *ap),
     NOTNULL(call_state_item *sti) )
-        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(5);
 
 PARROT_API
 int Parrot_init_ret_nci( PARROT_INTERP,
-    NOTNULL(call_state *st),
+    ARGOUT(call_state *st),
     NOTNULL(const char *sig) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -147,11 +147,13 @@ void parrot_pass_args( PARROT_INTERP,
 
 PARROT_API
 void Parrot_PCCINVOKE( PARROT_INTERP,
-    PMC* pmc,
-    STRING *method_name,
-    const char *signature,
+    NULLOK(PMC* pmc),
+    NOTNULL(STRING *method_name),
+    NOTNULL(const char *signature),
     ... )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_API
 void Parrot_process_args( PARROT_INTERP,
@@ -160,17 +162,24 @@ void Parrot_process_args( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 opcode_t * parrot_pass_args_fromc( PARROT_INTERP,
-    const char *sig,
-    opcode_t *dest,
-    parrot_context_t *old_ctxp,
+    NOTNULL(const char *sig),
+    NOTNULL(opcode_t *dest),
+    NOTNULL(parrot_context_t *old_ctxp),
     va_list ap )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 int Parrot_store_arg( PARROT_INTERP, NOTNULL(call_state *st) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 void* set_retval( PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
@@ -187,12 +196,16 @@ INTVAL set_retval_i( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC* set_retval_p( PARROT_INTERP,
     int sig_ret,
     NOTNULL(parrot_context_t *ctx) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING* set_retval_s( PARROT_INTERP,
     int sig_ret,
     NOTNULL(parrot_context_t *ctx) )
@@ -202,8 +215,8 @@ STRING* set_retval_s( PARROT_INTERP,
 /* HEADERIZER END: src/inter_call.c */
 
 #define ASSERT_SIG_PMC(sig) \
-    assert(PObj_is_PMC_TEST(sig) && \
-           sig->vtable->base_type == enum_class_FixedIntegerArray)
+    PARROT_ASSERT(PObj_is_PMC_TEST(sig)); \
+    PARROT_ASSERT(sig->vtable->base_type == enum_class_FixedIntegerArray)
 
 #define SIG_ELEMS(sig) PMC_int_val(sig)
 #define SIG_ARRAY(sig) (INTVAL*)PMC_data(sig)
