@@ -1,7 +1,7 @@
 /* key.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: key.h 19914 2007-07-16 21:06:48Z petdance $
+ *     $Id: key.h 20042 2007-07-20 20:07:50Z petdance $
  *  Overview:
  *     This is the API header for the PMC subsystem
  *  Data Structure and Algorithms:
@@ -39,11 +39,14 @@ typedef enum {
 /* HEADERIZER BEGIN: src/key.c */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_IGNORABLE_RESULT
 PMC * key_append( SHIM_INTERP, NOTNULL(PMC *key1), NOTNULL(PMC *key2) )
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL key_integer( PARROT_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -54,39 +57,58 @@ void key_mark( PARROT_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_new( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_new_cstring( PARROT_INTERP, NULLOK(const char *value) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_new_integer( PARROT_INTERP, INTVAL value )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_new_number( PARROT_INTERP, FLOATVAL value )
         __attribute__nonnull__(1);
 
 PARROT_API
-PMC * key_new_pmc( PARROT_INTERP, PMC *value )
-        __attribute__nonnull__(1);
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+PMC * key_new_pmc( PARROT_INTERP, NOTNULL(PMC *value) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
-PMC * key_new_string( PARROT_INTERP, STRING *value )
-        __attribute__nonnull__(1);
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+PMC * key_new_string( PARROT_INTERP, NOTNULL(STRING *value) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_next( SHIM_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 FLOATVAL key_number( PARROT_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC * key_pmc( PARROT_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -100,9 +122,10 @@ void key_set_number( SHIM_INTERP, NOTNULL(PMC *key), FLOATVAL value )
         __attribute__nonnull__(2);
 
 PARROT_API
-void key_set_pmc( PARROT_INTERP, NOTNULL(PMC *key), PMC *value )
+void key_set_pmc( PARROT_INTERP, NOTNULL(PMC *key), NOTNULL(PMC *value) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_API
 void key_set_register( SHIM_INTERP,
@@ -112,19 +135,25 @@ void key_set_register( SHIM_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_API
-void key_set_string( SHIM_INTERP, NOTNULL(PMC *key), STRING *value )
-        __attribute__nonnull__(2);
+void key_set_string( SHIM_INTERP, NOTNULL(PMC *key), NOTNULL(STRING *value) )
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING * key_set_to_string( PARROT_INTERP, NULLOK(PMC *key) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * key_string( PARROT_INTERP, NOTNULL(PMC *key) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL key_type( SHIM_INTERP, NOTNULL(const PMC *key) )
         __attribute__nonnull__(2);
 
