@@ -1,7 +1,7 @@
 /* longopt.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: longopt.h 19064 2007-06-17 15:23:33Z petdance $
+ *     $Id: longopt.h 19833 2007-07-13 22:02:25Z petdance $
  *  Overview:
  *     Command line option parsing (for pre-initialized code)
  *  Data Structure and Algorithms:
@@ -46,20 +46,18 @@ struct longopt_opt_info {
 
 /* HEADERIZER BEGIN: src/longopt.c */
 
-PARROT_API int longopt_get( Interp *interp /*NN*/,
+PARROT_API
+int longopt_get( PARROT_INTERP,
     int argc,
-    char* argv[] /*NN*/,
-    const struct longopt_opt_decl options[] /*NN*/,
-    struct longopt_opt_info* info_buf /*NN*/ )
+    NOTNULL(char* argv[]),
+    NOTNULL(const struct longopt_opt_decl options[]),
+    NOTNULL(struct longopt_opt_info* info_buf) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4)
         __attribute__nonnull__(5);
 
 /* HEADERIZER END: src/longopt.c */
-PARROT_API int longopt_get(Parrot_Interp, int argc, char* argv[],
-                const struct longopt_opt_decl options[],
-                struct longopt_opt_info* info_buf);
 
 #endif /* PARROT_LONGOPT_H_GUARD */
 
