@@ -1,7 +1,7 @@
 /* pmc.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: pmc.h 19885 2007-07-15 04:02:08Z petdance $
+ *     $Id: pmc.h 20342 2007-07-30 05:57:58Z petdance $
  *  Overview:
  *     This is the api header for the pmc subsystem
  *  Data Structure and Algorithms:
@@ -22,20 +22,26 @@
 /* HEADERIZER BEGIN: src/pmc.c */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 PMC * constant_pmc_new( PARROT_INTERP, INTVAL base_type )
         __attribute__nonnull__(1);
 
 PARROT_API
-PMC * constant_pmc_new_init( PARROT_INTERP, INTVAL base_type, PMC *init )
+PARROT_CANNOT_RETURN_NULL
+PMC * constant_pmc_new_init( PARROT_INTERP,
+    INTVAL base_type,
+    NULLOK(PMC *init) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 PMC * constant_pmc_new_noinit( PARROT_INTERP, INTVAL base_type )
         __attribute__nonnull__(1);
 
 PARROT_API
-void dod_register_pmc( PARROT_INTERP, PMC* pmc )
-        __attribute__nonnull__(1);
+void dod_register_pmc( PARROT_INTERP, NOTNULL(PMC* pmc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 void Parrot_create_mro( PARROT_INTERP, INTVAL type )
@@ -47,11 +53,13 @@ void Parrot_create_pmc_proxy( PARROT_INTERP, int type_num )
 
 PARROT_API
 PARROT_CANNOT_RETURN_NULL
+PARROT_MALLOC
 PMC * pmc_new( PARROT_INTERP, INTVAL base_type )
         __attribute__nonnull__(1);
 
 PARROT_API
-PMC * pmc_new_init( PARROT_INTERP, INTVAL base_type, PMC *init )
+PARROT_CANNOT_RETURN_NULL
+PMC * pmc_new_init( PARROT_INTERP, INTVAL base_type, NULLOK(PMC *init) )
         __attribute__nonnull__(1);
 
 PARROT_API
@@ -60,15 +68,16 @@ PMC * pmc_new_noinit( PARROT_INTERP, INTVAL base_type )
         __attribute__nonnull__(1);
 
 PARROT_API
-INTVAL pmc_register( PARROT_INTERP, STRING *name )
-        __attribute__nonnull__(1);
+INTVAL pmc_register( PARROT_INTERP, NOTNULL(STRING *name) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_CANNOT_RETURN_NULL
 PMC* pmc_reuse( PARROT_INTERP,
     NOTNULL(PMC *pmc),
     INTVAL new_type,
-    UINTVAL flags )
+    NULLOK(UINTVAL flags) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
@@ -83,8 +92,9 @@ INTVAL pmc_type_p( PARROT_INTERP, NOTNULL(PMC *name) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void dod_unregister_pmc( PARROT_INTERP, PMC* pmc )
-        __attribute__nonnull__(1);
+void dod_unregister_pmc( PARROT_INTERP, NOTNULL(PMC* pmc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 /* HEADERIZER END: src/pmc.c */
 
