@@ -1,7 +1,7 @@
 /* pobj.h
  *  Copyright (C) 2001-2005, The Perl Foundation.
  *  SVN Info
- *     $Id: pobj.h 19733 2007-07-09 20:01:28Z paultcochrane $
+ *     $Id: pobj.h 20258 2007-07-27 08:00:07Z petdance $
  *  Overview:
  *     Parrot Object data members and flags enum
  *  Data Structure and Algorithms:
@@ -129,8 +129,8 @@ struct parrot_string_t {
     char *strstart;
     UINTVAL strlen;
     /*    parrot_string_representation_t representation;*/
-    struct _encoding *encoding;
-    struct _charset *charset;
+    const struct _encoding *encoding;
+    const struct _charset *charset;
     UINTVAL hashval; /* cached hash value computation; not yet used */
 };
 
@@ -183,7 +183,7 @@ typedef struct PMC_EXT {
 #ifdef NDEBUG
 #  define PMC_ext_checked(pmc)             (pmc)->pmc_ext
 #else
-#  define PMC_ext_checked(pmc)             (assert((pmc)->pmc_ext), (pmc)->pmc_ext)
+#  define PMC_ext_checked(pmc)             (PARROT_ASSERT((pmc)->pmc_ext), (pmc)->pmc_ext)
 #endif /* NDEBUG */
 #if PMC_DATA_IN_EXT
 #  define PMC_data(pmc)                   PMC_ext_checked(pmc)->data
