@@ -1,7 +1,7 @@
 /* sub.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: sub.h 19833 2007-07-13 22:02:25Z petdance $
+ *     $Id: sub.h 20210 2007-07-26 18:54:21Z jonathan $
  *  Overview:
  *  Data Structure and Algorithms:
  *     Subroutine, coroutine, closure and continuation structures
@@ -58,6 +58,7 @@ typedef enum {
     SUB_COMP_FLAG_BIT_0     = SUB_FLAG(0),
     SUB_COMP_FLAG_BIT_1     = SUB_FLAG(1),
     SUB_COMP_FLAG_BIT_2     = SUB_FLAG(2),
+    SUB_COMP_FLAG_METHOD    = SUB_COMP_FLAG_BIT_2,
     SUB_COMP_FLAG_BIT_3     = SUB_FLAG(3),
     SUB_COMP_FLAG_BIT_4     = SUB_FLAG(4),
     SUB_COMP_FLAG_BIT_5     = SUB_FLAG(5),
@@ -87,7 +88,7 @@ typedef enum {
     SUB_COMP_FLAG_BIT_28    = SUB_FLAG(28),
     SUB_COMP_FLAG_BIT_29    = SUB_FLAG(29),
     SUB_COMP_FLAG_BIT_30    = SUB_FLAG(30),
-    SUB_COMP_FLAG_MASK      = 0x00000400
+    SUB_COMP_FLAG_MASK      = 0x00000404
 } sub_comp_flags_enum;
 #undef SUB_FLAG
 
@@ -209,7 +210,9 @@ typedef struct Parrot_Context_info {
 /* HEADERIZER BEGIN: src/sub.c */
 
 PARROT_API
-PMC * new_ret_continuation_pmc( PARROT_INTERP, opcode_t *address )
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
+PMC * new_ret_continuation_pmc( PARROT_INTERP, NULLOK(opcode_t *address) )
         __attribute__nonnull__(1);
 
 PARROT_API
@@ -221,18 +224,25 @@ int Parrot_Context_get_info( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING* Parrot_Context_infostr( PARROT_INTERP,
     NOTNULL(parrot_context_t *ctx) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING* Parrot_full_sub_name( PARROT_INTERP, NULLOK(PMC* sub) )
         __attribute__nonnull__(1);
 
 PARROT_API
-PMC* parrot_new_closure( PARROT_INTERP, PMC *sub_pmc )
-        __attribute__nonnull__(1);
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+PMC* parrot_new_closure( PARROT_INTERP, NOTNULL(PMC *sub_pmc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 void invalidate_retc_context( PARROT_INTERP, NOTNULL(PMC *cont) )
         __attribute__nonnull__(1)
@@ -242,25 +252,38 @@ void mark_context( PARROT_INTERP, NOTNULL(parrot_context_t* ctx) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Parrot_sub * new_closure( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Parrot_cont * new_continuation( PARROT_INTERP, NULLOK(Parrot_cont *to) )
         __attribute__nonnull__(1);
 
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Parrot_coro * new_coroutine( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Parrot_cont * new_ret_continuation( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Parrot_sub * new_sub( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC* Parrot_find_pad( PARROT_INTERP,
-    STRING *lex_name,
+    NOTNULL(STRING *lex_name),
     NOTNULL(parrot_context_t *ctx) )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
 /* HEADERIZER END: src/sub.c */
