@@ -19,23 +19,33 @@ package pirVisitor;
 .sub '__start' :anon :main
     .param pmc args
 #    print "start Lua\n"
+    load_bytecode 'languages/lua/lib/luaaux.pbc'
     load_bytecode 'languages/lua/lib/luabasic.pbc'
     load_bytecode 'languages/lua/lib/luacoroutine.pbc'
     load_bytecode 'languages/lua/lib/luapackage.pbc'
     load_bytecode 'languages/lua/lib/luastring.pbc'
+    load_bytecode 'languages/lua/lib/luaregex.pbc'
     load_bytecode 'languages/lua/lib/luatable.pbc'
     load_bytecode 'languages/lua/lib/luamath.pbc'
     load_bytecode 'languages/lua/lib/luaio.pbc'
+    load_bytecode 'languages/lua/lib/luafile.pbc'
     load_bytecode 'languages/lua/lib/luaos.pbc'
     load_bytecode 'languages/lua/lib/luadebug.pbc'
     load_bytecode 'languages/lua/lib/luaperl.pbc'
+    collectoff
+    lua_openlibs()
+    collecton
     .local pmc env
-    env = get_global '_G'
+    env = get_hll_global '_G'
     .local pmc vararg
     vararg = argstolua(env, args)
     .const .Sub main = '_main'
     main.'setfenv'(env)
-    docall(main, vararg :flat)
+    ($I0, $P0) = docall(main, vararg :flat)
+    unless $I0 goto L1
+    printerr 'luac.pl: '
+    printerr $P0
+  L1:
 .end
 
 };
@@ -149,7 +159,7 @@ package pirVisitor;
         my $self = shift;
         my ($op) = @_;
         my $FH   = $self->{fh};
-        print {$FH} "    $op->{result}->{symbol} = new $op->{arg1}\n";
+        print {$FH} "    $op->{result}->{symbol} = new '$op->{arg1}'\n";
         return;
     }
 
