@@ -1,4 +1,4 @@
-/* $Id: eval.c 16301 2006-12-28 22:26:04Z paultcochrane $ */
+/* $Id: eval.c 20092 2007-07-22 09:02:26Z paultcochrane $ */
 
 /* GNU m4 -- A simple macro processor
    Copyright (C) 1989, 90, 91, 92, 93, 94 Free Software Foundation, Inc.
@@ -115,7 +115,7 @@ eval_undo (void)
 
 static eval_token eval_lex( eval_t *val )
 {
-  while ( isspace(*eval_text) ) eval_text++;
+  while ( isspace((unsigned char)*eval_text) ) eval_text++;
 
   last_text = eval_text;
 
