@@ -1,5 +1,5 @@
 # Copyright (C) 2007, The Perl Foundation.
-# $Id: Test.pm 19254 2007-06-22 07:06:54Z chromatic $
+# $Id: Test.pm 20234 2007-07-27 00:19:01Z pmichaud $
 
 ## This is a temporary Test.pm to get us started until we get pugs's Test.pm
 ## working. It's shamelessly stolen & adapted from MiniPerl6 in the pugs repo.
@@ -53,8 +53,8 @@ sub proclaim($cond, $desc) {
     $num_of_tests_run = $num_of_tests_run + 1;
 
     if ( $cond ) {
-        say "ok ", $num_of_tests_run;
+        say "ok ", $num_of_tests_run, " - ", $desc;
     } else {
-        say "not ok ", $num_of_tests_run;
+        say "not ok ", $num_of_tests_run, " - ", $desc;
     }
 }
