@@ -1,4 +1,4 @@
-# $Id: Partridge.pm 17096 2007-02-20 20:20:48Z paultcochrane $
+# $Id: Partridge.pm 19183 2007-06-20 19:39:06Z bernhard $
 
 package Parrot::Test::Plumhead::Partridge;
 
@@ -8,6 +8,9 @@ use warnings;
 
 use base 'Parrot::Test::Plumhead';
 
+# Generate output_is(), output_isnt() and output_like() in current package.
+Parrot::Test::generate_languages_functions(); 
+
 sub get_out_fn {
     my $self = shift;
     my ( $count, $options ) = @_;
