@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: Configure.pm 19876 2007-07-14 20:32:31Z paultcochrane $
+# $Id: Configure.pm 20603 2007-08-11 02:11:28Z jkeenan $
 
 =head1 NAME
 
@@ -19,7 +19,7 @@ Parrot::Configure - Conducts the execution of Configuration Steps
 =head1 DESCRIPTION
 
 This module provides provides a means for registering, executing, and
-coordinating one or more Configuration steps.  Please see
+coordinating one or more configuration steps.  Please see
 F<docs/configuration.pod> for further details about the configuration
 framework.
 
@@ -39,6 +39,7 @@ use warnings;
 
 use lib qw(config);
 use Carp qw(carp);
+use Storable qw(nstore retrieve);
 use Parrot::Configure::Data;
 
 use Class::Struct;
@@ -96,9 +97,9 @@ Accepts no arguments and returns a L<Parrot::Configure::Data> object.
 =cut
 
 sub data {
-    my $self = shift;
+    my $conf = shift;
 
-    return $self->{data};
+    return $conf->{data};
 }
 
 =item * C<options()>
@@ -111,9 +112,9 @@ Accepts no arguments and returns a L<Parrot::Configure::Data> object.
 =cut
 
 sub options {
-    my $self = shift;
+    my $conf = shift;
 
-    return $self->{options};
+    return $conf->{options};
 }
 
 =item * C<steps()>
@@ -128,9 +129,9 @@ scalar context.
 =cut
 
 sub steps {
-    my $self = shift;
+    my $conf = shift;
 
-    return wantarray ? @{ $self->{steps} } : $self->{steps};
+    return wantarray ? @{ $conf->{steps} } : $conf->{steps};
 }
 
 =item * C<add_step()>
@@ -145,9 +146,13 @@ Accepts a list and modifies the data structure within the L<Parrot::Configure> o
 =cut
 
 sub add_step {
-    my ( $self, $step, @params ) = @_;
+    my ( $conf, $step, @params ) = @_;
 
-    push @{ $self->{steps} }, Parrot::Configure::Task->new( step => $step, params => \@params );
+    push @{ $conf->{steps} },
+        Parrot::Configure::Task->new(
+            step    => $step,
+            params  => \@params,
+        );
 
     return 1;
 }
@@ -161,10 +166,13 @@ Accepts a list of new steps and modifies the data structure within the L<Parrot:
 =cut
 
 sub add_steps {
-    my ( $self, @new_steps ) = @_;
+    my ( $conf, @new_steps ) = @_;
 
-    foreach my $step (@new_steps) {
-        $self->add_step($step);
+    $conf->{list_of_steps} = [ @new_steps ];
+
+    for (my $i = 0; $i <= $#new_steps; $i++) {
+        $conf->add_step($new_steps[$i]);
+        $conf->{hash_of_steps}->{$new_steps[$i]} = $i + 1;
     }
 
     return 1;
@@ -182,15 +190,15 @@ Accepts no arguments and modifies the data structure within the L<Parrot::Config
 =cut
 
 sub runsteps {
-    my $self = shift;
+    my $conf = shift;
 
     my $n = 0;    # step number
     my ( $verbose, $verbose_step, $ask ) =
-        $self->options->get( qw( verbose verbose-step ask ) );
+        $conf->options->get( qw( verbose verbose-step ask ) );
 
-    foreach my $task ( $self->steps ) {
+    foreach my $task ( $conf->steps ) {
         $n++;
-        $self->_run_this_step( {
+        $conf->_run_this_step( {
             task            => $task,
             verbose         => $verbose,
             verbose_step    => $verbose_step,
@@ -201,7 +209,7 @@ sub runsteps {
     return 1;
 }
 
-=item * C<runstep()>
+=item * C<run_single_step()>
 
 The invoking L<Parrot::Configure> object is passed as the first argument to
 each step's C<runstep()> method, followed by any parameters that were
@@ -211,16 +219,16 @@ Accepts no arguments and modifies the data structure within the L<Parrot::Config
 
 =cut
 
-sub runstep {
-    my $self     = shift;
+sub run_single_step {
+    my $conf     = shift;
     my $taskname = shift;
 
     my ( $verbose, $verbose_step, $ask ) =
-        $self->options->get( qw( verbose verbose-step ask ) );
+        $conf->options->get( qw( verbose verbose-step ask ) );
 
-    for my $task ( $self->steps() ) {
+    for my $task ( $conf->steps() ) {
         if ( $task->{"Parrot::Configure::Task::step"} eq $taskname ) {
-            $self->_run_this_step( {
+            $conf->_run_this_step( {
                 task            => $task,
                 verbose         => $verbose,
                 verbose_step    => $verbose_step,
@@ -232,7 +240,7 @@ sub runstep {
 }
 
 sub _run_this_step {
-    my $self = shift;
+    my $conf = shift;
     my $args = shift;
 
     my $step_name   = $args->{task}->step;
@@ -241,6 +249,11 @@ sub _run_this_step {
     eval "use $step_name;";
     die $@ if $@;
 
+    my $conftrace = [];
+    my $sto = q{.configure_trace.sto};
+    if ($conf->options->get(q{configure_trace}) and (-e $sto)) {
+        $conftrace = retrieve($sto);
+    }
     my $step = $step_name->new;
 
     # RT#43675 This works. but is probably not a good design.
@@ -250,20 +263,32 @@ sub _run_this_step {
 
     # set per step verbosity
     if ( defined $args->{verbose_step} ) {
-
-        # by step number
-        if ( $args->{verbose_step} =~ /^\d+$/ && $args->{n} == $args->{verbose_step} ) {
-            $self->options->set( verbose => 2 );
-        }
-
-        # by description
-        elsif ( $description =~ /$args->{verbose_step}/ ) {
-            $self->options->set( verbose => 2 );
+        if (
+            (
+                # by step number
+                ( $args->{verbose_step} =~ /^\d+$/ )
+                    and
+                ( $args->{n} == $args->{verbose_step} )
+            )
+                or
+            (
+                # by step name
+                ( ${$conf->{hash_of_steps}}{$args->{verbose_step}} )
+                    and
+                ( $args->{verbose_step} eq $step_name )
+            )
+                or
+            (
+                # by description
+                $description =~ /$args->{verbose_step}/
+            )
+        ) {
+            $conf->options->set( verbose => 2 );
         }
     }
 
     # RT#43673 cc_build uses this verbose setting, why?
-    $self->data->set( verbose => $args->{verbose} ) if $args->{n} > 2;
+    $conf->data->set( verbose => $args->{verbose} ) if $args->{n} > 2;
 
     print "\n", $description, '...';
     print "\n" if $args->{verbose} && $args->{verbose} == 2;
@@ -272,10 +297,10 @@ sub _run_this_step {
     eval {
         if (@step_params)
         {
-            $ret = $step->runstep( $self, @step_params );
+            $ret = $step->runstep( $conf, @step_params );
         }
         else {
-            $ret = $step->runstep($self);
+            $ret = $step->runstep($conf);
         }
     };
     if ($@) {
@@ -298,11 +323,51 @@ sub _run_this_step {
     print "..." if $args->{verbose} && $args->{verbose} == 2;
     print "." x ( 71 - length($description) - length($result) );
     print "$result." unless $step =~ m{^inter/} && $args->{ask};
-
     # reset verbose value for the next step
-    $self->options->set( verbose => $args->{verbose} );
+    $conf->options->set( verbose => $args->{verbose} );
+
+    if ($conf->options->get(q{configure_trace}) ) {
+        if (! defined $conftrace->[0]) {
+            $conftrace->[0] = [];
+        }
+        push @{$conftrace->[0]}, $step_name;
+        my $evolved_data = {
+            options => $conf->{options},
+            data    => $conf->{data},
+        };
+        push @{$conftrace}, $evolved_data;
+        nstore($conftrace, $sto);
+    }
 }
 
+=item * C<option_or_data($arg)>
+
+Are you tired of this construction all over the place?
+
+    my $opt = $conf->options->get( $arg );
+       $opt = $conf->data->get( $arg ) unless defined $opt;
+
+It gives you the user-specified option for I<$arg>, and if there
+isn't one, it gets it from the created data.  You do it all the
+time, but oh! the wear and tear on your fingers!
+
+Toil no more!  Use this simple construction:
+
+    my $opt = $conf->option_or_data($arg);
+
+and save your fingers for some real work!
+
+=cut
+
+sub option_or_data {
+    my $conf = shift;
+    my $arg = shift;
+
+    my $opt = $conf->options->get( $arg );
+    return defined $opt ? $opt : $conf->data->get( $arg );
+}
+
+
 =back
 
 =head1 CREDITS
