@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: Options.pm 19521 2007-07-01 15:30:20Z jkeenan $
+# $Id: Options.pm 20603 2007-08-11 02:11:28Z jkeenan $
 package Parrot::Configure::Options;
 use strict;
 use warnings;
@@ -18,6 +18,7 @@ sub get_valid_options {
         ccflags
         ccwarn
         cgoto
+        configure_trace
         cxx
         datadir
         debugging
@@ -77,7 +78,6 @@ sub get_valid_options {
 
 sub process_options {
     my $optionsref = shift;
-    my @valid_opts = get_valid_options();
     $optionsref->{argv} = []
         unless defined $optionsref->{argv};
     $optionsref->{script} = q{Configure.pl}
@@ -86,6 +86,7 @@ sub process_options {
         unless $optionsref->{parrot_version};
     die "Must provide argument 'svnid'"
         unless $optionsref->{svnid};
+    my @valid_opts = get_valid_options();
     my %args;
     for ( @{ $optionsref->{argv} } ) {
         my ( $key, $value ) = m/--([-\w]+)(?:=(.*))?/;
@@ -137,6 +138,7 @@ General Options:
    --verbose            Output extra information
    --verbose=2          Output every setting change
    --verbose-step=N     Set verbose for step N only
+   --verbose-step=name  Set verbose for step some::step only
    --verbose-step=regex Set verbose for step matching description
    --nomanicheck        Don't check the MANIFEST
    --step=(gen::languages)
@@ -270,7 +272,7 @@ Parrot::Configure::Options - Process command-line options to F<Configure.pl>
         script          => $0,
         parrot_version  => $parrot_version,
         svnid           =>
-            '$Id: Options.pm 19521 2007-07-01 15:30:20Z jkeenan $',
+            '$Id: Options.pm 20603 2007-08-11 02:11:28Z jkeenan $',
     } );
 
     @valid_options = get_valid_options();
