@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: Step.pm 19876 2007-07-14 20:32:31Z paultcochrane $
+# $Id: Step.pm 20748 2007-08-21 01:56:29Z jkeenan $
 
 =head1 NAME
 
@@ -61,10 +61,10 @@ sub integrate {
     my ( $orig, $new ) = @_;
 
     # Rather than sprinkling "if defined(...)", everywhere,
-    # config/inter/progs.pl just passes in potentially undefined
-    # strings.  Just pass back the original in that case.  Don't
-    # bother warning.  --AD, 12 Sep 2005
-    # warn "String to be integrated in to '$orig' undefined";
+    # various inter::* steps (coded in config/inter/*.pm) permit simply 
+    # passing in potentially undefined strings.
+    # In these instances, we simply pass back the original string without
+    # generating a warning.
     return $orig unless defined $new;
 
     if ( $new =~ /\S/ ) {
@@ -283,7 +283,7 @@ sub genfile {
     }
 
     # this loop can not be implemented as a foreach loop as the body
-    # is dependant on <IN> being evaluated lazily
+    # is dependent on <IN> being evaluated lazily
 
     while ( my $line = <$in> ) {
 
@@ -295,7 +295,7 @@ sub genfile {
             local *OUT = $out;
             my $text = do { local $/; <$in> };
 
-            # interoplate @foo@ values
+            # interpolate @foo@ values
             $text =~ s{ \@ (\w+) \@ }{\$conf->data->get("$1")}gx;
             eval $text;
             die $@ if $@;
@@ -453,9 +453,9 @@ sub _run_command {
                 && ( $_ ne File::Spec->devnull )
                 && ( !m/^&/ ) )
             {
-                open( my $out, '<', $_ );
-                print <$out>;
-                close $out;
+                open( my $verbose_handle, '<', $_ );
+                print <$verbose_handle>;
+                close $verbose_handle;
             }
         }
     }
@@ -463,6 +463,19 @@ sub _run_command {
     return $exit_code;
 }
 
+=item C<_build_compile_command( $cc, $ccflags, $cc_args )>
+
+Constructs a command-line to do the compile.
+
+=cut
+
+sub _build_compile_command {
+    my ( $cc, $ccflags, $cc_args ) = @_;
+    $_ ||= '' for ( $cc, $ccflags, $cc_args );
+
+    return "$cc $ccflags $cc_args -I./include -c test.c";
+}
+
 =item C<cc_gen($source)>
 
 Generates F<test.c> from the specified source file.
@@ -499,7 +512,8 @@ sub cc_build {
             cc_exe_out exe libs)
         );
 
-    _run_command( "$cc $ccflags $cc_args -I./include -c test.c", 'test.cco', 'test.cco', $verbose )
+    my $compile_command = _build_compile_command( $cc, $ccflags, $cc_args );
+    _run_command( $compile_command, 'test.cco', 'test.cco', $verbose )
         and confess "C compiler failed (see test.cco)";
 
     _run_command( "$link $linkflags test$o $link_args ${cc_exe_out}test$exe $libs",
