@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2005, The Perl Foundation.
-# $Id: Base.pm 17090 2007-02-20 20:06:55Z paultcochrane $
+# $Id: Base.pm 20629 2007-08-15 02:26:21Z jkeenan $
 
 =pod
 
@@ -46,8 +46,7 @@ Accepts no arguments and returns a L<Parrot::Configure::Step::> object.
 
 sub new {
     my $class = shift;
-
-    return bless {}, ref $class || $class;
+    return bless {}, $class;
 }
 
 =back
@@ -101,7 +100,7 @@ This method also works as an object method.
 
 sub description {
     my $class = shift;
-    $class = ref $class || $class;
+    $class = (ref $class) ? ref $class : $class;
     {
         no strict 'refs';
         return ${ $class . "::description" };
@@ -119,7 +118,7 @@ This method also works as an object method.
 
 sub args {
     my $class = shift;
-    $class = ref $class || $class;
+    $class = (ref $class) ? ref $class : $class;
     {
         no strict 'refs';
         return @{ $class . "::args" };
