@@ -1,5 +1,5 @@
 # Copyright (C) 2004, The Perl Foundation.
-# $Id: Perl.pm 16245 2006-12-25 22:15:39Z paultcochrane $
+# $Id: Perl.pm 20733 2007-08-20 09:59:28Z fperrad $
 
 =head1 NAME
 
@@ -50,7 +50,8 @@ sub new {
         ),
         $self->new_group(
             'PMCs', '',
-            $self->new_item( '', 'lib/Parrot/Pmc2c.pm' ),
+            'lib/Parrot/Pmc2c',
+            'lib/Parrot/Pmc2c/PMC',
             $self->new_item( '', 'lib/Parrot/Vtable.pm' ),
         ),
         $self->new_group( 'IO', '', 'lib/Parrot/Distribution.pm', 'lib/Parrot/IO' ),
