@@ -65,6 +65,7 @@ sub body {
 
     # Do we have a return value?
     my $return = $method->{type} =~ /void/ ? '' : 'return ';
+    my $void_return = $method->{type} =~ /void/ ? 'return;' : '';
 
     # work out what the null return should be so that we can quieten the "no
     # return from non-void function" warnings.
@@ -74,7 +75,13 @@ sub body {
     # icc), so we add a workaround for the null return from a FLOATVAL
     # function
     my $null_return;
-    if ($method->{type} =~ /void|PMC|INTVAL|STRING|opcode_t/) {
+    if ($method->{type} eq 'void') {
+        $null_return = '';
+    }
+    elsif ($method->{type} eq 'void*') {
+        $null_return = 'return NULL;';
+    }
+    elsif ($method->{type} =~ /PMC|INTVAL|STRING|opcode_t/) {
         $null_return = "return ($method->{type})NULL;";
     }
     # workaround for gcc because the general case doesn't work there
@@ -86,7 +93,7 @@ sub body {
     }
 
     my $l = $self->line_directive( $line + 1, "\L$self->{class}.c" );
-    return <<EOC;
+    my $generated = <<EOC;
 $l
 $decl {
     Parrot_Object * const obj = PARROT_OBJECT(pmc);
@@ -104,13 +111,24 @@ $decl {
         /* If it's from this universe or the class doesn't inherit from
          * anything outside of it... */
         if (all_in_universe || VTABLE_isa(interp, cur_class, string_from_literal(interp, "Class"))) {
+EOC
+
+    # We shouldn't allow overrides of get_pointer and friends,
+    # since it's unsafe.
+    if ($meth !~ /get_pointer/) {
+        $generated .= <<EOC;
             const Parrot_Class * const class_info = PARROT_CLASS(cur_class);
-            if (VTABLE_exists_keyed_str(interp, class_info->vtable_methods, string_from_literal(interp, "$meth"))) {
+            if (VTABLE_exists_keyed_str(interp, class_info->vtable_overrides, string_from_literal(interp, "$meth"))) {
                 /* Found it; call. */
                 PMC * const meth = VTABLE_get_pmc_keyed_str(interp,
-                    class_info->vtable_methods, string_from_literal(interp, "$meth"));
+                    class_info->vtable_overrides, string_from_literal(interp, "$meth"));
                 ${return}Parrot_run_meth_fromc_args$ret_type(interp, meth, pmc, string_from_literal(interp, "$meth"), "$sig"$arg);
+                $void_return
             }
+EOC
+    }
+
+    $generated .= <<EOC;
         }
         else {
             /* Get the PMC instance and call the vtable method on that. */
@@ -122,13 +140,15 @@ $decl {
     $null_return
 }
 EOC
+
+    return $generated;
 }
 
 
 sub ctype_to_sigchar($) {
     my $ctype = shift;
     $ctype =~ s/\s//g;
-    if (!$ctype) {
+    if (!$ctype || $ctype =~ /void/) {
         return "v";
     } elsif ($ctype =~ /opcode_t\*/) {
         # Only invoke's return needs this; we'll get away with this.
