@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2004-2007, The Perl Foundation.
-$Id: ascii.c 19882 2007-07-14 22:33:30Z petdance $
+$Id: ascii.c 20252 2007-07-27 03:33:24Z petdance $
 
 =head1 NAME
 
@@ -17,7 +17,6 @@ charset functionality for similar charsets like iso-8859-1.
 
 #include "parrot/parrot.h"
 #include "ascii.h"
-#include <assert.h>
 
 #ifdef EXCEPTION
 #  undef EXCEPTION
@@ -36,11 +35,13 @@ charset functionality for similar charsets like iso-8859-1.
 
 /* HEADERIZER BEGIN: static */
 
-static STRING* compose( PARROT_INTERP, NULLOK(STRING *src) )
-        __attribute__nonnull__(1);
+static STRING* compose( PARROT_INTERP, NOTNULL(STRING *src) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
-static STRING* decompose( PARROT_INTERP, NULLOK(STRING *src) )
-        __attribute__nonnull__(1);
+static STRING* decompose( PARROT_INTERP, NOTNULL(STRING *src) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 static void downcase( PARROT_INTERP, NOTNULL(STRING *source_string) )
         __attribute__nonnull__(1)
@@ -76,11 +77,13 @@ static INTVAL is_cclass( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 static void set_graphemes( PARROT_INTERP,
-    STRING *source_string,
+    NOTNULL(STRING *source_string),
     UINTVAL offset,
     UINTVAL replace_count,
-    STRING *insert_string )
-        __attribute__nonnull__(1);
+    NOTNULL(STRING *insert_string) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(5);
 
 PARROT_WARN_UNUSED_RESULT
 static STRING * string_from_codepoint( PARROT_INTERP, UINTVAL codepoint )
@@ -121,22 +124,23 @@ static void upcase_first( PARROT_INTERP, NOTNULL(STRING *source_string) )
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
-static UINTVAL validate( PARROT_INTERP, STRING *src )
-        __attribute__nonnull__(1);
+static UINTVAL validate( PARROT_INTERP, NOTNULL(STRING *src) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 /* HEADERIZER END: static */
 
 PARROT_WARN_UNUSED_RESULT
 STRING *
-ascii_get_graphemes(PARROT_INTERP, STRING *source_string,
+ascii_get_graphemes(PARROT_INTERP, NOTNULL(STRING *source_string),
         UINTVAL offset, UINTVAL count)
 {
     return ENCODING_GET_BYTES(interp, source_string, offset, count);
 }
 
 static void
-set_graphemes(PARROT_INTERP, STRING *source_string,
-        UINTVAL offset, UINTVAL replace_count, STRING *insert_string)
+set_graphemes(PARROT_INTERP, NOTNULL(STRING *source_string),
+        UINTVAL offset, UINTVAL replace_count, NOTNULL(STRING *insert_string))
 {
     ENCODING_SET_BYTES(interp, source_string, offset,
             replace_count, insert_string);
@@ -144,9 +148,10 @@ set_graphemes(PARROT_INTERP, STRING *source_string,
 }
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
-ascii_get_graphemes_inplace(PARROT_INTERP, STRING *source_string,
-        UINTVAL offset, UINTVAL count, STRING *dest_string)
+ascii_get_graphemes_inplace(PARROT_INTERP, NOTNULL(STRING *source_string),
+        UINTVAL offset, UINTVAL count, NOTNULL(STRING *dest_string))
 {
     return ENCODING_GET_BYTES_INPLACE(interp, source_string,
             offset, count, dest_string);
@@ -215,14 +220,14 @@ to_charset(PARROT_INTERP, NOTNULL(STRING *src), NULLOK(STRING *dest))
 
 /* A noop. can't compose ascii */
 static STRING*
-compose(PARROT_INTERP, NULLOK(STRING *src))
+compose(PARROT_INTERP, NOTNULL(STRING *src))
 {
     return string_copy(interp, src);
 }
 
 /* A noop. can't decompose ascii */
 static STRING*
-decompose(PARROT_INTERP, NULLOK(STRING *src))
+decompose(PARROT_INTERP, NOTNULL(STRING *src))
 {
     return string_copy(interp, src);
 }
@@ -238,7 +243,7 @@ upcase(PARROT_INTERP, NOTNULL(STRING *source_string))
         UINTVAL offset;
 
         for (offset = 0; offset < n; offset++) {
-            buffer[offset] = toupper(buffer[offset]);
+            buffer[offset] = toupper((unsigned char)buffer[offset]);
         }
     }
 }
@@ -254,7 +259,7 @@ downcase(PARROT_INTERP, NOTNULL(STRING *source_string))
         UINTVAL offset;
 
         for (offset = 0; offset < n; offset++) {
-            buffer[offset] = tolower(buffer[offset]);
+            buffer[offset] = tolower((unsigned char)buffer[offset]);
         }
     }
 }
@@ -269,9 +274,9 @@ titlecase(PARROT_INTERP, NOTNULL(STRING *source_string))
         char * const buffer = source_string->strstart;
         UINTVAL offset;
 
-        buffer[0] = toupper(buffer[0]);
+        buffer[0] = toupper((unsigned char)buffer[0]);
         for (offset = 1; offset < n; offset++) {
-            buffer[offset] = tolower(buffer[offset]);
+            buffer[offset] = tolower((unsigned char)buffer[offset]);
         }
     }
 }
@@ -283,7 +288,7 @@ upcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = toupper(buffer[0]);
+        buffer[0] = toupper((unsigned char)buffer[0]);
     }
 }
 
@@ -294,7 +299,7 @@ downcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = tolower(buffer[0]);
+        buffer[0] = tolower((unsigned char)buffer[0]);
     }
 }
 
@@ -305,7 +310,7 @@ titlecase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = toupper(buffer[0]);
+        buffer[0] = toupper((unsigned char)buffer[0]);
     }
 }
 
@@ -387,7 +392,7 @@ ascii_cs_index(PARROT_INTERP, NOTNULL(STRING *source_string),
                 offset);
     }
 
-    assert(source_string->encoding == Parrot_fixed_8_encoding_ptr);
+    PARROT_ASSERT(source_string->encoding == Parrot_fixed_8_encoding_ptr);
     retval = Parrot_byte_index(interp, source_string,
             search_string, offset);
     return retval;
@@ -403,7 +408,7 @@ ascii_cs_rindex(PARROT_INTERP, NOTNULL(STRING *source_string),
         real_exception(interp, NULL, UNIMPLEMENTED, "Cross-charset index not supported");
     }
 
-    assert(source_string->encoding == Parrot_fixed_8_encoding_ptr);
+    PARROT_ASSERT(source_string->encoding == Parrot_fixed_8_encoding_ptr);
     retval = Parrot_byte_rindex(interp, source_string,
             search_string, offset);
     return retval;
@@ -411,7 +416,7 @@ ascii_cs_rindex(PARROT_INTERP, NOTNULL(STRING *source_string),
 
 PARROT_WARN_UNUSED_RESULT
 static UINTVAL
-validate(PARROT_INTERP, STRING *src)
+validate(PARROT_INTERP, NOTNULL(STRING *src))
 {
     UINTVAL offset;
     String_iter iter;
@@ -459,7 +464,7 @@ find_cclass(PARROT_INTERP, INTVAL flags, NOTNULL(STRING *source_string),
     UINTVAL pos = offset;
     UINTVAL end = offset + count;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string != 0);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         const UINTVAL codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -477,7 +482,7 @@ find_not_cclass(PARROT_INTERP, INTVAL flags, NOTNULL(STRING *source_string),
     UINTVAL pos = offset;
     UINTVAL end = offset + count;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string != 0);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         const UINTVAL codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -499,7 +504,7 @@ ascii_compute_hash(SHIM_INTERP, NOTNULL(const STRING *source_string), size_t see
     const char *buffptr = (const char *)source_string->strstart;
     UINTVAL len = source_string->strlen;
 
-    assert(source_string->encoding == Parrot_fixed_8_encoding_ptr);
+    PARROT_ASSERT(source_string->encoding == Parrot_fixed_8_encoding_ptr);
     while (len--) {
         hashval += hashval << 5;
         hashval += *buffptr++;
@@ -507,7 +512,8 @@ ascii_compute_hash(SHIM_INTERP, NOTNULL(const STRING *source_string), size_t see
     return hashval;
 }
 
-CHARSET *
+PARROT_CANNOT_RETURN_NULL
+const CHARSET *
 Parrot_charset_ascii_init(PARROT_INTERP)
 {
     CHARSET * const return_set = Parrot_new_charset(interp);
