@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2004-2007, The Perl Foundation.
-$Id: iso-8859-1.c 19911 2007-07-16 19:22:55Z petdance $
+$Id: iso-8859-1.c 20342 2007-07-30 05:57:58Z petdance $
 
 =head1 NAME
 
@@ -25,7 +25,7 @@ static STRING* compose( PARROT_INTERP, NOTNULL(STRING *src) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static STRING* decompose( PARROT_INTERP, STRING *src )
+static STRING* decompose( PARROT_INTERP, SHIM(STRING *src) )
         __attribute__nonnull__(1);
 
 static void downcase( PARROT_INTERP, NOTNULL(STRING *source_string) )
@@ -38,10 +38,11 @@ static void downcase_first( PARROT_INTERP, NOTNULL(STRING *source_string) )
 
 static INTVAL find_cclass( PARROT_INTERP,
     INTVAL flags,
-    STRING *source_string,
+    NOTNULL(STRING *source_string),
     UINTVAL offset,
     UINTVAL count )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 static INTVAL find_not_cclass( PARROT_INTERP,
     INTVAL flags,
@@ -70,8 +71,9 @@ static void set_graphemes( PARROT_INTERP,
 static STRING * string_from_codepoint( PARROT_INTERP, UINTVAL codepoint )
         __attribute__nonnull__(1);
 
-static void titlecase( PARROT_INTERP, STRING *source_string )
-        __attribute__nonnull__(1);
+static void titlecase( PARROT_INTERP, NOTNULL(STRING *source_string) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 static void titlecase_first( PARROT_INTERP, NOTNULL(STRING *source_string) )
         __attribute__nonnull__(1)
@@ -97,8 +99,9 @@ static STRING * to_unicode( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static void upcase( PARROT_INTERP, STRING *source_string )
-        __attribute__nonnull__(1);
+static void upcase( PARROT_INTERP, NOTNULL(STRING *source_string) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 static void upcase_first( PARROT_INTERP, NOTNULL(STRING *source_string) )
         __attribute__nonnull__(1)
@@ -175,11 +178,8 @@ to_unicode(PARROT_INTERP, NOTNULL(STRING *src), NULLOK(STRING *dest))
         dest->strlen  = iter.charpos;
         return dest;
     }
-    else {
-        real_exception(interp, NULL, UNIMPLEMENTED,
-                "to_unicode inplace for iso-8859-1 not implemented");
-    }
-    return NULL;
+    real_exception(interp, NULL, UNIMPLEMENTED,
+            "to_unicode inplace for iso-8859-1 not implemented");
 }
 
 PARROT_WARN_UNUSED_RESULT
@@ -209,18 +209,16 @@ decompose(PARROT_INTERP, SHIM(STRING *src))
 {
     real_exception(interp, NULL, UNIMPLEMENTED,
             "decompose for iso-8859-1 not implemented");
-    return NULL;
 }
 
 static void
-upcase(PARROT_INTERP, STRING *source_string)
+upcase(PARROT_INTERP, NOTNULL(STRING *source_string))
 {
     unsigned char *buffer;
     UINTVAL offset = 0;
 
-    if (!source_string->strlen) {
+    if (!source_string->strlen)
         return;
-    }
 
     Parrot_unmake_COW(interp, source_string);
     buffer = (unsigned char *)source_string->strstart;
@@ -229,7 +227,7 @@ upcase(PARROT_INTERP, STRING *source_string)
         if (c >= 0xe0 && c != 0xf7)
             c &= ~0x20;
         else
-            c = toupper(c);
+            c = toupper((unsigned char)c);
         buffer[offset] = c;
     }
 }
@@ -248,29 +246,29 @@ downcase(PARROT_INTERP, NOTNULL(STRING *source_string))
             if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
                 c |= 0x20;
             else
-                c = tolower(c);
+                c = tolower((unsigned char)c);
             buffer[offset] = c;
         }
     }
 }
 
 static void
-titlecase(PARROT_INTERP, STRING *source_string)
+titlecase(PARROT_INTERP, NOTNULL(STRING *source_string))
 {
     unsigned char *buffer;
     unsigned int c;
     UINTVAL offset;
 
-    if (!source_string->strlen) {
+    if (!source_string->strlen)
         return;
-    }
+
     Parrot_unmake_COW(interp, source_string);
     buffer = (unsigned char *)source_string->strstart;
     c = buffer[0];
     if (c >= 0xe0 && c != 0xf7)
         c &= ~0x20;
     else
-        c = toupper(c);
+        c = toupper((unsigned char)c);
     buffer[0] = c;
 
     for (offset = 1; offset < source_string->strlen; offset++) {
@@ -278,7 +276,7 @@ titlecase(PARROT_INTERP, STRING *source_string)
         if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
             c |= 0x20;
         else
-            c = tolower(c);
+            c = tolower((unsigned char)c);
         buffer[offset] = c;
     }
 }
@@ -296,7 +294,7 @@ upcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
         if (c >= 0xe0 && c != 0xf7)
             c &= ~0x20;
         else
-            c = toupper(c);
+            c = toupper((unsigned char)c);
         buffer[0] = c;
     }
 }
@@ -314,9 +312,9 @@ downcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
         if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
             c &= ~0x20;
         else
-            c = tolower(c);
+            c = tolower((unsigned char)c);
         buffer[0] = c;
-        buffer[0] = toupper(buffer[0]);
+        buffer[0] = toupper((unsigned char)buffer[0]);
     }
 }
 
@@ -358,13 +356,13 @@ is_cclass(PARROT_INTERP, INTVAL flags,
 
 static INTVAL
 find_cclass(PARROT_INTERP, INTVAL flags,
-            STRING *source_string, UINTVAL offset, UINTVAL count)
+            NOTNULL(STRING *source_string), UINTVAL offset, UINTVAL count)
 {
     UINTVAL pos = offset;
     UINTVAL end = offset + count;
     UINTVAL codepoint;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string != 0);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -382,7 +380,7 @@ find_not_cclass(PARROT_INTERP, INTVAL flags,
     UINTVAL pos = offset;
     UINTVAL end = offset + count;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         const UINTVAL codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -403,7 +401,8 @@ string_from_codepoint(PARROT_INTERP, UINTVAL codepoint)
     return return_string;
 }
 
-CHARSET *
+PARROT_CANNOT_RETURN_NULL
+const CHARSET *
 Parrot_charset_iso_8859_1_init(PARROT_INTERP)
 {
     CHARSET * const return_set = Parrot_new_charset(interp);
