@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2005-2007, The Perl Foundation.
-$Id: unicode.c 19914 2007-07-16 21:06:48Z petdance $
+$Id: unicode.c 20342 2007-07-30 05:57:58Z petdance $
 
 =head1 NAME
 
@@ -40,12 +40,12 @@ static size_t compute_hash( PARROT_INTERP,
 
 static INTVAL cs_rindex( PARROT_INTERP,
     NOTNULL(STRING *source_string),
-    STRING *search_string,
+    SHIM(STRING *search_string),
     UINTVAL offset )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static STRING* decompose( PARROT_INTERP, STRING *src )
+static STRING* decompose( PARROT_INTERP, SHIM(STRING *src) )
         __attribute__nonnull__(1);
 
 static void downcase( PARROT_INTERP, NOTNULL(STRING *src) )
@@ -99,9 +99,10 @@ static void set_graphemes( PARROT_INTERP,
     NOTNULL(STRING *source_string),
     UINTVAL offset,
     UINTVAL replace_count,
-    STRING *insert_string )
+    NOTNULL(STRING *insert_string) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(5);
 
 static STRING * string_from_codepoint( PARROT_INTERP, UINTVAL codepoint )
         __attribute__nonnull__(1);
@@ -156,7 +157,7 @@ static UINTVAL validate( PARROT_INTERP, NOTNULL(STRING *src) )
 
 static void
 set_graphemes(PARROT_INTERP, NOTNULL(STRING *source_string),
-        UINTVAL offset, UINTVAL replace_count, STRING *insert_string)
+        UINTVAL offset, UINTVAL replace_count, NOTNULL(STRING *insert_string))
 {
     ENCODING_SET_CODEPOINTS(interp, source_string, offset,
             replace_count, insert_string);
@@ -224,7 +225,7 @@ compose(PARROT_INTERP, NOTNULL(STRING *src))
                                      * icu version */
                 (UChar *)dest->strstart, dest_len,
                 &err);
-        assert(U_SUCCESS(err));
+        PARROT_ASSERT(U_SUCCESS(err));
         dest->bufused = dest_len * sizeof (UChar);
     }
     dest->strlen = dest_len;
@@ -232,7 +233,6 @@ compose(PARROT_INTERP, NOTNULL(STRING *src))
 #else
     real_exception(interp, NULL, E_LibraryNotLoadedError,
             "no ICU lib loaded");
-    return NULL;
 #endif
 }
 
@@ -240,7 +240,6 @@ static STRING*
 decompose(PARROT_INTERP, SHIM(STRING *src))
 {
     UNIMPL;
-    return NULL;
 }
 
 static void
@@ -298,7 +297,7 @@ upcase(PARROT_INTERP, NOTNULL(STRING *src))
             (UChar *)src->strstart, src_len,
             NULL,       /* locale = default */
             &err);
-    assert(U_SUCCESS(err));
+    PARROT_ASSERT(U_SUCCESS(err));
     src->bufused = dest_len * sizeof (UChar);
     /* downgrade if possible */
     if (dest_len == (int)src->strlen)
@@ -351,7 +350,7 @@ u_strToLower(UChar *dest, int32_t destCapacity,
                 (UChar *)src->strstart, src_len,
                 NULL,       /* locale = default */
                 &err);
-        assert(U_SUCCESS(err));
+        PARROT_ASSERT(U_SUCCESS(err));
     }
     /* downgrade if possible */
     if (dest_len == (int)src->strlen)
@@ -399,7 +398,7 @@ u_strToTitle(UChar *dest, int32_t destCapacity,
                 (UChar *)src->strstart, src_len,
                 NULL, NULL,
                 &err);
-        assert(U_SUCCESS(err));
+        PARROT_ASSERT(U_SUCCESS(err));
     }
     /* downgrade if possible */
     if (dest_len == (int)src->strlen)
@@ -461,7 +460,6 @@ cs_rindex(PARROT_INTERP, NOTNULL(STRING *source_string),
         SHIM(STRING *search_string), UINTVAL offset)
 {
     UNIMPL;
-    return 0;
 }
 
 static UINTVAL
@@ -570,7 +568,7 @@ find_cclass(PARROT_INTERP, INTVAL flags,
     UINTVAL end = offset + count;
     UINTVAL codepoint;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string != 0);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -596,7 +594,7 @@ find_not_cclass(PARROT_INTERP, INTVAL flags,
     UINTVAL codepoint;
     int bit;
 
-    assert(source_string != 0);
+    PARROT_ASSERT(source_string);
     end = source_string->strlen < end ? source_string->strlen : end;
     for (; pos < end; ++pos) {
         codepoint = ENCODING_GET_CODEPOINT(interp, source_string, pos);
@@ -646,7 +644,8 @@ compute_hash(PARROT_INTERP, NOTNULL(const STRING *src), size_t seed)
     return hashval;
 }
 
-CHARSET *
+PARROT_CANNOT_RETURN_NULL
+const CHARSET *
 Parrot_charset_unicode_init(PARROT_INTERP)
 {
     CHARSET * const return_set = Parrot_new_charset(interp);
