@@ -1,7 +1,7 @@
 /* unicode.h
  *  Copyright (C) 2005-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: unicode.h 19796 2007-07-11 22:09:12Z petdance $
+ *     $Id: unicode.h 20003 2007-07-19 03:33:27Z petdance $
  *  Overview:
  *     This is the header for the unicode charset functions
  *  Data Structure and Algorithms:
@@ -20,7 +20,8 @@
 
 /* HEADERIZER BEGIN: src/charset/unicode.c */
 
-CHARSET * Parrot_charset_unicode_init( PARROT_INTERP )
+PARROT_CANNOT_RETURN_NULL
+const CHARSET * Parrot_charset_unicode_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/charset/unicode.c */
