@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2002-2007, The Perl Foundation.
 License:  Artistic/GPL, see README and LICENSES for details
-$Id: datatypes.c 18975 2007-06-13 16:48:54Z bernhard $
+$Id: datatypes.c 19881 2007-07-14 22:05:00Z petdance $
 
 =head1 NAME
 
@@ -19,7 +19,7 @@ F<include/parrot/datatypes.h>.
 
 #include "parrot/parrot.h"
 
-/* HEADER: include/parrot/datatypes.h */
+/* HEADERIZER HFILE: include/parrot/datatypes.h */
 
 /*
 
@@ -29,9 +29,9 @@ Return datatype C<enum> for C<STRING*> type_name.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL
-Parrot_get_datatype_enum(Interp *interp, const STRING *type_name /*NN*/)
-    /* PURE, WARN_UNUSED */
+Parrot_get_datatype_enum(PARROT_INTERP, NOTNULL(const STRING *type_name))
 {
     char * const type = string_to_cstring(interp, type_name);
     int i;
@@ -56,9 +56,10 @@ Return datatype name for C<type>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
-Parrot_get_datatype_name(Interp *interp, INTVAL type)
-    /* WARN_UNUSED */
+Parrot_get_datatype_name(PARROT_INTERP, INTVAL type)
 {
     const char * const s =
         (type < enum_first_type || type >= enum_last_type)
