@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: debug.c 19881 2007-07-14 22:05:00Z petdance $
+$Id: debug.c 20488 2007-08-05 11:26:43Z paultcochrane $
 
 =head1 NAME
 
@@ -15,7 +15,6 @@ debugger, and the C<debug> ops.
 
 */
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "parrot/parrot.h"
@@ -30,6 +29,8 @@ debugger, and the C<debug> ops.
 PARROT_API
 void
 IMCC_warning(PARROT_INTERP, NOTNULL(const char *fmt), ...);
+extern void imcc_init(PARROT_INTERP);
+
 
 
 /* HEADERIZER HFILE: include/parrot/debug.h */
@@ -43,23 +44,33 @@ static int GDB_B( PARROT_INTERP, NOTNULL(char *s) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static const char* GDB_P( PARROT_INTERP, NOTNULL(const char *s) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static char const * nextarg( NOTNULL(char const *command) )
         __attribute__nonnull__(1);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char * parse_command(
     NOTNULL(const char *command),
     NOTNULL(unsigned long *cmdP) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char * parse_int( NOTNULL(const char *str), NOTNULL(int *intP) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char* parse_key( PARROT_INTERP,
     NOTNULL(const char *str),
     NOTNULL(PMC **keyP) )
@@ -67,6 +78,8 @@ static const char* parse_key( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char * parse_string( PARROT_INTERP,
     NOTNULL(const char *str),
     NOTNULL(STRING **strP) )
@@ -74,9 +87,12 @@ static const char * parse_string( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CANNOT_RETURN_NULL
 static const char * skip_command( NOTNULL(const char *str) )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char * skip_ws( NOTNULL(const char *str) )
         __attribute__nonnull__(1);
 
@@ -93,6 +109,8 @@ debugger commands. This function is used for C<eval>.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static char const *
 nextarg(NOTNULL(char const *command))
 {
@@ -100,12 +118,12 @@ nextarg(NOTNULL(char const *command))
      * and it is either alphanumeric, a comma or a closing bracket,
      * continue looking for the next argument.
      */
-    while (*command && (isalnum((int) *command) || *command == ',' ||
+    while (*command && (isalnum((unsigned char) *command) || *command == ',' ||
         *command == ']'))
             command++;
 
     /* eat as much space as possible */
-    while (*command && isspace((int) *command))
+    while (*command && isspace((unsigned char) *command))
         command++;
 
     return command;
@@ -119,11 +137,13 @@ Returns the pointer past any whitespace.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char *
 skip_ws(NOTNULL(const char *str))
 {
     /* as long as str is not NULL and it contains space, skip it */
-    while (*str && isspace((int) *str))
+    while (*str && isspace((unsigned char) *str))
         str++;
 
     return str;
@@ -138,17 +158,18 @@ alternative to the C<skip_command()> macro above.)
 
 */
 
+PARROT_CANNOT_RETURN_NULL
 static const char *
 skip_command(NOTNULL(const char *str))
 {
     /* while str is not null and it contains a command (no spaces),
      * skip the character
      */
-    while (*str && !isspace((int) *str))
+    while (*str && !isspace((unsigned char) *str))
         str++;
 
     /* eat all space after that */
-    while (*str && isspace((int) *str))
+    while (*str && isspace((unsigned char) *str))
         str++;
 
     return str;
@@ -163,6 +184,8 @@ The output parameter C<intP> contains the parsed value.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char *
 parse_int(NOTNULL(const char *str), NOTNULL(int *intP))
 {
@@ -183,6 +206,8 @@ C<STRING> and placed in the output parameter C<strP>.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char *
 parse_string(PARROT_INTERP, NOTNULL(const char *str), NOTNULL(STRING **strP))
 {
@@ -225,6 +250,8 @@ after the key. Currently only string and integer keys are allowed.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char*
 parse_key(PARROT_INTERP, NOTNULL(const char *str), NOTNULL(PMC **keyP))
 {
@@ -245,7 +272,7 @@ parse_key(PARROT_INTERP, NOTNULL(const char *str), NOTNULL(PMC **keyP))
         *keyP = key_new_string(interp, parrot_string);
     }
     /* if this is a numeric key */
-    else if (isdigit((int) *str)) {
+    else if (isdigit((unsigned char) *str)) {
         int value;
         str   = parse_int(str, &value);
         *keyP = key_new_integer(interp, (INTVAL) value);
@@ -272,6 +299,8 @@ that can be used as a switch key for fast lookup.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char *
 parse_command(NOTNULL(const char *command), NOTNULL(unsigned long *cmdP))
 {
@@ -280,11 +309,11 @@ parse_command(NOTNULL(const char *command), NOTNULL(unsigned long *cmdP))
 
     if (*command == '\0') {
         *cmdP = c;
-        return 0;
+        return NULL;
     }
 
-    for (i = 0; *command && isalpha((int) *command); command++, i++)
-        c += (tolower((int) *command) + (i + 1)) * ((i + 1) * 255);
+    for (i = 0; *command && isalpha((unsigned char) *command); command++, i++)
+        c += (tolower((unsigned char) *command) + (i + 1)) * ((i + 1) * 255);
 
     /* Nonempty and did not start with a letter */
     if (c == 0)
@@ -319,7 +348,6 @@ PDB_get_command(PARROT_INTERP)
     int           ch;
     char         *c;
     PDB_t        * const pdb = interp->pdb;
-    PDB_line_t   *line;
 
     /* flush the buffered data */
     fflush(stdout);
@@ -336,7 +364,7 @@ PDB_get_command(PARROT_INTERP)
 
     /* if the program is stopped and running show the next line to run */
     if ((pdb->state & PDB_STOPPED) && (pdb->state & PDB_RUNNING)) {
-        line = pdb->file->line;
+        PDB_line_t *line = pdb->file->line;
 
         while (pdb->cur_opcode != line->opcode)
             line = line->next;
@@ -344,7 +372,7 @@ PDB_get_command(PARROT_INTERP)
         PIO_eprintf(interp, "%li  ", line->number);
         c = pdb->file->source + line->source_offset;
 
-        while (*c != '\n'  && c)
+        while (c && (*c != '\n'))
             PIO_eprintf(interp, "%c", *(c++));
     }
 
@@ -358,10 +386,10 @@ PDB_get_command(PARROT_INTERP)
     /* skip leading whitespace */
     do {
         ch = fgetc(stdin);
-    } while (isspace(ch) && ch != '\n');
+    } while (isspace((unsigned char)ch) && ch != '\n');
 
     /* generate string (no more than 255 chars) */
-     while (ch != EOF && ch != '\n' && (i < 255)) {
+    while (ch != EOF && ch != '\n' && (i < 255)) {
         c[i++] = ch;
         ch     = fgetc(stdin);
     }
@@ -406,7 +434,7 @@ PDB_script_file(PARROT_INTERP, NOTNULL(const char *command))
         fgets(buf, 1024, fd);
 
         /* skip spaces */
-        for(ptr=(char *)&buf;*ptr&&isspace(*ptr);ptr=ptr+1);
+        for(ptr=(char *)&buf;*ptr&&isspace((unsigned char)*ptr);ptr=ptr+1);
 
         /* avoid null blank and commented lines */
         if (*buf == '\0' || *buf == '#')
@@ -437,6 +465,7 @@ Hash the command to make a simple switch calling the correct handler.
 
 */
 
+PARROT_IGNORABLE_RESULT
 int
 PDB_run_command(PARROT_INTERP, NOTNULL(const char *command))
 {
@@ -451,6 +480,8 @@ PDB_run_command(PARROT_INTERP, NOTNULL(const char *command))
 
     if (command)
         skip_command(command);
+    else
+        return 0;
 
     switch (c) {
         case c_script_file:
@@ -554,7 +585,7 @@ PDB_next(PARROT_INTERP, NULLOK(const char *command))
 
     command = nextarg(command);
     /* Get the number of operations to execute if any */
-    if (command && isdigit((int) *command))
+    if (command && isdigit((unsigned char) *command))
         n = atol(command);
 
     /* Erase the stopped flag */
@@ -597,7 +628,7 @@ PDB_trace(PARROT_INTERP, NULLOK(const char *command))
 
     command = nextarg(command);
     /* if the number of ops to run is specified, convert to a long */
-    if (command && isdigit((int) *command))
+    if (command && isdigit((unsigned char) *command))
         n = atol(command);
 
     /* clear the PDB_STOPPED flag, we'll be running n ops now */
@@ -630,6 +661,7 @@ Analyzes a condition from the user input.
 
 */
 
+PARROT_CAN_RETURN_NULL
 PDB_condition_t *
 PDB_cond(PARROT_INTERP, NOTNULL(const char *command))
 {
@@ -737,7 +769,7 @@ INV_COND:   PIO_eprintf(interp, "Invalid condition\n");
         return NULL;
     }
 
-    if (isalpha((int)*command)) {
+    if (isalpha((unsigned char)*command)) {
         /* It's a register - we first check that it's the correct type */
         switch (*command) {
             case 'i':
@@ -845,19 +877,19 @@ Set a break point, the source code file must be loaded.
 */
 
 void
-PDB_set_break(PARROT_INTERP, const char *command /*NULL*/)
+PDB_set_break(PARROT_INTERP, NOTNULL(const char *command))
 {
     PDB_t            * const pdb      = interp->pdb;
     PDB_breakpoint_t *newbreak = NULL;
     PDB_breakpoint_t *sbreak;
     PDB_condition_t  *condition;
     PDB_line_t       *line;
-    long              ln, i;
+    long              i;
 
     command = nextarg(command);
     /* If no line number was specified, set it at the current line */
     if (command && *command) {
-        ln = atol(command);
+        const long ln = atol(command);
 
         /* Move to the line where we will set the break point */
         line = pdb->file->line;
@@ -947,8 +979,6 @@ Init the program.
 
 */
 
-extern void imcc_init(PARROT_INTERP);
-
 void
 PDB_init(PARROT_INTERP, SHIM(const char *command))
 {
@@ -1003,11 +1033,13 @@ exist or if no breakpoint was specified.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PDB_breakpoint_t *
 PDB_find_breakpoint(PARROT_INTERP, NOTNULL(const char *command))
 {
     command = nextarg(command);
-    if (isdigit((int) *command)) {
+    if (isdigit((unsigned char) *command)) {
         const long n = atol(command);
         PDB_breakpoint_t *breakpoint = interp->pdb->breakpoint;
 
@@ -1187,6 +1219,7 @@ Returns true if the condition was met.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 char
 PDB_check_condition(PARROT_INTERP, NOTNULL(PDB_condition_t *condition))
 {
@@ -1270,6 +1303,7 @@ Returns true if we have to stop running.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 char
 PDB_break(PARROT_INTERP)
 {
@@ -1332,6 +1366,8 @@ Escapes C<">, C<\r>, C<\n>, C<\t>, C<\a> and C<\\>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 char *
 PDB_escape(NOTNULL(const char *string), INTVAL length)
 {
@@ -1397,7 +1433,7 @@ Do inplace unescape of C<\r>, C<\n>, C<\t>, C<\a> and C<\\>.
 */
 
 int
-PDB_unescape(char *string)
+PDB_unescape(NOTNULL(char *string))
 {
     int l = 0;
 
@@ -1470,7 +1506,7 @@ PDB_disassemble_op(PARROT_INTERP, NOTNULL(char *dest), int space,
         FLOATVAL  f;
         PMC      *k;
 
-        assert(size + 2 < space);
+        PARROT_ASSERT(size + 2 < space);
 
         switch (info->types[j-1]) {
         case PARROT_ARG_I:
@@ -1512,7 +1548,7 @@ PDB_disassemble_op(PARROT_INTERP, NOTNULL(char *dest), int space,
             if (i == 0)
                 i = (INTVAL) op[j];
 
-            assert(size + 20 < space);
+            PARROT_ASSERT(size + 20 < space);
 
             sprintf(&dest[size], INTVAL_FMT, i);
             size += strlen(&dest[size]);
@@ -1522,7 +1558,7 @@ PDB_disassemble_op(PARROT_INTERP, NOTNULL(char *dest), int space,
             if (j == 1 && info->types[j-1] == PARROT_ARG_IC
                 && (strcmp(info->name, "infix") == 0
                     || strcmp(info->name, "n_infix") == 0)) {
-                assert(size + 20 < space);
+                PARROT_ASSERT(size + 20 < space);
 
                 sprintf(&dest[size], " [%s]",
                         /* [kludge: the "2+" skips the leading underscores.  --
@@ -1951,7 +1987,7 @@ that instruction and check that is the correct one.
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 char
-PDB_hasinstruction(const char *c)
+PDB_hasinstruction(NOTNULL(const char *c))
 {
     char h = 0;
 
@@ -1959,7 +1995,7 @@ PDB_hasinstruction(const char *c)
     while (*c && *c != '#' && *c != '\n') {
         /* ... and c is alphanumeric or a quoted string then the line contains
          * an instruction. */
-        if (isalnum((int) *c) || *c == '"') {
+        if (isalnum((unsigned char) *c) || *c == '"') {
             h = 1;
         }
         else if (*c == ':') {
@@ -1998,7 +2034,7 @@ PDB_list(PARROT_INTERP, NOTNULL(const char *command))
 
     command = nextarg(command);
     /* set the list line if provided */
-    if (isdigit((int) *command)) {
+    if (isdigit((unsigned char) *command)) {
         line_number = atol(command) - 1;
         if (line_number < 0)
             pdb->file->list_line = 0;
@@ -2012,7 +2048,7 @@ PDB_list(PARROT_INTERP, NOTNULL(const char *command))
     }
 
     /* set the number of lines to print */
-    if (isdigit((int) *command)) {
+    if (isdigit((unsigned char) *command)) {
         n = atol(command);
         skip_command(command);
     }
@@ -2083,6 +2119,7 @@ which generates a malloced string.
 
 */
 
+PARROT_CAN_RETURN_NULL
 opcode_t *
 PDB_compile(PARROT_INTERP, NOTNULL(const char *command))
 {
@@ -2165,7 +2202,7 @@ Print an entry from the user stack.
 */
 
 void
-PDB_print_user_stack(PARROT_INTERP, const char *command)
+PDB_print_user_stack(PARROT_INTERP, NOTNULL(const char *command))
 {
     Stack_Entry_t *entry;
     long           depth = 0;
@@ -2487,6 +2524,8 @@ PDB_backtrace(PARROT_INTERP)
  * TODO more, more
  */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static const char*
 GDB_P(PARROT_INTERP, NOTNULL(const char *s))
 {
@@ -2498,7 +2537,7 @@ GDB_P(PARROT_INTERP, NOTNULL(const char *s))
         case 'P': t = REGNO_PMC; break;
         default: return "no such reg";
     }
-    if (s[1] && isdigit(s[1]))
+    if (s[1] && isdigit((unsigned char)s[1]))
         n = atoi(s + 1);
     else
         return "no such reg";
