@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: embed.c 19833 2007-07-13 22:02:25Z petdance $
+$Id: embed.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -18,7 +18,35 @@ This file implements the Parrot embedding interface.
 #include "parrot/embed.h"
 #include "parrot/oplib/ops.h"
 
-/* HEADERIZER HFILE: include/parrot/embed.h */
+/* HEADERIZER HFILE: none */ /* The visible types are different than what we use in here */
+
+/* HEADERIZER BEGIN: static */
+
+static FLOATVAL calibrate( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+static const char * op_name( PARROT_INTERP, int k )
+        __attribute__nonnull__(1);
+
+static void print_debug( PARROT_INTERP, SHIM(int status), SHIM(void *p) )
+        __attribute__nonnull__(1);
+
+static void print_profile( PARROT_INTERP, SHIM(int status), SHIM(void *p) )
+        __attribute__nonnull__(1);
+
+static int prof_sort_f( NOTNULL(const void *a), NOTNULL(const void *b) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+static PMC* set_current_sub( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+static PMC* setup_argv( PARROT_INTERP, int argc, char ** argv )
+        __attribute__nonnull__(1);
+
+/* HEADERIZER END: static */
+
+extern int Parrot_exec_run;
 
 /*
 
@@ -577,7 +605,7 @@ print_profile(PARROT_INTERP, SHIM(int status), SHIM(void *p))
         UINTVAL op_count = 0;
         UINTVAL call_count = 0;
         FLOATVAL sum_time = 0.0;
-        FLOATVAL empty = calibrate(interp);
+        const FLOATVAL empty = calibrate(interp);
 
         PIO_printf(interp,
                    " Code J Name                         "
@@ -756,7 +784,6 @@ Parrot_runcode(PARROT_INTERP, int argc, char *argv[])
 
     /* s. runops_exec interpreter.c */
     if (Interp_core_TEST(interp, PARROT_EXEC_CORE)) {
-        extern int Parrot_exec_run;
         Parrot_exec_run = 1;
     }
 
@@ -833,7 +860,6 @@ PARROT_API
 void
 Parrot_disassemble(PARROT_INTERP)
 {
-    char       *c;
     PDB_t      *pdb             = mem_allocate_zeroed_typed(PDB_t);
     PDB_line_t *line;
     int         debugs;
@@ -860,6 +886,7 @@ Parrot_disassemble(PARROT_INTERP)
     }
 
     while (line->next) {
+        const char *c;
 
         /* PIO_printf(interp, "%i < %i %i == %i \n", curr_mapping,
          * num_mappings, op_code_seq_num,
@@ -867,7 +894,7 @@ Parrot_disassemble(PARROT_INTERP)
 
         if (debugs && curr_mapping < num_mappings) {
             if ( op_code_seq_num == interp->code->debugs->mappings[curr_mapping]->offset) {
-                int filename_const_offset = interp->code->debugs->mappings[curr_mapping]->u.filename;
+                const int filename_const_offset = interp->code->debugs->mappings[curr_mapping]->u.filename;
                 PIO_printf(interp, "Current Source Filename %Ss\n", interp->code->const_table->constants[filename_const_offset]->u.string);
                 curr_mapping++;
             }
@@ -886,7 +913,7 @@ Parrot_disassemble(PARROT_INTERP)
 
         c = pdb->file->source + line->source_offset;
 
-        while (*c != '\n' && c)
+        while (c && *c != '\n')
             PIO_printf(interp, "%c", *(c++));
 
         PIO_printf(interp, "\n");
