@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: exceptions.c 19940 2007-07-17 22:16:08Z particle $
+$Id: exceptions.c 20560 2007-08-08 15:03:29Z petdance $
 
 =head1 NAME
 
@@ -29,6 +29,10 @@ Define the internal interpreter exceptions.
 #include "parrot/parrot.h"
 #include "parrot/exceptions.h"
 
+#ifdef PARROT_HAS_GLIBC_BACKTRACE
+#  include <execinfo.h>
+#endif
+
 /* HEADERIZER HFILE: include/parrot/exceptions.h */
 
 /* HEADERIZER BEGIN: static */
@@ -44,8 +48,9 @@ static size_t dest2offset( PARROT_INTERP, NOTNULL(const opcode_t *dest) )
 
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-static PMC * find_exception_handler( PARROT_INTERP, PMC *exception )
-        __attribute__nonnull__(1);
+static PMC * find_exception_handler( PARROT_INTERP, NOTNULL(PMC *exception) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 static void run_cleanup_action( PARROT_INTERP, NOTNULL(Stack_Entry_t *e) )
         __attribute__nonnull__(1)
@@ -108,7 +113,7 @@ Panic handler.
 PARROT_DOES_NOT_RETURN
 void
 do_panic(NULLOK_INTERP, NULLOK(const char *message),
-         NULLOK(const char *file), int line)
+         NULLOK(const char *file), unsigned int line)
 {
     /* Note: we can't format any floats in here--Parrot_sprintf
     ** may panic because of floats.
@@ -118,7 +123,7 @@ do_panic(NULLOK_INTERP, NULLOK(const char *message),
     fprintf(stderr, "Parrot VM: PANIC: %s!\n",
                message ? message : "(no message available)");
 
-    fprintf(stderr, "C file %s, line %d\n",
+    fprintf(stderr, "C file %s, line %ud\n",
                file ? file : "(not available)", line);
 
     fprintf(stderr, "Parrot file (not available), ");
@@ -237,7 +242,7 @@ Find the exception handler for C<exception>.
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
 static PMC *
-find_exception_handler(PARROT_INTERP, PMC *exception)
+find_exception_handler(PARROT_INTERP, NOTNULL(PMC *exception))
 {
     char *m;
     int exit_status, print_location;
@@ -279,15 +284,19 @@ find_exception_handler(PARROT_INTERP, PMC *exception)
         string_cstring_free(m);
     }
     else {
-        const INTVAL severity =
-            VTABLE_get_integer_keyed_int(interp, exception, 2);
-        if (severity == EXCEPT_exit) {
-            print_location = 0;
-            exit_status =
-                (int)VTABLE_get_integer_keyed_int(interp, exception, 1);
+        if (m)
+            string_cstring_free(m); /* coverity fix, m was allocated but was "\0" */
+        /* new block for const assignment */
+        {
+            const INTVAL severity = VTABLE_get_integer_keyed_int(interp, exception, 2);
+            if (severity == EXCEPT_exit) {
+                print_location = 0;
+                exit_status =
+                    (int)VTABLE_get_integer_keyed_int(interp, exception, 1);
+            }
+            else
+                fprintf(stderr, "No exception handler and no message\n");
         }
-        else
-            fprintf(stderr, "No exception handler and no message\n");
     }
     /* caution against output swap (with PDB_backtrace) */
     fflush(stderr);
@@ -310,8 +319,6 @@ find_exception_handler(PARROT_INTERP, PMC *exception)
      * is freed during Parrot_exit
      */
     Parrot_exit(interp, exit_status);
-
-    return NULL;
 }
 
 /*
@@ -455,8 +462,8 @@ rethrow_c_exception(PARROT_INTERP)
 {
     Parrot_exception * const the_exception = interp->exceptions;
 
-    PMC * const exception = NULL;   /* TODO */
-    PMC * const handler = find_exception_handler(interp, exception);
+    PMC * const exception = PMCNULL;   /* TODO */
+    PMC * const handler   = find_exception_handler(interp, exception);
 
     /* XXX we should only peek for the next handler */
     push_exception(interp, handler);
@@ -748,6 +755,83 @@ Parrot_init_exceptions(PARROT_INTERP) {
     }
 }
 
+
+
+/*
+
+FUNCDOC: Parrot_confess
+
+A better version of assert() that gives a backtrace if possible.
+
+*/
+
+PARROT_API
+PARROT_DOES_NOT_RETURN
+void
+Parrot_confess(NOTNULL(const char *cond), NOTNULL(const char *file), unsigned int line)
+{
+    fprintf(stderr, "%s:%u: failed assertion '%s'\n", file, line, cond);
+    Parrot_print_backtrace();
+    abort();
+}
+
+void
+Parrot_print_backtrace(void)
+{
+#ifdef PARROT_HAS_GLIBC_BACKTRACE
+#  define BACKTRACE_DEPTH 32
+/*#  define BACKTRACE_VERBOSE */
+    /* stolen from http://www.delorie.com/gnu/docs/glibc/libc_665.html */
+    void *array[BACKTRACE_DEPTH];
+    size_t i;
+#  ifndef BACKTRACE_VERBOSE
+    int ident;
+    char *caller;
+    size_t callerLength;
+    size_t j;
+#  endif
+
+    const size_t size = backtrace (array, BACKTRACE_DEPTH);
+    char ** const strings = backtrace_symbols (array, size);
+
+    fprintf (stderr, "Backtrace - Obtained %zd stack frames (max trace depth is %d).\n", size, BACKTRACE_DEPTH);
+#  ifndef BACKTRACE_VERBOSE
+    for (i = 0; i < size; i++) {
+        /* always ident */
+        ident = 2;  /* initial indent */
+        ident += 2 * i; /* nesting depth */
+        fprintf(stderr, "%*s", ident, "");
+
+        /* if the caller was an anon function then strchr won't
+        find a '(' in the string and will return NULL */
+        caller = strchr(strings[i], '(');
+        if (caller) {
+            /* skip over the '(' */
+            caller++;
+            /* find the end of the symbol name */
+            callerLength = abs(strchr(caller, '+') - caller);
+            /* print just the symbol name */
+            for (j = 0; j < callerLength; j++) {
+                fputc(caller[j], stderr);
+            }
+            fprintf(stderr, "\n");
+        }
+        else {
+            fprintf(stderr, "(unknown)\n");
+        }
+    }
+#  else
+    for (i = 0; i < size; i++)
+        fprintf (stderr, "%s\n", strings[i]);
+#  endif
+
+    free (strings);
+
+#  undef BACKTRACE_DEPTH
+#endif /* ifdef PARROT_HAS_GLIBC_BACKTRACE */
+}
+
+
 /*
 
 =head1 SEE ALSO
