@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: dod.c 19886 2007-07-15 06:19:53Z petdance $
+$Id: dod.c 20687 2007-08-18 20:16:33Z chromatic $
 
 =head1 NAME
 
@@ -9,7 +9,7 @@ src/dod.c - Dead object destruction of the various headers
 =head1 DESCRIPTION
 
 This file implements I<dead object destruction>. This is documented in
-PDD 9 with supplementary notes in F<docs/dev/dod.dev>.
+PDD 9 with supplementary notes in F<docs/dev/dod.pod>.
 
 It's possible to turn on/off the checking of the system stack and
 processor registers. The actual checking is implemented in F<src/cpu_dep.c>.
@@ -23,7 +23,6 @@ There's also a verbose mode for garbage collection.
 #define DOD_C_SOURCE
 #include "parrot/parrot.h"
 #include "parrot/dod.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/dod.h */
 
@@ -95,11 +94,12 @@ mark_special(PARROT_INTERP, NOTNULL(PMC *obj))
      */
     if (PObj_is_PMC_shared_TEST(obj)) {
         interp = PMC_sync(obj)->owner;
-        assert(interp);
+        PARROT_ASSERT(interp);
         /* XXX FIXME hack */
         if (!interp->arena_base->dod_mark_ptr)
             interp->arena_base->dod_mark_ptr = obj;
     }
+
     arena_base = interp->arena_base;
 
     if (PObj_needs_early_DOD_TEST(obj))
@@ -165,9 +165,9 @@ pobject_lives(PARROT_INTERP, NOTNULL(PObj *obj))
     } while (0);
 #else /* not PARROT_GC_GMS */
     /* if object is live or on free list return */
-    if (PObj_is_live_or_free_TESTALL(obj)) {
+    if (PObj_is_live_or_free_TESTALL(obj))
         return;
-    }
+
 #  if ! DISABLE_GC_DEBUG
 #    if GC_VERBOSE
     if (CONSERVATIVE_POINTER_CHASING) {
@@ -241,9 +241,7 @@ Parrot_dod_trace_root(PARROT_INTERP, int trace_stack)
     Arenas           * const arena_base = interp->arena_base;
     parrot_context_t *ctx;
 
-    /*
-     * note: adding locals here did cause increased DOD runs
-     */
+    /* note: adding locals here did cause increased DOD runs */
     unsigned int i = 0;
 
     if (trace_stack == 2) {
@@ -373,9 +371,9 @@ int
 Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
 {
     Arenas * const arena_base = interp->arena_base;
-    const int     lazy_dod    = arena_base->lazy_dod;
-    PMC          *current     = arena_base->dod_mark_start;
-    PMC          *next;
+    const int      lazy_dod   = arena_base->lazy_dod;
+    PMC           *current    = arena_base->dod_mark_start;
+    PMC           *next       = PMCNULL;
 
     const UINTVAL mask = PObj_data_is_PMC_array_FLAG | PObj_custom_mark_FLAG;
 
@@ -403,14 +401,10 @@ Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
 
         arena_base->dod_trace_ptr = current;
 
-        /*
-         * short-term hack to color objects black
-         */
+        /* short-term hack to color objects black */
         PObj_get_FLAGS(current) |= PObj_custom_GC_FLAG;
 
-        /*
-         * clearing the flag is much more expensive then testing
-         */
+        /* clearing the flag is much more expensive then testing */
         if (!PObj_needs_early_DOD_TEST(current))
             PObj_high_priority_DOD_CLEAR(current);
 
@@ -436,15 +430,14 @@ Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
             }
             else {
                 /* All that's left is the custom */
-                assert(!PObj_on_free_list_TEST(current));
+                PARROT_ASSERT(!PObj_on_free_list_TEST(current));
                 VTABLE_mark(interp, current);
             }
         }
 
-        /* this value may be null */
         next = PMC_next_for_GC(current);
 
-        if (next && next == current)
+        if (!PMC_IS_NULL(next) && next == current)
             break;
 
         if (--how_many == 0) {
@@ -563,13 +556,12 @@ void
 Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
     Arenas * const arena_base = interp->arena_base;
-    UINTVAL i, total_used = 0;
+
+    UINTVAL i;
+    UINTVAL total_used    = 0;
     UINTVAL object_size   = pool->object_size;
 
     Small_Object_Arena *cur_arena;
-#if REDUCE_ARENAS
-    UINTVAL free_arenas = 0, old_total_used = 0;
-#endif
 
 #if GC_VERBOSE
     if (Interp_trace_TEST(interp, 1)) {
@@ -590,7 +582,7 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
             NULL != cur_arena; cur_arena = cur_arena->prev) {
         Buffer *b = (Buffer *)cur_arena->start_objects;
 
-        for (i = 0; i < cur_arena->used; i++) {
+        for (i = 0; i < cur_arena->total_objects; i++) {
             if (PObj_on_free_list_TEST(b))
                 ; /* if it's on free list, do nothing */
             else if (PObj_live_TEST(b)) {
@@ -627,7 +619,7 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 
                 /* if object is a PMC and needs destroying */
                 if (PObj_is_PMC_TEST(b)) {
-                    PMC * const p = (PMC*)b;
+                    PMC * const p = (PMC *)b;
 
                     /* then destroy it here
                      *
@@ -641,9 +633,9 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 
                     if (PObj_is_PMC_EXT_TEST(p) && p->pmc_ext != NULL) {
                         /* if the PMC has a PMC_EXT structure,
-                         * return it to the pool/arena
-                         */
-                        Small_Object_Pool * const ext_pool = arena_base->pmc_ext_pool;
+                         * return it to the pool/arena */
+                        Small_Object_Pool * const ext_pool
+                            = arena_base->pmc_ext_pool;
 
                         if (PObj_is_PMC_shared_TEST(p) && PMC_sync(p)) {
                             MUTEX_DESTROY(PMC_sync(p)->pmc_lock);
@@ -651,7 +643,7 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
                             PMC_sync(p) = NULL;
                         }
 
-                        ext_pool->add_free_object(interp, ext_pool, p->pmc_ext);
+                        ext_pool->add_free_object(interp, ext_pool, (PObj *)p->pmc_ext);
                     }
 #ifndef NDEBUG
                     /*
@@ -679,8 +671,10 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
                         if (PObj_COW_TEST(b)) {
                             INTVAL *refcount = ((INTVAL *)PObj_bufstart(b) - 1);
 
-                            if (!--(*refcount))
+                            if (!--(*refcount)) {
                                 free(refcount); /* the actual bufstart */
+                                refcount = NULL;
+                            }
                         }
                         else
                             free((INTVAL*)PObj_bufstart(b) - 1);
@@ -742,14 +736,12 @@ find_common_mask(PARROT_INTERP, size_t val1, size_t val2)
     }
 
     if (val1 == val2) {
-        assert(i == bound);
+        PARROT_ASSERT(i == bound);
         return 0;
     }
 
     real_exception(interp, NULL, INTERP_ERROR,
             "Unexpected condition in find_common_mask()!\n");
-
-    return 0;
 }
 
 /*
@@ -814,6 +806,7 @@ trace_mem_block(PARROT_INTERP, size_t lo_var_ptr, size_t hi_var_ptr)
             }
         }
     }
+
     return;
 }
 #endif
@@ -921,7 +914,8 @@ Parrot_dod_ms_run_init(PARROT_INTERP)
 }
 
 static int
-sweep_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag, NOTNULL(void *arg))
+sweep_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag,
+    NOTNULL(void *arg))
 {
     int * const total_free = (int *) arg;
 
@@ -1000,6 +994,8 @@ Parrot_dod_ms_run(PARROT_INTERP, int flags)
 
     /* Now go trace the PMCs */
     if (trace_active_PMCs(interp, flags & DOD_trace_stack_FLAG)) {
+        int ignored;
+
         arena_base->dod_trace_ptr = NULL;
         arena_base->dod_mark_ptr  = NULL;
 
@@ -1007,7 +1003,7 @@ Parrot_dod_ms_run(PARROT_INTERP, int flags)
         pt_DOD_stop_mark(interp);
 
         /* Now put unused PMCs and Buffers on the free list */
-        Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_PMC,
+        ignored = Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_PMC,
             (void*)&total_free, sweep_cb);
 
         if (interp->profile)
@@ -1019,12 +1015,7 @@ Parrot_dod_ms_run(PARROT_INTERP, int flags)
         /* successful lazy DOD count */
         ++arena_base->lazy_dod_runs;
 
-        /* it was an aborted lazy dod run - we should clear the live bits, but
-         * e.g. t/pmc/timer_7 succeeds w/o this */
-
-#if 1
         Parrot_dod_clear_live_bits(interp);
-#endif
         if (interp->profile)
             Parrot_dod_profile_end(interp, PARROT_PROF_DOD_p2);
     }
