@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: gc_gms.c 19905 2007-07-16 04:44:05Z petdance $
+$Id: gc_gms.c 20331 2007-07-30 01:12:29Z chromatic $
 
 =head1 NAME
 
@@ -110,7 +110,6 @@ A chained list of headers used e.g. for the IGP list.
 
 #include "parrot/parrot.h"
 #include "parrot/dod.h"
-#include <assert.h>
 
 #if PARROT_GC_GMS
 
@@ -132,7 +131,7 @@ static int end_cycle_cb( PARROT_INTERP,
 
 static void gc_gms_add_free_object( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool),
-    NOTNULL(void *to_add) )
+    NOTNULL(PObj *to_add) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
@@ -160,6 +159,7 @@ static void gc_gms_clear_igp( PARROT_INTERP, NOTNULL(Gc_gms_gen *gen) )
         __attribute__nonnull__(2);
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static Gc_gms_gen * gc_gms_create_gen( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool),
     size_t gen_no )
@@ -170,13 +170,16 @@ static void gc_gms_end_cycle( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static Gc_gms_gen * gc_gms_find_gen( PARROT_INTERP,
     NOTNULL(Gc_gms_hdr *h),
     UINTVAL gen_no )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static void * gc_gms_get_free_object( PARROT_INTERP,
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+static PObj * gc_gms_get_free_object( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -276,7 +279,7 @@ static void parrot_gc_gms_deinit( PARROT_INTERP )
 static void parrot_gc_gms_run( PARROT_INTERP, int flags )
         __attribute__nonnull__(1);
 
-PARROT_WARN_UNUSED
+PARROT_WARN_UNUSED_RESULT
 static int set_gen_cb( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool),
     int flag,
@@ -464,7 +467,7 @@ Run a GC cycle or allocate new objects for the given pool.
 
 static void
 gc_gms_add_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
-        NOTNULL(void *to_add))
+        NOTNULL(PObj *to_add))
 {
     real_exception(interp, NULL, 1, "gms abuse");
 }
@@ -517,7 +520,7 @@ gc_gms_chain_objects(PARROT_INTERP,
 
     p = new_arena->start_objects;
     marker = &pool->marker;
-    assert(pool->free_list == marker);
+    PARROT_ASSERT(pool->free_list == marker);
 
     /* update pool statistics */
     n = new_arena->total_objects;
@@ -548,11 +551,11 @@ gc_gms_chain_objects(PARROT_INTERP,
 #  endif
         next = p;
     }
-    assert(p == new_arena->start_objects);
+    PARROT_ASSERT(p == new_arena->start_objects);
     p->prev = prev;
     prev->next = p;
     pool->free_list = p;
-    assert(p != marker);
+    PARROT_ASSERT(p != marker);
 }
 
 static void
@@ -598,7 +601,9 @@ gc_gms_more_objects(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
  *    the free ptr moves towards the marker
  */
 
-static void *
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+static PObj *
 gc_gms_get_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
     PObj *ptr;
@@ -650,6 +655,7 @@ Initalize the generation system by creating the first two generations.
  */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static Gc_gms_gen *
 gc_gms_create_gen(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), size_t gen_no)
 {
@@ -659,9 +665,12 @@ gc_gms_create_gen(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), size_t gen_no
     gen->pool = pool;
     gen->timely_destruct_obj_sofar = 0;
     gen->black_color = b_PObj_live_FLAG;
-    gen->prev = gen->next = NULL;
+    gen->prev = NULL;
+    gen->next = NULL;
     gen->first = gen->last = gen->fin = &pool->marker;
-    gen->igp.first = gen->igp.last = NULL;
+    gen->igp.first = NULL;
+    gen->igp.last = NULL;
+
     return gen;
 }
 
@@ -696,12 +705,15 @@ to other items, and promote it to the old generation.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static Gc_gms_gen *
 gc_gms_find_gen(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), UINTVAL gen_no)
 {
     Gc_gms_gen *gen;
     Small_Object_Pool * const pool = h->gen->pool;
 
+    PARROT_ASSERT(pool);
+
     for (gen = pool->first_gen; gen; gen = gen->next) {
         if (gen_no == gen->gen_no)
             return gen;
@@ -717,7 +729,6 @@ gc_gms_find_gen(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), UINTVAL gen_no)
      */
     real_exception(interp, NULL, 1, "generation %d not found for hdr %p",
             gen_no, h);
-    return NULL;
 }
 
 static void
@@ -738,8 +749,8 @@ gc_gms_promote(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), UINTVAL gen_no)
 
     /* locate generation pointer */
     gen = gc_gms_find_gen(interp, h, gen_no);
-    assert(gen->last);
-    assert(gen->first);
+    PARROT_ASSERT(gen->last);
+    PARROT_ASSERT(gen->first);
 
     /* TODO if it needs destroy put it in front */
     next = gen->last;
@@ -769,7 +780,7 @@ gc_gms_store_hdr_list(PARROT_INTERP, NOTNULL(Gc_gms_hdr_list *l), NOTNULL(Gc_gms
         s->next = NULL;
         /* chain new store to old one */
         if (l->first) {
-            assert(l->last);
+            PARROT_ASSERT(l->last);
             l->last->next = s;
         }
         else {
@@ -810,7 +821,7 @@ gc_gms_clear_igp(PARROT_INTERP, NOTNULL(Gc_gms_gen *gen))
 }
 
 void
-parrot_gc_gms_wb(PARROT_INTERP, PMC *agg, void *old, NOTNULL(void *new))
+parrot_gc_gms_wb(PARROT_INTERP, NOTNULL(PMC *agg), NOTNULL(void *old), NOTNULL(void *new))
 {
     Gc_gms_hdr * const nh = PObj_to_GMSH(new);
     Gc_gms_hdr * const ah = PObj_to_GMSH(agg);
@@ -832,8 +843,8 @@ parrot_gc_gms_wb(PARROT_INTERP, PMC *agg, void *old, NOTNULL(void *new))
 }
 
 void
-parrot_gc_gms_wb_key(PARROT_INTERP, PMC *agg,
-        void *old, void *old_key, void *new, void *new_key)
+parrot_gc_gms_wb_key(PARROT_INTERP, NOTNULL(PMC *agg),
+        NOTNULL(void *old), NOTNULL(void *old_key), NOTNULL(void *new), NOTNULL(void *new_key))
 {
     Gc_gms_hdr *nh, *ah;
 
@@ -906,7 +917,7 @@ gc_gms_use_gen(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
 
 }
 
-PARROT_WARN_UNUSED
+PARROT_WARN_UNUSED_RESULT
 static int
 set_gen_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag, NOTNULL(void *arg))
 {
@@ -1052,7 +1063,7 @@ gc_gms_setto_gray(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), int priority)
             }
         }
     }
-    assert(h != pool->white);
+    PARROT_ASSERT(h != pool->white);
     /* verify all these pointer moves */
 #  if GC_GMS_DEBUG
     gms_debug_verify(interp, pool, "to_gray");
@@ -1086,8 +1097,8 @@ gc_gms_setto_black(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), int priority)
      * if the white is adjacent to black, move pointer
      */
     if (pool->black == h) {
-        assert(pool->gray == h);
-        assert(pool->white == h);
+        PARROT_ASSERT(pool->gray == h);
+        PARROT_ASSERT(pool->white == h);
         pool->white = h->next;
         pool->gray = h->next;
     }
@@ -1115,8 +1126,8 @@ gc_gms_setto_black(PARROT_INTERP, NOTNULL(Gc_gms_hdr *h), int priority)
             pool->black = h;
         }
     }
-    assert(h != pool->white);
-    assert(h != pool->gray);
+    PARROT_ASSERT(h != pool->white);
+    PARROT_ASSERT(h != pool->gray);
 #  if GC_GMS_DEBUG
     gms_debug_verify(interp, pool, "to_black");
 #  endif
