@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: gc_ims.c 19805 2007-07-12 04:24:21Z petdance $
+$Id: gc_ims.c 20687 2007-08-18 20:16:33Z chromatic $
 
 =head1 NAME
 
@@ -331,7 +331,6 @@ a sleep opcode.
 
 #include "parrot/parrot.h"
 #include "parrot/dod.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/dod.h */
 
@@ -347,7 +346,7 @@ static int collect_cb( PARROT_INTERP,
 
 static void gc_ims_add_free_object( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool),
-    NOTNULL(void *to_add) )
+    NOTNULL(PObj *to_add) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
@@ -357,7 +356,9 @@ static void gc_ims_alloc_objects( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static void * gc_ims_get_free_object( PARROT_INTERP,
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+static PObj * gc_ims_get_free_object( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -480,7 +481,7 @@ Allocate new objects for the given pool.
 */
 
 static void
-gc_ims_add_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), NOTNULL(void *to_add))
+gc_ims_add_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), NOTNULL(PObj *to_add))
 {
     *(void **)to_add = pool->free_list;
     pool->free_list  = to_add;
@@ -497,7 +498,9 @@ gc_ims_add_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), NOTNULL(
 }
 
 
-static void *
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+static PObj *
 gc_ims_get_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
     PObj *ptr;
@@ -528,7 +531,6 @@ gc_ims_alloc_objects(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
     Small_Object_Arena *new_arena;
     size_t size;
-    UINTVAL start, end;
 
     pool->objects_per_alloc  = ALLOCATION_BLOCK_SIZE / pool->object_size;
 
@@ -539,9 +541,7 @@ gc_ims_alloc_objects(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 
     Parrot_append_arena_in_pool(interp, pool, new_arena, size);
 
-    start = 0;
-    end = pool->objects_per_alloc;
-    Parrot_add_to_free_list(interp, pool, new_arena, start, end);
+    Parrot_add_to_free_list(interp, pool, new_arena);
 }
 
 static void
@@ -550,7 +550,7 @@ gc_ims_pool_init(SHIM_INTERP, NOTNULL(Small_Object_Pool *pool))
     pool->add_free_object = gc_ims_add_free_object;
     pool->get_free_object = gc_ims_get_free_object;
     pool->alloc_objects   = gc_ims_alloc_objects;
-    pool->more_objects = pool->alloc_objects;
+    pool->more_objects    = pool->alloc_objects;
 }
 
 static void
@@ -643,13 +643,12 @@ parrot_gc_ims_mark(PARROT_INTERP)
     /*
      * use statistics from the previous run
      */
-    if (g_ims->n_objects) {
+    if (g_ims->n_objects)
         work_factor = (double)g_ims->n_extended_PMCs / g_ims->n_objects;
-    }
     else
         work_factor = 1.0;
     todo = (size_t)(g_ims->alloc_trigger * g_ims->throttle * work_factor);
-    assert(arena_base->lazy_dod == 0);
+    PARROT_ASSERT(arena_base->lazy_dod == 0);
     Parrot_dod_trace_children(interp, todo);
     /*
      * check if we are finished with marking - the end is
@@ -686,33 +685,34 @@ sweep_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag, NOTNULL(void
 static void
 parrot_gc_ims_sweep(PARROT_INTERP)
 {
-    Arenas * const arena_base = interp->arena_base;
-    Gc_ims_private *g_ims;
-    size_t n_objects;
+    Arenas * const  arena_base = interp->arena_base;
+    Gc_ims_private *g_ims      = (Gc_ims_private *)arena_base->gc_private;
+    size_t          n_objects;
+    int             ignored;
 
     IMS_DEBUG((stderr, "\nSWEEP\n"));
-    g_ims = (Gc_ims_private *)arena_base->gc_private;
     /*
      * as we are now gonna kill objects, make sure that we
      * have traced the current stack
      * except for a lazy run, which is invoked from the run loop
      */
+
     /* TODO mark volatile roots */
     Parrot_dod_trace_root(interp, g_ims->lazy ? 0 : DOD_trace_stack_FLAG);
-    /*
-     * mark (again) rest of children
-     */
+
+    /* mark (again) rest of children */
     Parrot_dod_trace_children(interp, (size_t) -1);
-    /*
-     * now sweep all
-     */
+
+    /* now sweep all */
     n_objects = 0;
-    Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_PMC,
+    ignored   = Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_PMC,
             (void*)&n_objects, sweep_cb);
+
     if (interp->profile)
         Parrot_dod_profile_end(interp, PARROT_PROF_DOD_cb);
-    g_ims->state = GC_IMS_COLLECT;
-    g_ims->n_objects = n_objects;
+
+    g_ims->state           = GC_IMS_COLLECT;
+    g_ims->n_objects       = n_objects;
     g_ims->n_extended_PMCs = arena_base->num_extended_PMCs;
 }
 
@@ -725,7 +725,7 @@ memory.
 
 */
 
-#if !GC_IS_MALLOC
+#if !defined(GC_IS_MALLOC) || !GC_IS_MALLOC
 static int
 collect_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag, NOTNULL(void *arg))
 {
@@ -766,7 +766,7 @@ collect_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag, NOTNULL(vo
 static int
 parrot_gc_ims_collect(PARROT_INTERP, int check_only)
 {
-#if GC_IS_MALLOC
+#if defined(GC_IS_MALLOC) && GC_IS_MALLOC
     UNUSED(interp);
     UNUSED(check_only);
 #else
@@ -805,9 +805,9 @@ parrot_gc_ims_run_increment(PARROT_INTERP)
     Arenas * const arena_base    = interp->arena_base;
     Gc_ims_private * const g_ims = (Gc_ims_private *)arena_base->gc_private;
 
-    if (arena_base->DOD_block_level || g_ims->state == GC_IMS_DEAD) {
+    if (arena_base->DOD_block_level || g_ims->state == GC_IMS_DEAD)
         return;
-    }
+
     ++g_ims->increments;
     IMS_DEBUG((stderr, "state = %d => ", g_ims->state));
 
@@ -819,6 +819,7 @@ parrot_gc_ims_run_increment(PARROT_INTERP)
             break;
         case GC_IMS_STARTING:
             /*  fall through and start */
+            /* FALLTHRU */
         case GC_IMS_RE_INIT:
             parrot_gc_ims_reinit(interp);
             break;
