@@ -1,6 +1,6 @@
 /*
-Copyright (C) 2001-2006, The Perl Foundation.
-$Id: register.c 19808 2007-07-12 05:07:56Z petdance $
+Copyright (C) 2001-2007, The Perl Foundation.
+$Id: register.c 20686 2007-08-18 20:15:31Z chromatic $
 
 =head1 NAME
 
@@ -126,6 +126,10 @@ destroy_context(PARROT_INTERP)
     context = CONTEXT(interp->ctx);
     while (context) {
         Parrot_Context * const prev = context->caller_ctx;
+        if (context->n_regs_used) {
+            mem_sys_free(context->n_regs_used);
+            context->n_regs_used = NULL;
+        }
         mem_sys_free(context);
         context = prev;
     }
@@ -153,7 +157,8 @@ Create initial interpreter context.
 void
 create_initial_context(PARROT_INTERP)
 {
-    static INTVAL num_regs[] ={32,32,32,32};
+    static INTVAL   num_regs[] = {32,32,32,32};
+    Parrot_Context *ignored;
 
     /* Create some initial free_list slots. */
 
@@ -162,11 +167,9 @@ create_initial_context(PARROT_INTERP)
     interp->ctx_mem.free_list    =
         (void **)mem_sys_allocate_zeroed(INITIAL_FREE_SLOTS * sizeof (void *));
 
-    /*
-     * For now create context with 32 regs each. Some src tests (and maybe other
-     * extenders) are assuming the presence of these registers
-     */
-    Parrot_alloc_context(interp, num_regs);
+    /* For now create context with 32 regs each. Some src tests (and maybe
+     * other extenders) assume the presence of these registers */
+    ignored = Parrot_alloc_context(interp, num_regs);
 }
 
 /*
@@ -187,7 +190,7 @@ parrot_gc_context(PARROT_INTERP)
     if (!interp->ctx_mem.threshold)
         return;
     LVALUE_CAST(char *, ctx.bp) = interp->ctx_mem.threshold -
-        sizeof (struct parrot_regs_t);
+        sizeof (parrot_regs_t);
     /* TODO */
 #else
     UNUSED(interp);
@@ -273,8 +276,9 @@ Duplicate the passed context
 */
 
 PARROT_WARN_UNUSED_RESULT
-struct Parrot_Context *
-Parrot_dup_context(PARROT_INTERP, NOTNULL(const struct Parrot_Context *old) )
+PARROT_CANNOT_RETURN_NULL
+Parrot_Context *
+Parrot_dup_context(PARROT_INTERP, NOTNULL(const Parrot_Context *old) )
 {
     size_t          diff;
     Parrot_Context *ctx;
@@ -284,17 +288,21 @@ Parrot_dup_context(PARROT_INTERP, NOTNULL(const struct Parrot_Context *old) )
     const int slot = CALCULATE_SLOT_NUM(reg_alloc);
     void * ptr = interp->ctx_mem.free_list[slot];
 
-    if (ptr) {
+    if (ptr)
         interp->ctx_mem.free_list[slot] = *(void **) ptr;
-    }
-    else {
+    else
         ptr = (void *)mem_sys_allocate(reg_alloc + ALIGNED_CTX_SIZE);
-    }
-    CONTEXT(interp->ctx) = ctx = (Parrot_Context *)ptr;
 
-    ctx->regs_mem_size   = reg_alloc;
-    ctx->n_regs_used     = old->n_regs_used;
-    diff                 = (long *)ctx - (long *)const_cast(old);
+    ctx = (Parrot_Context *)ptr;
+    CONTEXT(interp->ctx) = ctx;
+
+    ctx->regs_mem_size          = reg_alloc;
+    ctx->n_regs_used            = (INTVAL *)mem_sys_allocate(sizeof (INTVAL) * 4);
+    ctx->n_regs_used[REGNO_INT] = old->n_regs_used[REGNO_INT];
+    ctx->n_regs_used[REGNO_NUM] = old->n_regs_used[REGNO_NUM];
+    ctx->n_regs_used[REGNO_STR] = old->n_regs_used[REGNO_STR];
+    ctx->n_regs_used[REGNO_PMC] = old->n_regs_used[REGNO_PMC];
+    diff                        = (long *)ctx - (long *)const_cast(old);
 
     interp->ctx.bp.regs_i    += diff;
     interp->ctx.bp_ps.regs_s += diff;
@@ -312,7 +320,9 @@ C<Parrot_pop_context>.
 */
 
 PARROT_API
-struct Parrot_Context *
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+Parrot_Context *
 Parrot_push_context(PARROT_INTERP, NOTNULL(INTVAL *n_regs_used))
 {
     Parrot_Context * const old = CONTEXT(interp->ctx);
@@ -356,14 +366,15 @@ Parrot_pop_context(PARROT_INTERP)
 
 FUNCDOC: Parrot_alloc_context
 
-Allocate a new context and set the context pointer. Please note that
-the register usage C<n_regs_used> is not copied; just the pointer is
-stored.  The function returns the new context.
+Allocate a new context and set the context pointer. Please note that the
+register usage C<n_regs_used> is copied.  The function returns the new context.
 
 */
 
-struct Parrot_Context *
-Parrot_alloc_context(PARROT_INTERP, NOTNULL(INTVAL *n_regs_used))
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+Parrot_Context *
+Parrot_alloc_context(PARROT_INTERP, NOTNULL(INTVAL *number_regs_used))
 {
     Parrot_Context *old, *ctx;
     void *ptr, *p;
@@ -372,16 +383,23 @@ Parrot_alloc_context(PARROT_INTERP, NOTNULL(INTVAL *n_regs_used))
      * TODO (OPT) if we allocate a new context due to a self-recursive call
      *      create a specialized version that just uses caller's size
      */
-    const size_t size_i = sizeof (INTVAL)   * n_regs_used[REGNO_INT];
-    const size_t size_n = sizeof (FLOATVAL) * n_regs_used[REGNO_NUM];
-    const size_t size_s = sizeof (STRING*)  * n_regs_used[REGNO_STR];
-    const size_t size_p = sizeof (PMC*)     * n_regs_used[REGNO_PMC];
+    const size_t size_i = sizeof (INTVAL)   * number_regs_used[REGNO_INT];
+    const size_t size_n = sizeof (FLOATVAL) * number_regs_used[REGNO_NUM];
+    const size_t size_s = sizeof (STRING*)  * number_regs_used[REGNO_STR];
+    const size_t size_p = sizeof (PMC*)     * number_regs_used[REGNO_PMC];
 
     const size_t size_nip      = size_n + size_i + size_p;
     const size_t all_regs_size = size_n + size_i + size_p + size_s;
     const size_t reg_alloc     = ROUND_ALLOC_SIZE(all_regs_size);
     const int    slot          = CALCULATE_SLOT_NUM(reg_alloc);
 
+    /* this gets attached to the context, which should free it */
+    INTVAL *n_regs_used    = (INTVAL *)mem_sys_allocate(sizeof (INTVAL) * 4);
+    n_regs_used[REGNO_INT] = number_regs_used[REGNO_INT];
+    n_regs_used[REGNO_NUM] = number_regs_used[REGNO_NUM];
+    n_regs_used[REGNO_STR] = number_regs_used[REGNO_STR];
+    n_regs_used[REGNO_PMC] = number_regs_used[REGNO_PMC];
+
     /*
      * If slot is beyond the end of the allocated list, extend the list to
      * allocate more slots.
@@ -437,11 +455,14 @@ Parrot_alloc_context(PARROT_INTERP, NOTNULL(INTVAL *n_regs_used))
 
     /* regs start past the context */
     p = (void *) ((char *)ptr + ALIGNED_CTX_SIZE);
+
     /* ctx.bp points to I0, which has Nx on the left */
     interp->ctx.bp.regs_i = (INTVAL*)((char*)p + size_n);
+
     /* ctx.bp_ps points to S0, which has Px on the left */
     interp->ctx.bp_ps.regs_s = (STRING**)((char*)p + size_nip);
     init_context(interp, ctx, old);
+
     return ctx;
 }
 
@@ -456,7 +477,7 @@ return continuation invoke, else from the destructor of a continuation.
 
 PARROT_API
 void
-Parrot_free_context(PARROT_INTERP, NOTNULL(struct Parrot_Context *ctxp), int re_use)
+Parrot_free_context(PARROT_INTERP, NOTNULL(Parrot_Context *ctxp), int re_use)
 {
     /*
      * The context structure has a reference count, initially 0.  This field is
@@ -479,15 +500,20 @@ Parrot_free_context(PARROT_INTERP, NOTNULL(struct Parrot_Context *ctxp), int re_
 
             fprintf(stderr, "[free  ctx %p of sub '%s']\n",
                     (void *)ctxp,
-                    (doomed->name == (void*)0xdeadbeef
+                    (doomed && doomed->name == (void*)0xdeadbeef
                      ? "???"
                      : (char*)doomed->name->strstart));
         }
 #endif
+        if (ctxp->n_regs_used) {
+            mem_sys_free(ctxp->n_regs_used);
+            ctxp->n_regs_used = NULL;
+        }
+
         ptr  = ctxp;
         slot = CALCULATE_SLOT_NUM(ctxp->regs_mem_size);
 
-        assert(slot < interp->ctx_mem.n_free_slots);
+        PARROT_ASSERT(slot < interp->ctx_mem.n_free_slots);
         *(void **)ptr = interp->ctx_mem.free_list[slot];
         interp->ctx_mem.free_list[slot] = ptr;
     }
@@ -503,7 +529,7 @@ Mark the context as possible threshold.
 
 PARROT_API
 void
-Parrot_set_context_threshold(PARROT_INTERP, struct Parrot_Context *ctxp)
+Parrot_set_context_threshold(PARROT_INTERP, NULLOK(Parrot_Context *ctxp))
 {
     UNUSED(interp);
     UNUSED(ctxp);
@@ -625,7 +651,7 @@ mark_register_stack(PARROT_INTERP, NOTNULL(Stack_Chunk_t* chunk))
     for (; ; chunk = chunk->prev) {
         int i;
         save_regs_t   *save_r;
-        struct Interp_Context ctx;
+        Interp_Context ctx;
 
         pobject_lives(interp, (PObj*)chunk);
         if (chunk == chunk->prev)
