@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: resources.c 19886 2007-07-15 06:19:53Z petdance $
+$Id: resources.c 20687 2007-08-18 20:16:33Z chromatic $
 
 =head1 NAME
 
@@ -12,7 +12,6 @@ src/resources.c - Allocate and deallocate tracked resources
 
 */
 
-#include <assert.h>
 #include "parrot/parrot.h"
 #include "parrot/resources.h"
 
@@ -31,6 +30,8 @@ typedef void (*compact_f) (Interp *, Memory_Pool *);
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static char * aligned_mem( NOTNULL(const Buffer *buffer), NOTNULL(char *mem) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -44,7 +45,7 @@ PARROT_CONST_FUNCTION
 PARROT_WARN_UNUSED_RESULT
 static size_t aligned_string_size( size_t len );
 
-static void * alloc_new_block( PARROT_INTERP,
+static void alloc_new_block( PARROT_INTERP,
     size_t size,
     NOTNULL(Memory_Pool *pool),
     NOTNULL(const char *why) )
@@ -52,6 +53,8 @@ static void * alloc_new_block( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char* buffer_location( PARROT_INTERP, NOTNULL(const PObj *b) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -65,6 +68,7 @@ static void debug_print_buf( PARROT_INTERP, NOTNULL(const PObj *b) )
         __attribute__nonnull__(2);
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static void * mem_allocate( PARROT_INTERP,
     size_t size,
     NOTNULL(Memory_Pool *pool) )
@@ -78,6 +82,7 @@ static void merge_pools(
         __attribute__nonnull__(2);
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static Memory_Pool * new_memory_pool(
     size_t min_block,
     NULLOK(compact_f compact) );
@@ -92,7 +97,7 @@ the default size.  The given text is used for debugging.
 
 */
 
-static void *
+static void
 alloc_new_block(PARROT_INTERP, size_t size, NOTNULL(Memory_Pool *pool),
         NOTNULL(const char *why))
 {
@@ -120,7 +125,7 @@ alloc_new_block(PARROT_INTERP, size_t size, NOTNULL(Memory_Pool *pool),
     new_block->free  = alloc_size;
     new_block->size  = alloc_size;
 
-    new_block->next = NULL;
+    new_block->next  = NULL;
     new_block->start = (char *)new_block + sizeof (Memory_Block);
     new_block->top   = new_block->start;
 
@@ -136,8 +141,6 @@ alloc_new_block(PARROT_INTERP, size_t size, NOTNULL(Memory_Pool *pool),
 
     pool->top_block        = new_block;
     pool->total_allocated += alloc_size;
-
-    return new_block;
 }
 
 /*
@@ -176,13 +179,14 @@ Buffer memory layout:
 */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static void *
 mem_allocate(PARROT_INTERP, size_t size, NOTNULL(Memory_Pool *pool))
 {
     void *return_val;
 
     /* we always should have one block at least */
-    assert(pool->top_block);
+    PARROT_ASSERT(pool->top_block);
 
     /* If not enough room, try to find some */
     if (pool->top_block->free < size) {
@@ -235,6 +239,8 @@ mem_allocate(PARROT_INTERP, size_t size, NOTNULL(Memory_Pool *pool))
 }
 
 #if RESOURCE_DEBUG
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static const char*
 buffer_location(PARROT_INTERP, NOTNULL(const PObj *b))
 {
@@ -344,8 +350,9 @@ compact_pool(PARROT_INTERP, NOTNULL(Memory_Pool *pool))
 #endif
 
     /* Snag a block big enough for everything */
-    new_block = (Memory_Block *)alloc_new_block(interp, total_size, pool,
-            "inside compact");
+    alloc_new_block(interp, total_size, pool, "inside compact");
+
+    new_block = pool->top_block;
 
     /* Start at the beginning */
     cur_spot  = new_block->start;
@@ -388,11 +395,12 @@ compact_pool(PARROT_INTERP, NOTNULL(Memory_Pool *pool))
                     }
 
                     /* buffer has already been moved; just change the header */
-                    if (PObj_COW_TEST(b) && *ref_count & Buffer_moved_FLAG) {
+                    if (PObj_COW_TEST(b) &&
+                        (ref_count && *ref_count & Buffer_moved_FLAG)) {
                         /* Find out who else references our data */
                         Buffer * const hdr = *(Buffer **)(PObj_bufstart(b));
 
-                        assert(PObj_is_COWable_TEST(b));
+                        PARROT_ASSERT(PObj_is_COWable_TEST(b));
 
                         /* Make sure they know that we own it too */
                         PObj_COW_SET(hdr);
@@ -422,7 +430,7 @@ compact_pool(PARROT_INTERP, NOTNULL(Memory_Pool *pool))
 
                         /* If we're COW */
                         if (PObj_COW_TEST(b)) {
-                            assert(PObj_is_COWable_TEST(b));
+                            PARROT_ASSERT(PObj_is_COWable_TEST(b));
 
                             /* Let the old buffer know how to find us */
                             *(Buffer **)(PObj_bufstart(b)) = b;
@@ -453,7 +461,7 @@ compact_pool(PARROT_INTERP, NOTNULL(Memory_Pool *pool))
     /* First, where we allocate next */
     new_block->top = cur_spot;
 
-    assert(new_block->size >= (size_t)new_block->top -
+    PARROT_ASSERT(new_block->size >= (size_t)new_block->top -
             (size_t)new_block->start);
 
     /* How much is free. That's the total size minus the amount we used */
@@ -466,7 +474,7 @@ compact_pool(PARROT_INTERP, NOTNULL(Memory_Pool *pool))
     {
         Memory_Block *cur_block;
 
-        assert(new_block == pool->top_block);
+        PARROT_ASSERT(new_block == pool->top_block);
         cur_block = new_block->prev;
         while (cur_block) {
             Memory_Block * const next_block = cur_block->prev;
@@ -511,27 +519,26 @@ aligned_size(NOTNULL(const Buffer *buffer), size_t len)
 {
     if (PObj_is_COWable_TEST(buffer))
         len += sizeof (void*);
-    if (PObj_aligned_TEST(buffer)) {
+    if (PObj_aligned_TEST(buffer))
         len = (len + BUFFER_ALIGN_1) & BUFFER_ALIGN_MASK;
-    }
-    else {
+    else
         len = (len + WORD_ALIGN_1) & WORD_ALIGN_MASK;
-    }
     return len;
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static char *
 aligned_mem(NOTNULL(const Buffer *buffer), NOTNULL(char *mem))
 {
     if (PObj_is_COWable_TEST(buffer))
         mem += sizeof (void*);
-    if (PObj_aligned_TEST(buffer)) {
+    if (PObj_aligned_TEST(buffer))
         mem = (char*)(((unsigned long)(mem + BUFFER_ALIGN_1)) &
                 BUFFER_ALIGN_MASK);
-    }
-    else {
+    else
         mem = (char*)(((unsigned long)(mem + WORD_ALIGN_1)) & WORD_ALIGN_MASK);
-    }
+
     return mem;
 }
 
@@ -671,7 +678,7 @@ Parrot_reallocate_string(PARROT_INTERP, NOTNULL(STRING *str), size_t tosize)
         PObj_buflen(str) = new_size - sizeof (void*);
         return;
     }
-    assert(str->bufused <= tosize);
+    PARROT_ASSERT(str->bufused <= tosize);
     /* only copy used memory, not total string buffer */
     copysize = str->bufused;
 
@@ -711,7 +718,7 @@ Parrot_allocate(PARROT_INTERP, NOTNULL(Buffer *buffer), size_t size)
 {
     PObj_buflen(buffer) = 0;
     PObj_bufstart(buffer) = NULL;
-    assert((size & WORD_ALIGN_1) == 0);
+    PARROT_ASSERT((size & WORD_ALIGN_1) == 0);
     PObj_bufstart(buffer) = mem_allocate(interp, size,
             interp->arena_base->memory_pool);
     PObj_buflen(buffer) = size;
@@ -795,20 +802,19 @@ Create a new memory pool.
 */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static Memory_Pool *
 new_memory_pool(size_t min_block, NULLOK(compact_f compact))
 {
     Memory_Pool * const pool = mem_internal_allocate_typed(Memory_Pool);
 
-    if (pool) {
-        pool->top_block              = NULL;
-        pool->compact                = compact;
-        pool->minimum_block_size     = min_block;
-        pool->total_allocated        = 0;
-        pool->guaranteed_reclaimable = 0;
-        pool->possibly_reclaimable   = 0;
-        pool->reclaim_factor         = RECLAMATION_FACTOR;
-    }
+    pool->top_block              = NULL;
+    pool->compact                = compact;
+    pool->minimum_block_size     = min_block;
+    pool->total_allocated        = 0;
+    pool->guaranteed_reclaimable = 0;
+    pool->possibly_reclaimable   = 0;
+    pool->reclaim_factor         = RECLAMATION_FACTOR;
 
     return pool;
 }
@@ -832,8 +838,7 @@ Parrot_initialize_memory_pools(PARROT_INTERP)
     /* Constant strings - not compacted */
     arena_base->constant_string_pool = new_memory_pool(POOL_SIZE, NULL);
 
-    alloc_new_block(interp, POOL_SIZE,
-                    arena_base->constant_string_pool, "init");
+    alloc_new_block(interp, POOL_SIZE, arena_base->constant_string_pool,"init");
 }
 
 /*
