@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: smallobject.c 19914 2007-07-16 21:06:48Z petdance $
+$Id: smallobject.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -16,7 +16,6 @@ Handles the accessing of small object pools (header pools).
 
 #include "parrot/parrot.h"
 #include "parrot/smallobject.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/smallobject.h */
 
@@ -24,7 +23,7 @@ Handles the accessing of small object pools (header pools).
 
 static void gc_ms_add_free_object( SHIM_INTERP,
     NOTNULL(Small_Object_Pool *pool),
-    NOTNULL(void *to_add) )
+    NOTNULL(PObj *to_add) )
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
@@ -33,7 +32,9 @@ static void gc_ms_alloc_objects( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static void * gc_ms_get_free_object( PARROT_INTERP,
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+static PObj * gc_ms_get_free_object( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -104,7 +105,7 @@ Parrot_is_const_pmc(PARROT_INTERP, NOTNULL(PMC *pmc))
     const               int   c    = contained_in_pool(pool, pmc);
 
     /* some paranoia first */
-    assert(!!PObj_constant_TEST(pmc) == !!c);
+    PARROT_ASSERT(!!PObj_constant_TEST(pmc) == !!c);
 
     return c;
 }
@@ -149,10 +150,10 @@ Add an unused object back to the free pool for later reuse.
 */
 
 static void
-gc_ms_add_free_object(SHIM_INTERP, NOTNULL(Small_Object_Pool *pool), NOTNULL(void *to_add))
+gc_ms_add_free_object(SHIM_INTERP, NOTNULL(Small_Object_Pool *pool), NOTNULL(PObj *to_add))
 {
-    *(void **)to_add = pool->free_list;
-    pool->free_list  = to_add;
+    PMC_struct_val(to_add) = pool->free_list;
+    pool->free_list        = to_add;
 }
 
 /*
@@ -163,19 +164,24 @@ Get a new object from the free pool and return it.
 
 */
 
-static void *
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
+static PObj *
 gc_ms_get_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
-    void *ptr;
+    PObj *ptr;
+    PObj *free_list = (PObj *)pool->free_list;
 
     /* if we don't have any objects */
-    if (!pool->free_list)
+    if (!free_list) {
         (*pool->more_objects)(interp, pool);
+        free_list = (PObj *)pool->free_list;
+    }
 
-    ptr             = pool->free_list;
-    pool->free_list = *(void **)ptr;
+    ptr             = free_list;
+    pool->free_list = PMC_struct_val(ptr);
 
-    PObj_flags_SETTO((PObj*) ptr, 0);
+    PObj_flags_SETTO(ptr, 0);
 
     --pool->num_free_objects;
 
@@ -186,27 +192,26 @@ gc_ms_get_free_object(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 
 FUNCDOC: Parrot_add_to_free_list
 
-Adds the memory between C<start> and C<end> to the free list.
+Adds the objects in the newly allocated C<arena> to the free list.
 
 */
 
 void
 Parrot_add_to_free_list(PARROT_INTERP,
         NOTNULL(Small_Object_Pool  *pool),
-        NOTNULL(Small_Object_Arena *arena),
-        UINTVAL             start,
-        UINTVAL             end)
+        NOTNULL(Small_Object_Arena *arena))
 {
     UINTVAL  i;
-    void    *object;
+    PObj    *object;
+    UINTVAL  num_objects = pool->objects_per_alloc;
 
-    pool->total_objects += end - start;
-    arena->used          = end;
+    pool->total_objects += num_objects;
+    arena->used          = num_objects;
 
     /* Move all the new objects into the free list */
-    object = (void *)((char *)arena->start_objects + start * pool->object_size);
+    object = (PObj *)((char *)arena->start_objects);
 
-    for (i = start; i < end; i++) {
+    for (i = 0; i < num_objects; i++) {
         PObj_flags_SETTO((PObj *)object, PObj_on_free_list_FLAG);
         /*
          * during GC buflen is used to check for objects on the
@@ -214,10 +219,10 @@ Parrot_add_to_free_list(PARROT_INTERP,
          */
         PObj_buflen((PObj*)object) = 0;
         pool->add_free_object(interp, pool, object);
-        object = (void *)((char *)object + pool->object_size);
+        object = (PObj *)((char *)object + pool->object_size);
     }
 
-    pool->num_free_objects += end - start;
+    pool->num_free_objects += num_objects;
 }
 
 /*
@@ -267,7 +272,6 @@ static void
 gc_ms_alloc_objects(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
     size_t  size;
-    UINTVAL start, end;
 
     /* Setup memory for the new objects */
     Small_Object_Arena * const new_arena =
@@ -283,9 +287,7 @@ gc_ms_alloc_objects(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 
     Parrot_append_arena_in_pool(interp, pool, new_arena, size);
 
-    start = 0;
-    end   = pool->objects_per_alloc;
-    Parrot_add_to_free_list(interp, pool, new_arena, start, end);
+    Parrot_add_to_free_list(interp, pool, new_arena);
 
     /* Allocate more next time */
     if (GC_DEBUG(interp)) {
@@ -317,6 +319,7 @@ Creates a new C<Small_Object_Pool> and returns a pointer to it.
 */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 Small_Object_Pool *
 new_small_object_pool(size_t object_size, size_t objects_per_alloc)
 {
@@ -395,9 +398,9 @@ Parrot_small_object_pool_merge(PARROT_INTERP,
     }
 #endif
 
-    /* assert(source->total_objects); */
-    assert(dest->object_size == source->object_size);
-    assert((dest->name == NULL && source->name == NULL) ||
+    /* PARROT_ASSERT(source->total_objects); */
+    PARROT_ASSERT(dest->object_size == source->object_size);
+    PARROT_ASSERT((dest->name == NULL && source->name == NULL) ||
         0 == strcmp(dest->name, source->name));
 
     dest->total_objects += source->total_objects;
