@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: hash.c 19915 2007-07-16 22:08:22Z petdance $
+$Id: hash.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -24,7 +24,6 @@ don't apply.
 */
 
 #include "parrot/parrot.h"
-#include <assert.h>
 
 #define INITIAL_BUCKETS 16
 
@@ -73,10 +72,8 @@ static void init_hash(
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static int int_compare( SHIM_INTERP,
-    NOTNULL(const void *a),
-    NOTNULL(const void *b) )
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3);
+    NULLOK(const void *a),
+    NULLOK(const void *b) );
 
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
@@ -87,11 +84,13 @@ static size_t key_hash_cstring( SHIM_INTERP,
 
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
-static size_t key_hash_int( SHIM_INTERP, void *value, size_t seed );
+static size_t key_hash_int( SHIM_INTERP, NULLOK(void *value), size_t seed );
 
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
-static size_t key_hash_pointer( SHIM_INTERP, void *value, size_t seed );
+static size_t key_hash_pointer( SHIM_INTERP,
+    NULLOK(void *value),
+    size_t seed );
 
 PARROT_WARN_UNUSED_RESULT
 static size_t key_hash_STRING( PARROT_INTERP,
@@ -103,14 +102,16 @@ static size_t key_hash_STRING( PARROT_INTERP,
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static int pointer_compare( SHIM_INTERP,
-    const void * const a,
-    const void * const b );
+    NULLOK(const void *a),
+    NULLOK(const void *b) );
 
 PARROT_WARN_UNUSED_RESULT
 static int STRING_compare( PARROT_INTERP,
-    const void *search_key,
-    const void *bucket_key )
-        __attribute__nonnull__(1);
+    NOTNULL(const void *search_key),
+    NOTNULL(const void *bucket_key) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 /* HEADERIZER END: static */
 
@@ -146,7 +147,7 @@ Compares the two strings, returning 0 if they are identical.
 
 PARROT_WARN_UNUSED_RESULT
 static int
-STRING_compare(PARROT_INTERP, const void *search_key, const void *bucket_key)
+STRING_compare(PARROT_INTERP, NOTNULL(const void *search_key), NOTNULL(const void *bucket_key))
 {
     return string_equal(interp, (const STRING *)search_key, (const STRING *)bucket_key);
 }
@@ -161,7 +162,7 @@ Compares the two pointers, returning 0 if they are identical
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static int
-pointer_compare(SHIM_INTERP, const void * const a, const void * const b)
+pointer_compare(SHIM_INTERP, NULLOK(const void *a), NULLOK(const void *b))
 {
     return a != b;
 }
@@ -176,7 +177,7 @@ Returns a hashvalue for a pointer.
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static size_t
-key_hash_pointer(SHIM_INTERP, void *value, size_t seed)
+key_hash_pointer(SHIM_INTERP, NULLOK(void *value), size_t seed)
 {
     return ((size_t) value) ^ seed;
 }
@@ -221,7 +222,7 @@ Custom C<key_hash> function.
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static size_t
-key_hash_int(SHIM_INTERP, void *value, size_t seed)
+key_hash_int(SHIM_INTERP, NULLOK(void *value), size_t seed)
 {
     return (size_t)value ^ seed;
 }
@@ -236,7 +237,7 @@ Custom C<compare> function.
 PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 static int
-int_compare(SHIM_INTERP, NOTNULL(const void *a), NOTNULL(const void *b))
+int_compare(SHIM_INTERP, NULLOK(const void *a), NULLOK(const void *b))
 {
     return a != b;
 }
@@ -340,16 +341,17 @@ hash_thaw(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(visit_info* info))
                 break;
             default:
                 real_exception(interp, NULL, 1, "unimplemented key type");
-                b = NULL;
                 break;
         }
         switch (hash->entry_type) {
+            INTVAL i;
             case enum_hash_pmc:
                 info->thaw_ptr = (PMC**)&b->value;
                 (info->visit_pmc_now)(interp, NULL, info);
                 break;
             case enum_hash_int:
-                b->value = (void*)io->vtable->shift_integer(interp, io);
+                i        = io->vtable->shift_integer(interp, io);
+                b->value = (void *)i;
                 break;
             default:
                 real_exception(interp, NULL, 1, "unimplemented value type");
@@ -377,7 +379,6 @@ hash_freeze(PARROT_INTERP, NOTNULL(const Hash * const hash), NOTNULL(visit_info*
                     break;
                 default:
                     real_exception(interp, NULL, 1, "unimplemented key type");
-                    b = NULL;
                     break;
             }
             switch (hash->entry_type) {
@@ -608,7 +609,7 @@ init_hash(NOTNULL(Hash *hash),
      * TODO randomize
      */
     hash->seed = 3793;
-    assert(INITIAL_BUCKETS % 4 == 0);
+    PARROT_ASSERT(INITIAL_BUCKETS % 4 == 0);
     hash->mask = INITIAL_BUCKETS-1;
     hash->entries = 0;
 
@@ -738,6 +739,8 @@ C<PObj_constant_FLAG> or 0.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC*
 Parrot_new_INTVAL_hash(PARROT_INTERP, UINTVAL flags)
 {
@@ -769,7 +772,6 @@ parrot_hash_size(PARROT_INTERP, NOTNULL(const Hash *hash))
     if (hash)
         return hash->entries;
     real_exception(interp, NULL, 1, "parrot_hash_size asked to check a NULL hash\n");
-    return 0;
 }
 
 /*
@@ -832,8 +834,9 @@ Returns the bucket for C<key>.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 HashBucket *
-parrot_hash_get_bucket(PARROT_INTERP, NOTNULL(const Hash *hash), void *key)
+parrot_hash_get_bucket(PARROT_INTERP, NOTNULL(const Hash *hash), NOTNULL(void *key))
 {
     const UINTVAL  hashval = (hash->hash_val)(interp, key, hash->seed);
     HashBucket    *bucket  = hash->bi[hashval & hash->mask];
@@ -858,7 +861,7 @@ PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
 void *
-parrot_hash_get(PARROT_INTERP, NOTNULL(Hash *hash), void *key)
+parrot_hash_get(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(void *key))
 {
     const HashBucket * const bucket = parrot_hash_get_bucket(interp, hash, key);
     return bucket ? bucket->value : NULL;
@@ -874,7 +877,7 @@ Returns whether the key exists in the hash.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-parrot_hash_exists(PARROT_INTERP, NOTNULL(Hash *hash), void *key)
+parrot_hash_exists(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(void *key))
 {
     const HashBucket * const bucket = parrot_hash_get_bucket(interp, hash, key);
     return bucket ? 1 : 0;
@@ -889,8 +892,10 @@ copied.
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 HashBucket*
-parrot_hash_put(PARROT_INTERP, NOTNULL(Hash *hash), void *key, void *value)
+parrot_hash_put(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(void *key), NULLOK(void *value))
 {
     const UINTVAL hashval = (hash->hash_val)(interp, key, hash->seed);
     HashBucket   *bucket = hash->bi[hashval & hash->mask];
@@ -942,7 +947,7 @@ Deletes the key from the hash.
 
 PARROT_API
 void
-parrot_hash_delete(PARROT_INTERP, NOTNULL(Hash *hash), void *key)
+parrot_hash_delete(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(void *key))
 {
     HashBucket *bucket;
     HashBucket *prev = NULL;
@@ -1004,9 +1009,9 @@ parrot_hash_clone(PARROT_INTERP, NOTNULL(Hash *hash), NOTNULL(Hash **dest))
                 break;
 
             default:
+                valtmp = NULL; /* avoid warning */
                 real_exception(interp, NULL, -1, "hash corruption: type = %d\n",
                                    hash->entry_type);
-                valtmp = NULL; /* avoid warning */
             };
             parrot_hash_put(interp, *dest, key, valtmp);
             b = b->next;
