@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: headers.c 19905 2007-07-16 04:44:05Z petdance $
+$Id: headers.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -14,7 +14,6 @@ Handles getting of various headers, and pool creation.
 
 #include "parrot/parrot.h"
 #include "parrot/headers.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/headers.h */
 
@@ -43,7 +42,7 @@ static PMC_EXT * new_pmc_ext( PARROT_INTERP )
 
 static int sweep_cb_buf( PARROT_INTERP,
     NOTNULL(Small_Object_Pool *pool),
-    int flag,
+    SHIM(int flag),
     NOTNULL(void *arg) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -331,6 +330,7 @@ add_pmc_ext(PARROT_INTERP, NOTNULL(PMC *pmc))
     if (PObj_live_TEST(pmc))
         PObj_get_FLAGS(pmc) |= PObj_custom_GC_FLAG;
 #endif
+    PMC_next_for_GC(pmc) = PMCNULL;
 }
 
 /*
@@ -717,7 +717,7 @@ sweep_cb_pmc(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag,
 void
 Parrot_destroy_header_pools(PARROT_INTERP)
 {
-    INTVAL pass, start;
+    INTVAL pass, start, ignored;
 
     /* const/non const COW strings life in different pools
      * so in first pass
@@ -729,14 +729,15 @@ Parrot_destroy_header_pools(PARROT_INTERP)
 #else
     start = 2;
 #endif
-    Parrot_forall_header_pools(interp, POOL_PMC | POOL_CONST, NULL,
+    ignored = Parrot_forall_header_pools(interp, POOL_PMC | POOL_CONST, NULL,
             sweep_cb_pmc);
 
     for (pass = start; pass <= 2; pass++) {
-        Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_CONST,
+        ignored = Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_CONST,
                 (void *)pass, sweep_cb_buf);
 
     }
+
     free_pool(interp->arena_base->pmc_ext_pool);
     mem_internal_free(interp->arena_base->sized_header_pools);
 }
@@ -765,7 +766,7 @@ fix_pmc_syncs(NOTNULL(Interp *dest_interp), NOTNULL(Small_Object_Pool *pool))
                         /* fprintf(stderr, "BAD PMC: address=%p,
                                    base_type=%d\n",
                                    p, p->vtable->base_type); */
-                        assert(0);
+                        PARROT_ASSERT(0);
                     }
                 }
             }
@@ -814,9 +815,9 @@ Parrot_merge_header_pools(NOTNULL(Interp *dest_interp), NOTNULL(Interp *source_i
 
         if (i >= dest_arena->num_sized ||
             !dest_arena->sized_header_pools[i]) {
-            make_bufferlike_pool(dest_interp, i * sizeof (void *)
-                + sizeof (Buffer));
-            assert(dest_arena->sized_header_pools[i]);
+            Small_Object_Pool *ignored = make_bufferlike_pool(dest_interp,
+                i * sizeof (void *) + sizeof (Buffer));
+            PARROT_ASSERT(dest_arena->sized_header_pools[i]);
         }
 
         Parrot_small_object_pool_merge(dest_interp,
