@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2005-2007, The Perl Foundation.
-$Id: hll.c 19882 2007-07-14 22:33:30Z petdance $
+$Id: hll.c 20751 2007-08-21 03:56:22Z chromatic $
 
 =head1 NAME
 
@@ -56,25 +56,18 @@ the context.  If no type is registered, returns C<core_type>.
 
 #include "parrot/parrot.h"
 #include "parrot/dynext.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/hll.h */
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* new_hll_entry( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: static */
 
-
-enum {
-    e_HLL_name,
-    e_HLL_lib,
-    e_HLL_typemap,
-    e_HLL_MAX
-} HLL_enum_t;
-
 /* for shared HLL data, do COW stuff */
 #define START_READ_HLL_INFO(interp, hll_info)
 #define END_READ_HLL_INFO(interp, hll_info)
@@ -89,9 +82,11 @@ enum {
     } while (0)
 #define END_WRITE_HLL_INFO(interp, hll_info)
 
-#define ASSERT_CONST_STRING(src) assert(PObj_constant_TEST(src))
+#define ASSERT_CONST_STRING(src) PARROT_ASSERT(PObj_constant_TEST(src))
 
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 new_hll_entry(PARROT_INTERP)
 {
@@ -111,7 +106,7 @@ new_hll_entry(PARROT_INTERP)
 
 PARROT_API
 INTVAL
-Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), STRING *hll_lib)
+Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), NULLOK(STRING *hll_lib))
 {
     PMC *entry, *name, *type_hash, *ns_hash, *hll_info;
     INTVAL idx;
@@ -119,10 +114,33 @@ Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), STRING *hll_lib)
     /* TODO LOCK or disallow in threads */
 
     if (!hll_name) {
+        INTVAL nelements, i;
+
         /* .loadlib pragma */
         hll_info = interp->HLL_info;
 
         START_WRITE_HLL_INFO(interp, hll_info);
+
+        nelements = VTABLE_elements(interp, hll_info);
+
+        for (i = 0; i < nelements; ++i) {
+            STRING      *name;
+            PMC * const  entry    = VTABLE_get_pmc_keyed_int(interp, hll_info, i);
+            PMC * const  lib_name = VTABLE_get_pmc_keyed_int(interp, entry,
+                e_HLL_lib);
+
+            if (PMC_IS_NULL(lib_name))
+                continue;
+
+            name = VTABLE_get_string(interp, lib_name);
+
+            if (!string_equal(interp, name, hll_lib))
+                break;
+        }
+
+        if (i < nelements)
+            return i;
+
         entry    = new_hll_entry(interp);
 
         VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_name, PMCNULL);
@@ -189,8 +207,9 @@ Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), STRING *hll_lib)
     VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_typemap, type_hash);
 
     /* load lib */
-    if (string_length(interp, hll_lib))
-        Parrot_load_lib(interp, hll_lib, NULL);
+    if (string_length(interp, hll_lib)) {
+        PMC *ignored = Parrot_load_lib(interp, hll_lib, NULL);
+    }
 
     /* UNLOCK */
     END_WRITE_HLL_INFO(interp, hll_info);
@@ -230,6 +249,8 @@ Parrot_get_HLL_id(PARROT_INTERP, NULLOK(STRING *hll_name))
     return i < nelements ? i : -1;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PARROT_API
 STRING *
 Parrot_get_HLL_name(PARROT_INTERP, INTVAL id)
@@ -280,10 +301,10 @@ Parrot_register_HLL_type(PARROT_INTERP, INTVAL hll_id,
     START_WRITE_HLL_INFO(interp, hll_info);
 
     entry     = VTABLE_get_pmc_keyed_int(interp, hll_info, hll_id);
-    assert(!PMC_IS_NULL(entry));
+    PARROT_ASSERT(!PMC_IS_NULL(entry));
 
     type_hash = VTABLE_get_pmc_keyed_int(interp, entry, e_HLL_typemap);
-    assert(!PMC_IS_NULL(type_hash));
+    PARROT_ASSERT(!PMC_IS_NULL(type_hash));
 
     hash      = (Hash *)PMC_struct_val(type_hash);
 
@@ -355,6 +376,8 @@ Return root namespace of the current HLL.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
 Parrot_get_ctx_HLL_namespace(PARROT_INTERP)
 {
@@ -371,6 +394,8 @@ special value C<PARROT_HLL_NONE>, return the global root namespace.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
 Parrot_get_HLL_namespace(PARROT_INTERP, int hll_id)
 {
