@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: inter_call.c 19886 2007-07-15 06:19:53Z petdance $
+$Id: inter_call.c 20363 2007-07-31 00:37:25Z rgrjr $
 
 =head1 NAME
 
@@ -13,14 +13,9 @@ subroutines.
 
 =head2 Functions
 
-=over 4
-
-=cut
-
 */
 
 
-#include <assert.h>
 #include "parrot/parrot.h"
 #include "parrot/oplib/ops.h"
 #include "inter_call.str"
@@ -43,14 +38,19 @@ static void clone_key_arg( PARROT_INTERP, NOTNULL(call_state *st) )
 static void commit_last_arg( PARROT_INTERP,
     int index,
     int cur,
-    opcode_t *n_regs_used,
+    NOTNULL(opcode_t *n_regs_used),
     int seen_arrow,
-    PMC **sigs,
-    opcode_t **indexes,
-    parrot_context_t *ctx,
-    PMC *pmc,
-    va_list *list )
-        __attribute__nonnull__(1);
+    NOTNULL(PMC **sigs),
+    NOTNULL(opcode_t **indexes),
+    NOTNULL(parrot_context_t *ctx),
+    NULLOK(PMC *pmc),
+    NOTNULL(va_list *list) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(6)
+        __attribute__nonnull__(7)
+        __attribute__nonnull__(8)
+        __attribute__nonnull__(10);
 
 static void convert_arg_from_int( PARROT_INTERP, NOTNULL(call_state *st) )
         __attribute__nonnull__(1)
@@ -160,8 +160,9 @@ Initialize the return value passing state C<call_state> for the given NCI signat
 
 PARROT_API
 int
-Parrot_init_arg_nci(PARROT_INTERP, NOTNULL(call_state *st), NOTNULL(const char *sig))
+Parrot_init_arg_nci(PARROT_INTERP, ARGOUT(call_state *st), NOTNULL(const char *sig))
 {
+    init_call_stats(st);
     if (PMC_IS_NULL(interp->args_signature))
         Parrot_init_arg_op(interp, CONTEXT(interp->ctx), interp->current_args, &st->src);
     else
@@ -173,7 +174,7 @@ Parrot_init_arg_nci(PARROT_INTERP, NOTNULL(call_state *st), NOTNULL(const char *
 
 PARROT_API
 int
-Parrot_init_ret_nci(PARROT_INTERP, NOTNULL(call_state *st), NOTNULL(const char *sig))
+Parrot_init_ret_nci(PARROT_INTERP, ARGOUT(call_state *st), NOTNULL(const char *sig))
 {
     Parrot_Context *ctx                 = CONTEXT(interp->ctx);
     PMC            * const current_cont = ctx->current_cont;
@@ -266,11 +267,9 @@ Parrot_init_arg_op(PARROT_INTERP, NOTNULL(parrot_context_t *ctx), NULLOK(opcode_
 
 PARROT_API
 int
-Parrot_init_arg_sig(PARROT_INTERP, parrot_context_t *ctx, NOTNULL(const char *sig),
-    void *ap, NOTNULL(call_state_item *sti))
+Parrot_init_arg_sig(SHIM_INTERP, NOTNULL(parrot_context_t *ctx), NOTNULL(const char *sig),
+    NULLOK(void *ap), NOTNULL(call_state_item *sti))
 {
-    UNUSED(interp);
-
     sti->used = 1;
     sti->i = 0;
     sti->n = 0;
@@ -456,12 +455,12 @@ Parrot_fetch_arg(PARROT_INTERP, NOTNULL(call_state *st))
     /* check if we're at a :flat argument */
     if (st->src.mode & CALL_STATE_FLATTEN) {
         PMC *elem;
-        assert(st->src.slurp_i < st->src.slurp_n);
+        PARROT_ASSERT(st->src.slurp_i < st->src.slurp_n);
         if (!PMC_IS_NULL(st->key)) {
             st->src.slurp_i++;
             st->name = (STRING *)parrot_hash_get_idx(interp,
                             (Hash *)PMC_struct_val(st->src.slurp), st->key);
-            assert(st->name);
+            PARROT_ASSERT(st->name);
             elem = VTABLE_get_pmc_keyed_str(interp, st->src.slurp, st->name);
         }
         else {
@@ -508,7 +507,7 @@ Parrot_fetch_arg_nci(PARROT_INTERP, NOTNULL(call_state *st))
     next_arg_sig(&st->dest);
     if (st->dest.sig & PARROT_ARG_SLURPY_ARRAY) {
         PMC *slurped;
-        assert((st->dest.sig & PARROT_ARG_TYPE_MASK) == PARROT_ARG_PMC);
+        PARROT_ASSERT((st->dest.sig & PARROT_ARG_TYPE_MASK) == PARROT_ARG_PMC);
         slurped = pmc_new(interp, enum_class_ResizablePMCArray);
         while (Parrot_fetch_arg(interp, st)) {
             st->src.used = 1;
@@ -626,7 +625,7 @@ check_for_opt_flag(NOTNULL(call_state *st), int has_arg)
     /* we're at an :opt_flag argument, so actually store something */
     idx = st->dest.u.op.pc[st->dest.i];
     --st->params;
-    assert(idx >= 0);
+    PARROT_ASSERT(idx >= 0);
     CTX_REG_INT(st->dest.ctx, idx) = has_arg;
 }
 
@@ -769,9 +768,9 @@ store_current_arg(NOTNULL(call_state *st))
     if (st->dest.i >= st->dest.n)
         return 0;
 
-    assert(st->dest.mode & CALL_STATE_OP);
+    PARROT_ASSERT(st->dest.mode & CALL_STATE_OP);
     idx = st->dest.u.op.pc[st->dest.i];
-    assert(idx >= 0);
+    PARROT_ASSERT(idx >= 0);
     store_arg(st, idx);
 
     return 1;
@@ -869,7 +868,7 @@ check_named(PARROT_INTERP, NOTNULL(call_state *st))
 
             /* verify that an actual arg exists */
             arg_sig = st->dest.sig = SIG_ITEM(st->dest.u.op.signature, i);
-            assert(!(arg_sig & PARROT_ARG_NAME));
+            PARROT_ASSERT(!(arg_sig & PARROT_ARG_NAME));
             /* if this named arg is already filled, continue*/
             if (st->named_done & (1 << n_named)) {
                 arg_sig = st->dest.sig = SIG_ITEM(st->dest.u.op.signature, i+1);
@@ -968,7 +967,7 @@ Parrot_process_args(PARROT_INTERP, NOTNULL(call_state *st), arg_pass_t param_or_
                 null_val(st->dest.sig, st);
                 /* actually store the argument */
                 idx = st->dest.u.op.pc[st->dest.i];
-                assert(idx >= 0);
+                PARROT_ASSERT(idx >= 0);
                 store_arg(st, idx);
                 /* check for :opt_flag */
                 check_for_opt_flag(st, 0);
@@ -1004,7 +1003,7 @@ Parrot_process_args(PARROT_INTERP, NOTNULL(call_state *st), arg_pass_t param_or_
             return;
         /* actually store the argument */
         idx = st->dest.u.op.pc[st->dest.i];
-        assert(idx >= 0);
+        PARROT_ASSERT(idx >= 0);
         store_arg(st, idx);
 
         /* if we're at an :optional argument, we need to check for an :opt_flag */
@@ -1020,7 +1019,7 @@ Parrot_process_args(PARROT_INTERP, NOTNULL(call_state *st), arg_pass_t param_or_
         PMC * const array = pmc_new(interp, enum_class_ResizablePMCArray);
 
         const INTVAL idx = st->dest.u.op.pc[dest->i];
-        assert(idx >= 0);
+        PARROT_ASSERT(idx >= 0);
         /* Must register this PMC or it may get collected when only the struct
          * references it. */
         dod_register_pmc(interp, array);
@@ -1184,9 +1183,11 @@ Prerequisites are like above.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 opcode_t *
-parrot_pass_args_fromc(PARROT_INTERP, const char *sig,
-        opcode_t *dest, parrot_context_t *old_ctxp, va_list ap)
+parrot_pass_args_fromc(PARROT_INTERP, NOTNULL(const char *sig),
+        NOTNULL(opcode_t *dest), NOTNULL(parrot_context_t *old_ctxp), va_list ap)
 {
     call_state st;
 
@@ -1219,6 +1220,8 @@ set_retval_util(PARROT_INTERP, NOTNULL(const char *sig), NOTNULL(parrot_context_
 /*
  * handle void, and pointer (PMC*, STRING*) return values
  */
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 void*
 set_retval(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
 {
@@ -1276,6 +1279,8 @@ set_retval_f(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
 /*
  * handle STRING return value
  */
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING*
 set_retval_s(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
 {
@@ -1293,6 +1298,8 @@ set_retval_s(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
 /*
  * handle PMC return value
  */
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC*
 set_retval_p(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
 {
@@ -1312,8 +1319,9 @@ set_retval_p(PARROT_INTERP, int sig_ret, NOTNULL(parrot_context_t *ctx))
  * in the arg signature.  see Parrot_PCCINVOKE for signature syntax.
  */
 static void
-commit_last_arg(PARROT_INTERP, int index, int cur, opcode_t *n_regs_used, int seen_arrow,
-        PMC **sigs, opcode_t **indexes, parrot_context_t *ctx, PMC *pmc, va_list *list)
+commit_last_arg(PARROT_INTERP, int index, int cur, NOTNULL(opcode_t *n_regs_used),
+        int seen_arrow, NOTNULL(PMC **sigs), NOTNULL(opcode_t **indexes),
+        NOTNULL(parrot_context_t *ctx), NULLOK(PMC *pmc), NOTNULL(va_list *list))
 {
     int reg_offset = 0;
 
@@ -1361,8 +1369,7 @@ commit_last_arg(PARROT_INTERP, int index, int cur, opcode_t *n_regs_used, int se
 
 /*
 
-=item C<void
-Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signature, ... )>
+FUNCDOC: Parrot_PCCINVOKE
 
 pmc is the invocant.
 method_name is the method_name used in the find_method VTABLE call
@@ -1397,13 +1404,13 @@ Example signature
     a slurpy PMC: Ps
 
 invokes a PMC method
-=cut
 
 */
 
 PARROT_API
 void
-Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signature, ...)
+Parrot_PCCINVOKE(PARROT_INTERP, NULLOK(PMC* pmc), NOTNULL(STRING *method_name),
+        NOTNULL(const char *signature), ...)
 {
 #define PCC_ARG_MAX 1024
     /* variables from PCCINVOKE impl in PCCMETHOD.pm */
@@ -1464,7 +1471,7 @@ Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signa
         if (*x == '-') {
             seen_arrow = 1 ;
         }
-        else if (isupper(*x)) {
+        else if (isupper((unsigned char)*x)) {
              /* calculate needed length of arg and result sig FIAs */
             arg_ret_cnt[seen_arrow]++;
 
@@ -1548,7 +1555,7 @@ Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signa
             n_regs_used[3] = 0;
         }
         /* parse arg type */
-        else if (isupper(*x)) {
+        else if (isupper((unsigned char)*x)) {
             if (index >= 0)
                 commit_last_arg(interp, index, cur, n_regs_used, seen_arrow,
                     sigs, indexes, ctx, pmc, &list);
@@ -1567,7 +1574,7 @@ Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signa
 
         }
         /* parse arg adverbs */
-        else if (islower(*x)) {
+        else if (islower((unsigned char)*x)) {
             switch (*x) {
                 case 'n': cur |= PARROT_ARG_NAME;         break;
                 case 'f': cur |= PARROT_ARG_FLATTEN;      break;
@@ -1615,8 +1622,8 @@ Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signa
     index      = 0;
     seen_arrow = 1;
 
-    for (x=ret_x; *x; x++) {
-        if (isupper(*x)) {
+    for (x=ret_x; x && *x; x++) {
+        if (isupper((unsigned char)*x)) {
             switch (*x) {
                 case 'I':
                     {
@@ -1662,14 +1669,10 @@ Parrot_PCCINVOKE(PARROT_INTERP, PMC* pmc, STRING *method_name, const char *signa
 
 /*
 
-=back
-
 =head1 SEE ALSO
 
 F<include/parrot/interpreter.h>, F<src/inter_run.c>, F<src/pmc/sub.pmc>.
 
-=cut
-
 */
 
 /*
