@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2003, The Perl Foundation.
-$Id: inter_run.c 19790 2007-07-11 20:09:10Z petdance $
+$Id: inter_run.c 20376 2007-07-31 06:52:48Z petdance $
 
 =head1 NAME
 
@@ -15,7 +15,6 @@ Various functions that call the run loop.
 */
 
 
-#include <assert.h>
 #include "parrot/parrot.h"
 #include "parrot/oplib/ops.h"
 
@@ -23,13 +22,18 @@ Various functions that call the run loop.
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static parrot_context_t * runops_args( PARROT_INTERP,
-    PMC *sub,
-    PMC *obj,
-    STRING *meth,
-    const char* sig,
+    NOTNULL(PMC *sub),
+    NOTNULL(PMC *obj),
+    SHIM(STRING *meth),
+    NOTNULL(const char *sig),
     va_list ap )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(5);
 
 /* HEADERIZER END: static */
 
@@ -127,8 +131,10 @@ is an invocable C<Sub> PMC.
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 parrot_context_t *
-Parrot_runops_fromc(PARROT_INTERP, PMC *sub)
+Parrot_runops_fromc(PARROT_INTERP, NOTNULL(PMC *sub))
 {
     opcode_t offset, *dest;
     parrot_context_t *ctx;
@@ -136,7 +142,7 @@ Parrot_runops_fromc(PARROT_INTERP, PMC *sub)
     /* we need one return continuation with a NULL offset */
     PMC * const ret_c = new_ret_continuation_pmc(interp, NULL);
     interp->current_cont = ret_c;
-#if GC_VERBOSE
+#if defined GC_VERBOSE && GC_VERBOSE
     PObj_report_SET(ret_c);     /* s. also dod.c */
 #endif
     /* invoke the sub, which places the context of the sub in the
@@ -144,9 +150,8 @@ Parrot_runops_fromc(PARROT_INTERP, PMC *sub)
      * Passing a dummy true destination copies registers
      */
     dest = VTABLE_invoke(interp, sub, (void*) 1);
-    if (!dest) {
+    if (!dest)
         real_exception(interp, NULL, 1, "Subroutine returned a NULL address");
-    }
     ctx = CONTEXT(interp->ctx);
     offset = dest - interp->code->base.data;
     runops(interp, offset);
@@ -154,21 +159,19 @@ Parrot_runops_fromc(PARROT_INTERP, PMC *sub)
 }
 
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static parrot_context_t *
-runops_args(PARROT_INTERP, PMC *sub, PMC *obj,
-        STRING *meth, const char* sig, va_list ap)
+runops_args(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        SHIM(STRING *meth), NOTNULL(const char *sig), va_list ap)
 {
     opcode_t offset, *dest;
     parrot_context_t *ctx;
-    /*
-     * FIXME argument count limited - check strlen of sig
-     */
+
     char new_sig[10];
     const char *sig_p;
     parrot_context_t * const old_ctx = CONTEXT(interp->ctx);
 
-    UNUSED(meth); /* Maybe can be removed? */
-
     interp->current_cont  = new_ret_continuation_pmc(interp, NULL);
     interp->current_object = obj;
     dest = VTABLE_invoke(interp, sub, NULL);
@@ -255,15 +258,14 @@ Signatures are similar to NCI:
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 void *
-Parrot_run_meth_fromc(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth)
+Parrot_run_meth_fromc(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj), SHIM(STRING *meth))
 {
     parrot_context_t *ctx;
     opcode_t offset, *dest;
 
-    UNUSED(meth); /* Maybe we can remove? */
-
     interp->current_cont = new_ret_continuation_pmc(interp, NULL);
     interp->current_object = obj;
     dest = VTABLE_invoke(interp, sub, (void*)1);
@@ -276,9 +278,10 @@ Parrot_run_meth_fromc(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
-Parrot_runops_fromc_args(PARROT_INTERP, PMC *sub,
-        const char *sig, ...)
+Parrot_runops_fromc_args(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -300,23 +303,23 @@ didn't return properly.
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 void *
-Parrot_runops_fromc_args_event(PARROT_INTERP, PMC *sub,
-        const char *sig, ...)
+Parrot_runops_fromc_args_event(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
-    opcode_t *cargs, *params, *returns;
-    PMC *cont;
     void *retval;
     /*
      * running code from event handlers isn't fully reentrant due to
      * these interpreter variables - mainly related to calls
      */
-    cargs   = interp->current_args;
-    params  = interp->current_params;
-    returns = interp->current_returns;
-    cont    = interp->current_cont;
+    opcode_t * const cargs   = interp->current_args;
+    opcode_t * const params  = interp->current_params;
+    opcode_t * const returns = interp->current_returns;
+    PMC *      const cont    = interp->current_cont;
     /* what else ? */
 
     va_start(args, sig);
@@ -332,9 +335,10 @@ Parrot_runops_fromc_args_event(PARROT_INTERP, PMC *sub,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 INTVAL
-Parrot_runops_fromc_args_reti(PARROT_INTERP, PMC *sub,
-        const char *sig, ...)
+Parrot_runops_fromc_args_reti(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -346,9 +350,10 @@ Parrot_runops_fromc_args_reti(PARROT_INTERP, PMC *sub,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 FLOATVAL
-Parrot_runops_fromc_args_retf(PARROT_INTERP, PMC *sub,
-        const char *sig, ...)
+Parrot_runops_fromc_args_retf(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -360,9 +365,11 @@ Parrot_runops_fromc_args_retf(PARROT_INTERP, PMC *sub,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 void*
-Parrot_run_meth_fromc_args(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, ...)
+Parrot_run_meth_fromc_args(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -374,9 +381,10 @@ Parrot_run_meth_fromc_args(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 INTVAL
-Parrot_run_meth_fromc_args_reti(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, ...)
+Parrot_run_meth_fromc_args_reti(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -388,9 +396,10 @@ Parrot_run_meth_fromc_args_reti(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 FLOATVAL
-Parrot_run_meth_fromc_args_retf(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, ...)
+Parrot_run_meth_fromc_args_retf(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), ...)
 {
     va_list args;
     parrot_context_t *ctx;
@@ -402,42 +411,45 @@ Parrot_run_meth_fromc_args_retf(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 void *
-Parrot_runops_fromc_arglist(PARROT_INTERP, PMC *sub,
-        const char *sig, va_list args)
+Parrot_runops_fromc_arglist(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), va_list args)
 {
-    parrot_context_t *ctx;
+    parrot_context_t * const ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
 
-    ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
     return set_retval(interp, *sig, ctx);
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 INTVAL
-Parrot_runops_fromc_arglist_reti(PARROT_INTERP, PMC *sub,
-        const char *sig, va_list args)
+Parrot_runops_fromc_arglist_reti(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), va_list args)
 {
-    parrot_context_t *ctx;
+    parrot_context_t * const ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
 
-    ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
     return set_retval_i(interp, *sig, ctx);
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 FLOATVAL
-Parrot_runops_fromc_arglist_retf(PARROT_INTERP, PMC *sub,
-        const char *sig, va_list args)
+Parrot_runops_fromc_arglist_retf(PARROT_INTERP, NOTNULL(PMC *sub),
+        NOTNULL(const char *sig), va_list args)
 {
-    parrot_context_t *ctx;
+    parrot_context_t * const ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
 
-    ctx = runops_args(interp, sub, PMCNULL, NULL, sig, args);
     return set_retval_f(interp, *sig, ctx);
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 void*
-Parrot_run_meth_fromc_arglist(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, va_list args)
+Parrot_run_meth_fromc_arglist(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), va_list args)
 {
     parrot_context_t *ctx;
 
@@ -446,24 +458,24 @@ Parrot_run_meth_fromc_arglist(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 INTVAL
-Parrot_run_meth_fromc_arglist_reti(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, va_list args)
+Parrot_run_meth_fromc_arglist_reti(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), va_list args)
 {
-    parrot_context_t *ctx;
+    parrot_context_t * const ctx = runops_args(interp, sub, obj, meth, sig, args);
 
-    ctx = runops_args(interp, sub, obj, meth, sig, args);
     return set_retval_i(interp, *sig, ctx);
 }
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
 FLOATVAL
-Parrot_run_meth_fromc_arglist_retf(PARROT_INTERP,
-        PMC *sub, PMC *obj, STRING *meth, const char *sig, va_list args)
+Parrot_run_meth_fromc_arglist_retf(PARROT_INTERP, NOTNULL(PMC *sub), NOTNULL(PMC *obj),
+        NOTNULL(STRING *meth), NOTNULL(const char *sig), va_list args)
 {
-    parrot_context_t *ctx;
-
-    ctx = runops_args(interp, sub, obj, meth, sig, args);
+    parrot_context_t * const ctx = runops_args(interp, sub, obj, meth, sig, args);
     return set_retval_f(interp, *sig, ctx);
 }
 
