@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: interpreter.c 19905 2007-07-16 04:44:05Z petdance $
+$Id: interpreter.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -28,7 +28,6 @@ have the same number of elements because there is a one-to-one mapping.
 
 */
 
-#include <assert.h>
 #include "parrot/parrot.h"
 #include "interp_guts.h"
 #include "parrot/oplib/core_ops.h"
@@ -62,9 +61,8 @@ PARROT_CANNOT_RETURN_NULL
 static oplib_init_f get_op_lib_init( PARROT_INTERP,
     int core_op,
     int which,
-    NOTNULL(PMC *lib) )
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(4);
+    NULLOK(PMC *lib) )
+        __attribute__nonnull__(1);
 
 static void init_prederef( PARROT_INTERP, int which )
         __attribute__nonnull__(1);
@@ -79,23 +77,35 @@ static void notify_func_table( PARROT_INTERP,
         __attribute__nonnull__(2);
 
 static void prederef_args(
-    void **pc_prederef,
+    NOTNULL(void **pc_prederef),
     PARROT_INTERP,
     NOTNULL(opcode_t *pc),
     NOTNULL(const op_info_t *opinfo) )
+        __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
-static opcode_t * runops_cgp( PARROT_INTERP, opcode_t *pc )
-        __attribute__nonnull__(1);
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+static opcode_t * runops_cgp( PARROT_INTERP, NOTNULL(opcode_t *pc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
-static opcode_t * runops_exec( PARROT_INTERP, opcode_t *pc )
-        __attribute__nonnull__(1);
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
+static opcode_t * runops_exec( PARROT_INTERP, NOTNULL(opcode_t *pc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
-static opcode_t * runops_jit( PARROT_INTERP, opcode_t *pc )
-        __attribute__nonnull__(1);
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
+static opcode_t * runops_jit( PARROT_INTERP, NOTNULL(opcode_t *pc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * runops_switch( PARROT_INTERP, NOTNULL(opcode_t *pc) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -108,6 +118,10 @@ static void turn_ev_check( PARROT_INTERP, int on )
 
 /* HEADERIZER END: static */
 
+#if EXEC_CAPABLE
+    extern int Parrot_exec_run;
+#endif
+
 /*
 
 FUNCDOC: prederef_args
@@ -119,7 +133,7 @@ C<pc_prederef> is the current opcode.
 */
 
 static void
-prederef_args(void **pc_prederef, PARROT_INTERP,
+prederef_args(NOTNULL(void **pc_prederef), PARROT_INTERP,
         NOTNULL(opcode_t *pc), NOTNULL(const op_info_t *opinfo))
 {
     const PackFile_ConstTable * const const_table = interp->code->const_table;
@@ -324,7 +338,7 @@ C<ParrotLibrary> PMC.
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 static oplib_init_f
-get_op_lib_init(PARROT_INTERP, int core_op, int which, NOTNULL(PMC *lib))
+get_op_lib_init(PARROT_INTERP, int core_op, int which, NULLOK(PMC *lib))
 {
     if (core_op) {
         oplib_init_f init_func;
@@ -499,8 +513,10 @@ Initializes JIT function for the specified opcode and returns it.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 void *
-init_jit(PARROT_INTERP, opcode_t *pc)
+init_jit(PARROT_INTERP, NULLOK(opcode_t *pc))
 {
 #if JIT_CAPABLE
     opcode_t *code_start;
@@ -546,9 +562,10 @@ Prepares to run the interpreter's run core.
 void
 prepare_for_run(PARROT_INTERP)
 {
+    void *ignored;
     switch (interp->run_core) {
         case PARROT_JIT_CORE:
-            (void) init_jit(interp, interp->code->base.data);
+            ignored = init_jit(interp, interp->code->base.data);
             break;
         case PARROT_SWITCH_CORE:
         case PARROT_SWITCH_JIT_CORE:
@@ -573,8 +590,10 @@ Runs the JIT code for the specified opcode.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t *
-runops_jit(PARROT_INTERP, opcode_t *pc)
+runops_jit(PARROT_INTERP, NOTNULL(opcode_t *pc))
 {
 #if JIT_CAPABLE
 #  ifdef PARROT_EXEC_OS_AIX
@@ -590,7 +609,7 @@ runops_jit(PARROT_INTERP, opcode_t *pc)
 
     ((jit_f) D2FPTR(&ptrgl_t)) (interp, pc);
 #  else
-    jit_f jit_code = (jit_f) D2FPTR(init_jit(interp, pc));
+    jit_f jit_code = (jit_f)(init_jit(interp, pc));
     (jit_code) (interp, pc);
 #  endif
 #else
@@ -608,14 +627,15 @@ Runs the native executable version of the specified opcode.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t *
-runops_exec(PARROT_INTERP, opcode_t *pc)
+runops_exec(PARROT_INTERP, NOTNULL(opcode_t *pc))
 {
 #if EXEC_CAPABLE
     opcode_t *code_start;
     UINTVAL code_size;          /* in opcodes */
     opcode_t *code_end;
-    extern int Parrot_exec_run;
 
     code_start = interp->code->base.data;
     code_size = interp->code->base.size;
@@ -628,9 +648,10 @@ runops_exec(PARROT_INTERP, opcode_t *pc)
 #    endif
 #  endif
     if (Parrot_exec_run == 2) {
+        void *ignored;
         Parrot_exec_run = 0;
         Interp_core_SET(interp, PARROT_JIT_CORE);
-        runops_jit(interp, pc);
+        ignored = runops_jit(interp, pc);
         Interp_core_SET(interp, PARROT_EXEC_CORE);
     }
     else if (Parrot_exec_run == 1) {
@@ -655,11 +676,13 @@ Runs the C C<goto>, predereferenced core.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
-runops_cgp(PARROT_INTERP, opcode_t *pc)
+runops_cgp(PARROT_INTERP, NOTNULL(opcode_t *pc))
 {
 #ifdef HAVE_COMPUTED_GOTO
-    opcode_t *code_start = (opcode_t *)interp->code->base.data;
+    opcode_t * const code_start = (opcode_t *)interp->code->base.data;
     opcode_t *pc_prederef;
     init_prederef(interp, PARROT_CGP_CORE);
     pc_prederef = (opcode_t*)interp->code->prederef.code + (pc - code_start);
@@ -669,7 +692,6 @@ runops_cgp(PARROT_INTERP, opcode_t *pc)
     PIO_eprintf(interp,
             "Computed goto unavailable in this configuration.\n");
     Parrot_exit(interp, 1);
-    return NULL;
 #endif
 }
 
@@ -681,6 +703,8 @@ Runs the C<switch> core.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 runops_switch(PARROT_INTERP, NOTNULL(opcode_t *pc))
 {
@@ -707,8 +731,7 @@ void
 runops_int(PARROT_INTERP, size_t offset)
 {
     int lo_var_ptr;
-    opcode_t *(*core) (Interp *, opcode_t *) =
-        (opcode_t *(*) (Interp *, opcode_t *)) 0;
+    opcode_t *(*core) (PARROT_INTERP, opcode_t *) = NULL;
 
     if (!interp->lo_var_ptr) {
         /*
@@ -907,7 +930,7 @@ dynop_register(PARROT_INTERP, PMC* lib_pmc)
     n_tot = n_old + n_new;
     core = PARROT_CORE_OPLIB_INIT(1);
 
-    assert(interp->op_count == core->op_count);
+    PARROT_ASSERT(interp->op_count == core->op_count);
     new_evc_func_table = (op_func_t *)mem__sys_realloc(interp->evc_func_table,
             sizeof (op_func_t) * n_tot);
     if (core->flags & OP_FUNC_IS_ALLOCATED) {
@@ -1090,7 +1113,7 @@ notify_func_table(PARROT_INTERP, NOTNULL(op_func_t* table), int on)
         case PARROT_SLOW_CORE:      /* normal func core */
         case PARROT_FAST_CORE:      /* normal func core */
         case PARROT_CGOTO_CORE:      /* cgoto address list  */
-            assert(table);
+            PARROT_ASSERT(table);
             interp->op_func_table = table;
             break;
         case PARROT_CGP_CORE:
@@ -1119,7 +1142,7 @@ disable_event_checking(PARROT_INTERP)
     /*
      * restore func table
      */
-    assert(interp->save_func_table);
+    PARROT_ASSERT(interp->save_func_table);
     notify_func_table(interp, interp->save_func_table, 0);
 }
 
