@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io.c 19881 2007-07-14 22:05:00Z petdance $
+$Id: io.c 20694 2007-08-19 01:37:57Z chromatic $
 
 =head1 NAME
 
@@ -28,7 +28,6 @@ Parrot ops. The C<ParrotIO struct> is defined in F<src/io/io_private.h>.
 #include "io_private.h"
 
 #include <stdarg.h>
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/io.h */
 
@@ -58,6 +57,8 @@ Creates and returns a new C<ParrotIO> PMC.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 new_io_pmc(PARROT_INTERP, NULLOK(ParrotIO *io))
 {
@@ -98,6 +99,7 @@ allocate the STRING memory.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 PIO_make_io_string(PARROT_INTERP, NOTNULL(STRING **buf), size_t len)
 {
@@ -130,6 +132,7 @@ Currently C<iotype> is unused.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 ParrotIO *
 PIO_new(PARROT_INTERP, SHIM(INTVAL iotype), INTVAL flags, INTVAL mode)
 {
@@ -186,7 +189,7 @@ PIO_destroy(SHIM_INTERP, NOTNULL(PMC *pmc))
         for (p = io->stack; p;) {
             ParrotIOLayer *down;
             /* if top got copied, all have to be malloced */
-            assert(p->flags & PIO_L_LAYER_COPIED);
+            PARROT_ASSERT(p->flags & PIO_L_LAYER_COPIED);
             down = p->down;
             if (p->api->Delete)
                 (*p->api->Delete) (p);
@@ -246,7 +249,7 @@ PIO_init(PARROT_INTERP)
     if (interp->piodata == NULL)
         real_exception(interp, NULL, PIO_ERROR, "PIO alloc piodata failure.");
     interp->piodata->default_stack = NULL;
-    interp->piodata->table = mem_sys_allocate_zeroed(PIO_NR_OPEN * sizeof(ParrotIO *));
+    interp->piodata->table = (PMC **)mem_sys_allocate_zeroed(PIO_NR_OPEN * sizeof(ParrotIO *));
     if (interp->piodata->table == NULL)
         real_exception(interp, NULL, PIO_ERROR, "PIO alloc table failure.");
 
@@ -360,7 +363,7 @@ PIO_init_stacks(PARROT_INTERP)
     for (i = 0, p = interp->piodata->default_stack; p; p = p->down) {
         bottom = p;
         if (fill) {
-            assert(i < n); /* XXX n can be undefined at this point. */
+            PARROT_ASSERT(i < n); /* XXX n can be undefined at this point. */
             pio_registered_layers[i++] = p;
             pio_registered_layers[i] = NULL;
         }
@@ -384,8 +387,8 @@ PIO_init_stacks(PARROT_INTERP)
         }
     }
     if (fill) {
-        assert(i == 2);
-        assert(pio_registered_layers[2] == NULL);
+        PARROT_ASSERT(i == 2);
+        PARROT_ASSERT(pio_registered_layers[2] == NULL);
         pio_registered_layers[2] = PIO_utf8_register_layer();
         pio_registered_layers[3] = PIO_mmap_register_layer();
         pio_registered_layers[4] = PIO_string_register_layer();
@@ -627,6 +630,7 @@ Creates and returns a C<ParrotIO> PMC for C<*spath>.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_open(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(const char *spath),
          NOTNULL(const char *sflags))
@@ -665,6 +669,7 @@ OS IO handles (0,1,2).
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_fdopen(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), PIOHANDLE fd, NOTNULL(const char *sflags))
 {
@@ -727,12 +732,14 @@ PARROT_API
 void
 PIO_flush(PARROT_INTERP, NOTNULL(PMC *pmc))
 {
-    ParrotIOLayer * const l = (ParrotIOLayer *)PMC_struct_val(pmc);
-    ParrotIO * const io = (ParrotIO *)PMC_data0(pmc);
+    ParrotIOLayer * const l  = (ParrotIOLayer *)PMC_struct_val(pmc);
+    ParrotIO      * const io = (ParrotIO *)PMC_data0(pmc);
+    INTVAL                ignored;
+
     if (!io)
         return;
 
-    PIO_flush_down(interp, l, io);
+    ignored = PIO_flush_down(interp, l, io);
 }
 
 /*
@@ -750,28 +757,28 @@ Reads up to C<len> bytes from C<*pmc> and copies them into C<*buffer>.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 PIO_reads(PARROT_INTERP, NOTNULL(PMC *pmc), size_t len)
 {
-    STRING *res;
-    ParrotIOLayer * const l = (ParrotIOLayer *)PMC_struct_val(pmc);
-    ParrotIO * const io = (ParrotIO *)PMC_data0(pmc);
+    STRING               *res = NULL;
+    ParrotIOLayer * const l   = (ParrotIOLayer *)PMC_struct_val(pmc);
+    ParrotIO      * const io  = (ParrotIO *)PMC_data0(pmc);
+    INTVAL                ignored;
 
     if (!io)
         return new_string_header(interp, 0);
 
     if (io->b.flags & PIO_BF_MMAP) {
-        res = new_string_header(interp, 0);
-        res->charset = Parrot_iso_8859_1_charset_ptr;   /* XXX binary */
+        res           = new_string_header(interp, 0);
+        res->charset  = Parrot_iso_8859_1_charset_ptr;   /* XXX binary */
         res->encoding = Parrot_fixed_8_encoding_ptr;
     }
-    else {
-        res = NULL;
+    else
         res = PIO_make_io_string(interp, &res, len);
-    }
 
     res->bufused = len;
-    PIO_read_down(interp, l, io, &res);
+    ignored      = PIO_read_down(interp, l, io, &res);
 
     return res;
 }
@@ -923,8 +930,8 @@ PIO_putps(PARROT_INTERP, NOTNULL(PMC *pmc), NULLOK(STRING *s))
 {
     ParrotIOLayer * const l = (ParrotIOLayer *)PMC_struct_val(pmc);
     ParrotIO * const io = (ParrotIO *)PMC_data0(pmc);
-    assert((unsigned long)l != 0xdeadbeefUL);
-    assert(io != 0);
+    PARROT_ASSERT((unsigned long)l != 0xdeadbeefUL);
+    PARROT_ASSERT(io);
 
     if (!s)
         return 0;
@@ -1063,6 +1070,8 @@ Returns the C<ParrotIO> PMC for C<stdin>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_STDIN(PARROT_INTERP)
 {
@@ -1078,6 +1087,8 @@ Returns the C<ParrotIO> PMC for C<stdout>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_STDOUT(PARROT_INTERP)
 {
@@ -1093,6 +1104,8 @@ Returns the C<ParrotIO> PMC for C<stderr>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_STDERR(PARROT_INTERP)
 {
@@ -1168,7 +1181,7 @@ Returns the return value of the C<get_integer> vtable method on C<*pmc>.
 */
 
 PIOOFF_T
-PIO_make_offset_pmc(PARROT_INTERP, PMC *pmc)
+PIO_make_offset_pmc(PARROT_INTERP, NOTNULL(PMC *pmc))
 {
     /* XXX: Maybe use bignums here */
     return VTABLE_get_integer(interp, pmc);
@@ -1212,6 +1225,8 @@ the socket was successfully created.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 PIO_socket(PARROT_INTERP, INTVAL fam, INTVAL type, INTVAL proto)
 {
@@ -1340,6 +1355,8 @@ Returns C<NULL> on failure.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
 PIO_accept(PARROT_INTERP, NOTNULL(PMC *pmc))
 {
@@ -1400,8 +1417,6 @@ IO stream.
 
 Add support for loadable layers in Parrot bytecode.
 
-=cut
-
 */
 
 
