@@ -1,6 +1,6 @@
 /*
-Copyright (C) 2001-2006, The Perl Foundation.
-$Id: io_private.h 19882 2007-07-14 22:33:30Z petdance $
+Copyright (C) 2001-2007, The Perl Foundation.
+$Id: io_private.h 20085 2007-07-22 04:37:48Z petdance $
 
 =head1 NAME
 
@@ -143,38 +143,50 @@ struct _ParrotIOData {
 /* HEADERIZER BEGIN: src/io/io_passdown.c */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO * PIO_accept_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_bind_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
-    STRING *address )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING *address) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_close_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_connect_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
-    STRING *address )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING *address) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_eof_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO * PIO_fdopen_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
     PIOHANDLE fd,
@@ -184,27 +196,33 @@ ParrotIO * PIO_fdopen_down( PARROT_INTERP,
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_flush_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_listen_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
     INTVAL backlog )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO * PIO_open_async_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
     NOTNULL(const char *name),
     NOTNULL(const char *mode),
-    DummyCodeRef * dummy )
+    NOTNULL(DummyCodeRef *dummy) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3)
-        __attribute__nonnull__(4);
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO * PIO_open_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
     NOTNULL(const char *name),
@@ -215,97 +233,123 @@ ParrotIO * PIO_open_down( PARROT_INTERP,
 PARROT_WARN_UNUSED_RESULT
 size_t PIO_peek_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
-    STRING ** buf )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING **buf) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_poll_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
     INTVAL which,
     INTVAL sec,
     INTVAL usec )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 size_t PIO_read_async_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
-    STRING ** buf,
-    DummyCodeRef *dummy )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING **buf),
+    NOTNULL(DummyCodeRef *dummy) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5);
 
 PARROT_WARN_UNUSED_RESULT
 size_t PIO_read_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
-    STRING ** buf )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING **buf) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_recv_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
-    STRING **buf )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING **buf) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 PIOOFF_T PIO_seek_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
+    NOTNULL(ParrotIO *io),
     PIOOFF_T offset,
     INTVAL whence )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_send_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
-    ParrotIO *io,
-    STRING *buf )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIOLayer *layer),
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING *buf) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_setbuf_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
+    NOTNULL(ParrotIO *io),
     size_t bufsize )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL PIO_setlinebuf_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO * PIO_socket_down( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    NOTNULL(ParrotIOLayer *layer),
     INTVAL fam,
     INTVAL type,
     INTVAL proto )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
 PIOOFF_T PIO_tell_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
 size_t PIO_write_async_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
-    STRING *s,
-    DummyCodeRef *dummy )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING *s),
+    NOTNULL(DummyCodeRef *dummy) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5);
 
 PARROT_WARN_UNUSED_RESULT
 size_t PIO_write_down( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
-    ParrotIO * io,
-    STRING *s )
-        __attribute__nonnull__(1);
+    NOTNULL(ParrotIO *io),
+    NOTNULL(STRING *s) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 /* HEADERIZER END: src/io/io_passdown.c */
 
@@ -316,85 +360,85 @@ size_t PIO_write_down( PARROT_INTERP,
  * the specific call.
  */
 struct _ParrotIOLayerAPI {
-    INTVAL          (*Init)(Interp *, ParrotIOLayer * l);
+    INTVAL          (*Init)(PARROT_INTERP, ParrotIOLayer * l);
     ParrotIOLayer * (*New)(ParrotIOLayer * proto);
     void            (*Delete)(ParrotIOLayer * l);
     INTVAL          (*Pushed)(ParrotIOLayer * l, ParrotIO * io);
     INTVAL          (*Popped)(ParrotIOLayer * l, ParrotIO * io);
-    ParrotIO *      (*Open)(Interp *, ParrotIOLayer * l,
+    ParrotIO *      (*Open)(PARROT_INTERP, ParrotIOLayer * l,
                             const char * name, INTVAL flags);
-    ParrotIO *      (*Open2_Unused)(Interp *);
-    ParrotIO *      (*Open3_Unused)(Interp *);
-    ParrotIO *      (*Open_ASync)(Interp *, ParrotIOLayer * l,
+    ParrotIO *      (*Open2_Unused)(PARROT_INTERP);
+    ParrotIO *      (*Open3_Unused)(PARROT_INTERP);
+    ParrotIO *      (*Open_ASync)(PARROT_INTERP, ParrotIOLayer * l,
                                   const char * name, const char * mode,
                                   DummyCodeRef *);
-    ParrotIO *      (*FDOpen)(Interp *, ParrotIOLayer * l,
+    ParrotIO *      (*FDOpen)(PARROT_INTERP, ParrotIOLayer * l,
                               PIOHANDLE fd, INTVAL flags);
-    INTVAL          (*Close)(Interp *, ParrotIOLayer * l,
+    INTVAL          (*Close)(PARROT_INTERP, ParrotIOLayer * l,
                                 ParrotIO * io);
-    size_t          (*Write)(Interp *, ParrotIOLayer * l,
+    size_t          (*Write)(PARROT_INTERP, ParrotIOLayer * l,
                              ParrotIO * io, STRING *);
-    size_t          (*Write_ASync)(Interp *, ParrotIOLayer * layer,
+    size_t          (*Write_ASync)(PARROT_INTERP, ParrotIOLayer * layer,
                                    ParrotIO * io, STRING *, DummyCodeRef *);
-    size_t          (*Read)(Interp *, ParrotIOLayer * layer,
+    size_t          (*Read)(PARROT_INTERP, ParrotIOLayer * layer,
                             ParrotIO * io, STRING **);
-    size_t          (*Read_ASync)(Interp *, ParrotIOLayer * layer,
+    size_t          (*Read_ASync)(PARROT_INTERP, ParrotIOLayer * layer,
                                   ParrotIO * io, STRING **, DummyCodeRef *);
-    INTVAL          (*Flush)(Interp *, ParrotIOLayer * layer,
+    INTVAL          (*Flush)(PARROT_INTERP, ParrotIOLayer * layer,
                              ParrotIO * io);
-    size_t          (*Peek)(Interp *, ParrotIOLayer * layer,
+    size_t          (*Peek)(PARROT_INTERP, ParrotIOLayer * layer,
                             ParrotIO * io, STRING ** buf);
-    PIOOFF_T        (*Seek)(Interp *, ParrotIOLayer * layer,
+    PIOOFF_T        (*Seek)(PARROT_INTERP, ParrotIOLayer * layer,
                             ParrotIO * io, PIOOFF_T offset, INTVAL whence);
-    PIOOFF_T        (*Tell)(Interp *, ParrotIOLayer * layer,
+    PIOOFF_T        (*Tell)(PARROT_INTERP, ParrotIOLayer * layer,
                             ParrotIO * io);
-    INTVAL          (*SetBuf)(Interp *, ParrotIOLayer * layer,
+    INTVAL          (*SetBuf)(PARROT_INTERP, ParrotIOLayer * layer,
                               ParrotIO * io, size_t bufsize);
-    INTVAL          (*SetLineBuf)(Interp *, ParrotIOLayer * layer,
+    INTVAL          (*SetLineBuf)(PARROT_INTERP, ParrotIOLayer * layer,
                                   ParrotIO * io);
-    INTVAL          (*GetCount)(Interp *, ParrotIOLayer * layer);
-    INTVAL          (*Fill)(Interp *, ParrotIOLayer * layer);
-    INTVAL          (*Eof)(Interp *, ParrotIOLayer * l,
+    INTVAL          (*GetCount)(PARROT_INTERP, ParrotIOLayer * layer);
+    INTVAL          (*Fill)(PARROT_INTERP, ParrotIOLayer * layer);
+    INTVAL          (*Eof)(PARROT_INTERP, ParrotIOLayer * l,
                            ParrotIO * io);
     /* Network API */
-    INTVAL          (*Poll)(Interp *, ParrotIOLayer *l, ParrotIO *io,
+    INTVAL          (*Poll)(PARROT_INTERP, ParrotIOLayer *l, ParrotIO *io,
                             int which, int sec, int usec);
-    ParrotIO *      (*Socket)(Interp *, ParrotIOLayer *,
+    ParrotIO *      (*Socket)(PARROT_INTERP, ParrotIOLayer *,
                             int dom, int type, int proto);
-    INTVAL          (*Connect)(Interp *, ParrotIOLayer *, ParrotIO *,
+    INTVAL          (*Connect)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *,
                             STRING *);
-    INTVAL          (*Send)(Interp *, ParrotIOLayer *, ParrotIO *, STRING *);
-    INTVAL          (*Recv)(Interp *, ParrotIOLayer *, ParrotIO *, STRING **);
-    INTVAL          (*Bind)(Interp *, ParrotIOLayer *, ParrotIO *, STRING *);
-    INTVAL          (*Listen)(Interp *, ParrotIOLayer *, ParrotIO *, INTVAL);
-    ParrotIO *      (*Accept)(Interp *, ParrotIOLayer *, ParrotIO *);
+    INTVAL          (*Send)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING *);
+    INTVAL          (*Recv)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING **);
+    INTVAL          (*Bind)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING *);
+    INTVAL          (*Listen)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, INTVAL);
+    ParrotIO *      (*Accept)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *);
 };
 
 /* these are defined rather than using NULL because strictly-speaking, ANSI C
  * doesn't like conversions between function and non-function pointers. */
-#define PIO_null_init (INTVAL (*)(Interp *, ParrotIOLayer *))0
+#define PIO_null_init (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *))0
 #define PIO_null_push_layer (INTVAL (*)(ParrotIOLayer *, ParrotIO *))0
 #define PIO_null_pop_layer (INTVAL (*)(ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_open (ParrotIO * (*)(Interp *, ParrotIOLayer *, const char*, INTVAL))0
-#define PIO_null_open2 (ParrotIO * (*)(Interp *))0
-#define PIO_null_open3 (ParrotIO * (*)(Interp *))0
-#define PIO_null_open_async (ParrotIO * (*)(Interp *, ParrotIOLayer *, const char *, const char *, DummyCodeRef *))0
-#define PIO_null_fdopen (ParrotIO * (*)(Interp *, ParrotIOLayer *, PIOHANDLE, INTVAL))0
-#define PIO_null_close (INTVAL (*)(Interp *, ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_write (size_t (*)(Interp *, ParrotIOLayer *, ParrotIO *, STRING*))0
-#define PIO_null_write_async (size_t (*)(Interp *, ParrotIOLayer *, ParrotIO *, STRING *,DummyCodeRef *))0
-#define PIO_null_read (size_t (*)(Interp *, ParrotIOLayer *, ParrotIO *, STRING**))0
-#define PIO_null_read_async (size_t (*)(Interp *, ParrotIOLayer *, ParrotIO *, STRING **, DummyCodeRef *))0
-#define PIO_null_flush (INTVAL (*)(Interp *, ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_peek (size_t (*)(Interp *, ParrotIOLayer *, ParrotIO *, STRING**))0
-#define PIO_null_seek (PIOOFF_T (*)(Interp *, ParrotIOLayer *, ParrotIO *, PIOOFF_T, INTVAL))0
-#define PIO_null_tell (PIOOFF_T (*)(Interp *, ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_setbuf (INTVAL (*)(Interp *, ParrotIOLayer *, ParrotIO *, size_t))0
-#define PIO_null_setlinebuf (INTVAL (*)(Interp *, ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_getcount (INTVAL (*)(Interp *, ParrotIOLayer *))0
-#define PIO_null_fill (INTVAL (*)(Interp *, ParrotIOLayer *))0
-#define PIO_null_eof (INTVAL (*)(Interp *, ParrotIOLayer *, ParrotIO *))0
-#define PIO_null_socket (ParrotIO * (*)(Interp *, ParrotIOLayer *, int, int, int))0
+#define PIO_null_open (ParrotIO * (*)(PARROT_INTERP, ParrotIOLayer *, const char*, INTVAL))0
+#define PIO_null_open2 (ParrotIO * (*)(PARROT_INTERP))0
+#define PIO_null_open3 (ParrotIO * (*)(PARROT_INTERP))0
+#define PIO_null_open_async (ParrotIO * (*)(PARROT_INTERP, ParrotIOLayer *, const char *, const char *, DummyCodeRef *))0
+#define PIO_null_fdopen (ParrotIO * (*)(PARROT_INTERP, ParrotIOLayer *, PIOHANDLE, INTVAL))0
+#define PIO_null_close (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *))0
+#define PIO_null_write (size_t (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING*))0
+#define PIO_null_write_async (size_t (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING *,DummyCodeRef *))0
+#define PIO_null_read (size_t (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING**))0
+#define PIO_null_read_async (size_t (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING **, DummyCodeRef *))0
+#define PIO_null_flush (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *))0
+#define PIO_null_peek (size_t (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, STRING**))0
+#define PIO_null_seek (PIOOFF_T (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, PIOOFF_T, INTVAL))0
+#define PIO_null_tell (PIOOFF_T (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *))0
+#define PIO_null_setbuf (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *, size_t))0
+#define PIO_null_setlinebuf (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *))0
+#define PIO_null_getcount (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *))0
+#define PIO_null_fill (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *))0
+#define PIO_null_eof (INTVAL (*)(PARROT_INTERP, ParrotIOLayer *, ParrotIO *))0
+#define PIO_null_socket (ParrotIO * (*)(PARROT_INTERP, ParrotIOLayer *, int, int, int))0
 
 /*
  * more API XXX should be in io.h when things settle
