@@ -1,6 +1,6 @@
 /*
 Copyright (c) 2006-2007, The Perl Foundation.
-$Id: io_string.c 19915 2007-07-16 22:08:22Z petdance $
+$Id: io_string.c 20342 2007-07-30 05:57:58Z petdance $
 
 =head1 NAME
 
@@ -23,14 +23,14 @@ Capture output to a string PMC.
 
 static size_t PIO_string_read( SHIM_INTERP,
     NOTNULL(ParrotIOLayer *l),
-    ParrotIO *io,
+    SHIM(ParrotIO *io),
     NOTNULL(STRING **buf) )
         __attribute__nonnull__(2)
         __attribute__nonnull__(4);
 
 static size_t PIO_string_write( PARROT_INTERP,
     NOTNULL(ParrotIOLayer *l),
-    ParrotIO *io,
+    SHIM(ParrotIO *io),
     NOTNULL(STRING *s) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -81,6 +81,8 @@ ParrotIOLayer pio_string_layer = {
     NULL, NULL
 };
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 ParrotIOLayer *
 PIO_string_register_layer(void)
 {
