@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io_unix.c 19915 2007-07-16 22:08:22Z petdance $
+$Id: io_unix.c 20694 2007-08-19 01:37:57Z chromatic $
 
 =head1 NAME
 
@@ -46,13 +46,15 @@ ParrotIOLayer pio_unix_layer = {
 PARROT_CONST_FUNCTION
 static INTVAL flags_to_unix( INTVAL flags );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO * PIO_unix_accept( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
-static INTVAL PIO_unix_async( PARROT_INTERP,
+INTVAL PIO_unix_async( PARROT_INTERP,
     NOTNULL(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     INTVAL b )
@@ -61,31 +63,33 @@ static INTVAL PIO_unix_async( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 static INTVAL PIO_unix_bind( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING *l) )
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
 static INTVAL PIO_unix_close( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
 static INTVAL PIO_unix_connect( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NULLOK(STRING *r) )
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO * PIO_unix_fdopen( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     PIOHANDLE fd,
     INTVAL flags )
         __attribute__nonnull__(1);
 
 static INTVAL PIO_unix_flush( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
@@ -95,11 +99,13 @@ static INTVAL PIO_unix_init( PARROT_INTERP, NOTNULL(ParrotIOLayer *layer) )
 
 static INTVAL PIO_unix_isatty( PIOHANDLE fd );
 static INTVAL PIO_unix_listen( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     INTVAL sec )
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO * PIO_unix_open( PARROT_INTERP,
     NOTNULL(ParrotIOLayer *layer),
     NOTNULL(const char *spath),
@@ -108,15 +114,17 @@ static ParrotIO * PIO_unix_open( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO * PIO_unix_pipe( PARROT_INTERP,
-    ParrotIOLayer *l,
+    SHIM(ParrotIOLayer *l),
     NOTNULL(const char *cmd),
     int flags )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
 static INTVAL PIO_unix_poll( SHIM_INTERP,
-    ParrotIOLayer *l,
+    SHIM(ParrotIOLayer *l),
     NOTNULL(ParrotIO *io),
     int which,
     int sec,
@@ -124,7 +132,7 @@ static INTVAL PIO_unix_poll( SHIM_INTERP,
         __attribute__nonnull__(3);
 
 static size_t PIO_unix_read( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING **buf) )
         __attribute__nonnull__(1)
@@ -132,7 +140,7 @@ static size_t PIO_unix_read( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 static INTVAL PIO_unix_recv( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING **s) )
         __attribute__nonnull__(1)
@@ -140,33 +148,35 @@ static INTVAL PIO_unix_recv( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 static PIOOFF_T PIO_unix_seek( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     PIOOFF_T offset,
     INTVAL whence )
         __attribute__nonnull__(3);
 
 static INTVAL PIO_unix_send( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING *s) )
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO * PIO_unix_socket( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     int fam,
     int type,
     int proto )
         __attribute__nonnull__(1);
 
 static PIOOFF_T PIO_unix_tell( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
 static size_t PIO_unix_write( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING *s) )
         __attribute__nonnull__(3)
@@ -255,6 +265,8 @@ values.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO *
 PIO_unix_open(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer),
               NOTNULL(const char *spath), INTVAL flags)
@@ -353,7 +365,7 @@ Toggles the C<O_ASYNC> flag on the IO file descriptor.
 
 */
 
-static INTVAL
+INTVAL
 PIO_unix_async(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io), INTVAL b)
 {
     int rflags;
@@ -381,6 +393,8 @@ Returns a new C<ParrotIO> with file descriptor C<fd>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO *
 PIO_unix_fdopen(PARROT_INTERP, SHIM(ParrotIOLayer *layer), PIOHANDLE fd, INTVAL flags)
 {
@@ -677,6 +691,8 @@ C<inet_aton()>, etc.) and take this out of platform specific compilation
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 PIO_sockaddr_in(PARROT_INTERP, unsigned short port, NOTNULL(STRING *addr))
 {
@@ -684,7 +700,7 @@ PIO_sockaddr_in(PARROT_INTERP, unsigned short port, NOTNULL(STRING *addr))
     /* Hard coded to IPv4 for now */
     int family = AF_INET;
 
-    char * s = string_to_cstring(interp, addr);
+    char * const s = string_to_cstring(interp, addr);
     /*
      * due to a bug in OS/X, we've to zero the struct
      * else bind is failing erratically
@@ -733,6 +749,8 @@ socket type and protocol number.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO *
 PIO_unix_socket(PARROT_INTERP, SHIM(ParrotIOLayer *layer), int fam, int type, int proto)
 {
@@ -745,7 +763,7 @@ PIO_unix_socket(PARROT_INTERP, SHIM(ParrotIOLayer *layer), int fam, int type, in
         io->remote.sin_family = fam;
         return io;
     }
-    return 0;
+    return NULL;
 }
 
 /*
@@ -832,6 +850,8 @@ Accept a new connection and return a newly created C<ParrotIO> socket.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO *
 PIO_unix_accept(PARROT_INTERP, SHIM(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
@@ -1016,6 +1036,8 @@ XXX: Where does this fit, should it belong in the ParrotIOLayerAPI?
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static ParrotIO *
 PIO_unix_pipe(PARROT_INTERP, SHIM(ParrotIOLayer *l), NOTNULL(const char *cmd), int flags)
 {
@@ -1118,7 +1140,7 @@ const ParrotIOLayerAPI pio_unix_layer_api = {
     PIO_unix_read,
     PIO_null_read_async,
     PIO_unix_flush,
-    0, /* no peek */
+    NULL, /* no peek */
     PIO_unix_seek,
     PIO_unix_tell,
     PIO_null_setbuf,
