@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2006, The Perl Foundation.
-$Id: jit.c 19833 2007-07-13 22:02:25Z petdance $
+$Id: jit.c 20350 2007-07-30 15:15:17Z particle $
 
 =head1 NAME
 
@@ -22,6 +22,9 @@ used and not per subroutine or even opcode, it works per bytecode segment.
 
 */
 
+/* HEADERIZER HFILE: none */
+/* HEADERIZER STOP */
+
 #include <parrot/parrot.h>
 #include <assert.h>
 #if PARROT_EXEC_CAPABLE
