@@ -3,7 +3,7 @@
  *
  * ALPHA
  *
- * $Id: jit_emit.h 18945 2007-06-12 14:08:35Z fperrad $
+ * $Id: jit_emit.h 19833 2007-07-13 22:02:25Z petdance $
  */
 
 /*  Register usage:
@@ -143,7 +143,7 @@ enum { JIT_ALPHABRANCH, JIT_ALPHABSR };
 
 #  define emit_l_s_r(pc, opcode, Ra, Rb, Parrot_reg) \
       emit_mem(pc, opcode, Ra, Rb, \
-        (((char *)Parrot_reg) - (char *)&REG_INT(0)))
+        (((char *)Parrot_reg) - (char *)&REG_INT(interp, 0)))
 
 #  define emit_ldq_b(pc, Ra, addr, Rb) \
       emit_mem(pc, LDQ, Ra, Rb, addr)
@@ -248,7 +248,7 @@ enum { JIT_ALPHABRANCH, JIT_ALPHABSR };
 
 static void
 emit_load_intval_cpool(Parrot_jit_info_t *jit_info,
-    Interp *interp, alpha_register_t dest, INTVAL constant)
+    PARROT_INTERP, alpha_register_t dest, INTVAL constant)
 {
     char *new_arena;
 
@@ -314,7 +314,7 @@ calculate_displacement(long *s, long *d, long *high, long *low)
 }
 
 static char *
-emit_l_c(Parrot_jit_info_t *jit_info, Interp *interp,
+emit_l_c(Parrot_jit_info_t *jit_info, PARROT_INTERP,
     alpha_register_t target, long constant)
 {
     char *pc = jit_info->native_ptr;
@@ -361,7 +361,7 @@ emit_b(Parrot_jit_info_t *jit_info, branch_t opcode, alpha_register_t reg,
 
 void
 Parrot_jit_begin(Parrot_jit_info_t *jit_info,
-    Interp *interp)
+    PARROT_INTERP)
 {
     emit_lda_b(jit_info->native_ptr, REG30_sp, -16, REG30_sp);
     emit_stq_b(jit_info->native_ptr, REG26_ra, 0, REG30_sp);
@@ -379,7 +379,7 @@ Parrot_jit_begin(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
-    Interp *interp)
+    PARROT_INTERP)
 {
     Parrot_jit_fixup_t *fixup;
     char *fixup_ptr;
@@ -418,7 +418,7 @@ Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
 
 static char *
 emit_bsr(Parrot_jit_info_t *jit_info,
-    Interp *interp)
+    PARROT_INTERP)
 {
     char *pc = jit_info->native_ptr;
 
@@ -455,7 +455,7 @@ emit_bsr(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
-    Interp *interp)
+    PARROT_INTERP)
 {
     jit_emit_mov_ri_i(jit_info->native_ptr, REG16_a0, jit_info->cur_op);
     jit_emit_mov_rr(jit_info->native_ptr, REG9_s0, REG17_a1);
@@ -464,7 +464,7 @@ Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
-    Interp *interp)
+    PARROT_INTERP)
 {
     Parrot_jit_normal_op(jit_info,interp);
 
@@ -478,7 +478,7 @@ Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
 
 /* move reg to mem (i.e. intreg) */
 void
-Parrot_jit_emit_mov_mr(Interp *interp, char *mem, int reg)
+Parrot_jit_emit_mov_mr(PARROT_INTERP, char *mem, int reg)
 {
     jit_emit_mov_mr_i(
         ((Parrot_jit_info_t *)(interp->code->jit_info))->native_ptr, mem, reg);
@@ -486,7 +486,7 @@ Parrot_jit_emit_mov_mr(Interp *interp, char *mem, int reg)
 
 /* move mem (i.e. intreg) to reg */
 void
-Parrot_jit_emit_mov_rm(Interp *interp, int reg, char *mem)
+Parrot_jit_emit_mov_rm(PARROT_INTERP, int reg, char *mem)
 {
     jit_emit_mov_rm_i(
         ((Parrot_jit_info_t *)(interp->code->jit_info))->native_ptr, reg, mem);
@@ -494,13 +494,13 @@ Parrot_jit_emit_mov_rm(Interp *interp, int reg, char *mem)
 
 /* move reg to mem (i.e. numreg) */
 void
-Parrot_jit_emit_mov_mr_n(Interp *interp, char *mem, int reg)
+Parrot_jit_emit_mov_mr_n(PARROT_INTERP, char *mem, int reg)
 {
 }
 
 /* move mem (i.e. numreg) to reg */
 void
-Parrot_jit_emit_mov_rm_n(Interp *interp, int reg, char *mem)
+Parrot_jit_emit_mov_rm_n(PARROT_INTERP, int reg, char *mem)
 {
 }
 
