@@ -3,7 +3,7 @@
  *
  * ARM (I think this is all ARM2 or later, although it is APCS-32)
  *
- * $Id: jit_emit.h 18945 2007-06-12 14:08:35Z fperrad $
+ * $Id: jit_emit.h 19833 2007-07-13 22:02:25Z petdance $
  */
 
 #ifndef PARROT_ARM_JIT_EMIT_H_GUARD
@@ -609,7 +609,7 @@ static void emit_jump_to_op(Parrot_jit_info_t *jit_info, arm_cond_t cond,
 
 static char *
 emit_load_constant_from_pool(char *pc,
-                              Interp *interp,
+                              PARROT_INTERP,
                               arm_cond_t cond,
                               int value,
                               arm_register_t hwreg)
@@ -635,7 +635,7 @@ emit_load_constant_from_pool(char *pc,
 
 static char *
 emit_load_constant(char *pc,
-                    Interp *interp,
+                    PARROT_INTERP,
                     arm_cond_t cond,
                     int value,
                     arm_register_t hwreg)
@@ -660,7 +660,7 @@ emit_load_constant(char *pc,
 
 static void
 Parrot_jit_int_load(Parrot_jit_info_t *jit_info,
-                    Interp *interp,
+                    PARROT_INTERP,
                     arm_cond_t cond,
                     int param,
                     arm_register_t hwreg)
@@ -675,7 +675,7 @@ Parrot_jit_int_load(Parrot_jit_info_t *jit_info,
             offset = ((char *)&interp->int_reg.registers[val])
                 - (char *)interp;
             if (offset > 4095) {
-                internal_exception(JIT_ERROR,
+                real_exception(interp, NULL, JIT_ERROR,
                                    "integer load register %d generates offset %d, larger than 4095\n",
                            val, offset);
             }
@@ -696,7 +696,7 @@ Parrot_jit_int_load(Parrot_jit_info_t *jit_info,
                                                        hwreg);
             break;
         default:
-            internal_exception(JIT_ERROR,
+            real_exception(interp, NULL, JIT_ERROR,
                                "Unsupported op parameter type %d in jit_int_load\n",
                                op_type);
     }
@@ -704,7 +704,7 @@ Parrot_jit_int_load(Parrot_jit_info_t *jit_info,
 
 static void
 Parrot_jit_int_store(Parrot_jit_info_t *jit_info,
-                     Interp *interp,
+                     PARROT_INTERP,
                      arm_cond_t cond,
                      int param,
                      arm_register_t hwreg)
@@ -719,7 +719,7 @@ Parrot_jit_int_store(Parrot_jit_info_t *jit_info,
             offset = ((char *)&interp->int_reg.registers[val])
                 - (char *)interp;
             if (offset > 4095) {
-                internal_exception(JIT_ERROR,
+                real_exception(interp, NULL, JIT_ERROR,
                                    "integer store register %d generates offset %d, larger than 4095\n",
                            val, offset);
             }
@@ -735,7 +735,7 @@ Parrot_jit_int_store(Parrot_jit_info_t *jit_info,
 
         case PARROT_ARG_N:
         default:
-            internal_exception(JIT_ERROR,
+            real_exception(interp, NULL, JIT_ERROR,
                             "Unsupported op parameter type %d in jit_int_store\n",
                                op_type);
     }
@@ -743,7 +743,7 @@ Parrot_jit_int_store(Parrot_jit_info_t *jit_info,
 
 static void
 Parrot_jit_arith_const_alternate(Parrot_jit_info_t *jit_info,
-                                  Interp *interp,
+                                  PARROT_INTERP,
                                   arm_cond_t cond,
                                   enum constant_state alternate_on,
                                   alu_op_t normal, alu_op_t alternative,
@@ -804,7 +804,7 @@ Parrot_jit_arith_const_alternate(Parrot_jit_info_t *jit_info,
 */
 static void
 Parrot_jit_jumpif_const(Parrot_jit_info_t *jit_info,
-                         Interp *interp,
+                         PARROT_INTERP,
                          int src, int const_val, int where_to,
                          arm_cond_t when)
 {
@@ -859,7 +859,7 @@ Parrot_jump_to_op_in_reg(Parrot_jit_info_t *jit_info,
 #if JIT_EMIT == 2
 
 void Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
-                        Interp *interp)
+                        PARROT_INTERP)
 {
     Parrot_jit_fixup_t *fixup = jit_info->arena.fixups;
 
@@ -877,7 +877,7 @@ void Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
                 break;
             }
             default:
-                internal_exception(JIT_ERROR, "Unknown fixup type:%d\n",
+                real_exception(interp, NULL, JIT_ERROR, "Unknown fixup type:%d\n",
                                    fixup->type);
                 break;
         }
@@ -894,7 +894,7 @@ void Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_begin(Parrot_jit_info_t *jit_info,
-                 Interp *interp)
+                 PARROT_INTERP)
 {
     jit_info->native_ptr = emit_mov(jit_info->native_ptr, REG12_ip, REG13_sp);
     jit_info->native_ptr = emit_ldmstm(jit_info->native_ptr,
@@ -949,7 +949,7 @@ need to adr beyond:
 */
 void
 Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
-                     Interp *interp)
+                     PARROT_INTERP)
 {
     jit_info->native_ptr = emit_mov(jit_info->native_ptr, r1, r4);
 #  ifndef ARM_K_BUG
@@ -981,7 +981,7 @@ Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
    We want address of equivalent bit of jit code, which is stored as an
    address at the same offset in a jit table. */
 void Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
-                        Interp *interp)
+                        PARROT_INTERP)
 {
     Parrot_jit_normal_op(jit_info, interp);
     Parrot_jump_to_op_in_reg(jit_info, interp, r0);
@@ -989,25 +989,25 @@ void Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
 
 /* move reg to mem (i.e. intreg) */
 void
-Parrot_jit_emit_mov_mr(Interp *interp, char *mem, int reg)
+Parrot_jit_emit_mov_mr(PARROT_INTERP, char *mem, int reg)
 {
 }
 
 /* move mem (i.e. intreg) to reg */
 void
-Parrot_jit_emit_mov_rm(Interp *interp, int reg, char *mem)
+Parrot_jit_emit_mov_rm(PARROT_INTERP, int reg, char *mem)
 {
 }
 
 /* move reg to mem (i.e. numreg) */
 void
-Parrot_jit_emit_mov_mr_n(Interp *interp, char *mem, int reg)
+Parrot_jit_emit_mov_mr_n(PARROT_INTERP, char *mem, int reg)
 {
 }
 
 /* move mem (i.e. numreg) to reg */
 void
-Parrot_jit_emit_mov_rm_n(Interp *interp, int reg, char *mem)
+Parrot_jit_emit_mov_rm_n(PARROT_INTERP, int reg, char *mem)
 {
 }
 
