@@ -3,7 +3,7 @@
  *
  * HPPA
  *
- * $Id: jit_emit.h 18945 2007-06-12 14:08:35Z fperrad $
+ * $Id: jit_emit.h 19833 2007-07-13 22:02:25Z petdance $
  */
 
 #ifndef PARROT_HPPA_JIT_EMIT_H_GUARD
@@ -477,7 +477,7 @@ jit_emit_bc(Parrot_jit_info_t *jit_info, hppa_iregister_t s1,
 
 static void
 Parrot_emit_jump_to_ret(Parrot_jit_info_t *jit_info,
-                        Interp * interpreter)
+                        PARROT_INTERP)
 {
     /* This calculates (INDEX into op_map * 4) */
 
@@ -548,7 +548,7 @@ Parrot_emit_jump_to_ret(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
-                     Interp * interpreter)
+                     PARROT_INTERP)
 {
     jit_emit_mov_rr(jit_info->native_ptr, r25, BASE);
     jit_emit_mov_ri_i(jit_info->native_ptr, r26, ((int)(jit_info->cur_op)));
@@ -557,27 +557,27 @@ Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
 
     jit_info->arena.fixups->type = JIT_HPPA_CALL;
     jit_info->arena.fixups->param.fptr =
-        (void (*)(void))interpreter->op_func_table[*(jit_info->cur_op)];
+        (void (*)(void))interp->op_func_table[*(jit_info->cur_op)];
 
     jit_info->native_ptr += 32;
 }
 
 void
 Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
-                   Interp * interpreter)
+                   PARROT_INTERP)
 {
-    Parrot_jit_normal_op(jit_info, interpreter);
-    Parrot_emit_jump_to_ret(jit_info, interpreter);
+    Parrot_jit_normal_op(jit_info, interp);
+    Parrot_emit_jump_to_ret(jit_info, interp);
 }
 
 #  undef Parrot_jit_restart_op
 void
 Parrot_jit_restart_op(Parrot_jit_info_t *jit_info,
-                    Interp * interpreter)
+                    PARROT_INTERP)
 {
     char *jmp_ptr, *sav_ptr;
 
-    Parrot_jit_normal_op(jit_info, interpreter);
+    Parrot_jit_normal_op(jit_info, interp);
     /* remember PC */
     jmp_ptr = jit_info->native_ptr;
     jit_emit_cmpbf(jit_info->native_ptr, RET0, r0, emit_EQ, 0, 0);
@@ -588,7 +588,7 @@ Parrot_jit_restart_op(Parrot_jit_info_t *jit_info,
     jit_emit_cmpbf(jit_info->native_ptr, RET0, r0, emit_EQ,
         (long)(((sav_ptr - jmp_ptr) - 8) / 4), 0);
     jit_info->native_ptr = sav_ptr;
-    Parrot_emit_jump_to_ret(jit_info, interpreter);
+    Parrot_emit_jump_to_ret(jit_info, interp);
 }
 
 #endif /* JIT_EMIT */
@@ -598,7 +598,7 @@ Parrot_jit_restart_op(Parrot_jit_info_t *jit_info,
 
 void
 Parrot_jit_begin(Parrot_jit_info_t *jit_info,
-                 Interp * interpreter)
+                 PARROT_INTERP)
 {
     /* Save the return address in the stack. */
     emit_stw(jit_info->native_ptr, r30, r2, -0x14);
@@ -627,12 +627,12 @@ Parrot_jit_begin(Parrot_jit_info_t *jit_info,
     /* Move the interpreter to the base register. */
     jit_emit_mov_rr(jit_info->native_ptr, BASE, r26);
     jit_emit_mov_rr(jit_info->native_ptr, RET0, r25);
-    Parrot_emit_jump_to_ret(jit_info, interpreter);
+    Parrot_emit_jump_to_ret(jit_info, interp);
 }
 
 void
 Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
-                   Interp * interpreter)
+                   PARROT_INTERP)
 {
     Parrot_jit_fixup_t *fixup;
     char *fixup_ptr;
@@ -766,7 +766,7 @@ static const jit_arch_info arch_info = {
     }
 };
 const jit_arch_info*
-Parrot_jit_init(Interp *interpreter)
+Parrot_jit_init(PARROT_INTERP)
 {
     return &arch_info;
 }
