@@ -2,7 +2,7 @@
  * jit_emit.h
  *
  * SVN Info
- *    $Id: jit_emit.h 18945 2007-06-12 14:08:35Z fperrad $
+ *    $Id: jit_emit.h 19833 2007-07-13 22:02:25Z petdance $
  * Overview:
  *    MIPS JIT
  * History:
@@ -379,7 +379,7 @@ emit_if(Parrot_jit_info_t *jit_info, char opcode, mips_register_t rs,
 
 void
 Parrot_jit_begin(Parrot_jit_info_t *jit_info,
-                 Interp *interp)
+                 PARROT_INTERP)
 {
     emit_addiu(jit_info->native_ptr, sp, sp, -40);
     emit_sw(jit_info->native_ptr, ra, 32, sp);
@@ -420,39 +420,39 @@ Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
 }
 
 void
-Parrot_jit_normal_op(Parrot_jit_info_t *jit_info, Interp *interp)
+Parrot_jit_normal_op(Parrot_jit_info_t *jit_info, PARROT_INTERP)
 {
 }
 
 void
-Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info, Interp *interp)
+Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info, PARROT_INTERP)
 {
 }
 
 /*void
-Parrot_jit_restart_op(Parrot_jit_info_t *jit_info, Interp *interp)
+Parrot_jit_restart_op(Parrot_jit_info_t *jit_info, PARROT_INTERP)
 {
 }*/
 
 /* move reg to mem (i.e. intreg) */
 void
-Parrot_jit_emit_mov_mr_offs(Interp *interp, int base, size_t offs, int reg)
+Parrot_jit_emit_mov_mr_offs(PARROT_INTERP, int base, size_t offs, int reg)
 {
 }
 
 /* move mem (i.e. intreg) to reg */
 void
-Parrot_jit_emit_mov_rm_offs(Interp *interp, int reg, int base, size_t offs)
+Parrot_jit_emit_mov_rm_offs(PARROT_INTERP, int reg, int base, size_t offs)
 {
 }
 
 void
-Parrot_jit_emit_mov_mr_n_offs(Interp *interp, int base, size_t offs, int reg)
+Parrot_jit_emit_mov_mr_n_offs(PARROT_INTERP, int base, size_t offs, int reg)
 {
 }
 
 void
-Parrot_jit_emit_mov_rm_n_offs(Interp *interp, int reg, int base, size_t offs)
+Parrot_jit_emit_mov_rm_n_offs(PARROT_INTERP, int reg, int base, size_t offs)
 {
 }
 
