@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: key.c 19833 2007-07-13 22:02:25Z petdance $
+$Id: key.c 20042 2007-07-20 20:07:50Z petdance $
 
 =head1 NAME
 
@@ -27,6 +27,8 @@ Returns a new C<Key> PMC.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_new(PARROT_INTERP)
 {
@@ -43,6 +45,8 @@ Returns a new integer C<Key> PMC with value C<value>.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_new_integer(PARROT_INTERP, INTVAL value)
 {
@@ -62,6 +66,8 @@ Returns a new number C<Key> PMC with value C<value>.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_new_number(PARROT_INTERP, FLOATVAL value)
 {
@@ -75,14 +81,16 @@ key_new_number(PARROT_INTERP, FLOATVAL value)
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_new_string
 Returns a new string C<Key> PMC with value C<value>.
 
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-key_new_string(PARROT_INTERP, STRING *value)
+key_new_string(PARROT_INTERP, NOTNULL(STRING *value))
 {
     PMC * const key = pmc_new(interp, enum_class_Key);
 
@@ -94,13 +102,15 @@ key_new_string(PARROT_INTERP, STRING *value)
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_new_cstring
 Returns a new string C<Key> PMC with value C<value> converted to a
 C<STRING>.
 
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_new_cstring(PARROT_INTERP, NULLOK(const char *value))
 {
@@ -110,14 +120,16 @@ key_new_cstring(PARROT_INTERP, NULLOK(const char *value))
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_new_pmc
 Returns a new PMC C<Key> PMC with value C<value>.
 
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-key_new_pmc(PARROT_INTERP, PMC *value)
+key_new_pmc(PARROT_INTERP, NOTNULL(PMC *value))
 {
     PMC * const key = pmc_new(interp, enum_class_Key);
 
@@ -130,7 +142,7 @@ key_new_pmc(PARROT_INTERP, PMC *value)
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_set_integer
 Set the integer C<value> in C<key>.
 
 */
@@ -148,7 +160,7 @@ key_set_integer(SHIM_INTERP, NOTNULL(PMC *key), INTVAL value)
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_set_register
 Set the register C<value> in C<key>.
 
 */
@@ -167,7 +179,7 @@ key_set_register(SHIM_INTERP, NOTNULL(PMC *key), INTVAL value,
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_set_number
 Set the number C<value> in C<key>.
 
 */
@@ -185,14 +197,14 @@ key_set_number(SHIM_INTERP, NOTNULL(PMC *key), FLOATVAL value)
 
 /*
 
-FUNCDOC:
+FUNCDOC: key_set_string
 Set the string C<value> in C<key>.
 
 */
 
 PARROT_API
 void
-key_set_string(SHIM_INTERP, NOTNULL(PMC *key), STRING *value)
+key_set_string(SHIM_INTERP, NOTNULL(PMC *key), NOTNULL(STRING *value))
 {
     PObj_get_FLAGS(key) &= ~KEY_type_FLAGS;
     PObj_get_FLAGS(key) |= KEY_string_FLAG;
@@ -210,7 +222,7 @@ Set the PMC C<value> in C<key>.
 
 PARROT_API
 void
-key_set_pmc(PARROT_INTERP, NOTNULL(PMC *key), PMC *value)
+key_set_pmc(PARROT_INTERP, NOTNULL(PMC *key), NOTNULL(PMC *value))
 {
     PObj_get_FLAGS(key) &= ~KEY_type_FLAGS;
     PObj_get_FLAGS(key) |= KEY_pmc_FLAG;
@@ -232,6 +244,7 @@ Returns the type of C<key>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL
 key_type(SHIM_INTERP, NOTNULL(const PMC *key))
 {
@@ -239,6 +252,7 @@ key_type(SHIM_INTERP, NOTNULL(const PMC *key))
 }
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL
 key_integer(PARROT_INTERP, NOTNULL(PMC *key))
 {
@@ -267,6 +281,7 @@ key_integer(PARROT_INTERP, NOTNULL(PMC *key))
 }
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 FLOATVAL
 key_number(PARROT_INTERP, NOTNULL(PMC *key))
 {
@@ -290,6 +305,8 @@ key_number(PARROT_INTERP, NOTNULL(PMC *key))
 }
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 key_string(PARROT_INTERP, NOTNULL(PMC *key))
 {
@@ -323,6 +340,8 @@ possible. Otherwise they throws an exceptions.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_pmc(PARROT_INTERP, NOTNULL(PMC *key))
 {
@@ -342,6 +361,8 @@ Returns the next key if C<key> is in a sequence of linked keys.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 key_next(SHIM_INTERP, NOTNULL(PMC *key))
 {
@@ -361,6 +382,8 @@ Returns C<key1>.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_IGNORABLE_RESULT
 PMC *
 key_append(SHIM_INTERP, NOTNULL(PMC *key1), NOTNULL(PMC *key2))
 {
@@ -404,49 +427,51 @@ key_mark(PARROT_INTERP, NOTNULL(PMC *key))
 }
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING *
 key_set_to_string(PARROT_INTERP, NULLOK(PMC *key))
 {
     STRING * const semicolon = string_from_cstring(interp, " ; ", 3);
     STRING * const quote = string_from_cstring(interp, "'", 1);
-    STRING * const value = string_from_cstring(interp, "[ ", 2);
+    STRING *value = string_from_cstring(interp, "[ ", 2);
 
     for (; key; key = (PMC *)PMC_data(key)) {
         switch (PObj_get_FLAGS(key) & KEY_type_FLAGS) {
             case KEY_integer_FLAG:
-                string_append(interp, value, string_from_int(interp, PMC_int_val(key)));
+                value = string_append(interp, value, string_from_int(interp, PMC_int_val(key)));
                 break;
             case KEY_string_FLAG:
-                string_append(interp, value, quote);
-                string_append(interp, value, PMC_str_val(key));
-                string_append(interp, value, quote);
+                value = string_append(interp, value, quote);
+                value = string_append(interp, value, PMC_str_val(key));
+                value = string_append(interp, value, quote);
                 break;
             case KEY_pmc_FLAG:
-                string_append(interp, value, VTABLE_get_string(interp, key));
+                value = string_append(interp, value, VTABLE_get_string(interp, key));
                 break;
             case KEY_integer_FLAG | KEY_register_FLAG:
-                string_append(interp, value, string_from_int(interp, REG_INT(interp, PMC_int_val(key))));
+                value = string_append(interp, value, string_from_int(interp, REG_INT(interp, PMC_int_val(key))));
                 break;
             case KEY_string_FLAG | KEY_register_FLAG:
-                string_append(interp, value, quote);
-                string_append(interp, value, REG_STR(interp, PMC_int_val(key)));
-                string_append(interp, value, quote);
+                value = string_append(interp, value, quote);
+                value = string_append(interp, value, REG_STR(interp, PMC_int_val(key)));
+                value = string_append(interp, value, quote);
                 break;
             case KEY_pmc_FLAG | KEY_register_FLAG:
                 {
                 PMC * const reg = REG_PMC(interp, PMC_int_val(key));
-                string_append(interp, value, VTABLE_get_string(interp, reg));
+                value = string_append(interp, value, VTABLE_get_string(interp, reg));
                 }
                 break;
             default:
-                string_append(interp, value, string_from_literal(interp, "Key type unknown"));
+                value = string_append(interp, value, string_from_literal(interp, "Key type unknown"));
                 break;
         }
 
         if (PMC_data(key))
-            string_append(interp, value, semicolon);
+            value = string_append(interp, value, semicolon);
     }
-    string_append(interp, value, string_from_cstring(interp, " ]", 2));
+    value = string_append(interp, value, string_from_cstring(interp, " ]", 2));
     return value;
 }
 
