@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2004-2007, The Perl Foundation.
-$Id: library.c 19882 2007-07-14 22:33:30Z petdance $
+$Id: library.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -16,7 +16,6 @@ This file contains a C function to access Parrot's bytecode library functions.
 
 #include "parrot/parrot.h"
 #include "parrot/dynext.h"
-#include <assert.h>
 #include "library.str"
 
 /* HEADERIZER HFILE: include/parrot/library.h */
@@ -27,6 +26,7 @@ static void cnv_to_win32_filesep( NOTNULL(STRING *path) )
         __attribute__nonnull__(1);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC* get_search_paths( PARROT_INTERP, enum_lib_paths which )
         __attribute__nonnull__(1);
 
@@ -34,6 +34,8 @@ PARROT_PURE_FUNCTION
 static int is_abs_path( NOTNULL(const STRING *file) )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* path_append( PARROT_INTERP,
     NOTNULL(STRING *l_path),
     NOTNULL(STRING *r_path) )
@@ -41,6 +43,8 @@ static STRING* path_append( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* path_concat( PARROT_INTERP,
     NOTNULL(STRING *l_path),
     NOTNULL(STRING *r_path) )
@@ -48,21 +52,27 @@ static STRING* path_concat( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* path_finalize( PARROT_INTERP, NOTNULL(STRING *path) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* path_guarantee_trailing_separator( PARROT_INTERP,
     NOTNULL(STRING *path) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING* try_bytecode_extensions( PARROT_INTERP, NOTNULL(STRING* path) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING* try_load_path( PARROT_INTERP, NOTNULL(STRING* path) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -172,6 +182,7 @@ parrot_init_library_paths(PARROT_INTERP)
 }
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC*
 get_search_paths(PARROT_INTERP, enum_lib_paths which)
 {
@@ -188,11 +199,11 @@ is_abs_path(NOTNULL(const STRING *file))
     const char * const file_name = file->strstart;
     if (file->strlen <= 1)
         return 0;
-    assert(file->encoding == Parrot_fixed_8_encoding_ptr ||
+    PARROT_ASSERT(file->encoding == Parrot_fixed_8_encoding_ptr ||
             file->encoding == Parrot_utf8_encoding_ptr);
 #ifdef WIN32
     if (file_name[0] == '\\' || file_name[0] == '/' ||
-            (isalpha(file_name[0]) && file->strlen > 2 &&
+            (isalpha((unsigned char)file_name[0]) && file->strlen > 2 &&
              (strncmp(file_name+1, ":\\", 2) == 0 ||
               strncmp(file_name+1, ":/",  2) == 0)))
 #else
@@ -218,7 +229,7 @@ cnv_to_win32_filesep(NOTNULL(STRING *path))
 {
     char* cnv;
 
-    assert(path->encoding == Parrot_fixed_8_encoding_ptr ||
+    PARROT_ASSERT(path->encoding == Parrot_fixed_8_encoding_ptr ||
         path->encoding == Parrot_utf8_encoding_ptr);
 
     cnv = path->strstart;
@@ -228,6 +239,8 @@ cnv_to_win32_filesep(NOTNULL(STRING *path))
 
 #endif
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 path_finalize(PARROT_INTERP, NOTNULL(STRING *path))
 {
@@ -256,6 +269,8 @@ path_finalize(PARROT_INTERP, NOTNULL(STRING *path))
   there already.
  */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 path_guarantee_trailing_separator(PARROT_INTERP, NOTNULL(STRING *path))
 {
@@ -275,6 +290,8 @@ path_guarantee_trailing_separator(PARROT_INTERP, NOTNULL(STRING *path))
   argument and the right argument is appended
  */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 path_append(PARROT_INTERP, NOTNULL(STRING *l_path), NOTNULL(STRING *r_path))
 {
@@ -290,6 +307,8 @@ path_append(PARROT_INTERP, NOTNULL(STRING *l_path), NOTNULL(STRING *r_path))
   with a path-separator.
  */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 path_concat(PARROT_INTERP, NOTNULL(STRING *l_path), NOTNULL(STRING *r_path))
 {
@@ -315,6 +334,7 @@ static const char* load_ext_code[ LOAD_EXT_CODE_LAST + 1 ] = {
 };
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING*
 try_load_path(PARROT_INTERP, NOTNULL(STRING* path))
 {
@@ -322,11 +342,6 @@ try_load_path(PARROT_INTERP, NOTNULL(STRING* path))
 
     final = string_copy(interp, path);
 
-#if 0
-    printf("path is \"%s\"\n",
-           string_to_cstring(interp, final ));
-#endif
-
     final = path_finalize(interp, final);
 
     if (Parrot_stat_info_intval(interp, final, STAT_EXISTS)) {
@@ -343,6 +358,7 @@ try_load_path(PARROT_INTERP, NOTNULL(STRING* path))
  */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING*
 try_bytecode_extensions(PARROT_INTERP, NOTNULL(STRING* path))
 {
@@ -357,7 +373,8 @@ try_bytecode_extensions(PARROT_INTERP, NOTNULL(STRING* path))
 
     with_ext = string_copy(interp, path);
 
-    if ( (result = try_load_path(interp, with_ext) ) )
+    result = try_load_path(interp, with_ext);
+    if (result)
         return result;
 
     /*
@@ -371,7 +388,8 @@ try_bytecode_extensions(PARROT_INTERP, NOTNULL(STRING* path))
         with_ext = string_append(interp,
                                  with_ext, const_string(interp, load_ext_code[guess]));
 
-        if ( (result = try_load_path(interp, with_ext)) )
+        result = try_load_path(interp, with_ext);
+        if (result)
             return result;
     }
 
@@ -400,6 +418,7 @@ F<include/parrot/library.h>.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 STRING*
 Parrot_locate_runtime_file_str(PARROT_INTERP, NOTNULL(STRING *file),
         enum_runtime_ft type)
@@ -409,11 +428,6 @@ Parrot_locate_runtime_file_str(PARROT_INTERP, NOTNULL(STRING *file),
     INTVAL i, n;
     PMC *paths;
 
-#if 0
-    printf("requesting path: \"%s\"\n",
-           string_to_cstring(interp, file ));
-#endif
-
     /* if this is an absolute path return it as is */
     if (is_abs_path(file))
         return file;
@@ -459,13 +473,13 @@ Parrot_locate_runtime_file_str(PARROT_INTERP, NOTNULL(STRING *file),
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 char*
 Parrot_locate_runtime_file(PARROT_INTERP, NOTNULL(const char *file_name),
         enum_runtime_ft type)
 {
     STRING * const file = string_from_cstring(interp, file_name, 0);
-    STRING * const result = Parrot_locate_runtime_file_str(interp,
-            file, type);
+    STRING * const result = Parrot_locate_runtime_file_str(interp, file, type);
     /*
      * XXX valgrind shows e.g.
      *     invalid read of size 8 inside a string of length 69
@@ -474,7 +488,7 @@ Parrot_locate_runtime_file(PARROT_INTERP, NOTNULL(const char *file_name),
      *
      *     see also the log at #37814
      */
-    return string_to_cstring(interp, result);
+    return result ? string_to_cstring(interp, result) : NULL;
 }
 
 /*
@@ -491,6 +505,8 @@ whether we're searching for a STRING or a cstring.
 */
 
 PARROT_API
+PARROT_MALLOC
+PARROT_CAN_RETURN_NULL
 char*
 Parrot_get_runtime_prefix(PARROT_INTERP, NULLOK(STRING **prefix_str))
 {
@@ -549,8 +565,10 @@ extension and C<ext> to the extension or NULL.
 
 */
 
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
-parrot_split_path_ext(PARROT_INTERP, STRING *in,
+parrot_split_path_ext(PARROT_INTERP, NOTNULL(STRING *in),
         NOTNULL(STRING **wo_ext), NOTNULL(STRING **ext))
 {
     STRING * const slash1 = CONST_STRING(interp, "/");
@@ -566,7 +584,7 @@ parrot_split_path_ext(PARROT_INTERP, STRING *in,
     pos_dot = CHARSET_RINDEX(interp, in, dot, len);
 
     /* XXX directory parrot-0.4.1 or such */
-    if (pos_dot != -1 && isdigit(((char*)in->strstart)[pos_dot+1]))
+    if (pos_dot != -1 && isdigit((unsigned char)((char*)in->strstart)[pos_dot+1]))
         pos_dot = -1;
 
     ++pos_dot;
@@ -585,7 +603,7 @@ parrot_split_path_ext(PARROT_INTERP, STRING *in,
     else if (pos_sl) {
         stem = string_substr(interp, in, pos_sl, len - pos_sl, NULL, 0);
         *wo_ext = string_copy(interp, in);
-        *ext = 0;
+        *ext = NULL;
     }
     else {
         stem = string_copy(interp, in);
