@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2002-2007, The Perl Foundation.
 License:  Artistic/GPL, see README and LICENSES for details
-$Id: list.c 19885 2007-07-15 04:02:08Z petdance $
+$Id: list.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -179,12 +179,13 @@ Also all array usage depends on list.
 */
 
 #include "parrot/parrot.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/list.h */
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 static List_chunk * add_chunk( PARROT_INTERP,
     NOTNULL(List *list),
     int where,
@@ -192,6 +193,8 @@ static List_chunk * add_chunk( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static List_chunk * alloc_next_size( PARROT_INTERP,
     NOTNULL(List *list),
     int where,
@@ -200,6 +203,7 @@ static List_chunk * alloc_next_size( PARROT_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static List_chunk * allocate_chunk( PARROT_INTERP,
     NOTNULL(List *list),
     UINTVAL items,
@@ -287,6 +291,7 @@ Make a new chunk, size bytes big, holding items items.
 */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 static List_chunk *
 allocate_chunk(PARROT_INTERP, NOTNULL(List *list), UINTVAL items, UINTVAL size)
 {
@@ -644,6 +649,8 @@ Calculate size and items for next chunk and allocate it.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static List_chunk *
 alloc_next_size(PARROT_INTERP, NOTNULL(List *list), int where, UINTVAL idx)
 {
@@ -660,7 +667,7 @@ alloc_next_size(PARROT_INTERP, NOTNULL(List *list), int where, UINTVAL idx)
         do_sparse = 0;
     }
     else if (do_sparse) {
-        assert(where);
+        PARROT_ASSERT(where);
         /* don't add sparse chunk at start of list */
         if (!list->n_chunks) {
             list->grow_policy = enum_grow_fixed;
@@ -739,6 +746,8 @@ Add chunk at start or end.
 
 */
 
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 static List_chunk *
 add_chunk(PARROT_INTERP, NOTNULL(List *list), int where, UINTVAL idx)
 {
@@ -937,7 +946,7 @@ get_chunk(PARROT_INTERP, NOTNULL(List *list), NOTNULL(UINTVAL *idx))
             const UINTVAL slot = ld(*idx + chunk->items) - ld_first;
 
             /* we are in this growing area, so we are done */
-            assert(slot < chunk->n_chunks);
+            PARROT_ASSERT(slot < chunk->n_chunks);
             *idx -= (1 << (ld_first + slot)) - chunk->items;
             return chunk_list_ptr(list, i + slot);
         }
@@ -1038,7 +1047,7 @@ list_set(PARROT_INTERP, NOTNULL(List *list), NULLOK(void *item), INTVAL type, IN
     const INTVAL oidx = idx;
     List_chunk *chunk = get_chunk(interp, list, (UINTVAL *)&idx);
 
-    assert(chunk);
+    PARROT_ASSERT(chunk);
     /* if this is a sparse chunk: split in possibly 2 sparse parts before and
      * after then make a real chunk, rebuild chunk list and set item */
     if (chunk->flags & sparse) {
@@ -1046,8 +1055,8 @@ list_set(PARROT_INTERP, NOTNULL(List *list), NULLOK(void *item), INTVAL type, IN
         /* reget chunk and idx */
         idx = oidx;
         chunk = get_chunk(interp, list, (UINTVAL *)&idx);
-        assert(chunk);
-        assert(!(chunk->flags & sparse));
+        PARROT_ASSERT(chunk);
+        PARROT_ASSERT(!(chunk->flags & sparse));
     }
 
     switch (type) {
@@ -1167,6 +1176,7 @@ Returns a new list of type C<type>.
 
 PARROT_API
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 List *
 list_new(PARROT_INTERP, PARROT_DATA_TYPE type)
 {
@@ -1260,6 +1270,7 @@ list_new_init(PARROT_INTERP, PARROT_DATA_TYPE type, NOTNULL(PMC *init))
     for (i = 0; i < len; i += 2) {
         const INTVAL key = VTABLE_get_integer_keyed_int(interp, init, i);
         const INTVAL val = i + 1;
+        INTVAL       result;
         switch (key) {
             case 0:
                 size = VTABLE_get_integer_keyed_int(interp, init, val);
@@ -1268,7 +1279,8 @@ list_new_init(PARROT_INTERP, PARROT_DATA_TYPE type, NOTNULL(PMC *init))
                 multi_key = VTABLE_get_pmc_keyed_int(interp, init, val);
                 break;
             case 2:
-                type = VTABLE_get_integer_keyed_int(interp, init, val);
+                result = VTABLE_get_integer_keyed_int(interp, init, val);
+                type   = (PARROT_DATA_TYPE)result;
                 break;
             case 3:
                 item_size = VTABLE_get_integer_keyed_int(interp, init, val);
@@ -1446,14 +1458,14 @@ C<pinfo> is the visit info, (see include/parrot/pmc_freeze.h>).
 
 PARROT_API
 void
-list_visit(PARROT_INTERP, NOTNULL(List *list), void *pinfo)
+list_visit(PARROT_INTERP, NOTNULL(List *list), NOTNULL(void *pinfo))
 {
     List_chunk *chunk;
     visit_info * const info = (visit_info*) pinfo;
     UINTVAL idx;
 
     const UINTVAL n = list_length(interp, list);
-    assert(list->item_type == enum_type_PMC);
+    PARROT_ASSERT(list->item_type == enum_type_PMC);
     /* TODO intlist ... */
     for (idx = 0, chunk = list->first; chunk; chunk = chunk->next) {
         /* TODO deleted elements */
@@ -1537,9 +1549,9 @@ list_insert(PARROT_INTERP, NOTNULL(List *list), INTVAL idx, INTVAL n_items)
 {
     List_chunk *chunk;
 
-    assert(idx >= 0);
+    PARROT_ASSERT(idx >= 0);
     idx += list->start;
-    assert(n_items >= 0);
+    PARROT_ASSERT(n_items >= 0);
     if (n_items == 0)
         return;
     /* empty list */
@@ -1608,8 +1620,8 @@ list_delete(PARROT_INTERP, NOTNULL(List *list), INTVAL idx, INTVAL n_items)
 {
     List_chunk *chunk;
 
-    assert(idx >= 0);
-    assert(n_items >= 0);
+    PARROT_ASSERT(idx >= 0);
+    PARROT_ASSERT(n_items >= 0);
     if (n_items == 0)
         return;
     idx += list->start;
@@ -1701,7 +1713,7 @@ Pushes C<item> of type C<type> on to the start of the list.
 
 PARROT_API
 void
-list_unshift(PARROT_INTERP, NOTNULL(List *list), void *item, int type)
+list_unshift(PARROT_INTERP, NOTNULL(List *list), NULLOK(void *item), int type)
 {
     List_chunk *chunk;
 
@@ -1710,7 +1722,7 @@ list_unshift(PARROT_INTERP, NOTNULL(List *list), void *item, int type)
         list->start = chunk->items;
     }
     else
-        chunk = list->first;
+        chunk = list->first; /* XXX This chunk is unused.  Why are we getting it? */
     list_set(interp, list, item, type, --list->start);
     list->length++;
 }
@@ -1796,7 +1808,7 @@ Assigns C<item> of type C<type> to index C<idx>.
 
 PARROT_API
 void
-list_assign(PARROT_INTERP, NOTNULL(List *list), INTVAL idx, void *item, int type)
+list_assign(PARROT_INTERP, NOTNULL(List *list), INTVAL idx, NULLOK(void *item), int type)
 {
     const INTVAL length = list->length;
 
