@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2007, The Perl Foundation.
-$Id: mmd.c 19886 2007-07-15 06:19:53Z petdance $
+$Id: mmd.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -41,7 +41,6 @@ not highest type in table.
 #include "parrot/mmd.h"
 #include "parrot/oplib/ops.h"
 #include "mmd.str"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/mmd.h */
 
@@ -51,6 +50,8 @@ static INTVAL distance_cmp( SHIM_INTERP, INTVAL a, INTVAL b );
 static void dump_mmd( PARROT_INTERP, INTVAL function )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static funcptr_t get_mmd_dispatcher( PARROT_INTERP,
     NOTNULL(PMC *left),
     NOTNULL(PMC *right),
@@ -61,9 +62,13 @@ static funcptr_t get_mmd_dispatcher( PARROT_INTERP,
         __attribute__nonnull__(3)
         __attribute__nonnull__(5);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC* mmd_arg_tuple_func( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_arg_tuple_inline( PARROT_INTERP,
     NOTNULL(STRING *signature),
     va_list args )
@@ -87,16 +92,19 @@ static void mmd_create_builtin_multi_meth_2( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-static PMC * mmd_create_builtin_multi_stub( PARROT_INTERP,
-    NOTNULL(PMC* ns),
+void mmd_create_builtin_multi_stub( PARROT_INTERP,
+    NOTNULL(PMC *ns),
     INTVAL func_nr )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_cvt_to_types( PARROT_INTERP, NOTNULL(PMC *multi_sig) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_CANNOT_RETURN_NULL
 static PMC * mmd_deref( PARROT_INTERP, NOTNULL(PMC *value) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -119,9 +127,12 @@ static void mmd_expand_x( PARROT_INTERP, INTVAL func_nr, INTVAL new_x )
 static void mmd_expand_y( PARROT_INTERP, INTVAL func_nr, INTVAL new_y )
         __attribute__nonnull__(1);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_get_ns( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
 static int mmd_is_hidden( PARROT_INTERP,
     NOTNULL(PMC *multi),
     NOTNULL(PMC *cl) )
@@ -129,6 +140,8 @@ static int mmd_is_hidden( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_make_ns( PARROT_INTERP )
         __attribute__nonnull__(1);
 
@@ -163,6 +176,8 @@ static int mmd_search_cur_namespace( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_search_default( PARROT_INTERP,
     NOTNULL(STRING *meth),
     NOTNULL(PMC *arg_tuple) )
@@ -170,6 +185,8 @@ static PMC* mmd_search_default( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC* mmd_search_scopes( PARROT_INTERP, NOTNULL(STRING *meth) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -191,12 +208,12 @@ static void
 dump_mmd(PARROT_INTERP, INTVAL function)
 {
     UINTVAL x, y;
-    UINTVAL offset, x_funcs, y_funcs;
+    UINTVAL offset;
     MMD_table * const table = interp->binop_mmd_funcs + function;
-    funcptr_t func, def; /* XXX Looks like def is never defined */
+    funcptr_t func;
+    const UINTVAL x_funcs = table->x;
+    const UINTVAL y_funcs = table->y;
 
-    x_funcs = table->x;
-    y_funcs = table->y;
     printf("    ");
     for (x = 0; x < x_funcs; ++x) {
         if (! (x % 10))
@@ -211,7 +228,6 @@ dump_mmd(PARROT_INTERP, INTVAL function)
             offset = x_funcs * y + x;
             func = table->mmd_funcs[offset];
             printf("%c",
-                    func == def ?  '.' :
                     (UINTVAL)func & 1 ?  'P' :
                     !func  ? '0' : 'F');
         }
@@ -221,7 +237,7 @@ dump_mmd(PARROT_INTERP, INTVAL function)
         for (x = 0; x < x_funcs; ++x) {
             offset = x_funcs * y + x;
             func = table->mmd_funcs[offset];
-            if (func && func != def && !((UINTVAL) func & 1))
+            if (func && !((UINTVAL) func & 1))
                 printf("%3d %3d: %p\n", (int)x, (int)y, (void*) func);
         }
     }
@@ -247,8 +263,8 @@ get_mmd_dispatch_type(PARROT_INTERP, INTVAL func_nr, INTVAL left_type,
 #endif
 
     func = NULL;
-    assert(left_type >= 0);
-    assert(right_type >=0 ||
+    PARROT_ASSERT(left_type >= 0);
+    PARROT_ASSERT(right_type >=0 ||
             (right_type >= enum_type_INTVAL && right_type <= enum_type_PMC));
     r = right_type;
     if (right_type < 0)
@@ -295,7 +311,8 @@ get_mmd_dispatch_type(PARROT_INTERP, INTVAL func_nr, INTVAL left_type,
     return func_;
 }
 
-
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static funcptr_t
 get_mmd_dispatcher(PARROT_INTERP, NOTNULL(PMC *left), NOTNULL(PMC *right),
         INTVAL function, NOTNULL(int *is_pmc))
@@ -314,6 +331,7 @@ call on the 'real' value.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
 static PMC *
 mmd_deref(PARROT_INTERP, NOTNULL(PMC *value))
 {
@@ -378,6 +396,8 @@ Inplace dispatch functions for C<< left <op=> right >>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC*
 mmd_dispatch_p_ppp(PARROT_INTERP,
         NOTNULL(PMC *left), NOTNULL(PMC *right), NULLOK(PMC *dest), INTVAL func_nr)
@@ -406,6 +426,7 @@ mmd_dispatch_p_ppp(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC*
 mmd_dispatch_p_pip(PARROT_INTERP,
         NOTNULL(PMC *left), INTVAL right, NULLOK(PMC *dest), INTVAL func_nr)
@@ -439,6 +460,7 @@ mmd_dispatch_p_pip(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC*
 mmd_dispatch_p_pnp(PARROT_INTERP,
         NOTNULL(PMC *left), FLOATVAL right, NULLOK(PMC *dest), INTVAL func_nr)
@@ -467,6 +489,7 @@ mmd_dispatch_p_pnp(PARROT_INTERP,
 }
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC*
 mmd_dispatch_p_psp(PARROT_INTERP,
         NOTNULL(PMC *left), NOTNULL(STRING *right), NULLOK(PMC *dest), INTVAL func_nr)
@@ -519,6 +542,7 @@ mmd_dispatch_v_pp(PARROT_INTERP,
     }
 }
 
+PARROT_API
 void
 mmd_dispatch_v_pi(PARROT_INTERP,
         NOTNULL(PMC *left), INTVAL right, INTVAL func_nr)
@@ -569,7 +593,7 @@ mmd_dispatch_v_pn(PARROT_INTERP,
 PARROT_API
 void
 mmd_dispatch_v_ps(PARROT_INTERP,
-        NOTNULL(PMC *left), STRING *right, INTVAL func_nr)
+        NOTNULL(PMC *left), NOTNULL(STRING *right), INTVAL func_nr)
 {
     mmd_f_v_ps real_function;
     int is_pmc;
@@ -731,7 +755,7 @@ mmd_expand_y(PARROT_INTERP, INTVAL func_nr, INTVAL new_y)
     UINTVAL           new_size, old_size;
     MMD_table * const table = interp->binop_mmd_funcs + func_nr;
 
-    assert(table->x);
+    PARROT_ASSERT(table->x);
 
     old_size = sizeof (funcptr_t) * table->x * table->y;
     new_size = sizeof (funcptr_t) * table->x * new_y;
@@ -827,9 +851,9 @@ mmd_register(PARROT_INTERP,
     INTVAL offset;
     MMD_table *table;
 
-    assert(func_nr < (INTVAL)interp->n_binop_mmd_funcs);
-    assert(left_type >= 0);
-    assert(right_type >=0 ||
+    PARROT_ASSERT(func_nr < (INTVAL)interp->n_binop_mmd_funcs);
+    PARROT_ASSERT(left_type >= 0);
+    PARROT_ASSERT(right_type >=0 ||
             (right_type >= enum_type_INTVAL && right_type <= enum_type_PMC));
     if (right_type < 0)
         right_type -= enum_type_INTVAL;
@@ -900,6 +924,7 @@ C function pointer in PMC_struct_val.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
 PMC *
 mmd_vtfind(PARROT_INTERP, INTVAL func_nr, INTVAL left, INTVAL right)
@@ -920,8 +945,10 @@ mmd_vtfind(PARROT_INTERP, INTVAL func_nr, INTVAL left, INTVAL right)
 }
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_MMD_search_default_infix(PARROT_INTERP, STRING *meth,
+Parrot_MMD_search_default_infix(PARROT_INTERP, NOTNULL(STRING *meth),
         INTVAL left_type, INTVAL right_type)
 {
     PMC* const arg_tuple = pmc_new(interp, enum_class_FixedIntegerArray);
@@ -942,8 +969,10 @@ manhatten distance to the current args.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_mmd_sort_candidate_list(PARROT_INTERP, PMC *candidates)
+Parrot_mmd_sort_candidate_list(PARROT_INTERP, NOTNULL(PMC *candidates))
 {
     PMC *arg_tuple;
     INTVAL n;
@@ -979,6 +1008,8 @@ according to calling conventions.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_arg_tuple_inline(PARROT_INTERP, NOTNULL(STRING *signature), va_list args)
 {
@@ -1023,10 +1054,12 @@ mmd_arg_tuple_inline(PARROT_INTERP, NOTNULL(STRING *signature), va_list args)
     return arg_tuple;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC*
 mmd_arg_tuple_func(PARROT_INTERP)
 {
-    INTVAL sig_len, i, type, idx;
+    INTVAL sig_len, i, type;
     PMC* arg;
     PMC* args_array;    /* from recent set_args opcode */
     opcode_t *args_op;
@@ -1045,7 +1078,7 @@ mmd_arg_tuple_func(PARROT_INTERP)
     args_op = interp->current_args;
     if (!args_op)
         return arg_tuple;
-    assert(*args_op == PARROT_OP_set_args_pc);
+    PARROT_ASSERT(*args_op == PARROT_OP_set_args_pc);
     constants = interp->code->const_table->constants;
     ++args_op;
     args_array = constants[*args_op]->u.key;
@@ -1071,19 +1104,21 @@ mmd_arg_tuple_func(PARROT_INTERP)
                 VTABLE_push_integer(interp, arg_tuple, enum_type_STRING);
                 break;
             case PARROT_ARG_PMC:
-                idx = *args_op;
+                {
+                const int idx = *args_op;
                 if ((type & PARROT_ARG_CONSTANT))
                     arg = constants[idx]->u.key;
                 else
                     arg = REG_PMC(interp, idx);
                 type = VTABLE_type(interp, arg);
                 VTABLE_push_integer(interp, arg_tuple, type);
+                }
                 break;
             case PARROT_ARG_FLATTEN | PARROT_ARG_PMC:  {
                 /* expand flattening args */
                 int j, n;
 
-                idx = *args_op;
+                const int idx = *args_op;
                 arg = REG_PMC(interp, idx);
                 n = VTABLE_elements(interp, arg);
                 for (j = 0; j < n; ++j)  {
@@ -1113,6 +1148,8 @@ result
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_search_default(PARROT_INTERP, NOTNULL(STRING *meth), NOTNULL(PMC *arg_tuple))
 {
@@ -1233,6 +1270,8 @@ Create Manhattan Distance of sub C<pmc> against given argument types.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_cvt_to_types(PARROT_INTERP, NOTNULL(PMC *multi_sig))
 {
@@ -1433,15 +1472,16 @@ C<arg_tuple>.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_search_scopes(PARROT_INTERP, NOTNULL(STRING *meth))
 {
     PMC * const candidate_list = pmc_new(interp, enum_class_ResizablePMCArray);
 
     const int stop = mmd_search_cur_namespace(interp, meth, candidate_list);
-    if (stop)
-        return candidate_list;
-    mmd_search_builtin(interp, meth, candidate_list);
+    if (!stop)
+        mmd_search_builtin(interp, meth, candidate_list);
     return candidate_list;
 }
 
@@ -1454,6 +1494,7 @@ the candidate list C<cl>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static int
 mmd_is_hidden(PARROT_INTERP, NOTNULL(PMC *multi), NOTNULL(PMC *cl))
 {
@@ -1530,6 +1571,8 @@ mmd_search_cur_namespace(PARROT_INTERP, NOTNULL(STRING *meth), NOTNULL(PMC *cl))
     return pmc && mmd_maybe_candidate(interp, pmc, cl);
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_get_ns(PARROT_INTERP)
 {
@@ -1539,6 +1582,8 @@ mmd_get_ns(PARROT_INTERP)
     return ns;
 }
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static PMC*
 mmd_make_ns(PARROT_INTERP)
 {
@@ -1566,17 +1611,16 @@ mmd_search_builtin(PARROT_INTERP, NOTNULL(STRING *meth), NOTNULL(PMC *cl))
         mmd_maybe_candidate(interp, pmc, cl);
 }
 
-
-static PMC *
-mmd_create_builtin_multi_stub(PARROT_INTERP, NOTNULL(PMC* ns), INTVAL func_nr)
+void
+mmd_create_builtin_multi_stub(PARROT_INTERP, NOTNULL(PMC *ns), INTVAL func_nr)
 {
-    const char * name = Parrot_MMD_method_name(interp, func_nr);
+    const char * name    = Parrot_MMD_method_name(interp, func_nr);
+
     /* create in constant pool */
-    STRING * const s = const_string(interp, name);
-    PMC * multi = constant_pmc_new(interp, enum_class_MultiSub);
+    STRING     * const s = const_string(interp, name);
+    PMC        * multi   = constant_pmc_new(interp, enum_class_MultiSub);
 
     VTABLE_set_pmc_keyed_str(interp, ns, s, multi);
-    return ns;
 }
 
 static void
@@ -1588,7 +1632,7 @@ mmd_create_builtin_multi_meth_2(PARROT_INTERP, NOTNULL(PMC *ns),
     STRING *meth_name;
     PMC *method, *multi, *_class, *multi_sig;
 
-    assert(type != enum_class_Null && type != enum_class_delegate &&
+    PARROT_ASSERT(type != enum_class_Null && type != enum_class_delegate &&
             type != enum_class_Ref  && type != enum_class_SharedRef &&
             type != enum_class_deleg_pmc && type != enum_class_ParrotClass &&
             type != enum_class_ParrotObject);
@@ -1637,7 +1681,7 @@ mmd_create_builtin_multi_meth_2(PARROT_INTERP, NOTNULL(PMC *ns),
             VTABLE_push_pmc(interp, multi, method);
         }
         else {
-            assert(method->vtable->base_type == enum_class_MultiSub);
+            PARROT_ASSERT(method->vtable->base_type == enum_class_MultiSub);
             multi = method;
         }
         method = constant_pmc_new(interp, enum_class_NCI);
@@ -1663,7 +1707,7 @@ mmd_create_builtin_multi_meth_2(PARROT_INTERP, NOTNULL(PMC *ns),
      */
     multi = Parrot_find_global_n(interp, ns,
                                  const_string(interp, short_name));
-    assert(multi);
+    PARROT_ASSERT(multi);
     VTABLE_push_pmc(interp, multi, method);
 }
 
@@ -1698,9 +1742,7 @@ Parrot_mmd_register_table(PARROT_INTERP, INTVAL type,
         for (i = 0; i < MMD_USER_FIRST; ++i) {
             mmd_register(interp, i, enum_class_core_max - 1,
                     enum_class_core_max - 1, NULL);
-            /*
-             * create a MultiSub stub
-             */
+            /* create a MultiSub stub */
             mmd_create_builtin_multi_stub(interp, ns, i);
         }
     }
@@ -1712,7 +1754,7 @@ Parrot_mmd_register_table(PARROT_INTERP, INTVAL type,
          * but I'm assuming it's optimizer related.
          */
 #ifndef __INTEL_COMPILER
-        assert((PTR2UINTVAL(mmd_table[i].func_ptr) & 3) == 0);
+        PARROT_ASSERT((PTR2UINTVAL(mmd_table[i].func_ptr) & 3) == 0);
 #endif
         mmd_register(interp,
                 mmd_table[i].func_nr, type,
