@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: objects.c 19912 2007-07-16 20:19:17Z petdance $
+$Id: objects.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -17,7 +17,6 @@ Handles class and object manipulation.
 #define PARROT_IN_OBJECTS_C
 #include "parrot/parrot.h"
 #include "parrot/objects.h"
-#include <assert.h>
 
 #include "objects.str"
 
@@ -32,6 +31,8 @@ static INTVAL attr_str_2_num( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* C3_merge( PARROT_INTERP, NOTNULL(PMC *merge_list) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -42,6 +43,8 @@ static PMC* class_mro_merge( PARROT_INTERP, NOTNULL(PMC *seqs) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC* create_class_mro( PARROT_INTERP, NOTNULL(PMC *_class) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -71,6 +74,7 @@ static void fail_if_exist( PARROT_INTERP, NOTNULL(PMC *name) )
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC * find_method_direct_1( PARROT_INTERP,
     NOTNULL(PMC *_class),
     NOTNULL(STRING *method_name) )
@@ -79,6 +83,7 @@ static PMC * find_method_direct_1( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* find_vtable_meth_ns( PARROT_INTERP,
     NOTNULL(PMC *ns),
     INTVAL vtable_index )
@@ -86,6 +91,7 @@ static PMC* find_vtable_meth_ns( PARROT_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* get_init_meth( PARROT_INTERP,
     NOTNULL(PMC *_class),
     NOTNULL(STRING *prop_str),
@@ -137,7 +143,8 @@ static INTVAL register_type( PARROT_INTERP, NOTNULL(PMC *name) )
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_get_vtable_index
+
 Return index if C<name> is a valid vtable slot name.
 
 */
@@ -176,12 +183,14 @@ Parrot_get_vtable_index(PARROT_INTERP, NOTNULL(const STRING *name))
 
 /*
 
-FUNCDOC:
+FUNCDOC: find_vtable_meth_ns
+
 Return Sub PMC if a method with the vtable name exists in ns
 
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 find_vtable_meth_ns(PARROT_INTERP, NOTNULL(PMC *ns), INTVAL vtable_index)
 {
@@ -190,13 +199,15 @@ find_vtable_meth_ns(PARROT_INTERP, NOTNULL(PMC *ns), INTVAL vtable_index)
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_find_vtable_meth
+
 Given pmc, run through its mro looking for the meth vtable method.
 Return the vtable method PMC if found.
 
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC*
 Parrot_find_vtable_meth(PARROT_INTERP, NOTNULL(PMC *pmc), NOTNULL(STRING *meth))
 {
@@ -237,7 +248,8 @@ Parrot_find_vtable_meth(PARROT_INTERP, NOTNULL(PMC *pmc), NOTNULL(STRING *meth))
 
 /*
 
-FUNCDOC:
+FUNCDOC: readable_name
+
 Given a String or Key PMC return the STRING* representation
 
 XXX this function, key_set_to_string, and the key PMC get_repr should be consolidated
@@ -245,6 +257,8 @@ XXX this function, key_set_to_string, and the key PMC get_repr should be consoli
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING*
 readable_name(PARROT_INTERP, NOTNULL(PMC *name))
 {
@@ -257,7 +271,7 @@ readable_name(PARROT_INTERP, NOTNULL(PMC *name))
     join_on = CONST_STRING(interp, ";");
     array   = pmc_new(interp, enum_class_ResizableStringArray);
 
-    assert(name->vtable->base_type == enum_class_Key);
+    PARROT_ASSERT(name->vtable->base_type == enum_class_Key);
 
     while (name) {
         VTABLE_push_string(interp, array, key_string(interp, name));
@@ -269,7 +283,8 @@ readable_name(PARROT_INTERP, NOTNULL(PMC *name))
 
 /*
 
-FUNCDOC:
+FUNCDOC: fail_if_exist
+
 Throws an exception if a PMC or class with the same name already exists.
 
 XXX uses global class registry
@@ -367,7 +382,7 @@ rebuild_attrib_stuff(PARROT_INTERP, NOTNULL(PMC *_class))
     }
 
 #ifndef NDEBUG
-    assert(_class == orig_class);
+    PARROT_ASSERT(_class == orig_class);
 #endif
 
     /* And note the totals */
@@ -377,7 +392,8 @@ rebuild_attrib_stuff(PARROT_INTERP, NOTNULL(PMC *_class))
 
 /*
 
-FUNCDOC:
+FUNCDOC: create_deleg_pmc_vtable
+
 Create a vtable that dispatches either to the contained PMC in the first
 attribute (deleg_pmc) or to an overridden method (delegate), depending
 on the existence of the method for this class.
@@ -445,6 +461,7 @@ create_deleg_pmc_vtable(PARROT_INTERP, NOTNULL(PMC *_class), int full)
 /*
 
 FUNCDOC: Parrot_MMD_method_name
+
 Return the method name for the given MMD enum.
 
 */
@@ -455,7 +472,7 @@ PARROT_CAN_RETURN_NULL
 const char*
 Parrot_MMD_method_name(SHIM_INTERP, INTVAL idx)
 {
-    assert(idx >= 0);
+    PARROT_ASSERT(idx >= 0);
 
     if (idx >= MMD_USER_FIRST)
         return NULL;
@@ -490,7 +507,8 @@ Parrot_MMD_method_idx(SHIM_INTERP, NOTNULL(const char *name))
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_single_subclass
+
 Subclass a class. Single parent class, nice and straightforward. If
 C<child_class> is C<NULL>, this is an anonymous subclass we're creating,
 function.
@@ -498,8 +516,10 @@ function.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
-Parrot_single_subclass(PARROT_INTERP, PMC *base_class, NULLOK(PMC *name))
+Parrot_single_subclass(PARROT_INTERP, NOTNULL(PMC *base_class), NULLOK(PMC *name))
 {
     PMC      *child_class, *parents, *temp_pmc, *mro;
     SLOTTYPE *child_class_array;
@@ -595,14 +615,15 @@ Parrot_single_subclass(PARROT_INTERP, PMC *base_class, NULLOK(PMC *name))
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_new_class
+
 Creates a new class, named C<class_name>.
 
 */
 
 PARROT_API
 void
-Parrot_new_class(PARROT_INTERP, PMC *_class, PMC *name)
+Parrot_new_class(PARROT_INTERP, NOTNULL(PMC *_class), NOTNULL(PMC *name))
 {
     SLOTTYPE *class_array;
     PMC      *mro;
@@ -643,15 +664,18 @@ Parrot_new_class(PARROT_INTERP, PMC *_class, PMC *name)
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_class_lookup
+
 Looks for the class named C<class_name> and returns it if it exists.
 Otherwise it returns C<PMCNULL>.
 
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_class_lookup(PARROT_INTERP, STRING *class_name)
+Parrot_class_lookup(PARROT_INTERP, NOTNULL(STRING *class_name))
 {
     const INTVAL type = pmc_type(interp, class_name);
     PMC         *pmc;
@@ -660,20 +684,22 @@ Parrot_class_lookup(PARROT_INTERP, STRING *class_name)
         return PMCNULL;
 
     pmc = interp->vtables[type]->pmc_class;
-    assert(pmc);
+    PARROT_ASSERT(pmc);
     return pmc;
 }
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_class_lookup_p
 Looks for the class named C<class_name> and returns it if it exists.
 Otherwise it returns C<PMCNULL>.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_class_lookup_p(PARROT_INTERP, PMC *class_name)
+Parrot_class_lookup_p(PARROT_INTERP, NOTNULL(PMC *class_name))
 {
     const INTVAL type = pmc_type_p(interp, class_name);
     PMC         *pmc;
@@ -682,7 +708,7 @@ Parrot_class_lookup_p(PARROT_INTERP, PMC *class_name)
         return PMCNULL;
 
     pmc = interp->vtables[type]->pmc_class;
-    assert(pmc);
+    PARROT_ASSERT(pmc);
     return pmc;
 }
 
@@ -830,6 +856,7 @@ parrot_class_register(PARROT_INTERP, NOTNULL(PMC *name),
 }
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 get_init_meth(PARROT_INTERP, NOTNULL(PMC *_class),
         NOTNULL(STRING *prop_str), NOTNULL(STRING **meth_str))
@@ -912,7 +939,7 @@ do_initcall(PARROT_INTERP, NULLOK(PMC* _class), NULLOK(PMC *object),
              * but only if init isn't inherited
              * or rather just on the last non-class parent
              */
-            assert(i >= 1);
+            PARROT_ASSERT(i >= 1);
             next_parent = VTABLE_get_pmc_keyed_int(interp,
                     classsearch_array, i - 1);
             if (!PObj_is_class_TEST(next_parent))
@@ -1113,6 +1140,8 @@ class_mro_merge(PARROT_INTERP, NOTNULL(PMC *seqs))
 }
 
 /* create C3 MRO */
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PMC*
 create_class_mro(PARROT_INTERP, NOTNULL(PMC *_class))
 {
@@ -1145,7 +1174,7 @@ create_class_mro(PARROT_INTERP, NOTNULL(PMC *_class))
 
 PARROT_API
 void
-Parrot_add_parent(PARROT_INTERP, PMC *_class, PMC *parent)
+Parrot_add_parent(PARROT_INTERP, NOTNULL(PMC *_class), NOTNULL(PMC *parent))
 {
     PMC *current_parent_array;
 
@@ -1191,6 +1220,8 @@ This currently does nothing but return C<PMCNULL>.
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
 Parrot_remove_parent(PARROT_INTERP, NOTNULL(PMC *removed_class),
         NOTNULL(PMC *existing_class))
@@ -1210,6 +1241,8 @@ This currently does nothing but return C<PMCNULL>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
 Parrot_multi_subclass(PARROT_INTERP, NOTNULL(PMC *base_class_array),
     NOTNULL(STRING *child_class_name))
@@ -1229,6 +1262,7 @@ Returns whether the object C<pmc> is an instance of class C<_class>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 INTVAL
 Parrot_object_isa(PARROT_INTERP, NOTNULL(PMC *pmc), NOTNULL(PMC *_class))
 {
@@ -1260,6 +1294,8 @@ Currently it does nothing but return C<PMCNULL>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
 Parrot_new_method_cache(PARROT_INTERP)
 {
@@ -1390,8 +1426,10 @@ interpreter, and name of the method. Don't use a possible method cache.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_find_method_direct(PARROT_INTERP, PMC *_class, STRING *method_name)
+Parrot_find_method_direct(PARROT_INTERP, NOTNULL(PMC *_class), NOTNULL(STRING *method_name))
 {
     PMC * const found = find_method_direct_1(interp, _class, method_name);
     STRING *s1, *s2;
@@ -1422,15 +1460,17 @@ the name in the global stash.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_find_method_with_cache(PARROT_INTERP, PMC *_class, NOTNULL(STRING *method_name))
+Parrot_find_method_with_cache(PARROT_INTERP, NOTNULL(PMC *_class), NOTNULL(STRING *method_name))
 {
     UINTVAL type, bits;
 
     Caches           *mc;
     Meth_cache_entry *e, *old;
 
-    assert(method_name != 0);
+    PARROT_ASSERT(method_name != 0);
 
 #if DISABLE_METH_CACHE
     return Parrot_find_method_direct(interp, _class, method_name);
@@ -1526,6 +1566,7 @@ debug_trace_find_meth(PARROT_INTERP, NOTNULL(PMC *_class),
 #endif
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC *
 find_method_direct_1(PARROT_INTERP, NOTNULL(PMC *_class),
                               NOTNULL(STRING *method_name))
@@ -1551,7 +1592,8 @@ find_method_direct_1(PARROT_INTERP, NOTNULL(PMC *_class),
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_note_method
+
 Notes where in the hierarchy we just found a method. Used so that we
 can do a next and continue the search through the hierarchy for the
 next instance of this method.
@@ -1560,7 +1602,8 @@ next instance of this method.
 
 PARROT_API
 void
-Parrot_note_method_offset(PARROT_INTERP, UINTVAL offset, PMC *method) {
+Parrot_note_method_offset(PARROT_INTERP, UINTVAL offset, NOTNULL(PMC *method))
+{
     UNUSED(interp);
     UNUSED(offset);
     UNUSED(method);
@@ -1568,7 +1611,8 @@ Parrot_note_method_offset(PARROT_INTERP, UINTVAL offset, PMC *method) {
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_add_attribute
+
 Adds the attribute C<attr> to the class.
 
    Life is ever so much easier if a class keeps its attributes at the
@@ -1581,7 +1625,7 @@ Adds the attribute C<attr> to the class.
 
 PARROT_API
 INTVAL
-Parrot_add_attribute(PARROT_INTERP, PMC* _class, STRING* attr)
+Parrot_add_attribute(PARROT_INTERP, NOTNULL(PMC *_class), NOTNULL(STRING *attr))
 {
     STRING *full_attr_name;
     SLOTTYPE * const class_array = (SLOTTYPE *)PMC_data(_class);
@@ -1627,6 +1671,8 @@ is asking for the correct attribute number.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
 Parrot_get_attrib_by_num(PARROT_INTERP, NOTNULL(PMC *object), INTVAL attrib)
 {
@@ -1706,8 +1752,10 @@ Returns attribute with full qualified name C<attr> from C<object>.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 PMC *
-Parrot_get_attrib_by_str(PARROT_INTERP, PMC *object, STRING *attr)
+Parrot_get_attrib_by_str(PARROT_INTERP, NOTNULL(PMC *object), NOTNULL(STRING *attr))
 {
     return Parrot_get_attrib_by_num(interp, object,
                 attr_str_2_num(interp, object, attr));
@@ -1754,7 +1802,7 @@ Parrot_set_attrib_by_str(PARROT_INTERP, NOTNULL(PMC *object),
 
 PARROT_API
 INTVAL
-Parrot_class_offset(PARROT_INTERP, PMC *object, STRING *_class)
+Parrot_class_offset(PARROT_INTERP, NOTNULL(PMC *object), NOTNULL(STRING *_class))
 {
     PMC *class_pmc, *mro, *attribs;
     INTVAL offset, i, n, attr_count;
@@ -1801,8 +1849,9 @@ undefined, is pretty likely).
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC *
-Parrot_find_class_constructor(PARROT_INTERP, STRING *_class,
+Parrot_find_class_constructor(PARROT_INTERP, NOTNULL(STRING *_class),
                                    INTVAL classtoken)
 {
     UNUSED(interp);
@@ -1813,8 +1862,9 @@ Parrot_find_class_constructor(PARROT_INTERP, STRING *_class,
 }
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC *
-Parrot_find_class_destructor(PARROT_INTERP, STRING *_class,
+Parrot_find_class_destructor(PARROT_INTERP, NOTNULL(STRING *_class),
                                   INTVAL classtoken)
 {
     UNUSED(interp);
@@ -1825,8 +1875,9 @@ Parrot_find_class_destructor(PARROT_INTERP, STRING *_class,
 }
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 PMC *
-Parrot_find_class_fallback(PARROT_INTERP, STRING *_class,
+Parrot_find_class_fallback(PARROT_INTERP, NOTNULL(STRING *_class),
                                 INTVAL classtoken)
 {
     UNUSED(interp);
@@ -1838,8 +1889,8 @@ Parrot_find_class_fallback(PARROT_INTERP, STRING *_class,
 
 PARROT_API
 void
-Parrot_set_class_constructor(PARROT_INTERP, STRING *_class,
-                                  INTVAL classtoken, STRING *method)
+Parrot_set_class_constructor(PARROT_INTERP, NOTNULL(STRING *_class),
+                                  INTVAL classtoken, NOTNULL(STRING *method))
 {
     UNUSED(interp);
     UNUSED(_class);
@@ -1849,8 +1900,8 @@ Parrot_set_class_constructor(PARROT_INTERP, STRING *_class,
 
 PARROT_API
 void
-Parrot_set_class_destructor(PARROT_INTERP, STRING *_class,
-                                 INTVAL classtoken, STRING *method)
+Parrot_set_class_destructor(PARROT_INTERP, NOTNULL(STRING *_class),
+                                 INTVAL classtoken, NOTNULL(STRING *method))
 {
     UNUSED(interp);
     UNUSED(_class);
@@ -1860,8 +1911,8 @@ Parrot_set_class_destructor(PARROT_INTERP, STRING *_class,
 
 PARROT_API
 void
-Parrot_set_class_fallback(PARROT_INTERP, STRING *_class,
-                               INTVAL classtoken, STRING *method)
+Parrot_set_class_fallback(PARROT_INTERP, NOTNULL(STRING *_class),
+                               INTVAL classtoken, NOTNULL(STRING *method))
 {
     UNUSED(interp);
     UNUSED(_class);
@@ -1874,6 +1925,8 @@ Parrot_set_class_fallback(PARROT_INTERP, STRING *_class,
 /* ********* BELOW HERE IS NEW PPD15 IMPLEMENTATION RELATED STUFF ********* */
 /* ************************************************************************ */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 C3_merge(PARROT_INTERP, NOTNULL(PMC *merge_list))
 {
@@ -1959,17 +2012,19 @@ C3_merge(PARROT_INTERP, NOTNULL(PMC *merge_list))
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_ComputeMRO_C3
 Computes the C3 linearization for the given class.
 
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
-Parrot_ComputeMRO_C3(PARROT_INTERP, PMC *_class)
+Parrot_ComputeMRO_C3(PARROT_INTERP, NOTNULL(PMC *_class))
 {
     PMC *result;
-    PMC *merge_list = pmc_new(interp, enum_class_ResizablePMCArray);
+    PMC * const merge_list = pmc_new(interp, enum_class_ResizablePMCArray);
     PMC *immediate_parents;
     int i, parent_count;
 
@@ -1978,11 +2033,9 @@ Parrot_ComputeMRO_C3(PARROT_INTERP, PMC *_class)
          string_from_literal(interp, "parents"),
         "->P", &immediate_parents);
 
-    if (immediate_parents == NULL) {
+    if (immediate_parents == NULL)
         real_exception(interp, NULL, METH_NOT_FOUND,
             "Failed to get parents list from class!");
-        return PMCNULL;
-    }
 
     parent_count = VTABLE_elements(interp, immediate_parents);
 
@@ -2022,7 +2075,8 @@ Parrot_ComputeMRO_C3(PARROT_INTERP, PMC *_class)
 
 /*
 
-FUNCDOC:
+FUNCDOC: Parrot_ComposeRole
+
 Used by the Class and Object PMCs internally to compose a role into either of
 them. The C<role> parameter is the role that we are composing into the class
 or role. C<methods_hash> is the hash of method names to invokable PMCs that
@@ -2040,10 +2094,10 @@ the default implementation.
 
 PARROT_API
 void
-Parrot_ComposeRole(PARROT_INTERP, PMC *role,
-                        PMC *exclude, int got_exclude,
-                        PMC *alias,   int got_alias,
-                        PMC *methods_hash, PMC *roles_list)
+Parrot_ComposeRole(PARROT_INTERP, NOTNULL(PMC *role),
+                        NOTNULL(PMC *exclude), int got_exclude,
+                        NOTNULL(PMC *alias),   int got_alias,
+                        NOTNULL(PMC *methods_hash), NOTNULL(PMC *roles_list))
 {
     PMC *methods;
     PMC *methods_iter;
