@@ -2,7 +2,7 @@
 Copyright (C) 2001-2007, The Perl Foundation.
 This program is free software. It is subject to the same license as
 Parrot itself.
-$Id: packdump.c 18952 2007-06-12 17:02:45Z fperrad $
+$Id: packdump.c 19833 2007-07-13 22:02:25Z petdance $
 
 =head1 NAME
 
@@ -19,11 +19,19 @@ This is only used by the PBC dumper C<pdump>.
 #include "parrot/parrot.h"
 #include "parrot/packfile.h"
 
-/* HEADER: include/parrot/packfile.h */
+/* HEADERIZER HFILE: include/parrot/packfile.h */
+
+/* HEADERIZER BEGIN: static */
+
+static void PackFile_Constant_dump( PARROT_INTERP,
+    NOTNULL(PackFile_ConstTable *ct),
+    NOTNULL(PackFile_Constant *self) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+/* HEADERIZER END: static */
 
-static void
-PackFile_Constant_dump(Interp *interp /*NN*/, PackFile_ConstTable *ct /*NN*/,
-                       PackFile_Constant *self /*NN*/);
 
 /*
 
@@ -35,7 +43,7 @@ Dumps the constant table C<self>.
 
 PARROT_API
 void
-PackFile_ConstTable_dump(Interp *interp /*NN*/, const PackFile_ConstTable *self /*NN*/)
+PackFile_ConstTable_dump(PARROT_INTERP, NOTNULL(const PackFile_ConstTable *self))
 {
     opcode_t i;
 
@@ -54,8 +62,8 @@ Dumps the constant C<self>.
 */
 
 static void
-PackFile_Constant_dump(Interp *interp /*NN*/, PackFile_ConstTable *ct /*NN*/,
-                       PackFile_Constant *self /*NN*/)
+PackFile_Constant_dump(PARROT_INTERP, NOTNULL(PackFile_ConstTable *ct),
+                       NOTNULL(PackFile_Constant *self))
 {
     PMC *key;
     size_t i;
@@ -273,7 +281,7 @@ Dumps the fix-up table C<ft>.
 
 PARROT_API
 void
-PackFile_Fixup_dump(Interp *interp /*NN*/, const PackFile_FixupTable *ft /*NN*/)
+PackFile_Fixup_dump(PARROT_INTERP, NOTNULL(const PackFile_FixupTable *ft))
 {
     opcode_t i;
 
