@@ -2,7 +2,7 @@
 Copyright (C) 2001-2007, The Perl Foundation.
 This program is free software. It is subject to the same license as
 Parrot itself.
-$Id: packfile.c 19914 2007-07-16 21:06:48Z petdance $
+$Id: packfile.c 20711 2007-08-19 07:34:15Z chromatic $
 
 =head1 NAME
 
@@ -24,6 +24,10 @@ structure of the frozen bytecode.
 #include "parrot/embed.h"
 #include "parrot/packfile.h"
 #include "jit.h"
+#include "../compilers/imcc/imc.h"
+
+/* XXX Declare this elsewhere */
+void * IMCC_compile_file(PARROT_INTERP, const char *s);
 
 /* HEADERIZER HFILE: include/parrot/packfile.h */
 
@@ -32,10 +36,12 @@ structure of the frozen bytecode.
 static void byte_code_destroy( SHIM_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * byte_code_new( SHIM_INTERP,
-    PackFile *pf,
-    const char *name,
-    int add );
+    SHIM(PackFile *pf),
+    SHIM(const char *name),
+    SHIM(int add) );
 
 static void const_destroy( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(1)
@@ -44,10 +50,12 @@ static void const_destroy( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
 PARROT_MALLOC
 PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * const_new( SHIM_INTERP,
-    PackFile *pf,
-    const char *name,
-    int add );
+    SHIM(PackFile *pf),
+    SHIM(const char *name),
+    SHIM(int add) );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * create_seg( PARROT_INTERP,
     NOTNULL(PackFile_Directory *dir),
     pack_file_types t,
@@ -66,6 +74,8 @@ static void default_dump( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * default_pack(
     NOTNULL(const PackFile_Segment *self),
     NOTNULL(opcode_t *dest) )
@@ -76,6 +86,7 @@ static size_t default_packed_size( NOTNULL(const PackFile_Segment *self) )
         __attribute__nonnull__(1);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t * default_unpack(
     NOTNULL(PackFile_Segment *self),
     NOTNULL(opcode_t *cursor) )
@@ -91,11 +102,15 @@ static void directory_dump( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * directory_new( SHIM_INTERP,
-    PackFile *pf,
-    const char *name,
-    int add );
+    SHIM(PackFile *pf),
+    SHIM(const char *name),
+    SHIM(int add) );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * directory_pack( PARROT_INTERP,
     NOTNULL(PackFile_Segment *self),
     NOTNULL(opcode_t *cursor) )
@@ -108,6 +123,8 @@ static size_t directory_packed_size( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * directory_unpack( PARROT_INTERP,
     NOTNULL(PackFile_Segment *segp),
     NOTNULL(opcode_t *cursor) )
@@ -115,6 +132,8 @@ static opcode_t * directory_unpack( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* do_1_sub_pragma( PARROT_INTERP,
     NOTNULL(PMC *sub_pmc),
     int action )
@@ -123,10 +142,12 @@ static PMC* do_1_sub_pragma( PARROT_INTERP,
 
 static INTVAL find_const_iter( PARROT_INTERP,
     NOTNULL(PackFile_Segment *seg),
-    void *user_data )
+    NULLOK(void *user_data) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Constant ** find_constants( PARROT_INTERP,
     NOTNULL(PackFile_ConstTable *ct) )
         __attribute__nonnull__(1)
@@ -143,19 +164,24 @@ static PackFile_FixupEntry * find_fixup(
 
 static INTVAL find_fixup_iter( PARROT_INTERP,
     NOTNULL(PackFile_Segment *seg),
-    void *user_data )
+    NOTNULL(void *user_data) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 static void fixup_destroy( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * fixup_new( SHIM_INTERP,
-    PackFile *pf,
-    const char *name,
-    int add );
+    SHIM(PackFile *pf),
+    SHIM(const char *name),
+    SHIM(int add) );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * fixup_pack( PARROT_INTERP,
     NOTNULL(PackFile_Segment *self),
     NOTNULL(opcode_t *cursor) )
@@ -183,6 +209,8 @@ static void mark_1_seg( PARROT_INTERP, NOTNULL(PackFile_ConstTable *ct) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PackFile * PackFile_append_pbc( PARROT_INTERP,
     NULLOK(const char *filename) )
         __attribute__nonnull__(1);
@@ -197,11 +225,15 @@ static void pf_debug_dump( PARROT_INTERP, NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment * pf_debug_new( SHIM_INTERP,
-    PackFile *pf,
-    const char *name,
-    int add );
+    SHIM(PackFile *pf),
+    SHIM(const char *name),
+    SHIM(int add) );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * pf_debug_pack( SHIM_INTERP,
     NOTNULL(PackFile_Segment *self),
     NOTNULL(opcode_t *cursor) )
@@ -212,6 +244,8 @@ static size_t pf_debug_packed_size( SHIM_INTERP,
     NOTNULL(PackFile_Segment *self) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t * pf_debug_unpack( PARROT_INTERP,
     NOTNULL(PackFile_Segment *self),
     NOTNULL(opcode_t *cursor) )
@@ -224,6 +258,8 @@ static INTVAL pf_register_standard_funcs( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* run_sub( PARROT_INTERP, NOTNULL(PMC *sub_pmc) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -247,8 +283,9 @@ static int sub_pragma( PARROT_INTERP,
 
 /* HEADERIZER END: static */
 
-
-#include <assert.h>
+#if EXEC_CAPABLE
+    extern int Parrot_exec_run;
+#endif
 
 #define TRACE_PACKFILE 0
 #define TRACE_PACKFILE_PMC 0
@@ -380,6 +417,8 @@ Run the B<sub_pmc> due its B<:load>, B<:immediate>, ... pragma
 
 */
 
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 run_sub(PARROT_INTERP, NOTNULL(PMC *sub_pmc))
 {
@@ -411,6 +450,8 @@ Run autoloaded or immediate bytecode, mark MAIN subroutine entry
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 do_1_sub_pragma(PARROT_INTERP, NOTNULL(PMC *sub_pmc), int action)
 {
@@ -464,10 +505,10 @@ do_1_sub_pragma(PARROT_INTERP, NOTNULL(PMC *sub_pmc), int action)
                 if ((interp->resume_flag & RESUME_INITIAL) &&
                         interp->resume_offset == 0) {
                     const ptrdiff_t code = (ptrdiff_t) sub->seg->base.data;
+                    void           *ptr  = VTABLE_get_pointer(interp, sub_pmc);
 
-                    const size_t start_offs =
-                        ((ptrdiff_t) VTABLE_get_pointer(interp, sub_pmc)
-                         - code) / sizeof (opcode_t*);
+                    const size_t start_offs = ((ptrdiff_t)ptr - code) /
+                        sizeof (opcode_t*);
                     interp->resume_offset = start_offs;
                     PObj_get_FLAGS(sub_pmc) &= ~SUB_FLAG_PF_MAIN;
                     CONTEXT(interp->ctx)->current_sub = sub_pmc;
@@ -514,7 +555,7 @@ mark_1_seg(PARROT_INTERP, NOTNULL(PackFile_ConstTable *ct))
 }
 
 static INTVAL
-find_const_iter(PARROT_INTERP, NOTNULL(PackFile_Segment *seg), void *user_data)
+find_const_iter(PARROT_INTERP, NOTNULL(PackFile_Segment *seg), NULLOK(void *user_data))
 {
     if (seg->type == PF_DIR_SEG) {
         PackFile_map_segments(interp, (PackFile_Directory *)seg,
@@ -557,7 +598,7 @@ the eval PMC is kept alive be living subs.
 PARROT_API
 void
 do_sub_pragmas(PARROT_INTERP, NOTNULL(PackFile_ByteCode *self),
-        int action, PMC *eval_pmc)
+        int action, NULLOK(PMC *eval_pmc))
 {
     opcode_t i;
     PackFile_FixupTable * const ft = self->fixups;
@@ -613,77 +654,63 @@ do_sub_pragmas(PARROT_INTERP, NOTNULL(PackFile_ByteCode *self),
 
 FUNCDOC: PackFile_unpack
 
-Unpack a C<PackFile> from a block of memory. The format is:
-
-  byte     wordsize
-  byte     byteorder
-  byte     major
-  byte     minor
-  byte     intvalsize
-  byte     floattype
-  byte     pad[10] = fingerprint
-
-  opcode_t magic
-  opcode_t language type
-
-  opcode_t dir_format
-  opcode_t padding
-
-  directory segment
-    * segment
-    ...
-
-All segments have this common header:
-
-  - op_count    ... total segment size incl. this count
-  - itype       ... internal type of data
-  - id          ... id of data e.g. byte code nr.
-  - size        ... size of data oparray
-  - data[size]  ... data array e.g. bytecode
-  segment specific data follows here ...
-
-Checks to see if the magic matches the Parrot magic number for
-Parrot C<PackFiles>.
+Unpack a C<PackFile> from a block of memory, ensuring the the magic number is
+valid, the bytecode version can be read by this Parrot and doing any endian
+and word size transforms that are required.
 
 Returns size of unpacked if everything is OK, else zero (0).
 
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
 opcode_t
-PackFile_unpack(PARROT_INTERP, NOTNULL(PackFile *self), opcode_t *packed,
+PackFile_unpack(PARROT_INTERP, NOTNULL(PackFile *self), NOTNULL(opcode_t *packed),
     size_t packed_size)
 {
     PackFile_Header * const header = self->header;
     opcode_t *cursor;
+    int header_read_length;
+    opcode_t padding;
 
-    if (!self) {
-        PIO_eprintf(NULL, "PackFile_unpack: self == NULL!\n");
-        return 0;
-    }
-    self->src = packed;
+    self->src  = packed;
     self->size = packed_size;
 
-    /*
-     * Map the header on top of the buffer later when we are sure
-     * we have alignment done right.
-     */
-    cursor = packed + PACKFILE_HEADER_BYTES/sizeof (opcode_t);
+    /* Extract the header. */
     memcpy(header, packed, PACKFILE_HEADER_BYTES);
 
+    /* Ensure the magic is correct. */
+    if (memcmp(header->magic, "\376PBC\r\n\032\n", 8) != 0) {
+        PIO_eprintf(NULL, "PackFile_unpack: This is not a valid Parrot bytecode file\n");
+        return 0;
+    }
+
+    /* Ensure the bytecode version is one we can read. Currently, we only
+     * support bytecode versions matching the current one. */
+    if (header->bc_major != PARROT_PBC_MAJOR && header->bc_minor != PARROT_PBC_MINOR) {
+        PIO_eprintf(NULL, "PackFile_unpack: This Parrot cannot read bytecode "
+            "files with version %d.%d.\n", header->bc_major, header->bc_minor);
+        return 0;
+    }
+
+    /* Check wordsize, byte order and floating point number type are valid. */
     if (header->wordsize != 4 && header->wordsize != 8) {
         PIO_eprintf(NULL, "PackFile_unpack: Invalid wordsize %d\n",
                     header->wordsize);
         return 0;
     }
+    if (header->byteorder != 0 && header->byteorder != 1) {
+        PIO_eprintf(NULL, "PackFile_unpack: Invalid byte ordering %d\n",
+                    header->byteorder);
+        return 0;
+    }
     if (header->floattype != 0 && header->floattype != 1) {
         PIO_eprintf(NULL, "PackFile_unpack: Invalid floattype %d\n",
                     header->floattype);
         return 0;
     }
 
-    PackFile_assign_transforms(self);
-
+    /* Describe what was read for debugging. */
 #if TRACE_PACKFILE
     PIO_eprintf(NULL, "PackFile_unpack: Wordsize %d.\n", header->wordsize);
     PIO_eprintf(NULL, "PackFile_unpack: Floattype %d (%s).\n",
@@ -695,68 +722,50 @@ PackFile_unpack(PARROT_INTERP, NOTNULL(PackFile *self), opcode_t *packed,
                 header->byteorder, header->byteorder ? "big " : "little-");
 #endif
 
-    if (header->major != PARROT_MAJOR_VERSION ||
-            header->minor != PARROT_MINOR_VERSION) {
-        PIO_eprintf(NULL, "PackFile_unpack: Bytecode not valid for this "
-                "interpreter: version mismatch\n");
-        return 0;
+    /* Check the UUID type is valid and, if needed, read a UUID. */
+    if (header->uuid_type == 0) {
+        /* No UUID; fine, nothing more to do. */
+    }
+    else if (header->uuid_type == 1) {
+        /* Read in the UUID. We'll put it in a NULL-terminated string, just in
+         * case pepole use it that way. */
+        header->uuid_data = (unsigned char *)mem_sys_allocate(header->uuid_size + 1);
+        memcpy(header->uuid_data, packed + PACKFILE_HEADER_BYTES,
+            header->uuid_size);
+        header->uuid_data[header->uuid_size] = 0; /* NULL terminate */
+    }
+    else
+    {
+        /* Don't know this UUID type. */
+        PIO_eprintf(NULL, "PackFile_unpack: Invalid UUID type %d\n",
+                    header->uuid_type);
     }
 
-    /* check the fingerprint */
-    if (!PackFile_check_fingerprint(header->pad)) {
-        PIO_eprintf(NULL, "PackFile_unpack: Bytecode not valid for this "
-                    "interpreter: fingerprint mismatch\n");
-        return 0;
-    }
-    /*
-     * Unpack and verify the magic which is stored byteorder of the file:
-     */
-    header->magic = PF_fetch_opcode(self, &cursor);
+    /* Set cursor to position after what we've read, allowing for padding to a
+     * 16 byte boundary. */
+    header_read_length = PACKFILE_HEADER_BYTES + header->uuid_size;
+    header_read_length += header_read_length % 16 ?
+        16 - header_read_length % 16 : 0;
+    cursor = packed + (header_read_length / sizeof(opcode_t));
 
-    /*
-     * The magic and opcodetype fields are in native byteorder.
-     */
-    if (header->magic != PARROT_MAGIC) {
-        PIO_eprintf(NULL, "PackFile_unpack: Not a Parrot PackFile!\n");
-        PIO_eprintf(NULL, "Magic number was 0x%08x not 0x%08x\n",
-                    header->magic, PARROT_MAGIC);
-        return 0;
-    }
+    /* Set what transforms we need to do when reading the rest of
+     * the file. */
+    PackFile_assign_transforms(self);
 
-#if TRACE_PACKFILE
-    PIO_eprintf(NULL, "PackFile_unpack: Magic 0x%08x.\n",
-                header->magic);
-#endif
-
-    header->opcodetype = PF_fetch_opcode(self, &cursor);
-
-#if TRACE_PACKFILE
-    PIO_eprintf(NULL, "PackFile_unpack: Opcodetype 0x%x.\n",
-                header->opcodetype);
-#endif
-
-    /*
-     * Unpack the dir_format
-     */
-
-#if TRACE_PACKFILE
-    PIO_eprintf(NULL, "PackFile_unpack: Directory, offset %d.\n",
-                (INTVAL)cursor - (INTVAL)packed);
-#endif
+    /* Directory format. */
     header->dir_format = PF_fetch_opcode(self, &cursor);
-
-    /* dir_format 1 use directory */
     if (header->dir_format != PF_DIR_FORMAT) {
         PIO_eprintf(NULL,
                 "PackFile_unpack: Dir format was %d not %d\n",
                     header->dir_format, PF_DIR_FORMAT);
         return 0;
     }
-#if TRACE_PACKFILE
-    PIO_eprintf(NULL, "PackFile_unpack: Dirformat %d.\n", header->dir_format);
-#endif
 
-    (void)PF_fetch_opcode(self, &cursor); /* padding */
+    /* Padding. */
+    padding = PF_fetch_opcode(self, &cursor);
+    padding = PF_fetch_opcode(self, &cursor);
+    padding = PF_fetch_opcode(self, &cursor);
+
 #if TRACE_PACKFILE
     PIO_eprintf(NULL, "PackFile_unpack: Directory read, offset %d.\n",
                 (INTVAL)cursor - (INTVAL)packed);
@@ -802,7 +811,7 @@ PARROT_API
 INTVAL
 PackFile_map_segments(PARROT_INTERP, NOTNULL(PackFile_Directory *dir),
                        PackFile_map_segments_func_t callback,
-                       void *user_data)
+                       NULLOK(void *user_data))
 {
     size_t i;
 
@@ -858,6 +867,8 @@ returned, but its still owned by the C<PackFile>.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PackFile_Segment *
 PackFile_find_segment(PARROT_INTERP, NOTNULL(PackFile_Directory *dir),
     NOTNULL(const char *name), int sub_dir)
@@ -868,14 +879,14 @@ PackFile_find_segment(PARROT_INTERP, NOTNULL(PackFile_Directory *dir),
         return NULL;
     for (i=0; i < dir->num_segments; i++) {
         PackFile_Segment *seg = dir->segments[i];
-        if (seg && (strcmp(seg->name, name) == 0)) {
-            return seg;
-        }
-        if (sub_dir && seg->type == PF_DIR_SEG) {
-            seg = PackFile_find_segment(interp,
-                    (PackFile_Directory *)seg, name, sub_dir);
-            if (seg)
+        if (seg) {
+            if (strcmp(seg->name, name) == 0)
                 return seg;
+            if (sub_dir && seg->type == PF_DIR_SEG) {
+                seg = PackFile_find_segment(interp, (PackFile_Directory *)seg, name, sub_dir);
+                if (seg)
+                    return seg;
+            }
         }
     }
 
@@ -893,6 +904,8 @@ the user.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PackFile_Segment *
 PackFile_remove_segment_by_name(SHIM_INTERP, NOTNULL(PackFile_Directory *dir),
                                 NOTNULL(const char *name))
@@ -929,17 +942,18 @@ Fill a C<PackFile> header with system specific data.
 static void
 PackFile_set_header(NOTNULL(PackFile *self))
 {
+    memcpy(self->header->magic, "\376PBC\r\n\032\n", 8);
     self->header->wordsize = sizeof (opcode_t);
     self->header->byteorder = PARROT_BIGENDIAN;
     self->header->major = PARROT_MAJOR_VERSION;
     self->header->minor = PARROT_MINOR_VERSION;
-    self->header->intvalsize = sizeof (INTVAL);
+    self->header->patch = PARROT_PATCH_VERSION;
+    self->header->bc_major = PARROT_PBC_MAJOR;
+    self->header->bc_minor = PARROT_PBC_MINOR;
     if (NUMVAL_SIZE == 8)
         self->header->floattype = 0;
     else /* if XXX */
         self->header->floattype = 1;
-    /* write the fingerprint */
-    PackFile_write_fingerprint(self->header->pad);
 }
 
 /*
@@ -989,6 +1003,8 @@ A Segment Header has these entries:
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PackFile *
 PackFile_new(PARROT_INTERP, INTVAL is_mapped)
 {
@@ -1036,10 +1052,13 @@ doesn't load any bytecode, but is using Parrot_compile_string.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PackFile *
 PackFile_new_dummy(PARROT_INTERP, NOTNULL(const char *name))
 {
     PackFile * const pf = PackFile_new(interp, 0);
+    /* XXX PackFile_new needs to die on NULL, or else we have to check here */
 
     interp->initial_pf = pf;
     interp->code =
@@ -1074,15 +1093,14 @@ The default unpack function.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t *
 default_unpack(NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
 {
-    if (self->pf->header->dir_format) {
-        self->op_count = PF_fetch_opcode(self->pf, &cursor);
-        self->itype = PF_fetch_opcode(self->pf, &cursor);
-        self->id = PF_fetch_opcode(self->pf, &cursor);
-        self->size = PF_fetch_opcode(self->pf, &cursor);
-    }
+    self->op_count = PF_fetch_opcode(self->pf, &cursor);
+    self->itype = PF_fetch_opcode(self->pf, &cursor);
+    self->id = PF_fetch_opcode(self->pf, &cursor);
+    self->size = PF_fetch_opcode(self->pf, &cursor);
     if (self->size == 0)
         return cursor;
     /* if the packfile is mmap()ed just point to it if we don't
@@ -1101,7 +1119,7 @@ default_unpack(NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
         PIO_eprintf(NULL,
                 "PackFile_unpack: Unable to allocate data memory!\n");
         self->size = 0;
-        return 0;
+        return NULL;
     }
 
     if (!self->pf->need_endianize && !self->pf->need_wordsize) {
@@ -1245,6 +1263,8 @@ Create a new segment.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PackFile_Segment *
 PackFile_Segment_new_seg(PARROT_INTERP, NOTNULL(PackFile_Directory *dir),
         UINTVAL type, NOTNULL(const char *name), int add)
@@ -1260,6 +1280,8 @@ PackFile_Segment_new_seg(PARROT_INTERP, NOTNULL(PackFile_Directory *dir),
     return seg;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment *
 create_seg(PARROT_INTERP, NOTNULL(PackFile_Directory *dir), pack_file_types t,
            NOTNULL(const char *name), NOTNULL(const char *file_name), int add)
@@ -1286,6 +1308,8 @@ adding the segments to the directory.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PackFile_ByteCode *
 PF_create_default_segs(PARROT_INTERP, NOTNULL(const char *file_name), int add)
 {
@@ -1345,8 +1369,11 @@ PackFile_Segment_packed_size(PARROT_INTERP, NOTNULL(PackFile_Segment *self))
 }
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t *
-PackFile_Segment_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), opcode_t *cursor)
+PackFile_Segment_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self),
+        NOTNULL(opcode_t *cursor))
 {
     PackFile_Segment_pack_func_t f =
         self->pf->PackFuncs[self->type].pack;
@@ -1373,9 +1400,11 @@ If a special is defined this gets called after.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 opcode_t *
 PackFile_Segment_unpack(PARROT_INTERP, NOTNULL(PackFile_Segment *self),
-                        opcode_t *cursor)
+        NOTNULL(opcode_t *cursor))
 {
     PackFile_Segment_unpack_func_t f =
         self->pf->PackFuncs[self->type].unpack;
@@ -1417,6 +1446,8 @@ Returns a new C<PackFile_Directory> cast as a C<PackFile_Segment>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment *
 directory_new(SHIM_INTERP, SHIM(PackFile *pf), SHIM(const char *name), SHIM(int add))
 {
@@ -1467,6 +1498,8 @@ Unpacks the directory.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 directory_unpack(PARROT_INTERP, NOTNULL(PackFile_Segment *segp), NOTNULL(opcode_t *cursor))
 {
@@ -1689,6 +1722,8 @@ Packs the directory C<self>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 directory_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
 {
@@ -1709,25 +1744,12 @@ directory_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t
     align = 16/sizeof (opcode_t);
     if (align && (cursor - self->pf->src) % align)
         cursor += align - (cursor - self->pf->src) % align;
+
     /* now pack all segments into new format */
     for (i = 0; i < dir->num_segments; i++) {
         PackFile_Segment * const seg = dir->segments[i];
-        const size_t size = seg->op_count;
 
-        PackFile_Segment_pack(interp, seg, cursor);
-        /*
-         * XXX somehow it's smelling fishy here:
-         * - either cursor is unaligned
-         * - or the return result of _pack doesn't match
-         *   expected size
-         *
-         * likely in combination with pbc_merge
-         *
-         * the relevant code with size check is visible in:
-         *
-         * svn diff -r15516:15517
-         */
-        cursor += size;
+        cursor = PackFile_Segment_pack(interp, seg, cursor);
     }
 
     return cursor;
@@ -1769,6 +1791,8 @@ Create a new default section.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PackFile_Segment *
 PackFile_Segment_new(SHIM_INTERP, SHIM(PackFile *pf), SHIM(const char *name), SHIM(int add))
 {
@@ -1829,6 +1853,8 @@ Performs the default pack.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 default_pack(NOTNULL(const PackFile_Segment *self), NOTNULL(opcode_t *dest))
 {
@@ -1836,9 +1862,8 @@ default_pack(NOTNULL(const PackFile_Segment *self), NOTNULL(opcode_t *dest))
     *dest++ = self->itype;
     *dest++ = self->id;
     *dest++ = self->size;
-    if (self->size) {
+    if (self->size)
         STRUCT_COPY_N(dest, self->data, self->size);
-    }
     return dest + self->size;
 }
 
@@ -1885,6 +1910,8 @@ C<pf> and C<add> are ignored.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment *
 byte_code_new(SHIM_INTERP, SHIM(PackFile *pf), SHIM(const char *name), SHIM(int add))
 {
@@ -1929,6 +1956,8 @@ C<pf> and C<add> ignored.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment *
 pf_debug_new(SHIM_INTERP, SHIM(PackFile *pf), SHIM(const char *name), SHIM(int add))
 {
@@ -1986,6 +2015,8 @@ Pack the debug segment.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 pf_debug_pack(SHIM_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
 {
@@ -2025,6 +2056,8 @@ Unpack a debug segment into a PackFile_Debug structure.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 pf_debug_unpack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
 {
@@ -2156,6 +2189,8 @@ Create and append (or resize) a new debug seg for a code segment.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PackFile_Debug *
 Parrot_new_debug_seg(PARROT_INTERP, NOTNULL(PackFile_ByteCode *cs), size_t size)
 {
@@ -2211,7 +2246,7 @@ PARROT_API
 void
 Parrot_debug_add_mapping(PARROT_INTERP, NOTNULL(PackFile_Debug *debug),
                          opcode_t offset, int mapping_type,
-                         const char *filename, int source_seg)
+                         NOTNULL(const char *filename), int source_seg)
 {
     PackFile_DebugMapping *mapping;
     PackFile_ConstTable * const ct = debug->code->const_table;
@@ -2288,6 +2323,8 @@ that position.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 Parrot_debug_pc_to_filename(PARROT_INTERP, NOTNULL(PackFile_Debug *debug), opcode_t pc)
 {
@@ -2359,6 +2396,8 @@ Switch to a byte code segment C<new_cs>, returning the old segment.
 */
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 PackFile_ByteCode *
 Parrot_switch_to_cs(PARROT_INTERP, NOTNULL(PackFile_ByteCode *new_cs), int really)
 {
@@ -2398,6 +2437,8 @@ subroutines need to reference namespace pointers.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Constant *
 clone_constant(PARROT_INTERP, NOTNULL(PackFile_Constant *old_const)) {
     STRING * const _sub = interp->vtables[enum_class_Sub]->whoami;
@@ -2427,6 +2468,8 @@ clone_constant(PARROT_INTERP, NOTNULL(PackFile_Constant *old_const)) {
     }
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Constant **
 find_constants(PARROT_INTERP, NOTNULL(PackFile_ConstTable *ct))
 {
@@ -2438,7 +2481,7 @@ find_constants(PARROT_INTERP, NOTNULL(PackFile_ConstTable *ct))
         Hash *tables;
         PackFile_Constant **new_consts;
 
-        assert(interp->thread_data);
+        PARROT_ASSERT(interp->thread_data);
 
         if (!interp->thread_data->const_tables) {
             interp->thread_data->const_tables = mem_allocate_typed(Hash);
@@ -2604,6 +2647,8 @@ I<What does this do?>
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static opcode_t *
 fixup_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cursor))
 {
@@ -2623,7 +2668,6 @@ fixup_pack(PARROT_INTERP, NOTNULL(PackFile_Segment *self), NOTNULL(opcode_t *cur
                 break;
             default:
                 real_exception(interp, NULL, 1, "Unknown fixup type\n");
-                return 0;
         }
     }
     return cursor;
@@ -2637,6 +2681,8 @@ Returns a new C<PackFile_FixupTable> segment.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static PackFile_Segment *
 fixup_new(SHIM_INTERP, SHIM(PackFile *pf), SHIM(const char *name), SHIM(int add))
 {
@@ -2785,7 +2831,7 @@ I<What does this do?>
 */
 
 static INTVAL
-find_fixup_iter(PARROT_INTERP, NOTNULL(PackFile_Segment *seg), void *user_data)
+find_fixup_iter(PARROT_INTERP, NOTNULL(PackFile_Segment *seg), NOTNULL(void *user_data))
 {
     if (seg->type == PF_DIR_SEG) {
         if (PackFile_map_segments(interp, (PackFile_Directory *)seg,
@@ -2819,15 +2865,13 @@ PackFile_FixupEntry *
 PackFile_find_fixup_entry(PARROT_INTERP, INTVAL type, char *name)
 {
     /* TODO make a hash of all fixups */
-    PackFile_Directory *dir = interp->code->base.dir;
-    PackFile_FixupEntry *ep, e;
+    PackFile_Directory * const dir = interp->code->base.dir;
+    PackFile_FixupEntry *ep        = mem_allocate_typed(PackFile_FixupEntry);
     int found;
 
-    e.type = type;
-    e.name = name;
-    ep = &e;
-    found = PackFile_map_segments(interp, dir, find_fixup_iter,
-            (void *) &ep);
+    ep->type = type;
+    ep->name = name;
+    found    = PackFile_map_segments(interp, dir, find_fixup_iter, (void *) ep);
     return found ? ep : NULL;
 }
 
@@ -2884,14 +2928,11 @@ PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
 opcode_t *
 PackFile_ConstTable_unpack(PARROT_INTERP, NOTNULL(PackFile_Segment *seg),
-                           opcode_t *cursor)
+        NOTNULL(opcode_t *cursor))
 {
     opcode_t i;
     PackFile_ConstTable * const self = (PackFile_ConstTable *)seg;
     PackFile * const pf              = seg->pf;
-#if EXEC_CAPABLE
-    extern int Parrot_exec_run;
-#endif
 
     PackFile_ConstTable_clear(interp, self);
 
@@ -3140,9 +3181,11 @@ Unpack a constant PMC.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t *
-PackFile_Constant_unpack_pmc(PARROT_INTERP, PackFile_ConstTable *constt,
-                             PackFile_Constant *self, opcode_t *cursor)
+PackFile_Constant_unpack_pmc(PARROT_INTERP, NOTNULL(PackFile_ConstTable *constt),
+        NOTNULL(PackFile_Constant *self), NOTNULL(opcode_t *cursor))
 {
     PackFile * const pf = constt->base.pf;
     STRING *image, *_sub;
@@ -3198,22 +3241,23 @@ Returns cursor if everything is OK, else zero (0).
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t *
 PackFile_Constant_unpack_key(PARROT_INTERP, NOTNULL(PackFile_ConstTable *constt),
-                             PackFile_Constant *self, opcode_t *cursor)
+        NOTNULL(PackFile_Constant *self), NOTNULL(opcode_t *cursor))
 {
-    PMC *head;
-    PMC *tail;
-    opcode_t op;
     PackFile * const pf = constt->base.pf;
     int pmc_enum = enum_class_Key;
 
     INTVAL components = (INTVAL)PF_fetch_opcode(pf, &cursor);
-    head = tail = NULL;
+    PMC *head = NULL;
+    PMC *tail = NULL;
 
     while (components-- > 0) {
         opcode_t type = PF_fetch_opcode(pf, &cursor);
         const opcode_t slice_bits = type & PF_VT_SLICE_BITS;
+        opcode_t op;
 
         type &= ~PF_VT_SLICE_BITS;
         if (!head && slice_bits) {
@@ -3281,6 +3325,8 @@ Fixup sub addresses in newly loaded bytecode and run :load subs.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PackFile *
 PackFile_append_pbc(PARROT_INTERP, NULLOK(const char *filename))
 {
@@ -3307,8 +3353,6 @@ directory.
 /*
  * intermediate hook during changes
  */
-/* XXX Declare this elsewhere */
-void * IMCC_compile_file(PARROT_INTERP, const char *s);
 
 PARROT_API
 void
@@ -3370,7 +3414,7 @@ If C<eval> is given, set this is the owner of the subroutines.
 
 PARROT_API
 void
-PackFile_fixup_subs(PARROT_INTERP, pbc_action_enum_t what, PMC *eval)
+PackFile_fixup_subs(PARROT_INTERP, pbc_action_enum_t what, NULLOK(PMC *eval))
 {
     do_sub_pragmas(interp, interp->code, what, eval);
 }
