@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: pf_items.c 19885 2007-07-15 04:02:08Z petdance $
+$Id: pf_items.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -26,7 +26,6 @@ C<opcode_t> units.
 */
 
 #include "parrot/parrot.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/packfile.h */
 
@@ -400,6 +399,7 @@ Store an C<INTVAL> to stream as is.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t*
 PF_store_integer(NOTNULL(opcode_t *cursor), INTVAL val)
 {
@@ -476,6 +476,8 @@ Write a C<FLOATVAL> to the opcode stream as is.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t*
 PF_store_number(NOTNULL(opcode_t *cursor), NOTNULL(const FLOATVAL *val))
 {
@@ -572,6 +574,8 @@ Write a STRING to the opcode stream.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t*
 PF_store_string(NOTNULL(opcode_t *cursor), NOTNULL(STRING *s))
 {
@@ -612,7 +616,7 @@ PF_store_string(NOTNULL(opcode_t *cursor), NOTNULL(STRING *s))
             }
         }
     }
-    assert(((long)charcursor & 3) == 0);
+    PARROT_ASSERT(((long)charcursor & 3) == 0);
     cursor = (opcode_t *)charcursor;
 
     return cursor;
@@ -649,18 +653,17 @@ Fetch a cstring from bytecode and return an allocated copy
 */
 
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 char *
 PF_fetch_cstring(NOTNULL(PackFile *pf), NOTNULL(opcode_t **cursor))
 {
     const size_t str_len = strlen ((char *)(*cursor)) + 1;
     char * const p = (char *)mem_sys_allocate(str_len);
 
-    if (p) {
-        const int wordsize = pf->header->wordsize;
+    const int wordsize = pf->header->wordsize;
 
-        strcpy(p, (char*) (*cursor));
-        *((unsigned char **) (cursor)) += ROUND_UP_B(str_len, wordsize);
-    }
+    strcpy(p, (char*) (*cursor));
+    *((unsigned char **) (cursor)) += ROUND_UP_B(str_len, wordsize);
 
     return p;
 }
@@ -696,7 +699,7 @@ PF_size_cstring(NOTNULL(const char *s))
 {
     size_t str_len;
 
-    assert(s);
+    PARROT_ASSERT(s);
     str_len = strlen(s);
     return ROUND_UP(str_len + 1, sizeof (opcode_t));
 }
