@@ -2,7 +2,7 @@
 Copyright (C) 2001-2007, The Perl Foundation.
 This program is free software. It is subject to the same license as
 Parrot itself.
-$Id: packout.c 19833 2007-07-13 22:02:25Z petdance $
+$Id: packout.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -16,7 +16,6 @@ src/packout.c - Functions for writing out packfiles
 
 #include "parrot/parrot.h"
 #include "parrot/packfile.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/packfile.h */
 
@@ -32,11 +31,17 @@ opcode_t
 PackFile_pack_size(PARROT_INTERP, NOTNULL(PackFile *self))
 {
     size_t size;
+    size_t header_size = 0;
     PackFile_Directory * const dir = &self->directory;
 
-    size = PACKFILE_HEADER_BYTES / sizeof (opcode_t);
+    header_size = PACKFILE_HEADER_BYTES;
+    header_size += self->header->uuid_size;
+    header_size += header_size % 16 ?
+        16 - header_size % 16 : 0;
 
-    size += 4; /* magic + opcode type + directory type + pad */
+    size = header_size / sizeof (opcode_t);
+
+    size += 4; /* directory type + 3 padding zeros */
 
     dir->base.file_offset = size;
     size += PackFile_Segment_packed_size(interp, (PackFile_Segment *) dir);
@@ -69,19 +74,44 @@ PackFile_pack(PARROT_INTERP, NOTNULL(PackFile *self), NOTNULL(opcode_t *cursor))
     size_t size;
     PackFile_Directory * const dir = &self->directory;
     PackFile_Segment *seg;
+    int padding_size;
+    char *byte_cursor = (char*)cursor;
 
     self->src = cursor;
 
-    /* Pack the header */
+    /* Pack the fixed part of the header */
     mem_sys_memcopy(cursor, self->header, PACKFILE_HEADER_BYTES);
-    cursor += PACKFILE_HEADER_BYTES / sizeof (opcode_t);
-    *cursor++ = PARROT_MAGIC;           /* Pack the magic */
-    *cursor++ = OPCODE_TYPE_PERL;       /* Pack opcode type */
-    *cursor++ = PF_DIR_FORMAT;          /* dir format */
-    *cursor++ = 0;                      /* pad */
+    byte_cursor += PACKFILE_HEADER_BYTES;
+
+    /* Pack the UUID. */
+    if (self->header->uuid_size > 0)
+        mem_sys_memcopy(byte_cursor, self->header->uuid_data,
+            self->header->uuid_size);
+
+    /* Padding. */
+    padding_size = 16 - (PACKFILE_HEADER_BYTES + self->header->uuid_size) % 16;
+    if (padding_size < 16) {
+        int i;
+        for (i = 0; i < padding_size; i++)
+            *byte_cursor++ = 0;
+    }
+    else {
+        padding_size = 0;
+    }
+
+    /* Set cursor. */
+    cursor += (PACKFILE_HEADER_BYTES + self->header->uuid_size + padding_size)
+        / sizeof(opcode_t);
+
+    /* Directory format and padding. */
+    *cursor++ = PF_DIR_FORMAT;
+    *cursor++ = 0;
+    *cursor++ = 0;
+    *cursor++ = 0;
 
     /* pack the directory */
     seg = (PackFile_Segment *) dir;
+
     /* dir size */
     size = seg->op_count;
     ret = PackFile_Segment_pack(interp, seg, cursor);
@@ -128,18 +158,19 @@ C<PackFile_ConstTable_pack()>
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 opcode_t *
 PackFile_ConstTable_pack(PARROT_INTERP,
-        NOTNULL(PackFile_Segment *seg), opcode_t *cursor)
+        NOTNULL(PackFile_Segment *seg), NOTNULL(opcode_t *cursor))
 {
     PackFile_ConstTable * const self = (PackFile_ConstTable *)seg;
     opcode_t i;
 
     *cursor++ = self->const_count;
 
-    for (i = 0; i < self->const_count; i++) {
+    for (i = 0; i < self->const_count; i++)
         cursor = PackFile_Constant_pack(interp, self, self->constants[i], cursor);
-    }
 
     return cursor;
 }
@@ -189,6 +220,8 @@ The data is zero-padded to an opcode_t-boundary, so pad bytes may be added.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 opcode_t *
 PackFile_Constant_pack(PARROT_INTERP,
         NOTNULL(const PackFile_ConstTable *const_table),
