@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2003, The Perl Foundation.
-$Id: /parrotcode/trunk/src/pbc_info.c 3159 2007-04-14T03:15:44.800537Z chromatic  $
+$Id: pbc_info.c 19833 2007-07-13 22:02:25Z petdance $
 
 =head1 NAME
 
@@ -37,7 +37,7 @@ the name of each segment in the directory.
 */
 
 static INTVAL
-iter(Interp* interp, PackFile_Segment *seg, void *user_data)
+iter(PARROT_INTERP, PackFile_Segment *seg, void *user_data)
 {
     int ident = (int)user_data;
     printf("%*.0s%s\n", ident, "", seg->name);
