@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: pmc_freeze.c 19914 2007-07-16 21:06:48Z petdance $
+$Id: pmc_freeze.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -30,7 +30,6 @@ C<STRING> with a vtable.
 */
 
 #include "parrot/parrot.h"
-#include <assert.h>
 
 /* default.pmc thawing of properties */
 void Parrot_default_thaw(Interp* , PMC* pmc, visit_info *info);
@@ -178,6 +177,8 @@ static void push_opcode_string( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* run_thaw( PARROT_INTERP,
     NOTNULL(STRING* image),
     visit_enum_type what )
@@ -190,9 +191,13 @@ static INTVAL shift_ascii_integer( SHIM_INTERP, NOTNULL(IMAGE_IO *io) )
 static FLOATVAL shift_ascii_number( SHIM_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* shift_ascii_pmc( SHIM_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING* shift_ascii_string( PARROT_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -203,10 +208,14 @@ static INTVAL shift_opcode_integer( SHIM_INTERP, NOTNULL(IMAGE_IO *io) )
 static FLOATVAL shift_opcode_number( SHIM_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC* shift_opcode_pmc( PARROT_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* shift_opcode_string( PARROT_INTERP, NOTNULL(IMAGE_IO *io) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -259,10 +268,9 @@ static void visit_loop_next_for_GC( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 static void visit_loop_todo_list( PARROT_INTERP,
-    NOTNULL(PMC *current),
+    NULLOK(PMC *current),
     NOTNULL(visit_info *info) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
 static void visit_next_for_GC( PARROT_INTERP,
@@ -344,7 +352,7 @@ str_append(PARROT_INTERP, NOTNULL(STRING *s), NOTNULL(const void *b), size_t len
         if (new_size < PObj_buflen(s) - need_free + 512)
             new_size = PObj_buflen(s) - need_free + 512;
         Parrot_reallocate_string(interp, s, new_size);
-        assert(PObj_buflen(s) - used - len >= 15);
+        PARROT_ASSERT(PObj_buflen(s) - used - len >= 15);
     }
     mem_sys_memcopy((void *)((ptrcast_t)s->strstart + used), b, len);
     s->bufused += len;
@@ -450,10 +458,10 @@ shift_ascii_integer(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
     const INTVAL i = strtoul(p, &p, 10);
 
     ++p;
-    assert(p <= start + io->image->bufused);
+    PARROT_ASSERT(p <= start + io->image->bufused);
     io->image->strstart = p;
     io->image->bufused -= (p - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return i;
 }
 
@@ -473,10 +481,10 @@ shift_ascii_number(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
     const FLOATVAL f = (FLOATVAL) strtod(p, &p);
 
     ++p;
-    assert(p <= start + io->image->bufused);
+    PARROT_ASSERT(p <= start + io->image->bufused);
     io->image->strstart = p;
     io->image->bufused -= (p - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return f;
 }
 
@@ -488,6 +496,8 @@ Removes and returns an string from the start of the C<*io> "stream".
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STRING*
 shift_ascii_string(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
 {
@@ -499,10 +509,10 @@ shift_ascii_string(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
     while (*p != ' ')
         ++p;
     ++p;
-    assert(p <= start + io->image->bufused);
+    PARROT_ASSERT(p <= start + io->image->bufused);
     io->image->strstart = p;
     io->image->bufused -= (p - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     s = string_make(interp, start, p - start - 1, "iso-8859-1", 0);
 /*    s = string_make(interp, start, p - start - 1, "UTF-8", 0); */
     return s;
@@ -516,6 +526,8 @@ Removes and returns a PMC from the start of the C<*io> "stream".
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 shift_ascii_pmc(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
 {
@@ -523,10 +535,10 @@ shift_ascii_pmc(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
     char *p = start;
     const unsigned long i = strtoul(p, &p, 16);
     ++p;
-    assert(p <= start + io->image->bufused);
+    PARROT_ASSERT(p <= start + io->image->bufused);
     io->image->strstart = p;
     io->image->bufused -= (p - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return (PMC*) i;
 }
 
@@ -555,7 +567,7 @@ op_check_size(PARROT_INTERP, NOTNULL(STRING *s), size_t len)
         if (new_size < PObj_buflen(s) - need_free + 512)
             new_size = PObj_buflen(s) - need_free + 512;
         Parrot_reallocate_string(interp, s, new_size);
-        assert(PObj_buflen(s) - used - len >= 15);
+        PARROT_ASSERT(PObj_buflen(s) - used - len >= 15);
     }
 #ifndef DISABLE_GC_DEBUG
     Parrot_go_collect(interp);
@@ -592,7 +604,7 @@ XXX assumes sizeof (opcode_t) == sizeof (INTVAL).
 static void
 push_opcode_integer(PARROT_INTERP, NOTNULL(IMAGE_IO *io), INTVAL v)
 {
-    assert(sizeof (opcode_t) == sizeof (INTVAL));
+    PARROT_ASSERT(sizeof (opcode_t) == sizeof (INTVAL));
     op_append(interp, io->image, (opcode_t)v, sizeof (opcode_t));
 }
 
@@ -671,7 +683,7 @@ shift_opcode_integer(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
         PF_fetch_integer(io->pf, (opcode_t**) &io->image->strstart);
 
     io->image->bufused -= ((char*)io->image->strstart - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return i;
 }
 
@@ -685,10 +697,13 @@ Note that this actually reads a PMC id, not a PMC.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 shift_opcode_pmc(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
 {
-    return (PMC*) shift_opcode_integer(interp, io);
+    INTVAL i = shift_opcode_integer(interp, io);
+    return (PMC *)i;
 }
 
 /*
@@ -707,7 +722,7 @@ shift_opcode_number(SHIM_INTERP, NOTNULL(IMAGE_IO *io))
         PF_fetch_number(io->pf, (opcode_t**) &io->image->strstart);
 
     io->image->bufused -= ((char*)io->image->strstart - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return f;
 }
 
@@ -719,6 +734,8 @@ Removes and returns a string from the start of the C<*io> "stream".
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 shift_opcode_string(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
 {
@@ -727,7 +744,7 @@ shift_opcode_string(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
         PF_fetch_string(interp, io->pf, (opcode_t**) &io->image->strstart);
 
     io->image->bufused -= ((char*)io->image->strstart - start);
-    assert((int)io->image->bufused >= 0);
+    PARROT_ASSERT((int)io->image->bufused >= 0);
     return s;
 }
 
@@ -768,7 +785,7 @@ cleanup_next_for_GC_pool(NOTNULL(Small_Object_Pool *pool))
         for (i = 0; i < arena->used; i++) {
             if (!PObj_on_free_list_TEST(p)) {
                 if (p->pmc_ext)
-                    PMC_next_for_GC(p) = NULL;
+                    PMC_next_for_GC(p) = PMCNULL;
             }
             p++;
         }
@@ -834,6 +851,12 @@ ft_init(PARROT_INTERP, NOTNULL(visit_info *info))
     STRING *s = info->image;
     PackFile *pf;
 
+    /* We want to store a 16-byte aligned header, but the actual
+     * header may be shorter. */
+    const unsigned int header_length = PACKFILE_HEADER_BYTES +
+        (PACKFILE_HEADER_BYTES % 16 ?
+         16 - PACKFILE_HEADER_BYTES % 16 : 0);
+
     info->image_io = mem_allocate_typed(IMAGE_IO);
     info->image_io->image = s = info->image;
 #if FREEZE_ASCII
@@ -845,20 +868,20 @@ ft_init(PARROT_INTERP, NOTNULL(visit_info *info))
     if (info->what == VISIT_FREEZE_NORMAL ||
         info->what == VISIT_FREEZE_AT_DESTRUCT) {
 
-        op_check_size(interp, s, PACKFILE_HEADER_BYTES);
+        op_check_size(interp, s, header_length);
         mem_sys_memcopy(s->strstart, pf->header, PACKFILE_HEADER_BYTES);
-        s->bufused += PACKFILE_HEADER_BYTES;
-        s->strlen += PACKFILE_HEADER_BYTES;
+        s->bufused += header_length;
+        s->strlen += header_length;
     }
     else {
-        if (string_length(interp, s) < PACKFILE_HEADER_BYTES) {
+        if (string_length(interp, s) < header_length) {
             real_exception(interp, NULL, E_IOError,
                     "bad string to thaw");
         }
         mem_sys_memcopy(pf->header, s->strstart, PACKFILE_HEADER_BYTES);
         PackFile_assign_transforms(pf);
-        s->bufused -= PACKFILE_HEADER_BYTES;
-        LVALUE_CAST(char *, s->strstart) += 16;
+        s->bufused -= header_length;
+        LVALUE_CAST(char *, s->strstart) += header_length;
     }
 
     info->last_type = -1;
@@ -1096,7 +1119,7 @@ do_thaw(PARROT_INTERP, NOTNULL(PMC* pmc), NOTNULL(visit_info *info))
             VTABLE_thaw(interp, pmc, info);
         }
 #else
-        assert(must_have_seen);
+        PARROT_ASSERT(must_have_seen);
 #endif
         /*
          * that's a duplicate
@@ -1107,7 +1130,7 @@ do_thaw(PARROT_INTERP, NOTNULL(PMC* pmc), NOTNULL(visit_info *info))
         return;
     }
 
-    assert(!must_have_seen);
+    PARROT_ASSERT(!must_have_seen);
     pmc = thaw_create_pmc(interp, info, type);
 
     VTABLE_thaw(interp, pmc, info);
@@ -1145,7 +1168,7 @@ id_from_pmc(PARROT_INTERP, NOTNULL(PMC* pmc))
         ptr_diff = (ptrdiff_t)pmc - (ptrdiff_t)arena->start_objects;
         if (ptr_diff >= 0 && ptr_diff <
                 (ptrdiff_t)(arena->used * pool->object_size)) {
-            assert(ptr_diff % pool->object_size == 0);
+            PARROT_ASSERT(ptr_diff % pool->object_size == 0);
             id += ptr_diff / pool->object_size;
             return id << 2;
         }
@@ -1156,7 +1179,7 @@ id_from_pmc(PARROT_INTERP, NOTNULL(PMC* pmc))
         ptr_diff = (ptrdiff_t)pmc - (ptrdiff_t)arena->start_objects;
         if (ptr_diff >= 0 && ptr_diff <
                 (ptrdiff_t)(arena->used * pool->object_size)) {
-            assert(ptr_diff % pool->object_size == 0);
+            PARROT_ASSERT(ptr_diff % pool->object_size == 0);
             id += ptr_diff / pool->object_size;
             return id << 2;
         }
@@ -1212,7 +1235,7 @@ next_for_GC_seen(PARROT_INTERP, NULLOK(PMC *pmc), NOTNULL(visit_info *info),
      */
     if (pmc->pmc_ext) {
         /* already seen? */
-        if (PMC_next_for_GC(pmc)) {
+        if (!PMC_IS_NULL(PMC_next_for_GC(pmc))) {
             seen = 1;
             goto skip;
         }
@@ -1376,7 +1399,7 @@ The thaw loop.
 */
 
 static void
-visit_loop_todo_list(PARROT_INTERP, NOTNULL(PMC *current),
+visit_loop_todo_list(PARROT_INTERP, NULLOK(PMC *current),
         NOTNULL(visit_info *info))
 {
     List * const todo = (List *)PMC_data(info->todo);
@@ -1487,6 +1510,8 @@ nested containers, for which the C<next_for_GC> approach could be a win.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC*
 run_thaw(PARROT_INTERP, NOTNULL(STRING* image), visit_enum_type what)
 {
@@ -1529,7 +1554,7 @@ run_thaw(PARROT_INTERP, NOTNULL(STRING* image), visit_enum_type what)
      */
     LVALUE_CAST(char *, image->strstart) -= bufused;
     image->bufused = bufused;
-    assert(image->strstart >= (char *)PObj_bufstart(image));
+    PARROT_ASSERT(image->strstart >= (char *)PObj_bufstart(image));
 
     if (dod_block) {
         Parrot_unblock_DOD(interp);
@@ -1553,8 +1578,10 @@ interrupted by a DOD run.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 STRING*
-Parrot_freeze_at_destruct(PARROT_INTERP, PMC* pmc)
+Parrot_freeze_at_destruct(PARROT_INTERP, NOTNULL(PMC* pmc))
 {
     visit_info info;
 
@@ -1585,8 +1612,10 @@ Freeze using either method.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 STRING*
-Parrot_freeze(PARROT_INTERP, PMC* pmc)
+Parrot_freeze(PARROT_INTERP, NOTNULL(PMC* pmc))
 {
 #if FREEZE_USE_NEXT_FOR_GC
     /*
@@ -1622,8 +1651,10 @@ Thaw a PMC, called from the C<thaw> opcode.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
-Parrot_thaw(PARROT_INTERP, STRING* image)
+Parrot_thaw(PARROT_INTERP, NOTNULL(STRING* image))
 {
     return run_thaw(interp, image, VISIT_THAW_NORMAL);
 }
@@ -1638,6 +1669,8 @@ constants.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
 Parrot_thaw_constants(PARROT_INTERP, NOTNULL(STRING* image))
 {
@@ -1655,8 +1688,10 @@ PMC.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC*
-Parrot_clone(PARROT_INTERP, PMC* pmc)
+Parrot_clone(PARROT_INTERP, NOTNULL(PMC* pmc))
 {
     return Parrot_thaw(interp, Parrot_freeze(interp, pmc));
 }
