@@ -17,6 +17,7 @@ the low-level synchronization.
 */
 
 
+#include "stm_waitlist.h"
 #include "stm_internal.h"
 
 #define STM_DEBUG 0
@@ -39,10 +40,14 @@ static void STM_TRACE_SAFE(const char *x, ...) /* HEADERIZER SKIP */
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_read_record * alloc_read( PARROT_INTERP, NOTNULL(STM_tx_log *log) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_write_record * alloc_write( PARROT_INTERP,
     NOTNULL(STM_tx_log *log) )
         __attribute__nonnull__(1)
@@ -67,6 +72,7 @@ static int do_real_commit( PARROT_INTERP, NOTNULL(STM_tx_log *log) )
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_write_record * find_write_record( PARROT_INTERP,
     NOTNULL(STM_tx_log *log),
     Parrot_STM_PMC_handle handle,
@@ -74,9 +80,13 @@ static STM_write_record * find_write_record( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC * force_sharing( PARROT_INTERP, NULLOK(PMC *pmc) )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STM_read_record * get_read( PARROT_INTERP,
     NOTNULL(STM_tx_log *log),
     int i )
@@ -87,9 +97,13 @@ static int get_read_valid_depth( PARROT_INTERP, NOTNULL(STM_tx_log *log) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_tx_log_sub * get_sublog( NOTNULL(STM_tx_log *log), int i )
         __attribute__nonnull__(1);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STM_write_record * get_write( PARROT_INTERP,
     NOTNULL(STM_tx_log *log),
     int i )
@@ -129,6 +143,7 @@ static int merge_transactions( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static void * next_version( NOTNULL(const void *old_version) )
         __attribute__nonnull__(1);
 
@@ -148,6 +163,8 @@ static int setup_wait( PARROT_INTERP, NOTNULL(STM_tx_log *log) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static void * wait_for_version( PARROT_INTERP,
     NOTNULL(STM_tx_log *log),
     Parrot_STM_PMC_handle handle )
@@ -191,7 +208,7 @@ of the PMC will be a copy of C<pmc>.
 */
 
 Parrot_STM_PMC_handle
-Parrot_STM_alloc(PARROT_INTERP, PMC *pmc)
+Parrot_STM_alloc(PARROT_INTERP, NULLOK(PMC *pmc))
 {
     Parrot_STM_PMC_handle_data *handle;
     STM_TRACE("Parrot_STM_alloc");
@@ -236,19 +253,23 @@ Parrot_thaw_STM_PMC_handle(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
 }
 
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STM_write_record *
 get_write(PARROT_INTERP, NOTNULL(STM_tx_log *log), int i)
 {
-    assert(i >= 0);
-    assert(i <= log->last_write);
+    PARROT_ASSERT(i >= 0);
+    PARROT_ASSERT(i <= log->last_write);
     return &log->writes[i];
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static STM_read_record *
 get_read(PARROT_INTERP, NOTNULL(STM_tx_log *log), int i)
 {
-    assert(i >= 0);
-    assert(i <= log->last_read);
+    PARROT_ASSERT(i >= 0);
+    PARROT_ASSERT(i <= log->last_read);
     return &log->reads[i];
 }
 
@@ -258,6 +279,8 @@ get_read(PARROT_INTERP, NOTNULL(STM_tx_log *log), int i)
  * marked with the value uninitialized.
  */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_write_record *
 alloc_write(PARROT_INTERP, NOTNULL(STM_tx_log *log))
 {
@@ -276,6 +299,8 @@ alloc_write(PARROT_INTERP, NOTNULL(STM_tx_log *log))
     return write;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_read_record *
 alloc_read(PARROT_INTERP, NOTNULL(STM_tx_log *log))
 {
@@ -295,6 +320,7 @@ alloc_read(PARROT_INTERP, NOTNULL(STM_tx_log *log))
 }
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static void *
 next_version(NOTNULL(const void *old_version))
 {
@@ -311,11 +337,13 @@ is_version(NOTNULL(const void *maybe_version))
     return v & 1;
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_tx_log_sub *
 get_sublog(NOTNULL(STM_tx_log *log), int i)
 {
-    assert(i > 0);
-    assert(i <= log->depth);
+    PARROT_ASSERT(i > 0);
+    PARROT_ASSERT(i <= log->depth);
     return &log->inner[i - 1];
 }
 
@@ -361,8 +389,8 @@ Parrot_STM_start_transaction(PARROT_INTERP)
     newsub->first_write = log->last_write + 1;
 
     if (log->depth == 1) {
-        assert(newsub->first_read == 0);
-        assert(newsub->first_write == 0);
+        PARROT_ASSERT(newsub->first_read == 0);
+        PARROT_ASSERT(newsub->first_write == 0);
     }
 
     PARROT_ATOMIC_INT_SET(newsub->status, STM_STATUS_ACTIVE);
@@ -416,7 +444,7 @@ merge_transactions(PARROT_INTERP, NOTNULL(STM_tx_log *log),
                 }
             }
 
-            assert(write->saw_version != outer);
+            PARROT_ASSERT(write->saw_version != outer);
         }
 
         if (!successp) {
@@ -460,6 +488,8 @@ merge_transactions(PARROT_INTERP, NOTNULL(STM_tx_log *log),
     }
 }
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static PMC *
 force_sharing(PARROT_INTERP, NULLOK(PMC *pmc))
 {
@@ -469,7 +499,7 @@ force_sharing(PARROT_INTERP, NULLOK(PMC *pmc))
         return PMCNULL;
 
     ret = VTABLE_share_ro(interp, pmc);
-    assert(PObj_is_PMC_shared_TEST(ret));
+    PARROT_ASSERT(PObj_is_PMC_shared_TEST(ret));
     return ret;
 }
 
@@ -533,7 +563,7 @@ do_real_commit(PARROT_INTERP, NOTNULL(STM_tx_log *log)) {
     int successp;
     STM_tx_log_sub *inner;
 
-    assert(log->depth == 1);
+    PARROT_ASSERT(log->depth == 1);
 
     inner = get_sublog(log, 1);
 
@@ -575,7 +605,7 @@ do_real_commit(PARROT_INTERP, NOTNULL(STM_tx_log *log)) {
                        new_version, write->handle);
 
         /* no one should steal our ownership when we are committed */
-        assert(successp);
+        PARROT_ASSERT(successp);
 
         Parrot_STM_waitlist_signal(interp, &write->handle->change_waitlist);
         STM_TRACE_SAFE("done waitlist_signal");
@@ -706,7 +736,7 @@ Parrot_STM_commit(PARROT_INTERP)
         return 0;
     }
 
-    assert(log->depth > 0);
+    PARROT_ASSERT(log->depth > 0);
 
     PROFILE_TRIED_COMMIT(log);
 
@@ -717,7 +747,7 @@ Parrot_STM_commit(PARROT_INTERP)
             get_sublog(log, log->depth - 1), cursub, 1);
 
         /* should always return true, since we pass 1 for the always argument */
-        assert(successp);
+        PARROT_ASSERT(successp);
     }
     else
         successp = do_real_commit(interp, log);
@@ -754,7 +784,7 @@ Parrot_STM_abort(PARROT_INTERP)
         return;
     }
 
-    assert(log->depth > 0);
+    PARROT_ASSERT(log->depth > 0);
 
     PROFILE_ABORTED(log);
 
@@ -968,7 +998,7 @@ Parrot_STM_mark_pmc_handle(PARROT_INTERP, Parrot_STM_PMC_handle handle)
     value = handle->value;
 
     if (!PMC_IS_NULL(value)) {
-        assert(PObj_is_PMC_shared_TEST(value));
+        PARROT_ASSERT(PObj_is_PMC_shared_TEST(value));
         pobject_lives(interp, (PObj*) value);
     }
 }
@@ -997,6 +1027,9 @@ Parrot_STM_transaction_depth(PARROT_INTERP)
  *
  * This may mark us as aborted and return NULL.
  */
+
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static void *
 wait_for_version(PARROT_INTERP,
         NOTNULL(STM_tx_log *log), Parrot_STM_PMC_handle handle)
@@ -1021,7 +1054,7 @@ wait_for_version(PARROT_INTERP,
             break;
         }
 
-        if (start_wait == 0.0)
+        if (FLOAT_IS_ZERO(start_wait))
             start_wait = Parrot_floatval_time();
 
         ++wait_count;
@@ -1036,10 +1069,12 @@ wait_for_version(PARROT_INTERP,
          * FIXME XXX race in accessing n_interpreters?
          * FIXME XXX race if other log goes away
          */
-        assert(n_interpreters > 1);
-        other  = (STM_tx_log_sub*)version;
+        PARROT_ASSERT(n_interpreters > 1);
+        other  = (STM_tx_log_sub *)version;
+
+        PARROT_ASSERT(other < &log->inner[0] ||
+                      other > &log->inner[STM_MAX_TX_DEPTH]);
 
-        assert(other < &log->inner[0] || other > &log->inner[STM_MAX_TX_DEPTH]);
         curlog = get_sublog(log, log->depth);
 
         PARROT_ATOMIC_INT_GET(other_wait_len, other->wait_length);
@@ -1113,6 +1148,8 @@ Read the value stored in the PMC wrapped by C<handle>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC *
 Parrot_STM_read(PARROT_INTERP, Parrot_STM_PMC_handle handle)
 {
@@ -1220,6 +1257,7 @@ local_pmc_copy(PARROT_INTERP, NULLOK(PMC * const original))
  * so initialize it to PMCNULL.
  */
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STM_write_record *
 find_write_record(PARROT_INTERP, NOTNULL(STM_tx_log *log),
         Parrot_STM_PMC_handle handle, int overwrite_p)
@@ -1236,7 +1274,7 @@ find_write_record(PARROT_INTERP, NOTNULL(STM_tx_log *log),
     STM_TRACE("finding write record for %p", handle);
 
     log      = Parrot_STM_tx_log_get(interp);
-    assert(log->depth > 0);
+    PARROT_ASSERT(log->depth > 0);
 
     cursub   = get_sublog(log, log->depth);
     outersub = NULL;
@@ -1309,7 +1347,7 @@ find_write_record(PARROT_INTERP, NOTNULL(STM_tx_log *log),
                     read->saw_version, cursub);
             }
             else {
-                assert(outersub);
+                PARROT_ASSERT(outersub);
                 STM_TRACE("... from outer transaction's write record");
                 PARROT_ATOMIC_PTR_GET(write->saw_version,
                                       handle->owner_or_version);
@@ -1370,6 +1408,8 @@ be given it when they read the value).
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC *
 Parrot_STM_begin_update(PARROT_INTERP, Parrot_STM_PMC_handle handle)
 {
@@ -1395,17 +1435,14 @@ Write C<new_value> into the PMC wrapped by C<handle>.
 */
 
 void
-Parrot_STM_write(PARROT_INTERP, Parrot_STM_PMC_handle handle, PMC* new_value)
+Parrot_STM_write(PARROT_INTERP, Parrot_STM_PMC_handle handle, NULLOK(PMC* new_value))
 {
     /* XXX no transaction case */
     STM_write_record   *write;
     STM_tx_log * const log = Parrot_STM_tx_log_get(interp);
 
-    if (log->depth == 0) {
-        /* error for now */
+    if (log->depth == 0) /* error for now */
         real_exception(interp, NULL, 1, "STM_write outside transaction");
-        return;
-    }
 
     write        = find_write_record(interp, log, handle, 1);
     write->value = new_value;
@@ -1420,6 +1457,8 @@ enough to wait() for it to become valid. User access through STMLog PMC class.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 void *
 Parrot_STM_extract(PARROT_INTERP)
 {
@@ -1460,41 +1499,39 @@ in.
 */
 
 void
-Parrot_STM_replay_extracted(PARROT_INTERP, void *saved_log_data)
+Parrot_STM_replay_extracted(PARROT_INTERP, NULLOK(void *saved_log_data))
 {
-    STM_tx_log       *log;
-    STM_saved_tx_log *saved;
-    STM_tx_log_sub   *sublog;
-    int               i, start;
 
-    if (saved_log_data == NULL)
-        return;
+    if (saved_log_data) {
+        STM_saved_tx_log * const saved = (STM_saved_tx_log *)saved_log_data;
+        STM_tx_log * const       log   = Parrot_STM_tx_log_get(interp);
 
-    saved = (STM_saved_tx_log *)saved_log_data;
-    log   = Parrot_STM_tx_log_get(interp);
+        STM_tx_log_sub *sublog;
+        int i, start;
 
-    if (log->depth == 0)
-        real_exception(interp, NULL, 1, "replay_extracted outside of transaction");
+        if (log->depth == 0)
+            real_exception(interp, NULL, 1, "replay_extracted outside of transaction");
 
-    sublog = get_sublog(log, log->depth);
-    start  = log->last_read;
+        sublog = get_sublog(log, log->depth);
+        start  = log->last_read;
 
-    for (i = 0; i < saved->num_reads; ++i)
-        *(alloc_read(interp, log)) = saved->reads[i];
+        for (i = 0; i < saved->num_reads; ++i)
+            *(alloc_read(interp, log)) = saved->reads[i];
 
-    start = log->last_write;
+        start = log->last_write;
 
-    for (i = 0; i < saved->num_writes; ++i) {
-        int successp;
-        STM_write_record * const write = alloc_write(interp, log);
-        *write                         = saved->writes[i];
+        for (i = 0; i < saved->num_writes; ++i) {
+            int successp;
+            STM_write_record * const write = alloc_write(interp, log);
+            *write                         = saved->writes[i];
 
-        PARROT_ATOMIC_PTR_CAS(successp, write->handle->owner_or_version,
-                              write->saw_version, sublog);
+            PARROT_ATOMIC_PTR_CAS(successp, write->handle->owner_or_version,
+                                write->saw_version, sublog);
 
-        /* failed! */
-        if (!successp)
-            PARROT_ATOMIC_INT_SET(sublog->status, STM_STATUS_ABORTED);
+            /* failed! */
+            if (!successp)
+                PARROT_ATOMIC_INT_SET(sublog->status, STM_STATUS_ABORTED);
+        }
     }
 }
 
@@ -1507,21 +1544,18 @@ Mark GC-managed objects reachable through an extracted transaction log.
 */
 
 void
-Parrot_STM_mark_extracted(PARROT_INTERP, void *saved_log_data)
+Parrot_STM_mark_extracted(PARROT_INTERP, NULLOK(void *saved_log_data))
 {
-    STM_saved_tx_log *saved;
-    int               i;
+    if (saved_log_data) {
+        STM_saved_tx_log * const saved = (STM_saved_tx_log *)saved_log_data;
+        int i;
 
-    if (saved_log_data == NULL)
-        return;
+        for (i = 0; i < saved->num_reads; ++i)
+            mark_read_record(interp, &saved->reads[i]);
 
-    saved = (STM_saved_tx_log *)saved_log_data;
-
-    for (i = 0; i < saved->num_reads; ++i)
-        mark_read_record(interp, &saved->reads[i]);
-
-    for (i = 0; i < saved->num_writes; ++i)
-        mark_write_record(interp, &saved->writes[i]);
+        for (i = 0; i < saved->num_writes; ++i)
+            mark_write_record(interp, &saved->writes[i]);
+    }
 }
 
 /*
@@ -1533,17 +1567,15 @@ Free memory associated with an extracted transaction log.
 */
 
 void
-Parrot_STM_destroy_extracted(PARROT_INTERP, void *saved_log_data)
+Parrot_STM_destroy_extracted(PARROT_INTERP, NULLOK(void *saved_log_data))
 {
-    STM_saved_tx_log *saved;
+    if (saved_log_data) {
+        STM_saved_tx_log * const saved = (STM_saved_tx_log *)saved_log_data;
 
-    if (saved_log_data == NULL)
-        return;
-
-    saved = (STM_saved_tx_log *)saved_log_data;
-    mem_sys_free(saved->reads);
-    mem_sys_free(saved->writes);
-    mem_sys_free(saved);
+        mem_sys_free(saved->reads);
+        mem_sys_free(saved->writes);
+        mem_sys_free(saved);
+    }
 }
 
 #if STM_PROFILE
