@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: string.c 19911 2007-07-16 19:22:55Z petdance $
+$Id: string.c 20688 2007-08-18 21:06:33Z chromatic $
 
 =head1 NAME
 
@@ -25,7 +25,6 @@ strings.
 #include "parrot/string_funcs.h"
 #include "string_private_cstring.h"
 #include "parrot/resources.h"
-#include <assert.h>
 
 /*
  * this extra size is in the hope that some concat ops might
@@ -43,16 +42,16 @@ strings.
 
 
 #define saneify_string(s) \
-    assert(s->encoding); \
-    assert(s->charset); \
-    assert(!PObj_on_free_list_TEST(s))
+    PARROT_ASSERT(s->encoding); \
+    PARROT_ASSERT(s->charset); \
+    PARROT_ASSERT(!PObj_on_free_list_TEST(s))
 
 /* HEADERIZER HFILE: include/parrot/string_funcs.h */
 
 /* HEADERIZER BEGIN: static */
 
 static void make_writable( PARROT_INTERP,
-    NOTNULL(STRING **s),
+    ARGINOUT(STRING **s),
     const size_t len,
     parrot_string_representation_t representation )
         __attribute__nonnull__(1)
@@ -74,6 +73,8 @@ PARROT_API
 void
 Parrot_unmake_COW(PARROT_INTERP, NOTNULL(STRING *s))
 {
+    PARROT_ASSERT(s);
+
     /* COW_FLAG | constant_FLAG | external_FLAG) */
     if (PObj_is_cowed_TESTALL(s)) {
         STRING for_alloc;
@@ -118,15 +119,14 @@ allocating a new buffer.
 
 */
 PARROT_API
-PARROT_CAN_RETURN_NULL
+PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
 STRING *
-Parrot_make_COW_reference(PARROT_INTERP, NULLOK(STRING *s))
+Parrot_make_COW_reference(PARROT_INTERP, NOTNULL(STRING *s))
 {
     STRING *d;
 
-    if (!s)
-        return NULL;
+    PARROT_ASSERT(s);
 
     if (PObj_constant_TEST(s)) {
         d = new_string_header(interp, PObj_get_FLAGS(s) & ~PObj_constant_FLAG);
@@ -172,9 +172,11 @@ using the one passed in and returns it.
 
 PARROT_API
 PARROT_CANNOT_RETURN_NULL
-STRING*
+STRING *
 Parrot_reuse_COW_reference(SHIM_INTERP, NOTNULL(STRING *s), NOTNULL(STRING *d))
 {
+    PARROT_ASSERT(s);
+
     if (PObj_constant_TEST(s)) {
         PObj_COW_SET(s);
         STRUCT_COPY(d,s);
@@ -300,6 +302,8 @@ PARROT_PURE_FUNCTION
 UINTVAL
 string_capacity(SHIM_INTERP, NOTNULL(const STRING *s))
 {
+    PARROT_ASSERT(s);
+
     return ((ptrcast_t)PObj_bufstart(s) + PObj_buflen(s) -
             (ptrcast_t)s->strstart);
 }
@@ -312,6 +316,7 @@ Creates and returns an empty Parrot string.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_make_empty(PARROT_INTERP,
     parrot_string_representation_t representation, UINTVAL capacity)
@@ -323,7 +328,7 @@ string_make_empty(PARROT_INTERP,
      */
     if (representation == enum_stringrep_one) {
         s->charset = PARROT_DEFAULT_CHARSET;
-        s->encoding = CHARSET_GET_PREFERRED_ENCODING(interp, s);;
+        s->encoding = CHARSET_GET_PREFERRED_ENCODING(interp, s);
     }
     else {
         real_exception(interp, NULL, INVALID_CHARTYPE, "Unsupported representation");
@@ -350,9 +355,9 @@ Returs NULL, if no compatible string representation can be found.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-CHARSET *
+const CHARSET *
 string_rep_compatible(SHIM_INTERP,
-    NOTNULL(const STRING *a), NOTNULL(const STRING *b), NOTNULL(ENCODING **e))
+    NOTNULL(const STRING *a), NOTNULL(const STRING *b), ARGOUT(const ENCODING **e))
 {
     if (a->encoding == b->encoding && a->charset == b->charset) {
         *e = a->encoding;
@@ -409,18 +414,20 @@ So make sure to _use_ the return value.
 */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_append(PARROT_INTERP, NULLOK(STRING *a), NULLOK(STRING *b))
 {
     UINTVAL a_capacity;
     UINTVAL total_length;
-    CHARSET *cs;
-    ENCODING *enc;
+    const CHARSET *cs;
+    const ENCODING *enc;
 
     /* XXX should this be a CHARSET method? */
 
     /* If B isn't real, we just bail */
-    const UINTVAL b_len = string_length(interp, b);
+    const UINTVAL b_len = b ? string_length(interp, b) : 0;
     if (!b_len)
         return a;
 
@@ -486,8 +493,7 @@ PARROT_WARN_UNUSED_RESULT
 PARROT_MALLOC
 PARROT_CANNOT_RETURN_NULL
 STRING *
-string_from_cstring(PARROT_INTERP,
-    NULLOK(const char * const buffer), const UINTVAL len)
+string_from_cstring(PARROT_INTERP, NULLOK(const char * const buffer), const UINTVAL len)
 {
     return string_make_direct(interp, buffer, len ? len :
             buffer ? strlen(buffer) : 0,
@@ -533,6 +539,8 @@ PARROT_CANNOT_RETURN_NULL
 STRING *
 const_string(PARROT_INTERP, NOTNULL(const char *buffer))
 {
+    PARROT_ASSERT(buffer);
+
     /* TODO cache the strings */
     return string_make_direct(interp, buffer, strlen(buffer),
                        PARROT_DEFAULT_ENCODING, PARROT_DEFAULT_CHARSET,
@@ -572,8 +580,8 @@ STRING *
 string_make(PARROT_INTERP, NULLOK(const char *buffer),
         UINTVAL len, NULLOK(const char *charset_name), UINTVAL flags)
 {
-    ENCODING *encoding;
-    CHARSET *charset;
+    const ENCODING *encoding;
+    const CHARSET *charset;
 
     if (!charset_name)
         charset_name = "ascii";
@@ -592,7 +600,7 @@ PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 STRING *
 string_make_direct(PARROT_INTERP, NULLOK(const char *buffer), UINTVAL len,
-        NOTNULL(ENCODING *encoding), NOTNULL(CHARSET *charset), UINTVAL flags)
+        NOTNULL(const ENCODING *encoding), NOTNULL(const CHARSET *charset), UINTVAL flags)
 {
     STRING * const s = new_string_header(interp, flags);
     DECL_CONST_CAST;
@@ -669,12 +677,13 @@ Returns the number of characters in the specified Parrot string.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
 UINTVAL
-string_length(SHIM_INTERP, NULLOK(const STRING *s))
+string_length(SHIM_INTERP, NOTNULL(const STRING *s))
 {
-    return s ? s->strlen : 0;
+    PARROT_ASSERT(s);
+
+    return s->strlen;
 }
 
 /*
@@ -749,9 +758,9 @@ are treated as counting from the end of the string.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-string_ord(PARROT_INTERP, NULLOK(const STRING *s), INTVAL idx)
+string_ord(PARROT_INTERP, NOTNULL(const STRING *s), INTVAL idx)
 {
-    const UINTVAL len = string_length(interp, s);
+    const UINTVAL len = s ? string_length(interp, s) : 0;
 
     if (len == 0) {
         real_exception(interp, NULL, ORD_OUT_OF_STRING,
@@ -778,7 +787,6 @@ string_ord(PARROT_INTERP, NULLOK(const STRING *s), INTVAL idx)
 
         return string_index(interp, s, true_index);
     }
-    return -1;
 }
 
 /*
@@ -838,10 +846,12 @@ string.
 */
 
 PARROT_API
-PARROT_MAY_IGNORE_RESULT
+PARROT_IGNORABLE_RESULT
 INTVAL
 string_compute_strlen(PARROT_INTERP, NOTNULL(STRING *s))
 {
+    PARROT_ASSERT(s);
+
     s->strlen = CHARSET_CODEPOINTS(interp, s);
     return s->strlen;
 }
@@ -860,7 +870,7 @@ PARROT_WARN_UNUSED_RESULT
 INTVAL
 string_max_bytes(SHIM_INTERP, NOTNULL(const STRING *s), INTVAL nchars)
 {
-    assert(s->encoding);
+    PARROT_ASSERT(s->encoding);
     return ENCODING_MAX_BYTES_PER_CODEPOINT(interp, s) * nchars;
 }
 
@@ -877,19 +887,20 @@ created and returned.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_concat(PARROT_INTERP, NULLOK(STRING *a), NULLOK(STRING *b), UINTVAL Uflags)
 {
     if (a != NULL && a->strlen != 0) {
         if (b != NULL && b->strlen != 0) {
-            CHARSET *cs;
-            ENCODING *enc;
+            const CHARSET *cs;
+            const ENCODING *enc;
             STRING *result;
 
             cs = string_rep_compatible(interp, a, b, &enc);
             if (!cs) {
                 cs = a->charset;
-                enc =a->encoding;
+                enc = a->encoding;
             }
             result =
                 string_make_direct(interp, NULL,
@@ -906,12 +917,9 @@ string_concat(PARROT_INTERP, NULLOK(STRING *a), NULLOK(STRING *b), UINTVAL Uflag
         }
     }
     else {
-        if (b != NULL) {
-            return string_copy(interp, b);
-        }
-        else {
-            return string_make(interp, NULL, 0, NULL, Uflags);
-        }
+        return b
+            ? string_copy(interp, b)
+            : string_make(interp, NULL, 0, NULL, Uflags);
     }
 }
 
@@ -926,9 +934,10 @@ necessary.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_repeat(PARROT_INTERP, NOTNULL(const STRING *s),
-    UINTVAL num, NULLOK(STRING **d))
+    UINTVAL num, ARGOUT_NULLOK(STRING **d))
 {
     UINTVAL i;
 
@@ -965,9 +974,11 @@ necessary. The substring is also returned.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING *
 string_substr(PARROT_INTERP, NOTNULL(STRING *src), INTVAL offset, INTVAL length,
-        NULLOK(STRING **d), int replace_dest)
+        ARGOUT_NULLOK(STRING **d), int replace_dest)
 {
     STRING *dest;
     UINTVAL true_offset;
@@ -994,7 +1005,7 @@ string_substr(PARROT_INTERP, NOTNULL(STRING *src), INTVAL offset, INTVAL length,
 
     /* do in-place i.e. reuse existing header if one */
     if (replace_dest && d && *d) {
-        assert(src->encoding == Parrot_fixed_8_encoding_ptr);
+        PARROT_ASSERT(src->encoding == Parrot_fixed_8_encoding_ptr);
         dest           = *d;
 
         dest->encoding = src->encoding;
@@ -1042,15 +1053,15 @@ PARROT_API
 PARROT_CAN_RETURN_NULL
 STRING *
 string_replace(PARROT_INTERP, NOTNULL(STRING *src),
-    INTVAL offset, INTVAL length, NOTNULL(STRING *rep), NULLOK(STRING **d))
+    INTVAL offset, INTVAL length, NOTNULL(STRING *rep), ARGOUT_NULLOK(STRING **d))
 {
     STRING *dest = NULL;
     UINTVAL start_byte, end_byte;
     UINTVAL true_offset;
     UINTVAL true_length;
     INTVAL diff;
-    CHARSET *cs;
-    ENCODING *enc;
+    const CHARSET *cs;
+    const ENCODING *enc;
     String_iter iter;
 
     /* special case */
@@ -1185,9 +1196,9 @@ PARROT_CANNOT_RETURN_NULL
 STRING *
 string_chopn(PARROT_INTERP, NOTNULL(STRING *s), INTVAL n)
 {
-    s = string_copy(interp, s);
-    string_chopn_inplace(interp, s, n);
-    return s;
+    STRING * const chopped = string_copy(interp, s);
+    string_chopn_inplace(interp, chopped, n);
+    return chopped;
 }
 
 /*
@@ -1328,7 +1339,7 @@ has to be created.
 */
 
 static void
-make_writable(PARROT_INTERP, NOTNULL(STRING **s),
+make_writable(PARROT_INTERP, ARGINOUT(STRING **s),
     const size_t len, parrot_string_representation_t representation)
 {
     if (!*s)
@@ -1339,17 +1350,6 @@ make_writable(PARROT_INTERP, NOTNULL(STRING **s),
         Parrot_unmake_COW(interp, *s);
 }
 
-#define BITWISE_AND_STRINGS(type1, type2, restype, s1, s2, res, minlen) \
-do { \
-    const type1 *curr1 = (type1 *)s1->strstart; \
-    const type2 *curr2 = (type2 *)s2->strstart; \
-    restype     *dp    = (restype *)res->strstart; \
-    size_t       len   = minlen; \
- \
-    for (; len ; ++curr1, ++curr2, ++dp, --len) \
-        *dp = *curr1 & *curr2; \
-} while (0)
-
 /*
 
 FUNCDOC: string_bitwise_and
@@ -1361,27 +1361,24 @@ then it is reused, otherwise a new Parrot string is created.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_bitwise_and(PARROT_INTERP, NULLOK(STRING *s1),
-        NULLOK(STRING *s2), NULLOK(STRING **dest))
+        NULLOK(STRING *s2), ARGOUT_NULLOK(STRING **dest))
 {
-    STRING *res = NULL;
+    STRING *res;
     size_t minlen;
 
     /* we could also trans_charset to iso-8859-1 */
-    if (s1 && s1->encoding != Parrot_fixed_8_encoding_ptr) {
+    if (s1 && s1->encoding != Parrot_fixed_8_encoding_ptr)
         real_exception(interp, NULL, INVALID_ENCODING,
                 "string bitwise_and (%s/%s) unsupported",
-                ((ENCODING *)(s1->encoding))->name,
-                ((ENCODING *)(s2->encoding))->name);
-    }
+                s1->encoding->name, s2->encoding->name);
 
-    if (s2 && s2->encoding != Parrot_fixed_8_encoding_ptr) {
+    if (s2 && s2->encoding != Parrot_fixed_8_encoding_ptr)
         real_exception(interp, NULL, INVALID_ENCODING,
                 "string bitwise_and (%s/%s) unsupported",
-                ((ENCODING *)(s2->encoding))->name,
-                ((ENCODING *)(s2->encoding))->name);
-    }
+                s1->encoding->name, s2->encoding->name);
 
     /* think about case of dest string is one of the operands */
     if (s1 && s2)
@@ -1413,8 +1410,15 @@ string_bitwise_and(PARROT_INTERP, NULLOK(STRING *s1),
 
     make_writable(interp, &res, minlen, enum_stringrep_one);
 
-    BITWISE_AND_STRINGS(Parrot_UInt1, Parrot_UInt1,
-            Parrot_UInt1, s1, s2, res, minlen);
+    { /* bitwise AND the strings */
+        const Parrot_UInt1 *curr1 = (Parrot_UInt1 *)s1->strstart;
+        const Parrot_UInt1 *curr2 = (Parrot_UInt1 *)s2->strstart;
+        Parrot_UInt1       *dp    = (Parrot_UInt1 *)res->strstart;
+        size_t len = minlen;
+
+        while (len--)
+            *dp++ = *curr1++ & *curr2++;
+    }
 
     res->bufused = res->strlen = minlen;
 
@@ -1469,30 +1473,27 @@ then it is reused, otherwise a new Parrot string is created.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_bitwise_or(PARROT_INTERP, NULLOK(STRING *s1),
-        NULLOK(STRING *s2), NULLOK(STRING **dest))
+        NULLOK(STRING *s2), ARGOUT_NULLOK(STRING **dest))
 {
     STRING *res;
     size_t  maxlen = 0;
 
     if (s1) {
-        if (s1->encoding != Parrot_fixed_8_encoding_ptr) {
+        if (s1->encoding != Parrot_fixed_8_encoding_ptr)
             real_exception(interp, NULL, INVALID_ENCODING,
-                    "string bitwise_and (%s/%s) unsupported",
-                    ((ENCODING *)(s1->encoding))->name,
-                    ((ENCODING *)(s2->encoding))->name);
-        }
+                    "string bitwise_or (%s/%s) unsupported",
+                    s1->encoding->name, s2->encoding->name);
         maxlen = s1->bufused;
     }
 
     if (s2) {
-        if (s2->encoding != Parrot_fixed_8_encoding_ptr) {
+        if (s2->encoding != Parrot_fixed_8_encoding_ptr)
             real_exception(interp, NULL, INVALID_ENCODING,
-                    "string bitwise_and (%s/%s) unsupported",
-                    ((ENCODING *)(s2->encoding))->name,
-                    ((ENCODING *)(s2->encoding))->name);
-        }
+                    "string bitwise_or (%s/%s) unsupported",
+                    s1->encoding->name, s2->encoding->name);
 
         if (s2->bufused > maxlen)
             maxlen = s2->bufused;
@@ -1542,9 +1543,10 @@ then it is reused, otherwise a new Parrot string is created.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_bitwise_xor(PARROT_INTERP, NULLOK(STRING *s1),
-        NULLOK(STRING *s2), NULLOK(STRING **dest))
+        NULLOK(STRING *s2), ARGOUT_NULLOK(STRING **dest))
 {
     STRING *res;
     size_t  maxlen = 0;
@@ -1552,9 +1554,8 @@ string_bitwise_xor(PARROT_INTERP, NULLOK(STRING *s1),
     if (s1) {
         if (s1->encoding != Parrot_fixed_8_encoding_ptr) {
             real_exception(interp, NULL, INVALID_ENCODING,
-                    "string bitwise_and (%s/%s) unsupported",
-                    ((ENCODING *)(s1->encoding))->name,
-                    ((ENCODING *)(s2->encoding))->name);
+                    "string bitwise_xor (%s/%s) unsupported",
+                    s1->encoding->name, s2->encoding->name);
         }
         maxlen = s1->bufused;
     }
@@ -1562,9 +1563,8 @@ string_bitwise_xor(PARROT_INTERP, NULLOK(STRING *s1),
     if (s2) {
         if (s2->encoding != Parrot_fixed_8_encoding_ptr) {
             real_exception(interp, NULL, INVALID_ENCODING,
-                    "string bitwise_and (%s/%s) unsupported",
-                    ((ENCODING *)(s2->encoding))->name,
-                    ((ENCODING *)(s2->encoding))->name);
+                    "string bitwise_xor (%s/%s) unsupported",
+                    s1->encoding->name, s2->encoding->name);
         }
 
         if (s2->bufused > maxlen)
@@ -1626,19 +1626,18 @@ not C<NULL> then it is reused, otherwise a new Parrot string is created.
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
-string_bitwise_not(PARROT_INTERP, NULLOK(STRING *s), NULLOK(STRING **dest))
+string_bitwise_not(PARROT_INTERP, NULLOK(STRING *s), ARGOUT_NULLOK(STRING **dest))
 {
     STRING *res;
     size_t  len;
 
     if (s) {
-        if (s->encoding != Parrot_fixed_8_encoding_ptr) {
+        if (s->encoding != Parrot_fixed_8_encoding_ptr)
             real_exception(interp, NULL, INVALID_ENCODING,
-                    "string bitwise_and (%s/%s) unsupported",
-                    ((ENCODING *)(s->encoding))->name,
-                    ((ENCODING *)(s->encoding))->name);
-        }
+                    "string bitwise_not (%s/%s) unsupported",
+                    s->encoding->name, s->encoding->name);
         len = s->bufused;
     }
     else
@@ -1688,7 +1687,7 @@ if it is equal to anything other than C<0>, C<""> or C<"0">.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-string_bool(PARROT_INTERP, NULLOK(const STRING *s))
+string_bool(PARROT_INTERP, NOTNULL(const STRING *s))
 {
     const INTVAL len = string_length(interp, s);
 
@@ -1711,7 +1710,8 @@ string_bool(PARROT_INTERP, NULLOK(const STRING *s))
 
 /*
 
-FUNCDOC:
+FUNCDOC: string_nprintf
+
 This is like C<Parrot_snprintf()> except that it writes to and returns a
 Parrot string.
 
@@ -1724,9 +1724,10 @@ encoding of C<*dest>.
 */
 
 PARROT_API
-STRING*
+PARROT_CANNOT_RETURN_NULL
+STRING *
 string_nprintf(PARROT_INTERP,
-    STRING *dest, INTVAL bytelen, NOTNULL(const char *format), ...)
+    NULLOK(STRING *dest), INTVAL bytelen, NOTNULL(const char *format), ...)
 {
     STRING  *output;
     va_list  args;
@@ -1739,7 +1740,7 @@ string_nprintf(PARROT_INTERP,
      * XXX -leo: bytelen with strlen compare
      */
     if (bytelen > 0 && bytelen < (INTVAL)string_length(interp, output))
-        string_substr(interp, output, 0, bytelen, &output, 1);
+        output = string_substr(interp, output, 0, bytelen, &output, 1);
 
     if (dest == NULL)
         return output;
@@ -1758,7 +1759,8 @@ Writes and returns a Parrot string.
 */
 
 PARROT_API
-STRING*
+PARROT_CANNOT_RETURN_NULL
+STRING *
 string_printf(PARROT_INTERP, NOTNULL(const char *format), ...)
 {
     STRING *output;
@@ -1796,46 +1798,46 @@ number, rounding towards zero.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-string_to_int(SHIM_INTERP, NULLOK(const STRING *s))
+string_to_int(SHIM_INTERP, NOTNULL(const STRING *s))
 {
     INTVAL i = 0;
 
-    if (s) {
-        const char         *start = s->strstart;
-        const char * const  end   = start + s->bufused;
+    const char         *start = s->strstart;
+    const char * const  end   = start + s->bufused;
 
-        int    sign      = 1;
-        INTVAL in_number = 0;
+    int    sign      = 1;
+    INTVAL in_number = 0;
 
-        while (start < end) {
-            const unsigned char c = *start;
+    PARROT_ASSERT(s);
 
-            if (isdigit(c)) {
-                in_number = 1;
-                i = i * 10 + (c - '0');
-            }
-            else if (!in_number) {
-                /* we've not yet seen any digits */
-                if (c == '-') {
-                    sign = -1;
-                    in_number = 1;
-                }
-                else if (c == '+')
-                    in_number = 1;
-                else if (isspace(c))
-                    ;
-                else
-                    break;
-            }
-            else {
-                break;
-            }
-            ++start;
+    while (start < end) {
+        const unsigned char c = *start;
+
+        if (isdigit((unsigned char)c)) {
+            in_number = 1;
+            i = i * 10 + (c - '0');
         }
-
-        i = i * sign;
+        else if (!in_number) {
+            /* we've not yet seen any digits */
+            if (c == '-') {
+                sign = -1;
+                in_number = 1;
+            }
+            else if (c == '+')
+                in_number = 1;
+            else if (isspace((unsigned char)c))
+                ;
+            else
+                break;
+        }
+        else {
+            break;
+        }
+        ++start;
     }
 
+    i = i * sign;
+
     return i;
 }
 
@@ -1851,35 +1853,37 @@ returned.
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 FLOATVAL
-string_to_num(PARROT_INTERP, NULLOK(const STRING *s))
+string_to_num(PARROT_INTERP, NOTNULL(const STRING *s))
 {
     FLOATVAL f = 0.0;
+    char *cstr;
+    const char *p;
     DECL_CONST_CAST;
 
-    if (s) {
-        /*
-         * XXX C99 atof interprets 0x prefix
-         * XXX would strtod() be better for detecting malformed input?
-         */
-        char * const  cstr = string_to_cstring(interp, (STRING *)const_cast(s));
-        const char   *p    = cstr;
+    PARROT_ASSERT(s);
 
-        while (isspace(*p))
-            p++;
+    /*
+     * XXX C99 atof interprets 0x prefix
+     * XXX would strtod() be better for detecting malformed input?
+     */
+    cstr = string_to_cstring(interp, (STRING *)const_cast(s));
+    p    = cstr;
 
-        f = atof(p);
+    while (isspace((unsigned char)*p))
+        p++;
 
-        /* Not all atof()s return -0 from "-0" */
-        if (*p == '-' && f == 0.0)
+    f = atof(p);
+
+    /* Not all atof()s return -0 from "-0" */
+    if (*p == '-' && FLOAT_IS_ZERO(f))
 #if defined(_MSC_VER)
-            /* Visual C++ compiles -0.0 to 0.0, so we need to trick
-               the compiler. */
-            f = 0.0 * -1;
+        /* Visual C++ compiles -0.0 to 0.0, so we need to trick
+            the compiler. */
+        f = 0.0 * -1;
 #else
-            f = -0.0;
+        f = -0.0;
 #endif
-        string_cstring_free(cstr);
-    }
+    string_cstring_free(cstr);
 
     return f;
 }
@@ -1894,6 +1898,7 @@ Returns a Parrot string representation of the specified integer value.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_from_int(PARROT_INTERP, INTVAL i)
 {
@@ -1912,6 +1917,7 @@ value.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_from_num(PARROT_INTERP, FLOATVAL f)
 {
@@ -1932,20 +1938,14 @@ result in a memory leak.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_MALLOC
-PARROT_CAN_RETURN_NULL
+PARROT_CANNOT_RETURN_NULL
 char *
-string_to_cstring(SHIM_INTERP, NULLOK(const STRING *s))
+string_to_cstring(SHIM_INTERP, NOTNULL(const STRING *s))
 {
     char *p;
 
-    /*
-     * TODO always provide a NUL at end of strings
-     *      ICU needs this too for a lot of string functions
-     */
-    if (s == NULL)
-        return NULL;
+    PARROT_ASSERT(s);
 
     p = (char *)mem_sys_allocate(s->bufused + 1);
     memcpy(p, s->strstart, s->bufused);
@@ -2091,6 +2091,7 @@ greater than this as I<\x{hh...hh}>.
 */
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 STRING *
 string_escape_string(PARROT_INTERP, NULLOK(const STRING *src))
 {
@@ -2189,7 +2190,7 @@ string_escape_string_delimited(PARROT_INTERP,
             }
             if (c >= 0x20) {
                 dp[i++] = (unsigned char)c;
-                assert(i < charlen);
+                PARROT_ASSERT(i < charlen);
                 continue;
             }
         }
@@ -2211,7 +2212,7 @@ string_escape_string_delimited(PARROT_INTERP,
         charlen = PObj_buflen(result);
         dp      = (unsigned char *)result->strstart;
 
-        assert(i < charlen);
+        PARROT_ASSERT(i < charlen);
     }
 
     result->bufused = result->strlen = i;
@@ -2235,6 +2236,7 @@ Unescapes the specified C string. These sequences are covered:
 */
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING *
 string_unescape_cstring(PARROT_INTERP,
     NOTNULL(const char *cstring), char delimiter, NULLOK(const char *enc_char))
@@ -2245,8 +2247,8 @@ string_unescape_cstring(PARROT_INTERP,
     Parrot_UInt4 r;
     UINTVAL flags;
     String_iter iter;
-    ENCODING *encoding;
-    CHARSET *charset;
+    const ENCODING *encoding;
+    const CHARSET *charset;
     char *p;
 
     if (delimiter && clength)
@@ -2289,7 +2291,7 @@ string_unescape_cstring(PARROT_INTERP,
         r = (Parrot_UInt4)((unsigned char*)result->strstart)[offs];
 
         /* There cannot be any NULs within this string.  */
-        assert(r != '\0');
+        PARROT_ASSERT(r != '\0');
 
         if (r == '\\') {
             ++offs;
@@ -2305,7 +2307,7 @@ string_unescape_cstring(PARROT_INTERP,
             continue;
         }
 
-        assert(d < offs);
+        PARROT_ASSERT(d < offs);
         iter.set_and_advance(interp, &iter, r);
         ++d;
     }
@@ -2345,7 +2347,7 @@ TODO - implemented only for ASCII.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING *
 string_upcase(PARROT_INTERP, NOTNULL(const STRING *s))
@@ -2382,7 +2384,7 @@ Non-caseable characters are left unchanged.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING *
 string_downcase(PARROT_INTERP, NOTNULL(const STRING *s))
@@ -2425,7 +2427,7 @@ Non-caseable characters are left unchanged.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING *
 string_titlecase(PARROT_INTERP, NOTNULL(const STRING *s))
@@ -2463,10 +2465,12 @@ PARROT_API
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
 STRING *
-string_increment(PARROT_INTERP, NULLOK(const STRING *s))
+string_increment(PARROT_INTERP, NOTNULL(const STRING *s))
 {
     INTVAL o;
 
+    PARROT_ASSERT(s);
+
     if (string_length(interp, s) != 1)
         real_exception(interp, NULL, UNIMPLEMENTED, "increment only for length=1 done");
 
@@ -2491,8 +2495,8 @@ as constants -- i.e. do not resize the result.
 */
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
+PARROT_CANNOT_RETURN_NULL
 const char *
 Parrot_string_cstring(SHIM_INTERP, NOTNULL(const STRING *str))
 {
@@ -2562,7 +2566,7 @@ STRING*
 Parrot_string_trans_charset(PARROT_INTERP, NULLOK(STRING *src),
         INTVAL charset_nr, NULLOK(STRING *dest))
 {
-    CHARSET *new_charset;
+    const CHARSET *new_charset;
 
     if (!src)
         return NULL;
@@ -2616,7 +2620,7 @@ STRING*
 Parrot_string_trans_encoding(PARROT_INTERP, NULLOK(STRING *src),
         INTVAL encoding_nr, NULLOK(STRING *dest))
 {
-    ENCODING *new_encoding;
+    const ENCODING *new_encoding;
 
     if (!src)
         return NULL;
@@ -2678,12 +2682,14 @@ string_join(PARROT_INTERP, NULLOK(STRING *j), NOTNULL(PMC *ar))
         return string_make_empty(interp, enum_stringrep_one, 0);
 
     s   = VTABLE_get_string_keyed_int(interp, ar, 0);
-    res = string_copy(interp, s);
+    res = s ? string_copy(interp, s) : NULL;
 
     for (i = 1; i < ar_len; ++i) {
+        STRING *next;
+
         res = string_append(interp, res, j);
-        s   = VTABLE_get_string_keyed_int(interp, ar, i);
-        res = string_append(interp, res, s);
+        next = VTABLE_get_string_keyed_int(interp, ar, i);
+        res = string_append(interp, res, next);
     }
 
     return res;
@@ -2691,6 +2697,7 @@ string_join(PARROT_INTERP, NULLOK(STRING *j), NOTNULL(PMC *ar))
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC*
 string_split(PARROT_INTERP, NOTNULL(STRING *delim), NOTNULL(STRING *str))
 {
@@ -2760,6 +2767,7 @@ If C<minus> is true then C<-> is prepended to the string representation.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING*
 uint_to_str(PARROT_INTERP,
             NOTNULL(char *tc), UHUGEINTVAL num, char base, int minus)
@@ -2768,7 +2776,7 @@ uint_to_str(PARROT_INTERP,
     char *p = tc + sizeof (UHUGEINTVAL)*8 + 1;
     const char * const tail = p;
 
-    assert(base >= 2 && base <= 36);
+    PARROT_ASSERT(base >= 2 && base <= 36);
     do {
         const char cur = (char)(num % base);
         if (cur < 10) {
@@ -2799,6 +2807,7 @@ If C<< num < 0 >> then C<-> is prepended to the string representation.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING *
 int_to_str(PARROT_INTERP, NOTNULL(char *tc), HUGEINTVAL num, char base)
 {
