@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: trace.c 19882 2007-07-14 22:33:30Z petdance $
+$Id: trace.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -26,6 +26,7 @@ src/test_main.c
 /* HEADERIZER BEGIN: static */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING* trace_class_name( NOTNULL(const PMC* pmc) )
         __attribute__nonnull__(1);
 
@@ -41,6 +42,7 @@ Prints a PMC to C<stderr>.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING*
 trace_class_name(NOTNULL(const PMC* pmc))
 {
@@ -233,7 +235,7 @@ trace_op_dump(PARROT_INTERP, NOTNULL(const opcode_t *code_start), NOTNULL(const
     int len;
 #define ARGS_COLUMN 40
 
-    assert(debugger);
+    PARROT_ASSERT(debugger);
     sig = NULL; /* silence compiler uninit warning */
 
     s = 1;
