@@ -1,7 +1,7 @@
 /* trace.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: trace.h 19032 2007-06-16 04:15:37Z petdance $
+ *     $Id: trace.h 19833 2007-07-13 22:02:25Z petdance $
  *  Overview:
  *     Tracing support for runops_cores.c.
  *  Data Structure and Algorithms:
@@ -17,31 +17,31 @@
 
 /* HEADERIZER BEGIN: src/trace.c */
 
-int trace_key_dump( Interp *interp /*NN*/, const PMC *key /*NN*/ )
+int trace_key_dump( PARROT_INTERP, NOTNULL(const PMC *key) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void trace_op( Interp *interp,
-    const opcode_t *code_start,
-    const opcode_t *code_end,
-    const opcode_t *pc /*NULLOK*/ );
-
-void trace_op_dump( Interp *interp /*NN*/,
-    const opcode_t *code_start,
-    const opcode_t *pc /*NN*/ )
+void trace_op( PARROT_INTERP,
+    NOTNULL(const opcode_t *code_start),
+    NOTNULL(const opcode_t *code_end),
+    NULLOK(const opcode_t *pc) )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-void trace_pmc_dump( Interp *interp /*NN*/, PMC* pmc /*NN*/ )
+void trace_op_dump( PARROT_INTERP,
+    NOTNULL(const opcode_t *code_start),
+    NOTNULL(const opcode_t *pc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+void trace_pmc_dump( PARROT_INTERP, NOTNULL(PMC *pmc) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 /* HEADERIZER END: src/trace.c */
 
-
-void trace_op(Interp *interp, const opcode_t * code_start,
-              const opcode_t * code_end, const opcode_t * pc);
-
 #endif /* PARROT_TRACE_H_GUARD */
 
 
